@@ -1,8 +1,34 @@
 /* ldapdelete.c - simple program to delete an entry using LDAP */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.99 2003/04/06 00:59:15 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapdelete.c,v 1.109 2004/11/10 02:51:05 jongchoi Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1998-2003 Kurt D. Zeilenga.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1992-1996 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor.  The name of the
+ * University may not be used to endorse or promote products derived
+ * from this software without specific prior written permission.  This
+ * software is provided ``as is'' without express or implied warranty.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).  Additional significant contributors
+ * include:
+ *   Kurt D. Zeilenga
  */
 
 #include "portable.h"
@@ -48,7 +74,7 @@ usage( void )
 
 
 const char options[] = "r"
-	"cCd:D:e:f:h:H:IkKMnO:p:P:QR:U:vVw:WxX:y:Y:Z";
+	"cd:D:e:f:h:H:IkKMnO:p:P:QR:U:vVw:WxX:y:Y:Z";
 
 int
 handle_private_option( int i )
@@ -57,7 +83,7 @@ handle_private_option( int i )
 #if 0
 		int crit;
 		char *control, *cvalue;
-	case 'E': /* delete controls */
+	case 'E': /* delete extensions */
 		if( protocol == LDAP_VERSION2 ) {
 			fprintf( stderr, _("%s: -E incompatible with LDAPv%d\n"),
 				prog, protocol );
@@ -79,7 +105,7 @@ handle_private_option( int i )
 		if ( (cvalue = strchr( control, '=' )) != NULL ) {
 			*cvalue++ = '\0';
 		}
-		fprintf( stderr, _("Invalid delete control name: %s\n"), control );
+		fprintf( stderr, _("Invalid delete extension name: %s\n"), control );
 		usage();
 #endif
 
@@ -143,8 +169,9 @@ main( int argc, char **argv )
 
 	tool_bind( ld );
 
-	if ( authzid || manageDSAit || noop )
+	if ( assertion || authzid || manageDSAit || noop ) {
 		tool_server_controls( ld, NULL, 0 );
+	}
 
 	retval = rc = 0;
 
@@ -170,7 +197,7 @@ main( int argc, char **argv )
 		}
 	}
 
-    ldap_unbind( ld );
+	ldap_unbind_ext( ld, NULL, NULL );
 
     return( retval );
 }
@@ -250,7 +277,6 @@ static int dodelete(
 
 /*
  * Delete all the children of an entry recursively until leaf nodes are reached.
- *
  */
 static int deletechildren(
 	LDAP *ld,
@@ -259,9 +285,13 @@ static int deletechildren(
 	LDAPMessage *res, *e;
 	int entries;
 	int rc;
-	static char *attrs[] = { "1.1", NULL };
+	static char *attrs[] = { LDAP_NO_ATTRS, NULL };
+	LDAPControl c, *ctrls[2];
+	BerElement *ber = NULL;
+	LDAPMessage *res_se;
 
 	if ( verbose ) printf ( _("deleting children of: %s\n"), dn );
+
 	/*
 	 * Do a one level search at dn for children.  For each, delete its children.
 	 */
@@ -318,5 +348,73 @@ static int deletechildren(
 	}
 
 	ldap_msgfree( res );
+
+	/*
+	 * Do a one level search at dn for subentry children.
+	 */
+
+	if ((ber = ber_alloc_t(LBER_USE_DER)) == NULL) {
+		return EXIT_FAILURE;
+	}
+	rc = ber_printf( ber, "b", 1 );
+	if ( rc == -1 ) {
+		ber_free( ber, 1 );
+		fprintf( stderr, _("Subentries control encoding error!\n"));
+		return EXIT_FAILURE;
+	}
+	if ( ber_flatten2( ber, &c.ldctl_value, 0 ) == -1 ) {
+		return EXIT_FAILURE;
+	}
+	c.ldctl_oid = LDAP_CONTROL_SUBENTRIES;
+	c.ldctl_iscritical = 1;
+	ctrls[0] = &c;
+	ctrls[1] = NULL;
+
+	rc = ldap_search_ext_s( ld, dn, LDAP_SCOPE_ONELEVEL, NULL, attrs, 1,
+		ctrls, NULL, NULL, -1, &res_se );
+	if ( rc != LDAP_SUCCESS ) {
+		ldap_perror( ld, "ldap_search" );
+		return( rc );
+	}
+	ber_free( ber, 1 );
+
+	entries = ldap_count_entries( ld, res_se );
+
+	if ( entries > 0 ) {
+		int i;
+
+		for (e = ldap_first_entry( ld, res_se ), i = 0; e != NULL;
+			e = ldap_next_entry( ld, e ), i++ )
+		{
+			char *dn = ldap_get_dn( ld, e );
+
+			if( dn == NULL ) {
+				ldap_perror( ld, "ldap_prune" );
+				ldap_get_option( ld, LDAP_OPT_ERROR_NUMBER, &rc );
+				ber_memfree( dn );
+				return rc;
+			}
+
+			if ( verbose ) {
+				printf( _("\tremoving %s\n"), dn );
+			}
+
+			rc = ldap_delete_s( ld, dn );
+			if ( rc == -1 ) {
+				ldap_perror( ld, "ldap_delete" );
+				ber_memfree( dn );
+				return rc;
+
+			}
+			
+			if ( verbose ) {
+				printf( _("\t%s removed\n"), dn );
+			}
+
+			ber_memfree( dn );
+		}
+	}
+
+	ldap_msgfree( res_se );
 	return rc;
 }
