@@ -1,7 +1,36 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldappasswd.c,v 1.117 2003/04/12 01:46:45 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* ldappasswd -- a tool for change LDAP passwords */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldappasswd.c,v 1.127 2004/08/28 00:30:18 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1998-2003 Kurt D. Zeilenga.
+ * Portions Copyright 1998-2001 Net Boolean Incorporated.
+ * Portions Copyright 2001-2003 IBM Corporation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1992-1996 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor.  The name of the
+ * University may not be used to endorse or promote products derived
+ * from this software without specific prior written permission.  This
+ * software is provided ``as is'' without express or implied warranty.
+ */
+/* ACKNOWLEDGEMENTS:
+ * The original ldappasswd(1) tool was developed by Dave Storey (F5
+ * Network), based on other OpenLDAP client tools (which are, of
+ * course, based on U-MICH LDAP).  This version was rewritten
+ * by Kurt D. Zeilenga (based on other OpenLDAP client tools).
  */
 
 #include "portable.h"
@@ -38,7 +67,7 @@ usage( void )
 {
 	fprintf( stderr, _("Change password of an LDAP user\n\n"));
 	fprintf( stderr,_("usage: %s [options] [user]\n"), prog);
-	fprintf( stderr, _("  user: the autentication identity, commonly a DN\n"));
+	fprintf( stderr, _("  user: the authentication identity, commonly a DN\n"));
 	fprintf( stderr, _("Password change options:\n"));
 	fprintf( stderr, _("  -a secret  old password\n"));
 	fprintf( stderr, _("  -A         prompt for old password\n"));
@@ -52,14 +81,14 @@ usage( void )
 
 
 const char options[] = "a:As:St:T:"
-	"Cd:D:e:h:H:InO:p:QR:U:vVw:WxX:y:Y:Z";
+	"d:D:e:h:H:InO:p:QR:U:vVw:WxX:y:Y:Z";
 
 int
 handle_private_option( int i )
 {
 	switch ( i ) {
 #if 0
-	case 'E': /* passwd controls */ {
+	case 'E': /* passwd extensions */ {
 		int		crit;
 		char	*control, *cvalue;
 		if( protocol == LDAP_VERSION2 ) {
@@ -83,7 +112,7 @@ handle_private_option( int i )
 		if ( (cvalue = strchr( control, '=' )) != NULL ) {
 			*cvalue++ = '\0';
 		}
-		fprintf( stderr, _("Invalid passwd control name: %s\n"), control );
+		fprintf( stderr, _("Invalid passwd extension name: %s\n"), control );
 		usage();
 		}
 #endif
@@ -166,7 +195,7 @@ main( int argc, char *argv[] )
 	}
 
 	if( oldpwfile ) {
-		rc = lutil_get_filed_password( prog, &oldpw );
+		rc = lutil_get_filed_password( oldpwfile, &oldpw );
 		if( rc ) return EXIT_FAILURE;
 	}
 
@@ -187,7 +216,7 @@ main( int argc, char *argv[] )
 	}
 
 	if( newpwfile ) {
-		rc = lutil_get_filed_password( prog, &newpw );
+		rc = lutil_get_filed_password( newpwfile, &newpw );
 		if( rc ) return EXIT_FAILURE;
 	}
 
@@ -207,22 +236,22 @@ main( int argc, char *argv[] )
 		newpw.bv_len = strlen( newpw.bv_val );
 	}
 
-	if( want_bindpw && passwd.bv_val == NULL ) {
-		if ( pw_file ) {
-			rc = lutil_get_filed_password( pw_file, &passwd );
-			if( rc ) return EXIT_FAILURE;
-		} else {
-			passwd.bv_val = getpassphrase( _("Enter LDAP Password: ") );
-			passwd.bv_len = passwd.bv_val ? strlen( passwd.bv_val ) : 0;
-		}
+	if ( pw_file ) {
+		rc = lutil_get_filed_password( pw_file, &passwd );
+		if( rc ) return EXIT_FAILURE;
+
+	} else if ( want_bindpw ) {
+		passwd.bv_val = getpassphrase( _("Enter LDAP Password: ") );
+		passwd.bv_len = passwd.bv_val ? strlen( passwd.bv_val ) : 0;
 	}
 
 	ld = tool_conn_setup( 0, 0 );
 
 	tool_bind( ld );
 
-	if ( authzid || manageDSAit || noop )
+	if ( assertion || authzid || manageDSAit || noop ) {
 		tool_server_controls( ld, NULL, 0 );
+	}
 
 	if( user != NULL || oldpw.bv_val != NULL || newpw.bv_val != NULL ) {
 		/* build change password control */
@@ -230,7 +259,7 @@ main( int argc, char *argv[] )
 
 		if( ber == NULL ) {
 			perror( "ber_alloc_t" );
-			ldap_unbind( ld );
+			ldap_unbind_ext( ld, NULL, NULL );
 			return EXIT_FAILURE;
 		}
 
@@ -260,7 +289,7 @@ main( int argc, char *argv[] )
 
 		if( rc < 0 ) {
 			perror( "ber_flatten2" );
-			ldap_unbind( ld );
+			ldap_unbind_ext( ld, NULL, NULL );
 			return EXIT_FAILURE;
 		}
 	}
@@ -278,7 +307,7 @@ main( int argc, char *argv[] )
 
 	if( rc != LDAP_SUCCESS ) {
 		ldap_perror( ld, "ldap_extended_operation" );
-		ldap_unbind( ld );
+		ldap_unbind_ext( ld, NULL, NULL );
 		return EXIT_FAILURE;
 	}
 
@@ -310,7 +339,7 @@ main( int argc, char *argv[] )
 
 		if( ber == NULL ) {
 			perror( "ber_init" );
-			ldap_unbind( ld );
+			ldap_unbind_ext( ld, NULL, NULL );
 			return EXIT_FAILURE;
 		}
 
@@ -354,7 +383,7 @@ main( int argc, char *argv[] )
 
 skip:
 	/* disconnect from server */
-	ldap_unbind (ld);
+	ldap_unbind_ext( ld, NULL, NULL );
 
 	return code == LDAP_SUCCESS ? EXIT_SUCCESS : EXIT_FAILURE;
 }
