@@ -1,7 +1,39 @@
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.170.2.1 2003/05/31 19:01:39 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* ldapsearch -- a tool for searching LDAP directories */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapsearch.c,v 1.207.2.1 2004/12/29 16:37:58 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1998-2003 Kurt D. Zeilenga.
+ * Portions Copyright 1998-2001 Net Boolean Incorporated.
+ * Portions Copyright 2001-2003 IBM Corporation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1992-1996 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor.  The name of the
+ * University may not be used to endorse or promote products derived
+ * from this software without specific prior written permission.  This
+ * software is provided ``as is'' without express or implied warranty.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).  Additional significant contributors
+ * include:
+ *   Jong Hyuk Choi
+ *   Lynn Moss
+ *   Mikhail Sahalaev
+ *   Kurt D. Zeilenga
  */
 
 #include "portable.h"
@@ -47,6 +79,12 @@ static int sizelimit = -1;
 static char *def_tmpdir;
 static char *def_urlpre;
 
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+/* Turn off commandline globbing, otherwise you cannot search for
+ * attribute '*'
+ */
+int _CRT_glob = 0;
+#endif
 
 void
 usage( void )
@@ -64,41 +102,36 @@ usage( void )
 	fprintf( stderr, _("  -a deref   one of never (default), always, search, or find\n"));
 	fprintf( stderr, _("  -A         retrieve attribute names only (no values)\n"));
 	fprintf( stderr, _("  -b basedn  base dn for search\n"));
-	fprintf( stderr, _("  -E [!]<ctrl>[=<ctrlparam>] search controls (! indicates criticality)\n"));
+	fprintf( stderr, _("  -E [!]<ext>[=<extparam>] search extensions (! indicates criticality)\n"));
 #ifdef LDAP_CONTROL_X_DOMAIN_SCOPE
 	fprintf( stderr, _("             [!]domainScope              (domain scope)\n"));
 #endif
 	fprintf( stderr, _("             [!]mv=<filter>              (matched values filter)\n"));
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-	fprintf( stderr, _("             [!]pr=<size>                (paged results)\n"));
+	fprintf( stderr, _("             [!]pr=<size>[/prompt|noprompt]   (paged results/prompt)\n"));
 #endif
 #ifdef LDAP_CONTROL_SUBENTRIES
 	fprintf( stderr, _("             [!]subentries[=true|false]  (subentries)\n"));
 #endif
-#ifdef LDAP_CLIENT_UPDATE
-	fprintf( stderr, _("             [!]lcup=p/<cint>/<cookie>/<slimit> (LDAP client update)\n"));
-/*
- * "                      s/<cint>/<cookie>  (LDAP client update)\n"
- * "                     sp/<cint>/<cookie>/<slimit>\n"
- * */
-#endif
-#ifdef LDAP_SYNC
 	fprintf( stderr, _("             [!]sync=ro[/<cookie>]            (LDAP Sync refreshOnly)\n"));
 	fprintf( stderr, _("                     rp[/<cookie>][/<slimit>] (LDAP Sync refreshAndPersist)\n"));
-#endif
 	fprintf( stderr, _("  -F prefix  URL prefix for files (default: %s)\n"), def_urlpre);
-	fprintf( stderr, _("  -l limit   time limit (in seconds) for search\n"));
+	fprintf( stderr, _("  -l limit   time limit (in seconds, or \"none\" or \"max\") for search\n"));
 	fprintf( stderr, _("  -L         print responses in LDIFv1 format\n"));
 	fprintf( stderr, _("  -LL        print responses in LDIF format without comments\n"));
 	fprintf( stderr, _("  -LLL       print responses in LDIF format without comments\n"));
 	fprintf( stderr, _("             and version\n"));
+#ifdef LDAP_SCOPE_SUBORDINATE
+	fprintf( stderr, _("  -s scope   one of base, one, sub or children (search scope)\n"));
+#else /* ! LDAP_SCOPE_SUBORDINATE */
 	fprintf( stderr, _("  -s scope   one of base, one, or sub (search scope)\n"));
+#endif /* ! LDAP_SCOPE_SUBORDINATE */
 	fprintf( stderr, _("  -S attr    sort the results by attribute `attr'\n"));
 	fprintf( stderr, _("  -t         write binary values to files in temporary directory\n"));
 	fprintf( stderr, _("  -tt        write all values to files in temporary directory\n"));
 	fprintf( stderr, _("  -T path    write files to directory specified by path (default: %s)\n"), def_tmpdir);
 	fprintf( stderr, _("  -u         include User Friendly entry names in the output\n"));
-	fprintf( stderr, _("  -z limit   size limit (in entries) for search\n"));
+	fprintf( stderr, _("  -z limit   size limit (in entries, or \"none\" or \"max\") for search\n"));
 	tool_common_usage();
 	exit( EXIT_FAILURE );
 }
@@ -160,28 +193,17 @@ static char	*vrFilter = NULL;
 static int domainScope = 0;
 #endif
 
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
-static int lcup = 0;
 static int ldapsync = 0;
-#endif
-
-#ifdef LDAP_CLIENT_UPDATE
-static int lcup_cint = 0;
-static struct berval lcup_cookie = { 0, NULL };
-static int lcup_slimit = -1;
-#endif
-
-#ifdef LDAP_SYNC
 static struct berval sync_cookie = { 0, NULL };
 static int sync_slimit = -1;
-#endif
 
 #ifdef LDAP_CONTROL_PAGEDRESULTS
 static int pagedResults = 0;
+static int pagePrompt = 1;
 static ber_int_t pageSize = 0;
 static ber_int_t entriesLeft = 0;
 static ber_int_t morePagedResults = 1;
-static struct berval cookie = { 0, NULL };
+static struct berval page_cookie = { 0, NULL };
 static int npagedresponses;
 static int npagedentries;
 static int npagedreferences;
@@ -214,30 +236,31 @@ const char options[] = "a:Ab:E:F:l:Ls:S:tT:uz:"
 int
 handle_private_option( int i )
 {
-	int crit;
-	char *control, *cvalue;
+	int crit, ival;
+	char *control, *cvalue, *next;
 	switch ( i ) {
 	case 'a':	/* set alias deref option */
 		if ( strcasecmp( optarg, "never" ) == 0 ) {
-		deref = LDAP_DEREF_NEVER;
+			deref = LDAP_DEREF_NEVER;
 		} else if ( strncasecmp( optarg, "search", sizeof("search")-1 ) == 0 ) {
-		deref = LDAP_DEREF_SEARCHING;
+			deref = LDAP_DEREF_SEARCHING;
 		} else if ( strncasecmp( optarg, "find", sizeof("find")-1 ) == 0 ) {
-		deref = LDAP_DEREF_FINDING;
+			deref = LDAP_DEREF_FINDING;
 		} else if ( strcasecmp( optarg, "always" ) == 0 ) {
-		deref = LDAP_DEREF_ALWAYS;
+			deref = LDAP_DEREF_ALWAYS;
 		} else {
-		fprintf( stderr, _("alias deref should be never, search, find, or always\n") );
-		usage();
+			fprintf( stderr,
+				_("alias deref should be never, search, find, or always\n") );
+			usage();
 		}
 		break;
 	case 'A':	/* retrieve attribute names only -- no values */
 		++attrsonly;
 		break;
 	case 'b': /* search base */
-		base = strdup( optarg );
+		base = ber_strdup( optarg );
 		break;
-	case 'E': /* search controls */
+	case 'E': /* search extensions */
 		if( protocol == LDAP_VERSION2 ) {
 			fprintf( stderr, _("%s: -E incompatible with LDAPv%d\n"),
 				prog, protocol );
@@ -255,7 +278,7 @@ handle_private_option( int i )
 			optarg++;
 		}
 
-		control = strdup( optarg );
+		control = ber_strdup( optarg );
 		if ( (cvalue = strchr( control, '=' )) != NULL ) {
 			*cvalue++ = '\0';
 		}
@@ -263,7 +286,8 @@ handle_private_option( int i )
 		if ( strcasecmp( control, "mv" ) == 0 ) {
 			/* ValuesReturnFilter control */
 			if( valuesReturnFilter ) {
-				fprintf( stderr, _("ValuesReturnFilter previously specified\n"));
+				fprintf( stderr,
+					_("ValuesReturnFilter previously specified\n"));
 				exit( EXIT_FAILURE );
 			}
 			valuesReturnFilter= 1 + crit;
@@ -282,15 +306,39 @@ handle_private_option( int i )
 			int num, tmp;
 			/* PagedResults control */
 			if ( pagedResults != 0 ) {
-				fprintf( stderr, _("PagedResultsControl previously specified\n") );
+				fprintf( stderr,
+					_("PagedResultsControl previously specified\n") );
 				exit( EXIT_FAILURE );
 			}
-			
-			num = sscanf( cvalue, "%d", &tmp );
-			if ( num != 1 ) {
-				fprintf( stderr, _("Invalid value for PagedResultsControl, %s.\n"), cvalue);
-				exit( EXIT_FAILURE );
 
+			if( cvalue != NULL ) {
+				char *promptp;
+
+				promptp = strchr( cvalue, '/' );
+				if ( promptp != NULL ) {
+					*promptp++ = '\0';
+					if ( strcasecmp( promptp, "prompt" ) == 0 ) {
+						pagePrompt = 1;
+					} else if ( strcasecmp( promptp, "noprompt" ) == 0) {
+						pagePrompt = 0;
+					} else {
+						fprintf( stderr,
+							_("Invalid value for PagedResultsControl,"
+							" %s/%s.\n"), cvalue, promptp );
+						exit( EXIT_FAILURE );
+					}
+				}
+				num = sscanf( cvalue, "%d", &tmp );
+				if ( num != 1 ) {
+					fprintf( stderr,
+						_("Invalid value for PagedResultsControl, %s.\n"),
+						cvalue );
+					exit( EXIT_FAILURE );
+				}
+			} else {
+				fprintf( stderr, _("Invalid value for PagedResultsControl.\n"),
+					cvalue);
+				exit( EXIT_FAILURE );
 			}
 			pageSize = (ber_int_t) tmp;
 			pagedResults = 1 + crit;
@@ -314,7 +362,8 @@ handle_private_option( int i )
 #ifdef LDAP_CONTROL_SUBENTRIES
 		} else if ( strcasecmp( control, "subentries" ) == 0 ) {
 			if( subentries ) {
-				fprintf( stderr, _("subentries control previously specified\n"));
+				fprintf( stderr,
+					_("subentries control previously specified\n"));
 				exit( EXIT_FAILURE );
 			}
 			if( cvalue == NULL || strcasecmp( cvalue, "true") == 0 ) {
@@ -330,81 +379,15 @@ handle_private_option( int i )
 			if( crit ) subentries *= -1;
 #endif
 
-#ifdef LDAP_CLIENT_UPDATE
-                } else if ( strcasecmp( control, "lcup" ) == 0 ) {
-                        char *cookiep;
-                        char *slimitp;
-                        if ( lcup ) {
-                                fprintf( stderr, _("client update control previously specified\n"));
-                                exit( EXIT_FAILURE );
-                        }
-                        if ( ldapsync != -1 ) {
-                                fprintf( stderr, _("ldap sync control previously specified\n"));
-                                exit( EXIT_FAILURE );
-                        }
-                        if ( cvalue == NULL ) {
-                                fprintf( stderr,
-                                        _("missing specification of client update control\n"));
-                                exit( EXIT_FAILURE );
-                        }
-                        if ( strncasecmp( cvalue, "p", 1 ) == 0 ) {
-                                lcup = LDAP_CUP_PERSIST_ONLY;
-                                cvalue = strchr( cvalue, '/' );
-                                cvalue++;
-                                cookiep = strchr( cvalue, '/' );
-                                *cookiep++ = '\0';
-                                lcup_cint = atoi( cvalue );
-                                cvalue = cookiep;
-                                slimitp = strchr( cvalue, '/' );
-                                *slimitp++ = '\0';
-                                while ( isspace( (unsigned char) *cookiep ) )
-                                    cookiep++;
-                                ber_str2bv( cookiep, 0, 0, &lcup_cookie );
-                                lcup_slimit = atoi( slimitp );
-/*
-			} else if ( strncasecmp( cvalue, "s", 1 ) == 0 ) {
-				lcup = LDAP_CUP_SYNC_ONLY;
-				cvalue += 2;
-				cookiep = strchr( cvalue, '/' );
-				*cookiep++ = '\0';
-				lcup_cint = atoi( cvalue );
-				ber_str2bv( cookiep, 0, 0, &lcup_cookie );
-			} else if ( strncasecmp( cvalue, "sp", 2 ) == 0 ) {
-				lcup = LDAP_CUP_SYNC_AND_PERSIST;
-				cvalue += 3;
-				cookiep = strchr( cvalue, '/' );
-				*cookiep++ = '\0';
-				lcup_cint = atoi( cvalue );
-				cvalue = cookiep;
-				slimitp = strchr( cvalue, '/' );
-				*slimitp++ = '\0';
-				ber_str2bv( cookiep, 0, 0, &lcup_cookie );
-				lcup_slimit = atoi( slimitp );
-*/
-			} else {
-				fprintf( stderr,
-					_("client update control value \"%s\" invalid\n"),
-					cvalue );
-				exit( EXIT_FAILURE );
-			}
-			if ( crit ) lcup *= -1;
-#endif
-
-#ifdef LDAP_SYNC
 	} else if ( strcasecmp( control, "sync" ) == 0 ) {
 			char *cookiep;
 			char *slimitp;
 			if ( ldapsync ) {
-				fprintf( stderr, _("ldap sync control previously specified\n") );
-				exit( EXIT_FAILURE );
-			}
-			if ( lcup ) {
-				fprintf( stderr, _("client update control previously specified\n") );
+				fprintf( stderr, _("sync control previously specified\n") );
 				exit( EXIT_FAILURE );
 			}
 			if ( cvalue == NULL ) {
-				fprintf( stderr,
-					_("missing specification of ldap sync control\n"));
+				fprintf( stderr, _("missing specification of sync control\n"));
 				exit( EXIT_FAILURE );
 			}
 			if ( strncasecmp( cvalue, "ro", 2 ) == 0 ) {
@@ -429,19 +412,24 @@ handle_private_option( int i )
 				}
 				if ( cookiep != NULL && *cookiep != '\0' )
 					ber_str2bv( cookiep, 0, 0, &sync_cookie );
-				if ( slimitp != NULL && *slimitp != '\0' )
-					sync_slimit = atoi( slimitp );
+				if ( slimitp != NULL && *slimitp != '\0' ) {
+					ival = strtol( slimitp, &next, 10 );
+					if ( next == NULL || next[0] != '\0' ) {
+						fprintf( stderr, _("Unable to parse sync control value \"%s\"\n"), slimitp );
+						exit( EXIT_FAILURE );
+					}
+					sync_slimit = ival;
+				}
 			} else {
-				fprintf( stderr,
-					_("ldap sync control value \"%s\" invalid\n"),
+				fprintf( stderr, _("sync control value \"%s\" invalid\n"),
 					cvalue );
 				exit( EXIT_FAILURE );
 			}
 			if ( crit ) ldapsync *= -1;
-#endif
 
 		} else {
-			fprintf( stderr, _("Invalid control name: %s\n"), control );
+			fprintf( stderr, _("Invalid search extension name: %s\n"),
+				control );
 			usage();
 		}
 		break;
@@ -450,8 +438,22 @@ handle_private_option( int i )
 		urlpre = strdup( optarg );
 		break;
 	case 'l':	/* time limit */
-		timelimit = atoi( optarg );
-		if( timelimit < 0 ) {
+		if ( strcasecmp( optarg, "none" ) == 0 ) {
+			timelimit = 0;
+
+		} else if ( strcasecmp( optarg, "max" ) == 0 ) {
+			timelimit = LDAP_MAXINT;
+
+		} else {
+			ival = strtol( optarg, &next, 10 );
+			if ( next == NULL || next[0] != '\0' ) {
+				fprintf( stderr,
+					_("Unable to parse time limit \"%s\"\n"), optarg );
+				exit( EXIT_FAILURE );
+			}
+			timelimit = ival;
+		}
+		if( timelimit < 0 || timelimit > LDAP_MAXINT ) {
 			fprintf( stderr, _("%s: invalid timelimit (%d) specified\n"),
 				prog, timelimit );
 			exit( EXIT_FAILURE );
@@ -461,15 +463,21 @@ handle_private_option( int i )
 		++ldif;
 		break;
 	case 's':	/* search scope */
-		if ( strcasecmp( optarg, "base" ) == 0 ) {
-		scope = LDAP_SCOPE_BASE;
+		if ( strncasecmp( optarg, "base", sizeof("base"-1) ) == 0 ) {
+			scope = LDAP_SCOPE_BASE;
 		} else if ( strncasecmp( optarg, "one", sizeof("one")-1 ) == 0 ) {
-		scope = LDAP_SCOPE_ONELEVEL;
+			scope = LDAP_SCOPE_ONELEVEL;
+#ifdef LDAP_SCOPE_SUBORDINATE
+		} else if (( strcasecmp( optarg, "subordinate" ) == 0 )
+			|| ( strcasecmp( optarg, "children" ) == 0 ))
+		{
+			scope = LDAP_SCOPE_SUBORDINATE;
+#endif
 		} else if ( strncasecmp( optarg, "sub", sizeof("sub")-1 ) == 0 ) {
-		scope = LDAP_SCOPE_SUBTREE;
+			scope = LDAP_SCOPE_SUBTREE;
 		} else {
-		fprintf( stderr, _("scope should be base, one, or sub\n") );
-		usage();
+			fprintf( stderr, _("scope should be base, one, or sub\n") );
+			usage();
 		}
 		break;
 	case 'S':	/* sort attribute */
@@ -486,7 +494,26 @@ handle_private_option( int i )
 		++includeufn;
 		break;
 	case 'z':	/* size limit */
-		sizelimit = atoi( optarg );
+		if ( strcasecmp( optarg, "none" ) == 0 ) {
+			sizelimit = 0;
+
+		} else if ( strcasecmp( optarg, "max" ) == 0 ) {
+			sizelimit = LDAP_MAXINT;
+
+		} else {
+			ival = strtol( optarg, &next, 10 );
+			if ( next == NULL || next[0] != '\0' ) {
+				fprintf( stderr,
+					_("Unable to parse size limit \"%s\"\n"), optarg );
+				exit( EXIT_FAILURE );
+			}
+			sizelimit = ival;
+		}
+		if( sizelimit < 0 || sizelimit > LDAP_MAXINT ) {
+			fprintf( stderr, _("%s: invalid sizelimit (%d) specified\n"),
+				prog, sizelimit );
+			exit( EXIT_FAILURE );
+		}
 		break;
 	default:
 		return 0;
@@ -499,21 +526,26 @@ static void
 private_conn_setup( LDAP *ld )
 {
 	if (deref != -1 &&
-		ldap_set_option( ld, LDAP_OPT_DEREF, (void *) &deref ) != LDAP_OPT_SUCCESS )
+		ldap_set_option( ld, LDAP_OPT_DEREF, (void *) &deref )
+			!= LDAP_OPT_SUCCESS )
 	{
 		fprintf( stderr, _("Could not set LDAP_OPT_DEREF %d\n"), deref );
 		exit( EXIT_FAILURE );
 	}
-	if (timelimit != -1 &&
-		ldap_set_option( ld, LDAP_OPT_TIMELIMIT, (void *) &timelimit ) != LDAP_OPT_SUCCESS )
+	if (timelimit > 0 &&
+		ldap_set_option( ld, LDAP_OPT_TIMELIMIT, (void *) &timelimit )
+			!= LDAP_OPT_SUCCESS )
 	{
-		fprintf( stderr, _("Could not set LDAP_OPT_TIMELIMIT %d\n"), timelimit );
+		fprintf( stderr,
+			_("Could not set LDAP_OPT_TIMELIMIT %d\n"), timelimit );
 		exit( EXIT_FAILURE );
 	}
-	if (sizelimit != -1 &&
-		ldap_set_option( ld, LDAP_OPT_SIZELIMIT, (void *) &sizelimit ) != LDAP_OPT_SUCCESS )
+	if (sizelimit > 0 &&
+		ldap_set_option( ld, LDAP_OPT_SIZELIMIT, (void *) &sizelimit )
+			!= LDAP_OPT_SUCCESS )
 	{
-		fprintf( stderr, _("Could not set LDAP_OPT_SIZELIMIT %d\n"), sizelimit );
+		fprintf( stderr,
+			_("Could not set LDAP_OPT_SIZELIMIT %d\n"), sizelimit );
 		exit( EXIT_FAILURE );
 	}
 }
@@ -527,15 +559,9 @@ main( int argc, char **argv )
 	int			rc, i, first;
 	LDAP		*ld = NULL;
 	BerElement	*seber = NULL, *vrber = NULL, *prber = NULL;
-#ifdef LDAP_CLIENT_UPDATE
-	BerElement	*cuber = NULL;
-        struct berval   *cubvalp = NULL;
-#endif
 
-#ifdef LDAP_SYNC
 	BerElement      *syncber = NULL;
 	struct berval   *syncbvalp = NULL;
-#endif
 
 	tool_init();
 
@@ -546,8 +572,6 @@ main( int argc, char **argv )
 
 	prog = lutil_progname( "ldapsearch", argc, argv );
 
-	lutil_log_initialize(argc, argv);
-
 	if((def_tmpdir = getenv("TMPDIR")) == NULL &&
 	   (def_tmpdir = getenv("TMP")) == NULL &&
 	   (def_tmpdir = getenv("TEMP")) == NULL )
@@ -633,20 +657,16 @@ main( int argc, char **argv )
 	tool_bind( ld );
 
 getNextPage:
-	if ( manageDSAit || noop || subentries || valuesReturnFilter
+	if ( assertion || authzid || manageDSAit || noop
 #ifdef LDAP_CONTROL_X_DOMAIN_SCOPE
-			|| domainScope
+		|| domainScope
 #endif
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-			|| pageSize
+		|| pagedResults
 #endif
-#ifdef LDAP_CLIENT_UPDATE
-			|| lcup
-#endif
-#ifdef LDAP_SYNC
-			|| ldapsync
-#endif
-			) {
+		|| ldapsync
+		|| subentries || valuesReturnFilter )
+	{
 		int err;
 		int i=0;
 		LDAPControl c[6];
@@ -667,7 +687,7 @@ getNextPage:
 				return EXIT_FAILURE;
 			}
 
-			err = ber_printf( seber, "{b}", abs(subentries) == 1 ? 0 : 1 );
+			err = ber_printf( seber, "b", abs(subentries) == 1 ? 0 : 1 );
 	    	if ( err == -1 ) {
 				ber_free( seber, 1 );
 				fprintf( stderr, _("Subentries control encoding error!\n") );
@@ -684,37 +704,6 @@ getNextPage:
 		}
 #endif
 
-#ifdef LDAP_CLIENT_UPDATE
-                if ( lcup ) {
-                        if (( cuber = ber_alloc_t(LBER_USE_DER)) == NULL ) {
-                                return EXIT_FAILURE;
-                        }
-
-                        if ( lcup_cookie.bv_len == 0 ) {
-                                err = ber_printf( cuber, "{ei}", abs(lcup), lcup_cint );
-                        } else {
-                                err = ber_printf( cuber, "{ei{sO}}", abs(lcup), lcup_cint,
-                                    LDAP_CUP_COOKIE_OID, &lcup_cookie );
-                        }
-
-                        if ( err == LBER_ERROR ) {
-                                ber_free( cuber, 1 );
-                                fprintf( stderr, _("client update control encoding error!\n") );
-                                return EXIT_FAILURE;
-                        }
-
-                        if ( ber_flatten( cuber, &cubvalp ) == LBER_ERROR ) {
-                                return EXIT_FAILURE;
-                        }
-
-                        c[i].ldctl_oid = LDAP_CONTROL_CLIENT_UPDATE;
-                        c[i].ldctl_value = (*cubvalp);
-                        c[i].ldctl_iscritical = lcup < 0;
-                        i++;
-                }
-#endif
-
-#ifdef LDAP_SYNC
 		if ( ldapsync ) {
 			if (( syncber = ber_alloc_t(LBER_USE_DER)) == NULL ) {
 				return EXIT_FAILURE;
@@ -742,7 +731,6 @@ getNextPage:
 			c[i].ldctl_iscritical = ldapsync < 0;
 			i++;
 		}
-#endif
 
 		if ( valuesReturnFilter ) {
 	        if (( vrber = ber_alloc_t(LBER_USE_DER)) == NULL ) {
@@ -770,10 +758,14 @@ getNextPage:
 				return EXIT_FAILURE;
 			}
 
-			ber_printf( prber, "{iO}", pageSize, &cookie );
+			ber_printf( prber, "{iO}", pageSize, &page_cookie );
 			if ( ber_flatten2( prber, &c[i].ldctl_value, 0 ) == -1 ) {
 				return EXIT_FAILURE;
 			}
+			if ( page_cookie.bv_val != NULL ) {
+				ber_memfree( page_cookie.bv_val );
+				page_cookie.bv_val = NULL;
+			}
 			
 			c[i].ldctl_oid = LDAP_CONTROL_PAGEDRESULTS;
 			c[i].ldctl_iscritical = pagedResults > 1;
@@ -783,9 +775,13 @@ getNextPage:
 
 		tool_server_controls( ld, c, i );
 
+#ifdef LDAP_CONTROL_SUBENTRIES
 		ber_free( seber, 1 );
+#endif
 		ber_free( vrber, 1 );
+#ifdef LDAP_CONTROL_PAGEDRESULTS
 		ber_free( prber, 1 );
+#endif
 	}
 	
 	if ( verbose ) {
@@ -813,8 +809,17 @@ getNextPage:
 		printf( "#\n" );
 		printf(_("# LDAPv%d\n"), protocol);
 		printf(_("# base <%s> with scope %s\n"),
-			base ? base : "", (scope == LDAP_SCOPE_BASE) ? "base"
-		       : ((scope == LDAP_SCOPE_ONELEVEL) ? "one" : "sub"));
+			base ? base : "",
+			((scope == LDAP_SCOPE_BASE) ? "baseObject"
+				: ((scope == LDAP_SCOPE_ONELEVEL) ? "oneLevel"
+#ifdef LDAP_SCOPE_SUBORDINATE
+				: ((scope == LDAP_SCOPE_SUBORDINATE) ? "children"
+#endif
+				: "subtree"
+#ifdef LDAP_SCOPE_SUBORDINATE
+				)
+#endif
+				)));
 		printf(_("# filter%s: %s\n"), infile != NULL ? _(" pattern") : "",
 		       filtpattern);
 		printf(_("# requesting: "));
@@ -845,7 +850,7 @@ getNextPage:
 				valuesReturnFilter > 1 ? _("critical ") : "", vrFilter );
 		}
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-		if ( pageSize ) {
+		if ( pagedResults ) {
 			printf(_("\n# with pagedResults %scontrol: size=%d"),
 				(pagedResults > 1) ? _("critical ") : "", 
 				pageSize );
@@ -878,44 +883,46 @@ getNextPage:
 	}
 
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-	if ( ( pageSize != 0 ) && ( morePagedResults != 0 ) ) {
+	if ( ( rc == LDAP_SUCCESS ) &&  ( pageSize != 0 ) && ( morePagedResults != 0 ) ) {
 		char	buf[6];
 		int	i, moreEntries, tmpSize;
 
 		/* Loop to get the next pages when 
 		 * enter is pressed on the terminal.
 		 */
-		if ( entriesLeft > 0 ) {
-			printf( _("Estimate entries: %d\n"), entriesLeft );
-		}
-		printf( _("Press [size] Enter for the next {%d|size} entries.\n"),
-			(int)pageSize ); 
-		i = 0;
-		moreEntries = getchar();
-		while ( moreEntries != EOF && moreEntries != '\n' ) { 
-			if ( i < (int)sizeof(buf) - 1 ) {
-				buf[i] = moreEntries;
-				i++;
+		if ( pagePrompt != 0 ) {
+			if ( entriesLeft > 0 ) {
+				printf( _("Estimate entries: %d\n"), entriesLeft );
 			}
+			printf( _("Press [size] Enter for the next {%d|size} entries.\n"),
+				(int)pageSize );
+			i = 0;
 			moreEntries = getchar();
-		}
-		buf[i] = '\0';
+			while ( moreEntries != EOF && moreEntries != '\n' ) { 
+				if ( i < (int)sizeof(buf) - 1 ) {
+					buf[i] = moreEntries;
+					i++;
+				}
+				moreEntries = getchar();
+			}
+			buf[i] = '\0';
 
-		if ( i > 0 && isdigit( (unsigned char)buf[0] ) ) {
-			int num = sscanf( buf, "%d", &tmpSize );
-			if ( num != 1 ) {
-				fprintf( stderr, _("Invalid value for PagedResultsControl, %s.\n"), buf);
-				return EXIT_FAILURE;
-
+			if ( i > 0 && isdigit( (unsigned char)buf[0] ) ) {
+				int num = sscanf( buf, "%d", &tmpSize );
+				if ( num != 1 ) {
+					fprintf( stderr, _("Invalid value for PagedResultsControl, %s.\n"), buf);
+					return EXIT_FAILURE;
+	
+				}
+				pageSize = (ber_int_t)tmpSize;
 			}
-			pageSize = (ber_int_t)tmpSize;
 		}
 
-		goto getNextPage;	
+		goto getNextPage;
 	}
 #endif
 
-	ldap_unbind( ld );
+	ldap_unbind_ext( ld, NULL, NULL );
 #ifdef HAVE_CYRUS_SASL
 	sasl_done();
 #endif
@@ -948,12 +955,10 @@ static int dosearch(
 	int			npartial;
 	LDAPMessage		*res, *msg;
 	ber_int_t		msgid;
-#ifdef LDAP_SYNC
 	char			*retoid = NULL;
 	struct berval		*retdata = NULL;
 	int			nresponses_psearch = -1;
 	int			cancel_msgid = -1;
-#endif
 
 	if( filtpatt != NULL ) {
 		filter = malloc( strlen( filtpatt ) + strlen( value ) );
@@ -1011,10 +1016,8 @@ static int dosearch(
 			msg = ldap_next_message( ld, msg ) )
 		{
 			if ( nresponses++ ) putchar('\n');
-#if LDAP_SYNC
 			if ( nresponses_psearch >= 0 ) 
 				nresponses_psearch++;
-#endif
 
 			switch( ldap_msgtype( msg ) ) {
 			case LDAP_RES_SEARCH_ENTRY:
@@ -1036,46 +1039,36 @@ static int dosearch(
 					goto done;
 				}
 
-#ifdef LDAP_SYNC
 				if ( cancel_msgid != -1 &&
 						cancel_msgid == ldap_msgid( msg ) ) {
 					printf(_("Cancelled \n"));
 					printf(_("cancel_msgid = %d\n"), cancel_msgid);
 					goto done;
 				}
-#endif
 				break;
 
 			case LDAP_RES_SEARCH_RESULT:
 				rc = print_result( ld, msg, 1 );
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-				if ( pageSize != 0 ) { 
-					rc = parse_page_control( ld, msg, &cookie );
+				if ( pageSize != 0 ) {
+					if ( rc == LDAP_SUCCESS ) {
+						rc = parse_page_control( ld, msg, &page_cookie );
+					} else {
+						morePagedResults = 0;
+					}
+				} else {
+					morePagedResults = 0;
 				}
 #endif
 
-#ifdef LDAP_CLIENT_UPDATE
-				if ( lcup == LDAP_CUP_PERSIST_ONLY ||
-				     lcup == LDAP_CUP_SYNC_AND_PERSIST ) {
-					break;
-				}
-#endif
-#if defined(LDAP_CLIENT_UPDATE) && defined(LDAP_SYNC)
-				else
-#endif
-#ifdef LDAP_SYNC
 				if ( ldapsync == LDAP_SYNC_REFRESH_AND_PERSIST ) {
 					break;
 				}
-#endif
 
 				goto done;
 
 			case LDAP_RES_INTERMEDIATE:
 				npartial++;
-#ifndef LDAP_SYNC
-				print_partial( ld, msg );
-#else
 				ldap_parse_intermediate( ld, msg,
 					&retoid, &retdata, NULL, 0 );
 
@@ -1092,16 +1085,8 @@ static int dosearch(
 				ldap_memfree( retoid );
 				ber_bvfree( retdata );
 				goto done;
-#endif
 			}
 
-#ifdef LDAP_CLIENT_UPDATE
-			if ( lcup && lcup_slimit != -1 && nresponses >= lcup_slimit ) {
-				ldap_abandon (ld, ldap_msgid(msg));
-				goto done;
-			}
-#endif
-#ifdef LDAP_SYNC
 			if ( ldapsync && sync_slimit != -1 &&
 					nresponses_psearch >= sync_slimit ) {
 				BerElement *msgidber = NULL;
@@ -1113,8 +1098,6 @@ static int dosearch(
 						msgidvalp, NULL, NULL, &cancel_msgid);
 				nresponses_psearch = -1;
 			}
-#endif
-
 		}
 
 		ldap_msgfree( res );
@@ -1128,18 +1111,18 @@ static int dosearch(
 done:
 	ldap_msgfree( res );
 #ifdef LDAP_CONTROL_PAGEDRESULTS
-	if ( pageSize != 0 ) { 
-		npagedresponses = npagedresponses + nresponses;
-		npagedentries = npagedentries + nentries;
-		npagedreferences = npagedreferences + nreferences;
-		npagedextended = npagedextended + nextended;
-		npagedpartial = npagedpartial + npartial;
+	if ( pagedResults ) { 
+		npagedresponses += nresponses;
+		npagedentries += nentries;
+		npagedextended += nextended;
+		npagedpartial += npartial;
+		npagedreferences += nreferences;
 		if ( ( morePagedResults == 0 ) && ( ldif < 2 ) ) {
 			printf( _("\n# numResponses: %d\n"), npagedresponses );
-			if( nentries ) printf( _("# numEntries: %d\n"), npagedentries );
-			if( nextended ) printf( _("# numExtended: %d\n"), npagedextended );
-			if( npartial ) printf( _("# numPartial: %d\n"), npagedpartial );
-			if( nreferences ) printf( _("# numReferences: %d\n"), npagedreferences );
+			if( npagedentries ) printf( _("# numEntries: %d\n"), npagedentries );
+			if( npagedextended ) printf( _("# numExtended: %d\n"), npagedextended );
+			if( npagedpartial ) printf( _("# numPartial: %d\n"), npagedpartial );
+			if( npagedreferences ) printf( _("# numReferences: %d\n"), npagedreferences );
 		}
 	} else
 #endif
@@ -1154,116 +1137,6 @@ done:
 	return( rc );
 }
 
-#if 1
-/* This is the original version, the old way of doing things. */
-static void
-print_entry(
-	LDAP	*ld,
-	LDAPMessage	*entry,
-	int		attrsonly)
-{
-	char		*a, *dn, *ufn;
-	char	tmpfname[ 256 ];
-	char	url[ 256 ];
-	int			i, rc;
-	BerElement		*ber = NULL;
-	struct berval	**bvals;
-	LDAPControl **ctrls = NULL;
-	FILE		*tmpfp;
-
-	dn = ldap_get_dn( ld, entry );
-	ufn = NULL;
-
-	if ( ldif < 2 ) {
-		ufn = ldap_dn2ufn( dn );
-		write_ldif( LDIF_PUT_COMMENT, NULL, ufn, ufn ? strlen( ufn ) : 0 );
-	}
-	write_ldif( LDIF_PUT_VALUE, "dn", dn, dn ? strlen( dn ) : 0);
-
-	rc = ldap_get_entry_controls( ld, entry, &ctrls );
-
-	if( rc != LDAP_SUCCESS ) {
-		fprintf(stderr, _("print_entry: %d\n"), rc );
-		ldap_perror( ld, "ldap_get_entry_controls" );
-		exit( EXIT_FAILURE );
-	}
-
-	if( ctrls ) {
-		print_ctrls( ctrls );
-		ldap_controls_free( ctrls );
-	}
-
-	if ( includeufn ) {
-		if( ufn == NULL ) {
-			ufn = ldap_dn2ufn( dn );
-		}
-		write_ldif( LDIF_PUT_VALUE, "ufn", ufn, ufn ? strlen( ufn ) : 0 );
-	}
-
-	if( ufn != NULL ) ldap_memfree( ufn );
-	ldap_memfree( dn );
-
-	for ( a = ldap_first_attribute( ld, entry, &ber ); a != NULL;
-		a = ldap_next_attribute( ld, entry, ber ) )
-	{
-		if ( attrsonly ) {
-			write_ldif( LDIF_PUT_NOVALUE, a, NULL, 0 );
-
-		} else if (( bvals = ldap_get_values_len( ld, entry, a )) != NULL ) {
-			for ( i = 0; bvals[i] != NULL; i++ ) {
-				if ( vals2tmp > 1 || ( vals2tmp
-					&& ldif_is_not_printable( bvals[i]->bv_val, bvals[i]->bv_len ) ))
-				{
-					int tmpfd;
-					/* write value to file */
-					snprintf( tmpfname, sizeof tmpfname,
-						"%s" LDAP_DIRSEP "ldapsearch-%s-XXXXXX",
-						tmpdir, a );
-					tmpfp = NULL;
-
-					tmpfd = mkstemp( tmpfname );
-
-					if ( tmpfd < 0  ) {
-						perror( tmpfname );
-						continue;
-					}
-
-					if (( tmpfp = fdopen( tmpfd, "w")) == NULL ) {
-						perror( tmpfname );
-						continue;
-					}
-
-					if ( fwrite( bvals[ i ]->bv_val,
-						bvals[ i ]->bv_len, 1, tmpfp ) == 0 )
-					{
-						perror( tmpfname );
-						fclose( tmpfp );
-						continue;
-					}
-
-					fclose( tmpfp );
-
-					snprintf( url, sizeof url, "%s%s", urlpre,
-						&tmpfname[strlen(tmpdir) + sizeof(LDAP_DIRSEP) - 1] );
-
-					urlize( url );
-					write_ldif( LDIF_PUT_URL, a, url, strlen( url ));
-
-				} else {
-					write_ldif( LDIF_PUT_VALUE, a,
-						bvals[ i ]->bv_val, bvals[ i ]->bv_len );
-				}
-			}
-			ber_bvecfree( bvals );
-		}
-		ldap_memfree( a );
-	}
-
-	if( ber != NULL ) {
-		ber_free( ber, 0 );
-	}
-}
-#else
 /* This is the proposed new way of doing things.
  * It is more efficient, but the API is non-standard.
  */
@@ -1290,8 +1163,7 @@ print_entry(
 	}
 	write_ldif( LDIF_PUT_VALUE, "dn", bv.bv_val, bv.bv_len );
 
-	rc = ldap_int_get_controls( ber, &ctrls );
-
+	rc = ldap_get_entry_controls( ld, entry, &ctrls );
 	if( rc != LDAP_SUCCESS ) {
 		fprintf(stderr, _("print_entry: %d\n"), rc );
 		ldap_perror( ld, "ldap_get_entry_controls" );
@@ -1320,13 +1192,13 @@ print_entry(
 	{
 		if (bv.bv_val == NULL) break;
 
-		if ( attrsonly || !bvals ) {
+		if ( attrsonly ) {
 			write_ldif( LDIF_PUT_NOVALUE, bv.bv_val, NULL, 0 );
 
-		} else {
+		} else if ( bvals ) {
 			for ( i = 0; bvals[i].bv_val != NULL; i++ ) {
-				if ( vals2tmp > 1 || ( vals2tmp
-					&& ldif_is_not_printable( bvals[i].bv_val, bvals[i].bv_len ) ))
+				if ( vals2tmp > 1 || ( vals2tmp &&
+					ldif_is_not_printable( bvals[i].bv_val, bvals[i].bv_len )))
 				{
 					int tmpfd;
 					/* write value to file */
@@ -1376,7 +1248,6 @@ print_entry(
 		ber_free( ber, 0 );
 	}
 }
-#endif
 
 static void print_reference(
 	LDAP *ld,
@@ -1664,7 +1535,6 @@ parse_page_control(
 	LDAPControl *ctrlp = NULL;
 	BerElement *ber;
 	ber_tag_t tag;
-	struct berval servercookie = { 0, NULL };
 
 	rc = ldap_parse_result( ld, result,
 		&err, NULL, NULL, NULL, &ctrl, 0 );
@@ -1693,8 +1563,7 @@ parse_page_control(
 			return EXIT_FAILURE;
 		}
 
-		tag = ber_scanf( ber, "{im}", &entriesLeft, &servercookie );
-		ber_dupbv( cookie, &servercookie );
+		tag = ber_scanf( ber, "{io}", &entriesLeft, cookie );
 		(void) ber_free( ber, 1 );
 
 		if( tag == LBER_ERROR ) {
@@ -1709,6 +1578,10 @@ parse_page_control(
 			return EXIT_FAILURE;
 		}
 
+		if ( cookie->bv_len == 0 ) {
+			morePagedResults = 0;
+		}
+
 		ldap_controls_free( ctrl );
 
 	} else {
