@@ -1,16 +1,18 @@
-/* $OpenLDAP: pkg/ldap/include/lber.h,v 1.83.2.1 2003/05/31 19:01:39 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
+/* $OpenLDAP: pkg/ldap/include/lber.h,v 1.94 2004/11/09 04:39:09 lukeh Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted only as authorized by the OpenLDAP
- * Public License.  A copy of this license is available at
- * http://www.OpenLDAP.org/license.html or in file LICENSE in the
- * top-level directory of the distribution.
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -25,6 +27,7 @@
 #define _LBER_H
 
 #include <lber_types.h>
+#include <string.h>
 
 LDAP_BEGIN_DECL
 
@@ -149,13 +152,13 @@ typedef struct lber_memory_fns {
 #define LBER_OPT_SOCKBUF_DEBUG		0x1002
 
 /* on/off values */
-#define LBER_OPT_ON		((void *) 1)
+LBER_V( char ) ber_pvt_opt_on;
+#define LBER_OPT_ON		((void *) &ber_pvt_opt_on)
 #define LBER_OPT_OFF	((void *) 0)
 
 #define LBER_OPT_SUCCESS	(0)
 #define LBER_OPT_ERROR		(-1)
 
-#define LBER_ELEMENT_SIZEOF (256) /* must be >= sizeof(BerElement) */
 typedef struct berelement BerElement;
 typedef struct sockbuf Sockbuf;
 typedef struct seqorset Seqorset;
@@ -531,9 +534,7 @@ LBER_V( Sockbuf_IO ) ber_sockbuf_io_tcp;
 LBER_V( Sockbuf_IO ) ber_sockbuf_io_readahead;
 LBER_V( Sockbuf_IO ) ber_sockbuf_io_fd;
 LBER_V( Sockbuf_IO ) ber_sockbuf_io_debug;
-#ifdef LDAP_CONNECTIONLESS
 LBER_V( Sockbuf_IO ) ber_sockbuf_io_udp;
-#endif
 
 /*
  * LBER memory.c
@@ -582,15 +583,15 @@ ber_bvdup LDAP_P((
 	struct berval *src ));
 
 LBER_F( struct berval * )
-ber_str2bv LDAP_P((
-	LDAP_CONST char *, ber_len_t len, int dup, struct berval *bv));
+ber_mem2bv LDAP_P((
+	LDAP_CONST char *, ber_len_t len, int duplicate, struct berval *bv));
 
 LBER_F( struct berval * )
-ber_mem2bv LDAP_P((
-	LDAP_CONST char *, ber_len_t len, int dup, struct berval *bv));
+ber_str2bv LDAP_P((
+	LDAP_CONST char *, ber_len_t len, int duplicate, struct berval *bv));
 
-#define	ber_bvstr(a)	ber_str2bv(a, 0, 0, NULL)
-#define	ber_bvstrdup(a)	ber_str2bv(a, 0, 1, NULL)
+#define	ber_bvstr(a)	((ber_str2bv)((a), 0, 0, NULL))
+#define	ber_bvstrdup(a)	((ber_str2bv)((a), 0, 1, NULL))
 
 LBER_F( char * )
 ber_strdup LDAP_P((
