@@ -1,14 +1,18 @@
-/* $OpenLDAP: pkg/ldap/include/ldap_pvt_uc.h,v 1.25 2003/04/11 01:57:10 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
+/* $OpenLDAP: pkg/ldap/include/ldap_pvt_uc.h,v 1.28 2004/01/01 18:15:24 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted only as authorized by the OpenLDAP
- * Public License.  A copy of this license is available at
- * http://www.OpenLDAP.org/license.html or in file LICENSE in the
- * top-level directory of the distribution.
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 /*
  * ldap_pvt_uc.h - Header for Unicode functions.
  * These are meant to be used by the OpenLDAP distribution only.
@@ -20,6 +24,7 @@
 
 #include <lber.h>				/* get ber_slen_t */
 
+#include <ac/bytes.h>
 #include "../libraries/liblunicode/ucdata/ucdata.h"
 
 LDAP_BEGIN_DECL
