@@ -1,11 +1,19 @@
 /* dtest.c - lber decoding test program */
-/* $OpenLDAP: pkg/ldap/libraries/liblber/dtest.c,v 1.27 2002/01/04 20:17:36 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblber/dtest.c,v 1.35 2004/01/01 18:15:25 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -15,6 +23,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 #include "portable.h"
 
@@ -24,6 +36,7 @@
 #include <ac/string.h>
 #include <ac/socket.h>
 #include <ac/unistd.h>
+#include <ac/errno.h>
 
 #ifdef HAVE_CONSOLE_H
 #include <console.h>
@@ -40,7 +53,6 @@ int
 main( int argc, char **argv )
 {
 	char *s;
-	int rc;
 
 	ber_tag_t	tag;
 	ber_len_t	len;
@@ -68,18 +80,25 @@ main( int argc, char **argv )
 	ber_sockbuf_add_io( sb, &ber_sockbuf_io_fd, LBER_SBIOD_LEVEL_PROVIDER,
 		(void *)&fd );
 
-	if( (ber = ber_alloc_t(LBER_USE_DER)) == NULL ) {
+	ber = ber_alloc_t(LBER_USE_DER);
+	if( ber == NULL ) {
 		perror( "ber_alloc_t" );
 		return( EXIT_FAILURE );
 	}
 
-	if(( tag = ber_get_next( sb, &len, ber) ) == LBER_ERROR ) {
+	for (;;) {
+		tag = ber_get_next( sb, &len, ber);
+		if( tag != LBER_ERROR ) break;
+
+		if( errno == EWOULDBLOCK ) continue;
+		if( errno == EAGAIN ) continue;
+
 		perror( "ber_get_next" );
 		return( EXIT_FAILURE );
 	}
 
 	printf("decode: message tag 0x%lx and length %ld\n",
-	        (unsigned long) tag, (long) len );
+		(unsigned long) tag, (long) len );
 
 	for( s = argv[1]; *s; s++ ) {
 		char buf[128];
@@ -89,9 +108,9 @@ main( int argc, char **argv )
 
 		printf("decode: format %s\n", fmt );
 		len = sizeof(buf);
-		rc = ber_scanf( ber, fmt, &buf[0], &len );
+		tag = ber_scanf( ber, fmt, &buf[0], &len );
 
-		if( rc == LBER_ERROR ) {
+		if( tag == LBER_ERROR ) {
 			perror( "ber_scanf" );
 			return( EXIT_FAILURE );
 		}
