@@ -1,12 +1,31 @@
-/* test.c - lber encoding test program */
-/* $OpenLDAP: pkg/ldap/libraries/liblber/etest.c,v 1.28 2003/05/22 03:41:13 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* etest.c - lber encoding test program */
+/* $OpenLDAP: pkg/ldap/libraries/liblber/etest.c,v 1.33 2004/07/09 08:19:20 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
  * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
  */
 
 #include "portable.h"
@@ -34,11 +53,9 @@ static char* getbuf( void ) {
 	char *p;
 	static char buf[1024];
 
-	if ( fgets( buf, sizeof(buf), stdin ) == NULL )
-		return NULL;
+	if ( fgets( buf, sizeof(buf), stdin ) == NULL ) return NULL;
 
-	if ( (p = strchr( buf, '\n' )) != NULL )
-		*p = '\0';
+	if ( (p = strchr( buf, '\n' )) != NULL ) *p = '\0';
 
 	return buf;
 }
@@ -47,6 +64,7 @@ int
 main( int argc, char **argv )
 {
 	char	*s;
+	int tag;
 
 	int			fd, rc;
 	BerElement	*ber;
@@ -126,10 +144,14 @@ main( int argc, char **argv )
 			break;
 
 		case 's':	/* string */
-		case 't':	/* tag for the next element */
 			buf = getbuf();
 			rc = ber_printf( ber, fmt, buf );
 			break;
+		case 't':	/* tag for the next element */
+			buf = getbuf();
+			tag = atoi(buf);
+			rc = ber_printf( ber, fmt, tag );
+			break;
 
 		default:
 			fprintf( stderr, "encode: unknown fmt %c\n", *fmt );
