@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblber/lber-int.h,v 1.62 2003/04/09 11:25:11 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblber/lber-int.h,v 1.65 2004/09/04 02:54:28 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -37,23 +45,6 @@ struct lber_options {
 	long		lbo_meminuse;
 };
 
-#ifdef NEW_LOGGING
-/*
-#    ifdef LDAP_DEBUG
-#        ifdef LDAP_LOG
-#            undef LDAP_LOG
-#        endif
-#        define LDAP_LOG(a) ber_pvt_log_output a
- */
-#        define BER_DUMP(a) ber_output_dump a
-/*
-#    else
-#        define LDAP_LOG(a)
-#        define BER_DUMP(a)
-#    endif
- */
-#endif
-
 LBER_F( int ) ber_pvt_log_output(
 	const char *subsystem,
 	int level,
@@ -134,15 +125,6 @@ LBER_F (void) ber_rewind LDAP_P(( BerElement * ));
  */
 #define ber_log_printf ber_pvt_log_printf
 
-#ifdef NEW_LOGGING
-LBER_F( int )
-ber_output_dump LDAP_P((
-	const char *subsys,
-	int level,
-	BerElement *ber,
-	int inout ));
-#endif
-
 LBER_F( int )
 ber_log_bprint LDAP_P((
 	int errlvl,
