@@ -1,8 +1,18 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblber/nt_err.c,v 1.10 2003/01/03 19:20:51 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblber/nt_err.c,v 1.13 2004/01/01 18:15:25 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 #include "portable.h"
 
 #ifdef HAVE_WINSOCK2
@@ -15,8 +25,7 @@
 
 char *ber_pvt_wsa_err2string( int err )
 {
-	switch( err )
-	{
+	switch( err ) {
 		LBER_RETSTR( WSAEINTR )
 		LBER_RETSTR( WSAEBADF )
 		LBER_RETSTR( WSAEACCES )
@@ -84,4 +93,4 @@ char *ber_pvt_wsa_err2string( int err )
 		LBER_RETSTR( WSANO_DATA )
 	}
 	return "unknown WSA error";
-}
\ No newline at end of file
+}
