@@ -1,8 +1,18 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblber/options.c,v 1.34 2003/05/28 20:17:08 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblber/options.c,v 1.37 2004/01/01 18:15:25 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 #include "portable.h"
 
 #include <ac/stdlib.h>
@@ -10,6 +20,8 @@
 #include <ac/stdarg.h>
 #include "lber-int.h"
 
+char ber_pvt_opt_on;	/* used to get a non-NULL address for *_OPT_ON */
+
 struct lber_options ber_int_options = {
 	LBER_UNINITIALIZED, 0, 0, 0 };
 
