@@ -1,7 +1,20 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/cancel.c,v 1.3 2003/02/08 20:53:07 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/libldap/cancel.c,v 1.7 2004/03/14 22:32:01 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This program was orignally developed by Kurt D. Zeilenga for inclusion in
+ * OpenLDAP Software.
  */
 
 /*
@@ -20,8 +33,6 @@
 #include "ldap-int.h"
 #include "ldap_log.h"
 
-#ifdef LDAP_EXOP_X_CANCEL
-
 int
 ldap_cancel(
 	LDAP		*ld,
@@ -63,4 +74,3 @@ ldap_cancel_s(
 	return rc;
 }
 
-#endif /* LDAP_EXOP_X_CANCEL */
