@@ -1,16 +1,22 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/os-local.c,v 1.28.2.1 2003/05/31 19:01:39 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
-/*  Portions
- *  Copyright (c) 1995 Regents of the University of Michigan.
- *  All rights reserved. 
+/* os-local.c -- platform-specific domain socket code */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/os-local.c,v 1.37 2004/11/17 19:04:18 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- *  os-local.c -- platform-specific domain socket code
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Portions (C) Copyright PADL Software Pty Ltd. 1999
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved. 
+ */
+/* Portions (C) Copyright PADL Software Pty Ltd. 1999
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that this notice is preserved
  * and that due credit is given to PADL Software Pty Ltd. This software
@@ -45,13 +51,19 @@
 #include "ldap-int.h"
 #include "ldap_defaults.h"
 
-/* int ldap_int_tblsize = 0; */
+#ifdef LDAP_DEBUG
 
 #define oslocal_debug(ld,fmt,arg1,arg2,arg3) \
 do { \
 	ldap_log_printf(ld, LDAP_DEBUG_TRACE, fmt, arg1, arg2, arg3); \
 } while(0)
 
+#else
+
+#define oslocal_debug(ld,fmt,arg1,arg2,arg3) ((void)0)
+
+#endif /* LDAP_DEBUG */
+
 static void
 ldap_pvt_set_errno(int err)
 {
@@ -89,11 +101,12 @@ ldap_pvt_close_socket(LDAP *ld, int s)
 
 #undef TRACE
 #define TRACE do { \
+	char ebuf[128]; \
 	oslocal_debug(ld, \
 		"ldap_is_socket_ready: errror on socket %d: errno: %d (%s)\n", \
 		s, \
 		errno, \
-		STRERROR(errno) ); \
+		AC_STRERROR_R(errno, ebuf, sizeof ebuf)); \
 } while( 0 )
 
 /*
@@ -154,7 +167,6 @@ ldap_pvt_connect(LDAP *ld, ber_socket_t s, struct sockaddr_un *sa, int async)
 {
 	int rc;
 	struct timeval	tv, *opt_tv=NULL;
-	fd_set		wfds, *z=NULL;
 
 	if ( (opt_tv = ld->ld_options.ldo_tm_net) != NULL ) {
 		tv.tv_usec = opt_tv->tv_usec;
@@ -162,17 +174,15 @@ ldap_pvt_connect(LDAP *ld, ber_socket_t s, struct sockaddr_un *sa, int async)
 	}
 
 	oslocal_debug(ld, "ldap_connect_timeout: fd: %d tm: %ld async: %d\n",
-			s, opt_tv ? tv.tv_sec : -1L, async);
+		s, opt_tv ? tv.tv_sec : -1L, async);
 
-	if ( ldap_pvt_ndelay_on(ld, s) == -1 )
-		return ( -1 );
+	if ( ldap_pvt_ndelay_on(ld, s) == -1 ) return -1;
 
 	if ( connect(s, (struct sockaddr *) sa, sizeof(struct sockaddr_un))
 		!= AC_SOCKET_ERROR )
 	{
-		if ( ldap_pvt_ndelay_off(ld, s) == -1 ) {
-			return ( -1 );
-		}
+		if ( ldap_pvt_ndelay_off(ld, s) == -1 ) return -1;
+
 #ifdef DO_SENDMSG
 	/* Send a dummy message with access rights. Remote side will
 	 * obtain our uid/gid by fstat'ing this descriptor.
@@ -196,38 +206,76 @@ sendcred:
 			}
 		}
 #endif
-		return ( 0 );
+		return 0;
 	}
 
-	if ( errno != EINPROGRESS && errno != EWOULDBLOCK ) {
-		return ( -1 );
-	}
+	if ( errno != EINPROGRESS && errno != EWOULDBLOCK ) return -1;
 	
 #ifdef notyet
-	if ( async ) return ( -2 );
+	if ( async ) return -2;
 #endif
 
-	FD_ZERO(&wfds);
-	FD_SET(s, &wfds );
+#ifdef HAVE_POLL
+	{
+		struct pollfd fd;
+		int timeout = INFTIM;
 
-	do { 
-		rc = select(ldap_int_tblsize, z, &wfds, z, opt_tv ? &tv : NULL);
-	} while( rc == AC_SOCKET_ERROR && errno == EINTR &&
-		LDAP_BOOL_GET(&ld->ld_options, LDAP_BOOL_RESTART ));
+		if( opt_tv != NULL ) timeout = TV2MILLISEC( &tv );
 
-	if( rc == AC_SOCKET_ERROR ) return rc;
+		fd.fd = s;
+		fd.events = POLL_WRITE;
 
-	if ( FD_ISSET(s, &wfds) ) {
-		if ( ldap_pvt_is_socket_ready(ld, s) == -1 )
-			return ( -1 );
-		if ( ldap_pvt_ndelay_off(ld, s) == -1 )
-			return ( -1 );
+		do {
+			fd.revents = 0;
+			rc = poll( &fd, 1, timeout );
+		} while( rc == AC_SOCKET_ERROR && errno == EINTR &&
+			LDAP_BOOL_GET(&ld->ld_options, LDAP_BOOL_RESTART ));
+
+		if( rc == AC_SOCKET_ERROR ) return rc;
+
+		if( fd.revents & POLL_WRITE ) {
+			if ( ldap_pvt_is_socket_ready(ld, s) == -1 ) return -1;
+			if ( ldap_pvt_ndelay_off(ld, s) == -1 ) return -1;
 #ifdef DO_SENDMSG
-		goto sendcred;
+			goto sendcred;
 #else
-		return ( 0 );
+			return ( 0 );
 #endif
+		}
 	}
+#else
+	{
+		fd_set wfds, *z=NULL;
+
+#ifdef FD_SETSIZE
+		if ( s >= FD_SETSIZE ) {
+			rc = AC_SOCKET_ERROR;
+			tcp_close( s );
+			ldap_pvt_set_errno( EMFILE );
+			return rc;
+		}
+#endif
+		do { 
+			FD_ZERO(&wfds);
+			FD_SET(s, &wfds );
+			rc = select( ldap_int_tblsize, z, &wfds, z, opt_tv ? &tv : NULL );
+		} while( rc == AC_SOCKET_ERROR && errno == EINTR &&
+			LDAP_BOOL_GET(&ld->ld_options, LDAP_BOOL_RESTART ));
+
+		if( rc == AC_SOCKET_ERROR ) return rc;
+
+		if ( FD_ISSET(s, &wfds) ) {
+			if ( ldap_pvt_is_socket_ready(ld, s) == -1 ) return -1;
+			if ( ldap_pvt_ndelay_off(ld, s) == -1 ) return -1;
+#ifdef DO_SENDMSG
+			goto sendcred;
+#else
+			return ( 0 );
+#endif
+		}
+	}
+#endif
+
 	oslocal_debug(ld, "ldap_connect_timeout: timed out\n",0,0,0);
 	ldap_pvt_set_errno( ETIMEDOUT );
 	return ( -1 );
