@@ -1,7 +1,19 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/sasl.c,v 1.50 2003/05/22 14:21:14 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/libldap/sasl.c,v 1.58 2004/09/04 02:54:29 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (C) The Internet Society (1997)
+ * ASN.1 fragments are from RFC 2251; see RFC for full legal notices.
  */
 
 /*
@@ -62,11 +74,7 @@ ldap_sasl_bind(
 	int rc;
 	ber_int_t id;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( TRANSPORT, ENTRY, "ldap_sasl_bind\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_sasl_bind\n", 0, 0, 0 );
-#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
@@ -108,14 +116,14 @@ ldap_sasl_bind(
 			cred );
 		
 	} else if ( cred == NULL || cred->bv_val == NULL ) {
-		/* SASL bind w/o creditials */
+		/* SASL bind w/o credentials */
 		rc = ber_printf( ber, "{it{ist{sN}N}" /*}*/,
 			id, LDAP_REQ_BIND,
 			ld->ld_version, dn, LDAP_AUTH_SASL,
 			mechanism );
 
 	} else {
-		/* SASL bind w/ creditials */
+		/* SASL bind w/ credentials */
 		rc = ber_printf( ber, "{it{ist{sON}N}" /*}*/,
 			id, LDAP_REQ_BIND,
 			ld->ld_version, dn, LDAP_AUTH_SASL,
@@ -165,11 +173,7 @@ ldap_sasl_bind_s(
 	LDAPMessage	*result;
 	struct berval	*scredp = NULL;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( TRANSPORT, ENTRY, "ldap_sasl_bind_s\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_sasl_bind_s\n", 0, 0, 0 );
-#endif
 
 	/* do a quick !LDAPv3 check... ldap_sasl_bind will do the rest. */
 	if( servercredp != NULL ) {
@@ -251,11 +255,7 @@ ldap_parse_sasl_bind_result(
 	ber_tag_t tag;
 	BerElement	*ber;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( TRANSPORT, ENTRY, "ldap_parse_sasl_bind_result\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_parse_sasl_bind_result\n", 0, 0, 0 );
-#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
@@ -364,11 +364,7 @@ ldap_pvt_sasl_getmechs ( LDAP *ld, char **pmechlist )
 	char **values, *mechlist;
 	int rc;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( TRANSPORT, ENTRY, "ldap_pvt_sasl_getmech\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_pvt_sasl_getmech\n", 0, 0, 0 );
-#endif
 
 	rc = ldap_search_s( ld, "", LDAP_SCOPE_BASE,
 		NULL, attrs, 0, &res );
@@ -429,6 +425,7 @@ ldap_sasl_interactive_bind_s(
 	void *defaults )
 {
 	int rc;
+	char *smechs = NULL;
 
 #if defined( LDAP_R_COMPILE ) && defined( HAVE_CYRUS_SASL )
 	ldap_pvt_thread_mutex_lock( &ldap_int_sasl_mutex );
@@ -439,40 +436,34 @@ ldap_sasl_interactive_bind_s(
 		 * ask all the time. No, we don't ever actually bind, but I'll
 		 * let the final bind handler take care of saving the cdn.
 		 */
-		rc = ldap_simple_bind(ld, dn, NULL);
-		return rc < 0 ? rc : 0;
+		rc = ldap_simple_bind( ld, dn, NULL );
+		rc = rc < 0 ? rc : 0;
+		goto done;
 	} else
 #endif
+
+#ifdef HAVE_CYRUS_SASL
+	if( mechs == NULL || *mechs == '\0' ) {
+		mechs = ld->ld_options.ldo_def_sasl_mech;
+	}
+#endif
+		
 	if( mechs == NULL || *mechs == '\0' ) {
-		char *smechs;
-
 		rc = ldap_pvt_sasl_getmechs( ld, &smechs );
-
 		if( rc != LDAP_SUCCESS ) {
 			goto done;
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, 
-			"ldap_interactive_sasl_bind_s: server supports: %s\n", 
-			smechs, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"ldap_interactive_sasl_bind_s: server supports: %s\n",
+			"ldap_sasl_interactive_bind_s: server supports: %s\n",
 			smechs, 0, 0 );
-#endif
 
 		mechs = smechs;
 
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, 
-			"ldap_interactive_sasl_bind_s: user selected: %s\n", mechs, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"ldap_interactive_sasl_bind_s: user selected: %s\n",
+			"ldap_sasl_interactive_bind_s: user selected: %s\n",
 			mechs, 0, 0 );
-#endif
 	}
 
 	rc = ldap_int_sasl_bind( ld, dn, mechs,
@@ -483,6 +474,7 @@ done:
 #if defined( LDAP_R_COMPILE ) && defined( HAVE_CYRUS_SASL )
 	ldap_pvt_thread_mutex_unlock( &ldap_int_sasl_mutex );
 #endif
+	if ( smechs ) LDAP_FREE( smechs );
 
 	return rc;
 }
