@@ -1,9 +1,17 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.106 2003/05/06 13:00:58 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* tls.c - Handle tls/ssl using SSLeay or OpenSSL. */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.118 2004/11/23 02:48:09 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * tls.c - Handle tls/ssl using SSLeay or OpenSSL.
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -45,6 +53,9 @@ static char *tls_opt_keyfile = NULL;
 static char *tls_opt_cacertfile = NULL;
 static char *tls_opt_cacertdir = NULL;
 static int  tls_opt_require_cert = LDAP_OPT_X_TLS_DEMAND;
+#ifdef HAVE_OPENSSL_CRL
+static int  tls_opt_crlcheck = LDAP_OPT_X_TLS_CRL_NONE;
+#endif
 static char *tls_opt_ciphersuite = NULL;
 static char *tls_opt_randfile = NULL;
 
@@ -149,8 +160,7 @@ ldap_pvt_tls_init( void )
 {
 	static int tls_initialized = 0;
 
-	if ( tls_initialized ) return 0;
-	tls_initialized = 1;
+	if ( tls_initialized++ ) return 0;
 
 #ifdef HAVE_EBCDIC
 	{
@@ -220,15 +230,9 @@ ldap_pvt_tls_init_def_ctx( void )
 		int i;
 		tls_def_ctx = SSL_CTX_new( SSLv23_method() );
 		if ( tls_def_ctx == NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_init_def_ctx: "
-				"TLS could not allocate default ctx (%d).\n",
-				ERR_peek_error(), 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			   "TLS: could not allocate default ctx (%lu).\n",
 				ERR_peek_error(),0,0);
-#endif
 			rc = -1;
 			goto error_exit;
 		}
@@ -239,15 +243,9 @@ ldap_pvt_tls_init_def_ctx( void )
 		if ( tls_opt_ciphersuite &&
 			!SSL_CTX_set_cipher_list( tls_def_ctx, ciphersuite ) )
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_init_def_ctx: "
-				"TLS could not set cipher list %s.\n",
-				tls_opt_ciphersuite, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				   "TLS: could not set cipher list %s.\n",
 				   tls_opt_ciphersuite, 0, 0 );
-#endif
 			tls_report_error();
 			rc = -1;
 			goto error_exit;
@@ -258,20 +256,11 @@ ldap_pvt_tls_init_def_ctx( void )
 					cacertfile, cacertdir ) ||
 				!SSL_CTX_set_default_verify_paths( tls_def_ctx ) )
 			{
-#ifdef NEW_LOGGING
-				LDAP_LOG ( TRANSPORT, ERR, 
-					"ldap_pvt_tls_init_def_ctx: "
-					"TLS could not load verify locations "
-					"(file:`%s',dir:`%s').\n",
-					tls_opt_cacertfile ? tls_opt_cacertfile : "",
-					tls_opt_cacertdir ? tls_opt_cacertdir : "", 0 );
-#else
 				Debug( LDAP_DEBUG_ANY, "TLS: "
 					"could not load verify locations (file:`%s',dir:`%s').\n",
 					tls_opt_cacertfile ? tls_opt_cacertfile : "",
 					tls_opt_cacertdir ? tls_opt_cacertdir : "",
 					0 );
-#endif
 				tls_report_error();
 				rc = -1;
 				goto error_exit;
@@ -279,18 +268,11 @@ ldap_pvt_tls_init_def_ctx( void )
 
 			calist = get_ca_list( cacertfile, cacertdir );
 			if ( !calist ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_init_def_ctx: "
-					"TLS could not load client CA list (file: `%s',dir:`%s')\n",
-					tls_opt_cacertfile ? tls_opt_cacertfile : "",
-					tls_opt_cacertdir ? tls_opt_cacertdir : "", 0 );
-#else
 				Debug( LDAP_DEBUG_ANY, "TLS: "
 					"could not load client CA list (file:`%s',dir:`%s').\n",
 					tls_opt_cacertfile ? tls_opt_cacertfile : "",
 					tls_opt_cacertdir ? tls_opt_cacertdir : "",
 					0 );
-#endif
 				tls_report_error();
 				rc = -1;
 				goto error_exit;
@@ -303,14 +285,9 @@ ldap_pvt_tls_init_def_ctx( void )
 			!SSL_CTX_use_PrivateKey_file( tls_def_ctx,
 				keyfile, SSL_FILETYPE_PEM ) )
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_init_def_ctx: "
-				"TLS could not use key file `%s'.\n", tls_opt_keyfile, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"TLS: could not use key file `%s'.\n",
 				tls_opt_keyfile,0,0);
-#endif
 			tls_report_error();
 			rc = -1;
 			goto error_exit;
@@ -320,15 +297,9 @@ ldap_pvt_tls_init_def_ctx( void )
 			!SSL_CTX_use_certificate_file( tls_def_ctx,
 				certfile, SSL_FILETYPE_PEM ) )
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_init_def_ctx: "
-				"TLS could not use certificate `%s'.\n", 
-				tls_opt_certfile, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"TLS: could not use certificate `%s'.\n",
 				tls_opt_certfile,0,0);
-#endif
 			tls_report_error();
 			rc = -1;
 			goto error_exit;
@@ -337,15 +308,9 @@ ldap_pvt_tls_init_def_ctx( void )
 		if ( ( tls_opt_certfile || tls_opt_keyfile ) &&
 			!SSL_CTX_check_private_key( tls_def_ctx ) )
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, 
-				"ldap_pvt_tls_init_def_ctx: TLS private key mismatch.\n", 
-				0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"TLS: private key mismatch.\n",
 				0,0,0);
-#endif
 			tls_report_error();
 			rc = -1;
 			goto error_exit;
@@ -369,6 +334,17 @@ ldap_pvt_tls_init_def_ctx( void )
 			tls_verify_ok : tls_verify_cb );
 		SSL_CTX_set_tmp_rsa_callback( tls_def_ctx, tls_tmp_rsa_cb );
 		/* SSL_CTX_set_tmp_dh_callback( tls_def_ctx, tls_tmp_dh_cb ); */
+#ifdef HAVE_OPENSSL_CRL
+		if ( tls_opt_crlcheck ) {
+			X509_STORE *x509_s = SSL_CTX_get_cert_store( tls_def_ctx );
+			if ( tls_opt_crlcheck == LDAP_OPT_X_TLS_CRL_PEER ) {
+				X509_STORE_set_flags( x509_s, X509_V_FLAG_CRL_CHECK );
+			} else if ( tls_opt_crlcheck == LDAP_OPT_X_TLS_CRL_ALL ) {
+				X509_STORE_set_flags( x509_s, 
+						X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL  );
+			}
+		}
+#endif
 	}
 error_exit:
 	if ( rc == -1 && tls_def_ctx != NULL ) {
@@ -429,12 +405,7 @@ alloc_handle( void *ctx_arg )
 
 	ssl = SSL_new( ctx );
 	if ( ssl == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"alloc_handle: TLS can't create ssl handle.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,"TLS: can't create ssl handle.\n",0,0,0);
-#endif
 		return NULL;
 	}
 	return ssl;
@@ -742,6 +713,7 @@ ldap_int_tls_connect( LDAP *ld, LDAPConn *conn )
 		ber_sockbuf_ctrl( sb, LBER_SB_OPT_GET_SSL, (void *)&ssl );
 
 	} else {
+		struct ldapoptions *lo;
 		void *ctx = ld->ld_defconn
 			? ld->ld_defconn->lconn_tls_ctx : NULL;
 
@@ -757,8 +729,15 @@ ldap_int_tls_connect( LDAP *ld, LDAPConn *conn )
 			LBER_SBIOD_LEVEL_TRANSPORT, (void *)ssl );
 
 		if( ctx == NULL ) {
+			ctx = tls_def_ctx;
 			conn->lconn_tls_ctx = tls_def_ctx;
 		}
+		lo = &ld->ld_options;
+		if ( lo->ldo_tls_connect_cb )
+			lo->ldo_tls_connect_cb( ld, ssl, ctx, lo->ldo_tls_connect_arg );
+		lo = LDAP_INT_GLOBAL_OPT();   
+		if ( lo && lo->ldo_tls_connect_cb )
+			lo->ldo_tls_connect_cb( ld, ssl, ctx, lo->ldo_tls_connect_arg );
 	}
 
 	err = SSL_connect( ssl );
@@ -780,12 +759,7 @@ ldap_int_tls_connect( LDAP *ld, LDAPConn *conn )
 #endif
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"ldap_int_tls_connect: TLS can't connect.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,"TLS: can't connect.\n",0,0,0);
-#endif
 
 		ber_sockbuf_remove_io( sb, &sb_tls_sbio,
 			LBER_SBIOD_LEVEL_TRANSPORT );
@@ -832,12 +806,7 @@ ldap_pvt_tls_accept( Sockbuf *sb, void *ctx_arg )
 	if ( err <= 0 ) {
 		if ( update_flags( sb, ssl, err )) return 1;
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"ldap_pvt_tls_accept: TLS can't accept.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,"TLS: can't accept.\n",0,0,0 );
-#endif
 
 		tls_report_error();
 		ber_sockbuf_remove_io( sb, &sb_tls_sbio,
@@ -858,45 +827,6 @@ ldap_pvt_tls_inplace ( Sockbuf *sb )
 	return HAS_TLS( sb ) ? 1 : 0;
 }
 
-void *
-ldap_pvt_tls_sb_ctx( Sockbuf *sb )
-{
-	void			*p;
-	
-	if (HAS_TLS( sb )) {
-		ber_sockbuf_ctrl( sb, LBER_SB_OPT_GET_SSL, (void *)&p );
-		return p;
-	}
-
-	return NULL;
-}
-
-int
-ldap_pvt_tls_get_strength( void *s )
-{
-	SSL_CIPHER *c;
-
-	c = SSL_get_current_cipher((SSL *)s);
-	return SSL_CIPHER_get_bits(c, NULL);
-}
-
-
-int
-ldap_pvt_tls_get_my_dn( void *s, struct berval *dn, LDAPDN_rewrite_dummy *func, unsigned flags )
-{
-	X509 *x;
-	X509_NAME *xn;
-	int rc;
-
-	x = SSL_get_certificate((SSL *)s);
-
-	if (!x) return LDAP_INVALID_CREDENTIALS;
-	
-	xn = X509_get_subject_name(x);
-	rc = ldap_X509dn2bv(xn, dn, (LDAPDN_rewrite_func *)func, flags );
-	return rc;
-}
-
 static X509 *
 tls_get_cert( SSL *s )
 {
@@ -912,7 +842,8 @@ tls_get_cert( SSL *s )
 }
 
 int
-ldap_pvt_tls_get_peer_dn( void *s, struct berval *dn, LDAPDN_rewrite_dummy *func, unsigned flags )
+ldap_pvt_tls_get_peer_dn( void *s, struct berval *dn,
+	LDAPDN_rewrite_dummy *func, unsigned flags )
 {
 	X509 *x;
 	X509_NAME *xn;
@@ -981,15 +912,9 @@ ldap_pvt_tls_check_hostname( LDAP *ld, void *s, const char *name_in )
 
 	x = tls_get_cert((SSL *)s);
 	if (!x) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"ldap_pvt_tls_check_hostname: "
-			"TLS unable to get peer certificate.\n" , 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"TLS: unable to get peer certificate.\n",
 			0, 0, 0 );
-#endif
 		/* If this was a fatal condition, things would have
 		 * aborted long before now.
 		 */
@@ -1006,8 +931,7 @@ ldap_pvt_tls_check_hostname( LDAP *ld, void *s, const char *name_in )
 	} else 
 #endif
 	if ((ptr = strrchr(name, '.')) && isdigit((unsigned char)ptr[1])) {
-		if (inet_aton(name, (struct in_addr *)&addr))
-			ntype = IS_IP4;
+		if (inet_aton(name, (struct in_addr *)&addr)) ntype = IS_IP4;
 	}
 	
 	i = X509_get_ext_by_NID(x, NID_subject_alt_name, -1);
@@ -1040,24 +964,21 @@ ldap_pvt_tls_check_hostname( LDAP *ld, void *s, const char *name_in )
 					sn = (char *) ASN1_STRING_data(gn->d.ia5);
 					sl = ASN1_STRING_length(gn->d.ia5);
 
+					/* ignore empty */
+					if (sl == 0) continue;
+
 					/* Is this an exact match? */
 					if ((len1 == sl) && !strncasecmp(name, sn, len1)) {
 						break;
 					}
 
 					/* Is this a wildcard match? */
-					if ((*sn == '*') && domain && (len2 == sl-1) &&
-						!strncasecmp(domain, sn+1, len2)) {
+					if (domain && (sn[0] == '*') && (sn[1] == '.') &&
+						(len2 == sl-1) && !strncasecmp(domain, &sn[1], len2))
+					{
 						break;
 					}
 
-#if 0
-					/* Is this a RFC 2459 style wildcard match? */
-					if ((*sn == '.') && domain && (len2 == sl) &&
-						!strncasecmp(domain, sn, len2)) {
-						break;
-					}
-#endif
 				} else if (gn->type == GEN_IPADD) {
 					if (ntype == IS_DNS) continue;
 
@@ -1088,40 +1009,46 @@ ldap_pvt_tls_check_hostname( LDAP *ld, void *s, const char *name_in )
 	if (ret != LDAP_SUCCESS) {
 		X509_NAME *xn;
 		char buf[2048];
+		buf[0] = '\0';
 
 		xn = X509_get_subject_name(x);
-
 		if( X509_NAME_get_text_by_NID( xn, NID_commonName,
 			buf, sizeof(buf)) == -1)
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_check_hostname: "
-				"TLS unable to get common name from peer certificate.\n", 
-				0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"TLS: unable to get common name from peer certificate.\n",
 				0, 0, 0 );
-#endif
+       		ret = LDAP_CONNECT_ERROR;
 			ld->ld_error = LDAP_STRDUP(
 				_("TLS: unable to get CN from peer certificate"));
 
-		} else if (strcasecmp(name, buf)) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, "ldap_pvt_tls_check_hostname: "
-				"TLS hostname (%s) does not match "
-				"common name in certificate (%s).\n", name, buf, 0 );
-#else
-			Debug( LDAP_DEBUG_ANY, "TLS: hostname (%s) does not match "
-				"common name in certificate (%s).\n", 
-				name, buf, 0 );
-#endif
-			ret = LDAP_CONNECT_ERROR;
-			ld->ld_error = LDAP_STRDUP(
-				_("TLS: hostname does not match CN in peer certificate"));
-
-		} else {
+		} else if (strcasecmp(name, buf) == 0 ) {
 			ret = LDAP_SUCCESS;
+
+		} else if (( buf[0] == '*' ) && ( buf[1] == '.' )) {
+			char *domain = strchr(name, '.');
+			if( domain ) {
+				size_t dlen = 0;
+				size_t sl;
+
+				sl = strlen(name);
+				dlen = sl - (domain-name);
+				sl = strlen(buf);
+
+				/* Is this a wildcard match? */
+				if ((dlen == sl-1) && !strncasecmp(domain, &buf[1], dlen)) {
+					ret = LDAP_SUCCESS;
+				}
+			}
+		}
+
+		if( ret == LDAP_LOCAL_ERROR ) {
+       		 Debug( LDAP_DEBUG_ANY, "TLS: hostname (%s) does not match "
+       			 "common name in certificate (%s).\n", 
+       			 name, buf, 0 );
+       		 ret = LDAP_CONNECT_ERROR;
+       		 ld->ld_error = LDAP_STRDUP(
+       		  	 _("TLS: hostname does not match CN in peer certificate"));
 		}
 	}
 	X509_free(x);
@@ -1160,6 +1087,7 @@ ldap_int_tls_config( LDAP *ld, int option, const char *arg )
 	case LDAP_OPT_X_TLS_CERTFILE:
 	case LDAP_OPT_X_TLS_KEYFILE:
 	case LDAP_OPT_X_TLS_RANDOM_FILE:
+	case LDAP_OPT_X_TLS_CIPHER_SUITE:
 		return ldap_pvt_tls_set_option( ld, option, (void *) arg );
 
 	case LDAP_OPT_X_TLS_REQUIRE_CERT:
@@ -1189,8 +1117,22 @@ ldap_int_tls_config( LDAP *ld, int option, const char *arg )
 			return ldap_pvt_tls_set_option( ld, option, &i );
 		}
 		return -1;
+#ifdef HAVE_OPENSSL_CRL
+	case LDAP_OPT_X_TLS_CRLCHECK:
+		i = -1;
+		if ( strcasecmp( arg, "none" ) == 0 ) {
+			i = LDAP_OPT_X_TLS_CRL_NONE ;
+		} else if ( strcasecmp( arg, "peer" ) == 0 ) {
+			i = LDAP_OPT_X_TLS_CRL_PEER ;
+		} else if ( strcasecmp( arg, "all" ) == 0 ) {
+			i = LDAP_OPT_X_TLS_CRL_ALL ;
+		}
+		if (i >= 0) {
+			return ldap_pvt_tls_set_option( ld, option, &i );
+		}
+		return -1;
+#endif
 	}
-
 	return -1;
 }
 
@@ -1246,6 +1188,11 @@ ldap_pvt_tls_get_option( LDAP *ld, int option, void *arg )
 	case LDAP_OPT_X_TLS_REQUIRE_CERT:
 		*(int *)arg = tls_opt_require_cert;
 		break;
+#ifdef HAVE_OPENSSL_CRL
+	case LDAP_OPT_X_TLS_CRLCHECK:
+		*(int *)arg = tls_opt_crlcheck;
+		break;
+#endif
 	case LDAP_OPT_X_TLS_RANDOM_FILE:
 		*(char **)arg = tls_opt_randfile ?
 			LDAP_STRDUP( tls_opt_randfile ) : NULL;
@@ -1262,6 +1209,12 @@ ldap_pvt_tls_get_option( LDAP *ld, int option, void *arg )
 		*(void **)arg = retval;
 		break;
 	}
+	case LDAP_OPT_X_TLS_CONNECT_CB:
+		*(LDAP_TLS_CONNECT_CB **)arg = lo->ldo_tls_connect_cb;
+		break;
+	case LDAP_OPT_X_TLS_CONNECT_ARG:
+		*(void **)arg = lo->ldo_tls_connect_arg;
+		break;
 	default:
 		return -1;
 	}
@@ -1314,6 +1267,12 @@ ldap_pvt_tls_set_option( LDAP *ld, int option, void *arg )
 			ld->ld_defconn->lconn_tls_ctx = arg;
 		}
 		return 0;
+	case LDAP_OPT_X_TLS_CONNECT_CB:
+		lo->ldo_tls_connect_cb = (LDAP_TLS_CONNECT_CB *)arg;
+		return 0;
+	case LDAP_OPT_X_TLS_CONNECT_ARG:
+		lo->ldo_tls_connect_arg = arg;
+		return 0;
 	}
 
 	if ( ld != NULL ) {
@@ -1348,6 +1307,17 @@ ldap_pvt_tls_set_option( LDAP *ld, int option, void *arg )
 			return 0;
 		}
 		return -1;
+#ifdef HAVE_OPENSSL_CRL
+	case LDAP_OPT_X_TLS_CRLCHECK:
+		switch( *(int *) arg ) {
+		case LDAP_OPT_X_TLS_CRL_NONE:
+		case LDAP_OPT_X_TLS_CRL_PEER:
+		case LDAP_OPT_X_TLS_CRL_ALL:
+			tls_opt_crlcheck = * (int *) arg;
+			return 0;
+		}
+		return -1;
+#endif
 	case LDAP_OPT_X_TLS_CIPHER_SUITE:
 		if ( tls_opt_ciphersuite ) LDAP_FREE( tls_opt_ciphersuite );
 		tls_opt_ciphersuite = arg ? LDAP_STRDUP( (char *) arg ) : NULL;
@@ -1430,14 +1400,9 @@ tls_info_cb( const SSL *ssl, int where, int ret )
 	}
 #endif
 	if ( where & SSL_CB_LOOP ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, "tls_info_cb: "
-			"TLS trace: %s:%s\n", op, state, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			   "TLS trace: %s:%s\n",
 			   op, state, 0 );
-#endif
 
 	} else if ( where & SSL_CB_ALERT ) {
 		char *atype = (char *) SSL_alert_type_string_long( ret );
@@ -1453,40 +1418,22 @@ tls_info_cb( const SSL *ssl, int where, int ret )
 			__etoa( adesc );
 		}
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, 
-			"tls_info_cb: TLS trace: SSL3 alert %s:%s:%s\n", 
-			op, atype, adesc );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			   "TLS trace: SSL3 alert %s:%s:%s\n",
 			   op, atype, adesc );
-#endif
 #ifdef HAVE_EBCDIC
 		if ( atype ) LDAP_FREE( atype );
 		if ( adesc ) LDAP_FREE( adesc );
 #endif
 	} else if ( where & SSL_CB_EXIT ) {
 		if ( ret == 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, 
-				"tls_info_cb: TLS trace: %s:failed in %s\n", 
-				op, state, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				   "TLS trace: %s:failed in %s\n",
 				   op, state, 0 );
-#endif
 		} else if ( ret < 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( TRANSPORT, ERR, 
-				"tls_info_cb: TLS trace: %s:error in %s\n", 
-				op, state, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				   "TLS trace: %s:error in %s\n",
 				   op, state, 0 );
-#endif
 		}
 	}
 #ifdef HAVE_EBCDIC
@@ -1528,18 +1475,6 @@ tls_verify_cb( int ok, X509_STORE_CTX *ctx )
 		__etoa( certerr );
 	}
 #endif
-#ifdef NEW_LOGGING
-	LDAP_LOG( TRANSPORT, ERR,
-		   "TLS certificate verification: depth: %d, err: %d, subject: %s,",
-		   errdepth, errnum,
-		   sname ? sname : "-unknown-" );
-	LDAP_LOG( TRANSPORT, ERR, " issuer: %s\n", iname ? iname : "-unknown-", 0, 0 );
-	if ( !ok ) {
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"TLS certificate verification: Error, %s\n",
-			certerr, 0, 0 );
-	}
-#else
 	Debug( LDAP_DEBUG_TRACE,
 		   "TLS certificate verification: depth: %d, err: %d, subject: %s,",
 		   errdepth, errnum,
@@ -1550,7 +1485,6 @@ tls_verify_cb( int ok, X509_STORE_CTX *ctx )
 			"TLS certificate verification: Error, %s\n",
 			certerr, 0, 0 );
 	}
-#endif
 	if ( sname )
 		CRYPTO_free ( sname );
 	if ( iname )
@@ -1586,14 +1520,8 @@ tls_report_error( void )
 		}
 		__etoa( buf );
 #endif
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"tls_report_error: TLS %s %s:%d\n", 
-			buf, file, line );
-#else
 		Debug( LDAP_DEBUG_ANY, "TLS: %s %s:%d\n",
 			buf, file, line );
-#endif
 #ifdef HAVE_EBCDIC
 		if ( file ) LDAP_FREE( (void *)file );
 #endif
@@ -1610,15 +1538,9 @@ tls_tmp_rsa_cb( SSL *ssl, int is_export, int key_length )
 	tmp_rsa = RSA_generate_key( key_length, RSA_F4, NULL, NULL );
 
 	if ( !tmp_rsa ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, ERR, 
-			"tls_tmp_rsa_cb: TLS Failed to generate temporary %d-bit %s "
-			"RSA key\n", key_length, is_export ? "export" : "domestic", 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"TLS: Failed to generate temporary %d-bit %s RSA key\n",
 			key_length, is_export ? "export" : "domestic", 0 );
-#endif
 		return NULL;
 	}
 	return tmp_rsa;
@@ -1646,30 +1568,18 @@ tls_seed_PRNG( const char *randfile )
 	}
 
 	if (randfile == NULL) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, 
-			"tls_seed_PRNG: TLS Use configuration file or "
-			"$RANDFILE to define seed PRNG\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"TLS: Use configuration file or $RANDFILE to define seed PRNG\n",
 			0, 0, 0);
-#endif
 		return -1;
 	}
 
 	total = RAND_load_file(randfile, -1);
 
 	if (RAND_status() == 0) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( TRANSPORT, DETAIL1, 
-			"tls_seed_PRNG: TLS PRNG not been seeded with enough data\n", 
-			0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"TLS: PRNG not been seeded with enough data\n",
 			0, 0, 0);
-#endif
 		return -1;
 	}
 
@@ -1692,6 +1602,55 @@ tls_tmp_dh_cb( SSL *ssl, int is_export, int key_length )
 #endif
 #endif
 
+void *
+ldap_pvt_tls_sb_ctx( Sockbuf *sb )
+{
+#ifdef HAVE_TLS
+	void			*p;
+	
+	if (HAS_TLS( sb )) {
+		ber_sockbuf_ctrl( sb, LBER_SB_OPT_GET_SSL, (void *)&p );
+		return p;
+	}
+#endif
+
+	return NULL;
+}
+
+int
+ldap_pvt_tls_get_strength( void *s )
+{
+#ifdef HAVE_TLS
+	SSL_CIPHER *c;
+
+	c = SSL_get_current_cipher((SSL *)s);
+	return SSL_CIPHER_get_bits(c, NULL);
+#else
+	return 0;
+#endif
+}
+
+
+int
+ldap_pvt_tls_get_my_dn( void *s, struct berval *dn, LDAPDN_rewrite_dummy *func, unsigned flags )
+{
+#ifdef HAVE_TLS
+	X509 *x;
+	X509_NAME *xn;
+	int rc;
+
+	x = SSL_get_certificate((SSL *)s);
+
+	if (!x) return LDAP_INVALID_CREDENTIALS;
+	
+	xn = X509_get_subject_name(x);
+	rc = ldap_X509dn2bv(xn, dn, (LDAPDN_rewrite_func *)func, flags );
+	return rc;
+#else
+	return LDAP_NOT_SUPPORTED;
+#endif
+}
+
 int
 ldap_start_tls_s ( LDAP *ld,
 	LDAPControl **serverctrls,
