@@ -1,14 +1,24 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.74 2003/02/11 08:23:28 ando Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* LIBLDAP url.c -- LDAP URL (RFC 2255) related routines */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/url.c,v 1.84 2004/09/04 02:54:29 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*  Portions
- *  Copyright (c) 1996 Regents of the University of Michigan.
- *  All rights reserved.
- *
- *  LIBLDAP url.c -- LDAP URL (RFC 2255) related routines
- *
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
+ * All rights reserved.
+ */
+
+
+/*
  *  LDAP URLs look like this:
  *    ldap[is]://host:port[/[dn[?[attributes][?[scope][?[filter][?exts]]]]]]
  *
@@ -68,6 +78,34 @@ int ldap_pvt_url_scheme2proto( const char *scheme )
 	return -1;
 }
 
+int ldap_pvt_url_scheme_port( const char *scheme, int port )
+{
+	assert( scheme );
+
+	if( port ) return port;
+	if( scheme == NULL ) return port;
+
+	if( strcmp("ldap", scheme) == 0 ) {
+		return LDAP_PORT;
+	}
+
+	if( strcmp("ldapi", scheme) == 0 ) {
+		return -1;
+	}
+
+	if( strcmp("ldaps", scheme) == 0 ) {
+		return LDAPS_PORT;
+	}
+
+#ifdef LDAP_CONNECTIONLESS
+	if( strcmp("cldap", scheme) == 0 ) {
+		return LDAP_PORT;
+	}
+#endif
+
+	return -1;
+}
+
 int
 ldap_pvt_url_scheme2tls( const char *scheme )
 {
@@ -224,7 +262,7 @@ static int str2scope( const char *p )
 	if ( strcasecmp( p, "one" ) == 0 ) {
 		return LDAP_SCOPE_ONELEVEL;
 
-	} else if ( strcasecmp( p, "onetree" ) == 0 ) {
+	} else if ( strcasecmp( p, "onelevel" ) == 0 ) {
 		return LDAP_SCOPE_ONELEVEL;
 
 	} else if ( strcasecmp( p, "base" ) == 0 ) {
@@ -235,6 +273,14 @@ static int str2scope( const char *p )
 
 	} else if ( strcasecmp( p, "subtree" ) == 0 ) {
 		return LDAP_SCOPE_SUBTREE;
+
+#ifdef LDAP_SCOPE_SUBORDINATE
+	} else if ( strcasecmp( p, "subordinate" ) == 0 ) {
+		return LDAP_SCOPE_SUBORDINATE;
+
+	} else if ( strcasecmp( p, "children" ) == 0 ) {
+		return LDAP_SCOPE_SUBORDINATE;
+#endif
 	}
 
 	return( -1 );
@@ -331,10 +377,13 @@ char * ldap_url_desc2str( LDAPURLDesc *u )
 	if ( len ) len++; /* ? */
 
 	switch( u->lud_scope ) {
+		case LDAP_SCOPE_BASE:
 		case LDAP_SCOPE_ONELEVEL:
 		case LDAP_SCOPE_SUBTREE:
-		case LDAP_SCOPE_BASE:
-			len += sizeof("base");
+#ifdef LDAP_FEATURE_SUBORDINATE_SCOPE
+		case LDAP_SCOPE_SUBORDINATE:
+#endif
+			len += sizeof("subordinate");
 			if( !sep ) sep = 3;
 			break;
 
@@ -403,6 +452,12 @@ char * ldap_url_desc2str( LDAPURLDesc *u )
 		strcpy( &s[sofar], "sub" );
 		sofar += sizeof("sub") - 1;
 		break;
+#ifdef LDAP_FEATURE_SUBORDINATE_SCOPE
+	case LDAP_SCOPE_SUBORDINATE:
+		strcpy( &s[sofar], "children" );
+		sofar += sizeof("children") - 1;
+		break;
+#endif
 	}
 
 	if( sep < 4 ) goto done;
@@ -444,11 +499,7 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	 * because a call to LDAP_INT_GLOBAL_OPT() will try to allocate
 	 * the options and cause infinite recursion
 	 */
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "ldap_url_parse_ext(%s)\n", url_in, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_url_parse_ext(%s)\n", url_in, 0, 0 );
-#endif
 #endif
 
 	*ludpp = NULL;	/* pessimistic */
@@ -491,7 +542,6 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	ludp->lud_port = 0;
 	ludp->lud_dn = NULL;
 	ludp->lud_attrs = NULL;
-	ludp->lud_filter = NULL;
 	ludp->lud_scope = LDAP_SCOPE_DEFAULT;
 	ludp->lud_filter = NULL;
 	ludp->lud_exts = NULL;
@@ -527,6 +577,8 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	}
 
 	if ( q != NULL ) {
+		char	*next;
+
 		*q++ = '\0';
 		ldap_pvt_hex_unescape( q );
 
@@ -536,7 +588,12 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 			return LDAP_URL_ERR_BADURL;
 		}
 
-		ludp->lud_port = atoi( q );
+		ludp->lud_port = strtol( q, &next, 10 );
+		if ( next == NULL || next[0] != '\0' ) {
+			LDAP_FREE( url );
+			ldap_free_urldesc( ludp );
+			return LDAP_URL_ERR_BADURL;
+		}
 	}
 
 	ldap_pvt_hex_unescape( url );
@@ -691,7 +748,6 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 			return LDAP_URL_ERR_BADFILTER;
 		}
 
-		LDAP_FREE( ludp->lud_filter );
 		ludp->lud_filter = LDAP_STRDUP( p );
 
 		if( ludp->lud_filter == NULL ) {
@@ -898,7 +954,7 @@ ldap_url_parselist_ext (LDAPURLDesc **ludlist, const char *url, const char *sep
 
 	urls = ldap_str2charray(url, sep);
 	if (urls == NULL)
-		return LDAP_NO_MEMORY;
+		return LDAP_URL_ERR_MEM;
 
 	/* count the URLs... */
 	for (i = 0; urls[i] != NULL; i++) ;
@@ -915,7 +971,7 @@ ldap_url_parselist_ext (LDAPURLDesc **ludlist, const char *url, const char *sep
 		*ludlist = ludp;
 	}
 	ldap_charray_free(urls);
-	return LDAP_SUCCESS;
+	return LDAP_URL_SUCCESS;
 }
 
 int
@@ -976,9 +1032,14 @@ ldap_url_parsehosts(
 				}
 			}
 			if (p != NULL) {
+				char	*next;
+
 				*p++ = 0;
 				ldap_pvt_hex_unescape(p);
-				ludp->lud_port = atoi(p);
+				ludp->lud_port = strtol( p, &next, 10 );
+				if ( next == NULL || next[0] != '\0' ) {
+					return LDAP_PARAM_ERROR;
+				}
 			}
 		}
 		ldap_pvt_hex_unescape(ludp->lud_host);
