@@ -1,25 +1,28 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/util-int.c,v 1.45 2003/05/25 18:50:26 hallvard Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/libldap/util-int.c,v 1.51 2004/05/21 08:53:30 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1998 A. Hartgers.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Bart Hartgers for inclusion in
+ * OpenLDAP Software.
+ */
+
 /*
  * util-int.c	Various functions to replace missing threadsafe ones.
- *				  Without the real *_r funcs, things will
- *				  work, but might not be threadsafe. 
- * 
- * Written by Bart Hartgers.
- *
- * Copyright 1998, A. Hartgers, All rights reserved.
- * This software is not subject to any license of Eindhoven University of
- * Technology, since it was written in my spare time.
- *			
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
- */ 
-
+ *				Without the real *_r funcs, things will
+ *				work, but might not be threadsafe. 
+ */
 
 #include "portable.h"
 
@@ -40,6 +43,12 @@
 extern int h_errno;
 #endif
 
+#ifdef HAVE_HSTRERROR
+# define HSTRERROR(e)	hstrerror(e)
+#else
+# define HSTRERROR(e)	hp_strerror(e)
+#endif
+
 #ifndef LDAP_R_COMPILE
 # undef HAVE_REENTRANT_FUNCTIONS
 # undef HAVE_CTIME_R
@@ -186,7 +195,7 @@ int ldap_pvt_gethostbyname_a(
 #endif	
 }
 
-#ifndef GETNAMEINFO
+#if !defined( GETNAMEINFO ) && !defined( HAVE_HERROR )
 static const char *
 hp_strerror( int err )
 {
@@ -246,7 +255,7 @@ int ldap_pvt_get_hname(
 		alen = sizeof(sin->sin_addr);
 	} else {
 		rc = NO_RECOVERY;
-		*err = (char *)hp_strerror( rc );
+		*err = (char *)HSTRERROR( rc );
 		return rc;
 	}
 #if defined( HAVE_GETHOSTBYADDR_R )
@@ -278,7 +287,7 @@ int ldap_pvt_get_hname(
 	if (hp) {
 		strncpy( name, hp->h_name, namelen );
 	} else {
-		*err = (char *)hp_strerror( h_errno );
+		*err = (char *)HSTRERROR( h_errno );
 	}
 	LDAP_FREE(buf);
 #else /* HAVE_GETHOSTBYADDR_R */
@@ -396,6 +405,10 @@ void ldap_int_utils_init( void )
 	ldap_pvt_thread_mutex_init( &ldap_int_ctime_mutex );
 #endif
 	ldap_pvt_thread_mutex_init( &ldap_int_resolv_mutex );
+
+#ifdef HAVE_CYRUS_SASL
+	ldap_pvt_thread_mutex_init( &ldap_int_sasl_mutex );
+#endif
 #endif
 
 	/* call other module init functions here... */
