@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ure/ure.c,v 1.11 2002/01/04 20:17:43 kurt Exp $ */
-/*
- * Copyright 2000-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ure/ure.c,v 1.15 2004/01/01 18:15:28 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright 1997, 1998, 1999 Computing Research Labs,
+/* Copyright 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -2044,7 +2052,7 @@ ure_exec(ure_dfa_t dfa, int flags, ucs2_t *text, unsigned long textlen,
             }
 
             if (matched) {
-                if (ms == ~0)
+                if (ms == ~0UL)
                   ms = lp - text;
                 else
                   me = sp - text;
@@ -2119,5 +2127,5 @@ ure_exec(ure_dfa_t dfa, int flags, ucs2_t *text, unsigned long textlen,
     *match_start = ms;
     *match_end = me;
 
-    return (ms != ~0) ? 1 : 0;
+    return (ms != ~0UL) ? 1 : 0;
 }
