@@ -1,8 +1,18 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/passwd.c,v 1.74 2003/05/02 11:29:29 hallvard Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/passwd.c,v 1.92 2004/06/02 18:29:21 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 /*
  * int lutil_passwd(
  *	const struct berval *passwd,
@@ -31,16 +41,7 @@
 #	endif
 #endif
 
-#ifdef SLAPD_KPASSWD
-#	include <ac/krb.h>
-#	include <ac/krb5.h>
-#endif
-
-/* KPASSWD/krb.h brings in a conflicting des.h so don't use both.
- * configure currently requires OpenSSL to enable LMHASH. Obviously
- * this requirement can be fulfilled by the KRB DES library as well.
- */
-#if defined(SLAPD_LMHASH) && !defined(DES_ENCRYPT)
+#if defined(SLAPD_LMHASH)
 #	include <openssl/des.h>
 #endif /* SLAPD_LMHASH */
 
@@ -91,6 +92,8 @@ struct pw_slist {
 
 /* password check routines */
 
+#define	SALT_SIZE	4
+
 static LUTIL_PASSWD_CHK_FUNC chk_md5;
 static LUTIL_PASSWD_CHK_FUNC chk_smd5;
 static LUTIL_PASSWD_HASH_FUNC hash_smd5;
@@ -109,18 +112,10 @@ static LUTIL_PASSWD_CHK_FUNC chk_lanman;
 static LUTIL_PASSWD_HASH_FUNC hash_lanman;
 #endif
 
-#ifdef SLAPD_NS_MTA_MD5
-static LUTIL_PASSWD_CHK_FUNC chk_ns_mta_md5;
-#endif
-
 #ifdef SLAPD_SPASSWD
 static LUTIL_PASSWD_CHK_FUNC chk_sasl;
 #endif
 
-#ifdef SLAPD_KPASSWD
-static LUTIL_PASSWD_CHK_FUNC chk_kerberos;
-#endif
-
 #ifdef SLAPD_CRYPT
 static LUTIL_PASSWD_CHK_FUNC chk_crypt;
 static LUTIL_PASSWD_HASH_FUNC hash_crypt;
@@ -152,18 +147,10 @@ static const struct pw_scheme pw_schemes_default[] =
 	{ BER_BVC("{LANMAN}"),		chk_lanman, hash_lanman },
 #endif /* SLAPD_LMHASH */
 
-#ifdef SLAPD_NS_MTA_MD5
-	{ BER_BVC("{NS-MTA-MD5}"),	chk_ns_mta_md5, NULL },
-#endif /* SLAPD_NS_MTA_MD5 */
-
 #ifdef SLAPD_SPASSWD
 	{ BER_BVC("{SASL}"),		chk_sasl, NULL },
 #endif
 
-#ifdef SLAPD_KPASSWD
-	{ BER_BVC("{KERBEROS}"),	chk_kerberos, NULL },
-#endif
-
 #ifdef SLAPD_CRYPT
 	{ BER_BVC("{CRYPT}"),		chk_crypt, hash_crypt },
 # if defined( HAVE_GETPWNAM ) && defined( HAVE_PW_PASSWD )
@@ -198,11 +185,10 @@ int lutil_passwd_add(
 
 void lutil_passwd_init()
 {
-	struct pw_slist *ptr;
 	struct pw_scheme *s;
 
 	for( s=(struct pw_scheme *)pw_schemes_default; s->name.bv_val; s++) {
-		if ( lutil_passwd_add( &s->name, s->chk_fn, s->hash_fn )) break;
+		if ( lutil_passwd_add( &s->name, s->chk_fn, s->hash_fn ) ) break;
 	}
 }
 
@@ -326,26 +312,20 @@ lutil_passwd(
 	return 1;
 }
 
-struct berval * lutil_passwd_generate( ber_len_t len )
+int lutil_passwd_generate( struct berval *pw, ber_len_t len )
 {
-	struct berval *pw;
 
-	if( len < 1 ) return NULL;
-
-	pw = ber_memalloc( sizeof( struct berval ) );
-	if( pw == NULL ) return NULL;
+	if( len < 1 ) return -1;
 
 	pw->bv_len = len;
 	pw->bv_val = ber_memalloc( len + 1 );
 
 	if( pw->bv_val == NULL ) {
-		ber_memfree( pw );
-		return NULL;
+		return -1;
 	}
 
 	if( lutil_entropy( (unsigned char *) pw->bv_val, pw->bv_len) < 0 ) {
-		ber_bvfree( pw );
-		return NULL; 
+		return -1; 
 	}
 
 	for( len = 0; len < pw->bv_len; len++ ) {
@@ -355,68 +335,77 @@ struct berval * lutil_passwd_generate( ber_len_t len )
 
 	pw->bv_val[len] = '\0';
 	
-	return pw;
+	return 0;
 }
 
-struct berval * lutil_passwd_hash(
+int lutil_passwd_hash(
 	const struct berval * passwd,
 	const char * method,
+	struct berval *hash,
 	const char **text )
 {
 	const struct pw_scheme *sc = get_scheme( method );
 
-	if( text ) *text = NULL;
-	if( sc == NULL ) return NULL;
-	if( ! sc->hash_fn ) return NULL;
+	hash->bv_val = NULL;
+	hash->bv_len = 0;
 
-	return (sc->hash_fn)( &sc->name, passwd, text );
+	if( sc == NULL ) {
+		if( text ) *text = "scheme not recognized";
+		return -1;
+	}
+
+	if( ! sc->hash_fn ) {
+		if( text ) *text = "scheme provided no hash function";
+		return -1;
+	}
+
+	if( text ) *text = NULL;
+
+	return (sc->hash_fn)( &sc->name, passwd, hash, text );
 }
 
 /* pw_string is only called when SLAPD_LMHASH or SLAPD_CRYPT is defined */
 #if defined(SLAPD_LMHASH) || defined(SLAPD_CRYPT)
-static struct berval * pw_string(
+static int pw_string(
 	const struct berval *sc,
-	const struct berval *passwd )
+	struct berval *passwd )
 {
-	struct berval *pw = ber_memalloc( sizeof( struct berval ) );
-	if( pw == NULL ) return NULL;
+	struct berval pw;
 
-	pw->bv_len = sc->bv_len + passwd->bv_len;
-	pw->bv_val = ber_memalloc( pw->bv_len + 1 );
+	pw.bv_len = sc->bv_len + passwd->bv_len;
+	pw.bv_val = ber_memalloc( pw.bv_len + 1 );
 
-	if( pw->bv_val == NULL ) {
-		ber_memfree( pw );
-		return NULL;
+	if( pw.bv_val == NULL ) {
+		return LUTIL_PASSWD_ERR;
 	}
 
-	AC_MEMCPY( pw->bv_val, sc->bv_val, sc->bv_len );
-	AC_MEMCPY( &pw->bv_val[sc->bv_len], passwd->bv_val, passwd->bv_len );
+	AC_MEMCPY( pw.bv_val, sc->bv_val, sc->bv_len );
+	AC_MEMCPY( &pw.bv_val[sc->bv_len], passwd->bv_val, passwd->bv_len );
 
-	pw->bv_val[pw->bv_len] = '\0';
-	return pw;
+	pw.bv_val[pw.bv_len] = '\0';
+	*passwd = pw;
+
+	return LUTIL_PASSWD_OK;
 }
 #endif /* SLAPD_LMHASH || SLAPD_CRYPT */
 
-static struct berval * pw_string64(
+static int pw_string64(
 	const struct berval *sc,
 	const struct berval *hash,
+	struct berval *b64,
 	const struct berval *salt )
 {
 	int rc;
 	struct berval string;
-	struct berval *b64 = ber_memalloc( sizeof(struct berval) );
 	size_t b64len;
 
-	if( b64 == NULL ) return NULL;
-
 	if( salt ) {
 		/* need to base64 combined string */
 		string.bv_len = hash->bv_len + salt->bv_len;
 		string.bv_val = ber_memalloc( string.bv_len + 1 );
 
 		if( string.bv_val == NULL ) {
-			ber_memfree( b64 );
-			return NULL;
+			return LUTIL_PASSWD_ERR;
 		}
 
 		AC_MEMCPY( string.bv_val, hash->bv_val,
@@ -435,8 +424,7 @@ static struct berval * pw_string64(
 
 	if( b64->bv_val == NULL ) {
 		if( salt ) ber_memfree( string.bv_val );
-		ber_memfree( b64 );
-		return NULL;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	AC_MEMCPY(b64->bv_val, sc->bv_val, sc->bv_len);
@@ -448,14 +436,13 @@ static struct berval * pw_string64(
 	if( salt ) ber_memfree( string.bv_val );
 	
 	if( rc < 0 ) {
-		ber_bvfree( b64 );
-		return NULL;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	/* recompute length */
 	b64->bv_len = sc->bv_len + rc;
 	assert( strlen(b64->bv_val) == b64->bv_len );
-	return b64;
+	return LUTIL_PASSWD_OK;
 }
 
 /* PASSWORD CHECK ROUTINES */
@@ -472,22 +459,23 @@ static int chk_ssha1(
 	int rc;
 	unsigned char *orig_pass = NULL;
 
-	/* safety check */
+	/* safety check -- must have some salt */
 	if (LUTIL_BASE64_DECODE_LEN(passwd->bv_len) <= sizeof(SHA1digest)) {
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	/* decode base64 password */
 	orig_pass = (unsigned char *) ber_memalloc( (size_t) (
 		LUTIL_BASE64_DECODE_LEN(passwd->bv_len) + 1) );
 
-	if( orig_pass == NULL ) return -1;
+	if( orig_pass == NULL ) return LUTIL_PASSWD_ERR;
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 
-	if (rc <= sizeof(SHA1digest)) {
+	/* safety check -- must have some salt */
+	if (rc <= (int)(sizeof(SHA1digest))) {
 		ber_memfree(orig_pass);
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
  
 	/* hash credentials with salt */
@@ -502,7 +490,7 @@ static int chk_ssha1(
 	/* compare */
 	rc = memcmp((char *)orig_pass, (char *)SHA1digest, sizeof(SHA1digest));
 	ber_memfree(orig_pass);
-	return rc ? 1 : 0;
+	return rc ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
 }
 
 static int chk_sha1(
@@ -516,17 +504,22 @@ static int chk_sha1(
 	int rc;
 	unsigned char *orig_pass = NULL;
  
+	/* safety check */
+	if (LUTIL_BASE64_DECODE_LEN(passwd->bv_len) < sizeof(SHA1digest)) {
+		return LUTIL_PASSWD_ERR;
+	}
+
 	/* base64 un-encode password */
 	orig_pass = (unsigned char *) ber_memalloc( (size_t) (
 		LUTIL_BASE64_DECODE_LEN(passwd->bv_len) + 1) );
 
-	if( orig_pass == NULL ) return -1;
+	if( orig_pass == NULL ) return LUTIL_PASSWD_ERR;
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 
 	if( rc != sizeof(SHA1digest) ) {
 		ber_memfree(orig_pass);
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
  
 	/* hash credentials with salt */
@@ -538,7 +531,7 @@ static int chk_sha1(
 	/* compare */
 	rc = memcmp((char *)orig_pass, (char *)SHA1digest, sizeof(SHA1digest));
 	ber_memfree(orig_pass);
-	return rc ? 1 : 0;
+	return rc ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
 }
 #endif
 
@@ -555,20 +548,20 @@ static int chk_smd5(
 
 	/* safety check */
 	if (LUTIL_BASE64_DECODE_LEN(passwd->bv_len) <= sizeof(MD5digest)) {
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	/* base64 un-encode password */
 	orig_pass = (unsigned char *) ber_memalloc( (size_t) (
 		LUTIL_BASE64_DECODE_LEN(passwd->bv_len) + 1) );
 
-	if( orig_pass == NULL ) return -1;
+	if( orig_pass == NULL ) return LUTIL_PASSWD_ERR;
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 
-	if (rc <= sizeof(MD5digest)) {
+	if (rc <= (int)(sizeof(MD5digest))) {
 		ber_memfree(orig_pass);
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	/* hash credentials with salt */
@@ -584,7 +577,7 @@ static int chk_smd5(
 	/* compare */
 	rc = memcmp((char *)orig_pass, (char *)MD5digest, sizeof(MD5digest));
 	ber_memfree(orig_pass);
-	return rc ? 1 : 0;
+	return rc ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
 }
 
 static int chk_md5(
@@ -598,16 +591,21 @@ static int chk_md5(
 	int rc;
 	unsigned char *orig_pass = NULL;
 
+	/* safety check */
+	if (LUTIL_BASE64_DECODE_LEN(passwd->bv_len) < sizeof(MD5digest)) {
+		return LUTIL_PASSWD_ERR;
+	}
+
 	/* base64 un-encode password */
 	orig_pass = (unsigned char *) ber_memalloc( (size_t) (
 		LUTIL_BASE64_DECODE_LEN(passwd->bv_len) + 1) );
 
-	if( orig_pass == NULL ) return -1;
+	if( orig_pass == NULL ) return LUTIL_PASSWD_ERR;
 
 	rc = lutil_b64_pton(passwd->bv_val, orig_pass, passwd->bv_len);
 	if ( rc != sizeof(MD5digest) ) {
 		ber_memfree(orig_pass);
-		return -1;
+		return LUTIL_PASSWD_ERR;
 	}
 
 	/* hash credentials with salt */
@@ -620,528 +618,10 @@ static int chk_md5(
 	/* compare */
 	rc = memcmp((char *)orig_pass, (char *)MD5digest, sizeof(MD5digest));
 	ber_memfree(orig_pass);
-	return rc ? 1 : 0;
+	return rc ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
 }
 
 #ifdef SLAPD_LMHASH
-static int chk_lanman(
-	const struct berval *scheme,
-	const struct berval *passwd,
-	const struct berval *cred,
-	const char **text )
-{
-	struct berval *hash;
-
-	hash = hash_lanman( scheme, cred );
-	return memcmp( &hash->bv_val[scheme->bv_len], passwd->bv_val, 32);
-}
-#endif /* SLAPD_LMHASH */
-
-#ifdef SLAPD_NS_MTA_MD5
-static int chk_ns_mta_md5(
-	const struct berval *scheme,
-	const struct berval *passwd,
-	const struct berval *cred,
-	const char **text )
-{
-	lutil_MD5_CTX MD5context;
-	unsigned char MD5digest[LUTIL_MD5_BYTES], c;
-	char buffer[LUTIL_MD5_BYTES + LUTIL_MD5_BYTES + 1];
-	int i;
-
-	/* hash credentials with salt */
-	lutil_MD5Init(&MD5context);
-	lutil_MD5Update(&MD5context,
-		(const unsigned char *) &passwd->bv_val[32],
-		32 );
-
-	c = 0x59;
-	lutil_MD5Update(&MD5context,
-		(const unsigned char *) &c,
-		1 );
-
-	lutil_MD5Update(&MD5context,
-		(const unsigned char *) cred->bv_val,
-		cred->bv_len );
-
-	c = 0xF7;
-	lutil_MD5Update(&MD5context,
-		(const unsigned char *) &c,
-		1 );
-
-	lutil_MD5Update(&MD5context,
-		(const unsigned char *) &passwd->bv_val[32],
-		32 );
-
-	lutil_MD5Final(MD5digest, &MD5context);
-
-	for( i=0; i < sizeof( MD5digest ); i++ ) {
-		buffer[i+i]   = "0123456789abcdef"[(MD5digest[i]>>4) & 0x0F]; 
-		buffer[i+i+1] = "0123456789abcdef"[ MD5digest[i] & 0x0F]; 
-	}
-
-	/* compare */
-	return memcmp((char *)passwd->bv_val, (char *)buffer, sizeof(buffer))
-		? 1 : 0;
-}
-#endif
-
-#ifdef SLAPD_SPASSWD
-#ifdef HAVE_CYRUS_SASL
-sasl_conn_t *lutil_passwd_sasl_conn = NULL;
-#endif
-
-static int chk_sasl(
-	const struct berval *sc,
-	const struct berval * passwd,
-	const struct berval * cred,
-	const char **text )
-{
-	unsigned int i;
-	int rtn;
-
-	for( i=0; i<cred->bv_len; i++) {
-		if(cred->bv_val[i] == '\0') {
-			return 1;	/* NUL character in password */
-		}
-	}
-
-	if( cred->bv_val[i] != '\0' ) {
-		return 1;	/* cred must behave like a string */
-	}
-
-	for( i=0; i<passwd->bv_len; i++) {
-		if(passwd->bv_val[i] == '\0') {
-			return 1;	/* NUL character in password */
-		}
-	}
-
-	if( passwd->bv_val[i] != '\0' ) {
-		return 1;	/* passwd must behave like a string */
-	}
-
-	rtn = 1;
-
-#ifdef HAVE_CYRUS_SASL
-	if( lutil_passwd_sasl_conn != NULL ) {
-		int sc;
-# if SASL_VERSION_MAJOR < 2
-		sc = sasl_checkpass( lutil_passwd_sasl_conn,
-			passwd->bv_val, passwd->bv_len,
-			cred->bv_val, cred->bv_len,
-			text );
-# else
-		sc = sasl_checkpass( lutil_passwd_sasl_conn,
-			passwd->bv_val, passwd->bv_len,
-			cred->bv_val, cred->bv_len );
-# endif
-		rtn = ( sc != SASL_OK );
-	}
-#endif
-
-	return rtn;
-}
-#endif
-
-#ifdef SLAPD_KPASSWD
-static int chk_kerberos(
-	const struct berval *sc,
-	const struct berval * passwd,
-	const struct berval * cred,
-	const char **text )
-{
-	unsigned int i;
-	int rtn;
-
-	for( i=0; i<cred->bv_len; i++) {
-		if(cred->bv_val[i] == '\0') {
-			return 1;	/* NUL character in password */
-		}
-	}
-
-	if( cred->bv_val[i] != '\0' ) {
-		return 1;	/* cred must behave like a string */
-	}
-
-	for( i=0; i<passwd->bv_len; i++) {
-		if(passwd->bv_val[i] == '\0') {
-			return 1;	/* NUL character in password */
-		}
-	}
-
-	if( passwd->bv_val[i] != '\0' ) {
-		return 1;	/* passwd must behave like a string */
-	}
-
-	rtn = 1;
-
-#ifdef HAVE_KRB5 /* HAVE_HEIMDAL_KRB5 */
-	{
-/* Portions:
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska H\xf6gskolan
- * (Royal Institute of Technology, Stockholm, Sweden).
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * 3. Neither the name of the Institute nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-		krb5_context context;
-   		krb5_error_code ret;
-   		krb5_creds creds;
-   		krb5_get_init_creds_opt get_options;
-   		krb5_verify_init_creds_opt verify_options;
-		krb5_principal client, server;
-#ifdef notdef
-		krb5_preauthtype pre_auth_types[] = {KRB5_PADATA_ENC_TIMESTAMP};
-#endif
-
-		ret = krb5_init_context( &context );
-		if (ret) {
-			return 1;
-		}
-
-#ifdef notdef
-		krb5_get_init_creds_opt_set_preauth_list(&get_options,
-			pre_auth_types, 1);
-#endif
-
-   		krb5_get_init_creds_opt_init( &get_options );
-
-		krb5_verify_init_creds_opt_init( &verify_options );
-	
-		ret = krb5_parse_name( context, passwd->bv_val, &client );
-
-		if (ret) {
-			krb5_free_context( context );
-			return 1;
-		}
-
-		ret = krb5_get_init_creds_password( context,
-			&creds, client, cred->bv_val, NULL,
-			NULL, 0, NULL, &get_options );
-
-		if (ret) {
-			krb5_free_principal( context, client );
-			krb5_free_context( context );
-			return 1;
-		}
-
-		{
-			char *host = ldap_pvt_get_fqdn( NULL );
-
-			if( host == NULL ) {
-				krb5_free_principal( context, client );
-				krb5_free_context( context );
-				return 1;
-			}
-
-			ret = krb5_sname_to_principal( context,
-				host, "ldap", KRB5_NT_SRV_HST, &server );
-
-			ber_memfree( host );
-		}
-
-		if (ret) {
-			krb5_free_principal( context, client );
-			krb5_free_context( context );
-			return 1;
-		}
-
-		ret = krb5_verify_init_creds( context,
-			&creds, server, NULL, NULL, &verify_options );
-
-		krb5_free_principal( context, client );
-		krb5_free_principal( context, server );
-		krb5_free_cred_contents( context, &creds );
-		krb5_free_context( context );
-
-		rtn = !!ret;
-	}
-#elif	defined(HAVE_KRB4)
-	{
-		/* Borrowed from Heimdal kpopper */
-/* Portions:
- * Copyright (c) 1989 Regents of the University of California.
- * All rights reserved.  The Berkeley software License Agreement
- * specifies the terms and conditions for redistribution.
- */
-
-		int status;
-		char lrealm[REALM_SZ];
-		char tkt[MAXHOSTNAMELEN];
-
-		status = krb_get_lrealm(lrealm,1);
-		if (status == KFAILURE) {
-			return 1;
-		}
-
-		snprintf(tkt, sizeof(tkt), "%s_slapd.%u",
-			TKT_ROOT, (unsigned)getpid());
-		krb_set_tkt_string (tkt);
-
-		status = krb_verify_user( passwd->bv_val, "", lrealm,
-			cred->bv_val, 1, "ldap");
-
-		dest_tkt(); /* no point in keeping the tickets */
-
-		return status == KFAILURE;
-	}
-#endif
-
-	return rtn;
-}
-#endif /* SLAPD_KPASSWD */
-
-#ifdef SLAPD_CRYPT
-static int chk_crypt(
-	const struct berval *sc,
-	const struct berval * passwd,
-	const struct berval * cred,
-	const char **text )
-{
-	char *cr;
-	unsigned int i;
-
-	for( i=0; i<cred->bv_len; i++) {
-		if(cred->bv_val[i] == '\0') {
-			return 1;	/* NUL character in password */
-		}
-	}
-
-	if( cred->bv_val[i] != '\0' ) {
-		return -1;	/* cred must behave like a string */
-	}
-
-	if( passwd->bv_len < 2 ) {
-		return -1;	/* passwd must be at least two characters long */
-	}
-
-	for( i=0; i<passwd->bv_len; i++) {
-		if(passwd->bv_val[i] == '\0') {
-			return -1;	/* NUL character in password */
-		}
-	}
-
-	if( passwd->bv_val[i] != '\0' ) {
-		return -1;	/* passwd must behave like a string */
-	}
-
-	cr = crypt( cred->bv_val, passwd->bv_val );
-
-	if( cr == NULL || cr[0] == '\0' ) {
-		/* salt must have been invalid */
-		return -1;
-	}
-
-	return strcmp( passwd->bv_val, cr ) ? 1 : 0;
-}
-
-# if defined( HAVE_GETPWNAM ) && defined( HAVE_PW_PASSWD )
-static int chk_unix(
-	const struct berval *sc,
-	const struct berval * passwd,
-	const struct berval * cred,
-	const char **text )
-{
-	unsigned int i;
-	char *pw,*cr;
-
-	for( i=0; i<cred->bv_len; i++) {
-		if(cred->bv_val[i] == '\0') {
-			return -1;	/* NUL character in password */
-		}
-	}
-	if( cred->bv_val[i] != '\0' ) {
-		return -1;	/* cred must behave like a string */
-	}
-
-	for( i=0; i<passwd->bv_len; i++) {
-		if(passwd->bv_val[i] == '\0') {
-			return -1;	/* NUL character in password */
-		}
-	}
-
-	if( passwd->bv_val[i] != '\0' ) {
-		return -1;	/* passwd must behave like a string */
-	}
-
-	{
-		struct passwd *pwd = getpwnam(passwd->bv_val);
-
-		if(pwd == NULL) {
-			return -1;	/* not found */
-		}
-
-		pw = pwd->pw_passwd;
-	}
-#  ifdef HAVE_GETSPNAM
-	{
-		struct spwd *spwd = getspnam(passwd->bv_val);
-
-		if(spwd != NULL) {
-			pw = spwd->sp_pwdp;
-		}
-	}
-#  endif
-#  ifdef HAVE_AIX_SECURITY
-	{
-		struct userpw *upw = getuserpw(passwd->bv_val);
-
-		if (upw != NULL) {
-			pw = upw->upw_passwd;
-		}
-	}
-#  endif
-
-	if( pw == NULL || pw[0] == '\0' || pw[1] == '\0' ) {
-		/* password must must be at least two characters long */
-		return -1;
-	}
-
-	cr = crypt(cred->bv_val, pw);
-
-	if( cr == NULL || cr[0] == '\0' ) {
-		/* salt must have been invalid */
-		return -1;
-	}
-
-	return strcmp(pw, cr) ? 1 : 0;
-
-}
-# endif
-#endif
-
-/* PASSWORD GENERATION ROUTINES */
-
-#ifdef LUTIL_SHA1_BYTES
-static struct berval *hash_ssha1(
-	const struct berval *scheme,
-	const struct berval  *passwd,
-	const char **text )
-{
-	lutil_SHA1_CTX  SHA1context;
-	unsigned char   SHA1digest[LUTIL_SHA1_BYTES];
-	char            saltdata[4];
-	struct berval digest;
-	struct berval salt;
-
-	digest.bv_val = (char *) SHA1digest;
-	digest.bv_len = sizeof(SHA1digest);
-	salt.bv_val = saltdata;
-	salt.bv_len = sizeof(saltdata);
-
-	if( lutil_entropy( (unsigned char *) salt.bv_val, salt.bv_len) < 0 ) {
-		return NULL; 
-	}
-
-	lutil_SHA1Init( &SHA1context );
-	lutil_SHA1Update( &SHA1context,
-		(const unsigned char *)passwd->bv_val, passwd->bv_len );
-	lutil_SHA1Update( &SHA1context,
-		(const unsigned char *)salt.bv_val, salt.bv_len );
-	lutil_SHA1Final( SHA1digest, &SHA1context );
-
-	return pw_string64( scheme, &digest, &salt);
-}
-
-static struct berval *hash_sha1(
-	const struct berval *scheme,
-	const struct berval  *passwd,
-	const char **text )
-{
-	lutil_SHA1_CTX  SHA1context;
-	unsigned char   SHA1digest[LUTIL_SHA1_BYTES];
-	struct berval digest;
-	digest.bv_val = (char *) SHA1digest;
-	digest.bv_len = sizeof(SHA1digest);
-     
-	lutil_SHA1Init( &SHA1context );
-	lutil_SHA1Update( &SHA1context,
-		(const unsigned char *)passwd->bv_val, passwd->bv_len );
-	lutil_SHA1Final( SHA1digest, &SHA1context );
-            
-	return pw_string64( scheme, &digest, NULL);
-}
-#endif
-
-static struct berval *hash_smd5(
-	const struct berval *scheme,
-	const struct berval  *passwd,
-	const char **text )
-{
-	lutil_MD5_CTX   MD5context;
-	unsigned char   MD5digest[LUTIL_MD5_BYTES];
-	char            saltdata[4];
-	struct berval digest;
-	struct berval salt;
-
-	digest.bv_val = (char *) MD5digest;
-	digest.bv_len = sizeof(MD5digest);
-	salt.bv_val = saltdata;
-	salt.bv_len = sizeof(saltdata);
-
-	if( lutil_entropy( (unsigned char *) salt.bv_val, salt.bv_len) < 0 ) {
-		return NULL; 
-	}
-
-	lutil_MD5Init( &MD5context );
-	lutil_MD5Update( &MD5context,
-		(const unsigned char *) passwd->bv_val, passwd->bv_len );
-	lutil_MD5Update( &MD5context,
-		(const unsigned char *) salt.bv_val, salt.bv_len );
-	lutil_MD5Final( MD5digest, &MD5context );
-
-	return pw_string64( scheme, &digest, &salt );
-}
-
-static struct berval *hash_md5(
-	const struct berval *scheme,
-	const struct berval  *passwd,
-	const char **text )
-{
-	lutil_MD5_CTX   MD5context;
-	unsigned char   MD5digest[LUTIL_MD5_BYTES];
-
-	struct berval digest;
-
-	digest.bv_val = (char *) MD5digest;
-	digest.bv_len = sizeof(MD5digest);
-
-	lutil_MD5Init( &MD5context );
-	lutil_MD5Update( &MD5context,
-		(const unsigned char *) passwd->bv_val, passwd->bv_len );
-	lutil_MD5Final( MD5digest, &MD5context );
-
-	return pw_string64( scheme, &digest, NULL );
-;
-}
-
-#ifdef SLAPD_LMHASH 
 /* pseudocode from RFC2433
  * A.2 LmPasswordHash()
  * 
@@ -1217,9 +697,347 @@ static void lmPasswd_to_key(
 	des_set_odd_parity( key );
 }	
 
-static struct berval *hash_lanman(
+static int chk_lanman(
 	const struct berval *scheme,
 	const struct berval *passwd,
+	const struct berval *cred,
+	const char **text )
+{
+	int i;
+	char UcasePassword[15];
+	des_cblock key;
+	des_key_schedule schedule;
+	des_cblock StdText = "KGS!@#$%";
+	des_cblock PasswordHash1, PasswordHash2;
+	char PasswordHash[33], storedPasswordHash[33];
+	
+	for( i=0; i<cred->bv_len; i++) {
+		if(cred->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+	
+	if( cred->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
+	}
+	
+	strncpy( UcasePassword, cred->bv_val, 14 );
+	UcasePassword[14] = '\0';
+	ldap_pvt_str2upper( UcasePassword );
+	
+	lmPasswd_to_key( UcasePassword, &key );
+	des_set_key_unchecked( &key, schedule );
+	des_ecb_encrypt( &StdText, &PasswordHash1, schedule , DES_ENCRYPT );
+	
+	lmPasswd_to_key( &UcasePassword[7], &key );
+	des_set_key_unchecked( &key, schedule );
+	des_ecb_encrypt( &StdText, &PasswordHash2, schedule , DES_ENCRYPT );
+	
+	sprintf( PasswordHash, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
+		PasswordHash1[0],PasswordHash1[1],PasswordHash1[2],PasswordHash1[3],
+		PasswordHash1[4],PasswordHash1[5],PasswordHash1[6],PasswordHash1[7],
+		PasswordHash2[0],PasswordHash2[1],PasswordHash2[2],PasswordHash2[3],
+		PasswordHash2[4],PasswordHash2[5],PasswordHash2[6],PasswordHash2[7] );
+	
+	/* as a precaution convert stored password hash to lower case */
+	strncpy( storedPasswordHash, passwd->bv_val, 32 );
+	storedPasswordHash[32] = '\0';
+	ldap_pvt_str2lower( storedPasswordHash );
+	
+	return memcmp( PasswordHash, storedPasswordHash, 32) ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
+}
+#endif /* SLAPD_LMHASH */
+
+#ifdef SLAPD_SPASSWD
+#ifdef HAVE_CYRUS_SASL
+sasl_conn_t *lutil_passwd_sasl_conn = NULL;
+#endif
+
+static int chk_sasl(
+	const struct berval *sc,
+	const struct berval * passwd,
+	const struct berval * cred,
+	const char **text )
+{
+	unsigned int i;
+	int rtn;
+
+	for( i=0; i<cred->bv_len; i++) {
+		if(cred->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+
+	if( cred->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* cred must behave like a string */
+	}
+
+	for( i=0; i<passwd->bv_len; i++) {
+		if(passwd->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+
+	if( passwd->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
+	}
+
+	rtn = LUTIL_PASSWD_ERR;
+
+#ifdef HAVE_CYRUS_SASL
+	if( lutil_passwd_sasl_conn != NULL ) {
+		int sc;
+# if SASL_VERSION_MAJOR < 2
+		sc = sasl_checkpass( lutil_passwd_sasl_conn,
+			passwd->bv_val, passwd->bv_len,
+			cred->bv_val, cred->bv_len,
+			text );
+# else
+		sc = sasl_checkpass( lutil_passwd_sasl_conn,
+			passwd->bv_val, passwd->bv_len,
+			cred->bv_val, cred->bv_len );
+# endif
+		rtn = ( sc != SASL_OK ) ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
+	}
+#endif
+
+	return rtn;
+}
+#endif
+
+#ifdef SLAPD_CRYPT
+static int chk_crypt(
+	const struct berval *sc,
+	const struct berval * passwd,
+	const struct berval * cred,
+	const char **text )
+{
+	char *cr;
+	unsigned int i;
+
+	for( i=0; i<cred->bv_len; i++) {
+		if(cred->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+
+	if( cred->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* cred must behave like a string */
+	}
+
+	if( passwd->bv_len < 2 ) {
+		return LUTIL_PASSWD_ERR;	/* passwd must be at least two characters long */
+	}
+
+	for( i=0; i<passwd->bv_len; i++) {
+		if(passwd->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+
+	if( passwd->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
+	}
+
+	cr = crypt( cred->bv_val, passwd->bv_val );
+
+	if( cr == NULL || cr[0] == '\0' ) {
+		/* salt must have been invalid */
+		return LUTIL_PASSWD_ERR;
+	}
+
+	return strcmp( passwd->bv_val, cr ) ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
+}
+
+# if defined( HAVE_GETPWNAM ) && defined( HAVE_PW_PASSWD )
+static int chk_unix(
+	const struct berval *sc,
+	const struct berval * passwd,
+	const struct berval * cred,
+	const char **text )
+{
+	unsigned int i;
+	char *pw,*cr;
+
+	for( i=0; i<cred->bv_len; i++) {
+		if(cred->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+	if( cred->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* cred must behave like a string */
+	}
+
+	for( i=0; i<passwd->bv_len; i++) {
+		if(passwd->bv_val[i] == '\0') {
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
+		}
+	}
+
+	if( passwd->bv_val[i] != '\0' ) {
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
+	}
+
+	{
+		struct passwd *pwd = getpwnam(passwd->bv_val);
+
+		if(pwd == NULL) {
+			return LUTIL_PASSWD_ERR;	/* not found */
+		}
+
+		pw = pwd->pw_passwd;
+	}
+#  ifdef HAVE_GETSPNAM
+	{
+		struct spwd *spwd = getspnam(passwd->bv_val);
+
+		if(spwd != NULL) {
+			pw = spwd->sp_pwdp;
+		}
+	}
+#  endif
+#  ifdef HAVE_AIX_SECURITY
+	{
+		struct userpw *upw = getuserpw(passwd->bv_val);
+
+		if (upw != NULL) {
+			pw = upw->upw_passwd;
+		}
+	}
+#  endif
+
+	if( pw == NULL || pw[0] == '\0' || pw[1] == '\0' ) {
+		/* password must must be at least two characters long */
+		return LUTIL_PASSWD_ERR;
+	}
+
+	cr = crypt(cred->bv_val, pw);
+
+	if( cr == NULL || cr[0] == '\0' ) {
+		/* salt must have been invalid */
+		return LUTIL_PASSWD_ERR;
+	}
+
+	return strcmp(pw, cr) ? LUTIL_PASSWD_ERR : LUTIL_PASSWD_OK;
+
+}
+# endif
+#endif
+
+/* PASSWORD GENERATION ROUTINES */
+
+#ifdef LUTIL_SHA1_BYTES
+static int hash_ssha1(
+	const struct berval *scheme,
+	const struct berval  *passwd,
+	struct berval *hash,
+	const char **text )
+{
+	lutil_SHA1_CTX  SHA1context;
+	unsigned char   SHA1digest[LUTIL_SHA1_BYTES];
+	char            saltdata[SALT_SIZE];
+	struct berval digest;
+	struct berval salt;
+
+	digest.bv_val = (char *) SHA1digest;
+	digest.bv_len = sizeof(SHA1digest);
+	salt.bv_val = saltdata;
+	salt.bv_len = sizeof(saltdata);
+
+	if( lutil_entropy( (unsigned char *) salt.bv_val, salt.bv_len) < 0 ) {
+		return LUTIL_PASSWD_ERR; 
+	}
+
+	lutil_SHA1Init( &SHA1context );
+	lutil_SHA1Update( &SHA1context,
+		(const unsigned char *)passwd->bv_val, passwd->bv_len );
+	lutil_SHA1Update( &SHA1context,
+		(const unsigned char *)salt.bv_val, salt.bv_len );
+	lutil_SHA1Final( SHA1digest, &SHA1context );
+
+	return pw_string64( scheme, &digest, hash, &salt);
+}
+
+static int hash_sha1(
+	const struct berval *scheme,
+	const struct berval  *passwd,
+	struct berval *hash,
+	const char **text )
+{
+	lutil_SHA1_CTX  SHA1context;
+	unsigned char   SHA1digest[LUTIL_SHA1_BYTES];
+	struct berval digest;
+	digest.bv_val = (char *) SHA1digest;
+	digest.bv_len = sizeof(SHA1digest);
+     
+	lutil_SHA1Init( &SHA1context );
+	lutil_SHA1Update( &SHA1context,
+		(const unsigned char *)passwd->bv_val, passwd->bv_len );
+	lutil_SHA1Final( SHA1digest, &SHA1context );
+            
+	return pw_string64( scheme, &digest, hash, NULL);
+}
+#endif
+
+static int hash_smd5(
+	const struct berval *scheme,
+	const struct berval  *passwd,
+	struct berval *hash,
+	const char **text )
+{
+	lutil_MD5_CTX   MD5context;
+	unsigned char   MD5digest[LUTIL_MD5_BYTES];
+	char            saltdata[SALT_SIZE];
+	struct berval digest;
+	struct berval salt;
+
+	digest.bv_val = (char *) MD5digest;
+	digest.bv_len = sizeof(MD5digest);
+	salt.bv_val = saltdata;
+	salt.bv_len = sizeof(saltdata);
+
+	if( lutil_entropy( (unsigned char *) salt.bv_val, salt.bv_len) < 0 ) {
+		return LUTIL_PASSWD_ERR; 
+	}
+
+	lutil_MD5Init( &MD5context );
+	lutil_MD5Update( &MD5context,
+		(const unsigned char *) passwd->bv_val, passwd->bv_len );
+	lutil_MD5Update( &MD5context,
+		(const unsigned char *) salt.bv_val, salt.bv_len );
+	lutil_MD5Final( MD5digest, &MD5context );
+
+	return pw_string64( scheme, &digest, hash, &salt );
+}
+
+static int hash_md5(
+	const struct berval *scheme,
+	const struct berval  *passwd,
+	struct berval *hash,
+	const char **text )
+{
+	lutil_MD5_CTX   MD5context;
+	unsigned char   MD5digest[LUTIL_MD5_BYTES];
+
+	struct berval digest;
+
+	digest.bv_val = (char *) MD5digest;
+	digest.bv_len = sizeof(MD5digest);
+
+	lutil_MD5Init( &MD5context );
+	lutil_MD5Update( &MD5context,
+		(const unsigned char *) passwd->bv_val, passwd->bv_len );
+	lutil_MD5Final( MD5digest, &MD5context );
+
+	return pw_string64( scheme, &digest, hash, NULL );
+;
+}
+
+#ifdef SLAPD_LMHASH 
+static int hash_lanman(
+	const struct berval *scheme,
+	const struct berval *passwd,
+	struct berval *hash,
 	const char **text )
 {
 
@@ -1228,18 +1046,17 @@ static struct berval *hash_lanman(
 	des_cblock key;
 	des_key_schedule schedule;
 	des_cblock StdText = "KGS!@#$%";
-	des_cblock hash1, hash2;
-	char lmhash[33];
-	struct berval hash;
+	des_cblock PasswordHash1, PasswordHash2;
+	char PasswordHash[33];
 	
 	for( i=0; i<passwd->bv_len; i++) {
 		if(passwd->bv_val[i] == '\0') {
-			return NULL;	/* NUL character in password */
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
 		}
 	}
 	
 	if( passwd->bv_val[i] != '\0' ) {
-		return NULL;	/* passwd must behave like a string */
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
 	}
 	
 	strncpy( UcasePassword, passwd->bv_val, 14 );
@@ -1248,45 +1065,47 @@ static struct berval *hash_lanman(
 	
 	lmPasswd_to_key( UcasePassword, &key );
 	des_set_key_unchecked( &key, schedule );
-	des_ecb_encrypt( &StdText, &hash1, schedule , DES_ENCRYPT );
+	des_ecb_encrypt( &StdText, &PasswordHash1, schedule , DES_ENCRYPT );
 	
 	lmPasswd_to_key( &UcasePassword[7], &key );
 	des_set_key_unchecked( &key, schedule );
-	des_ecb_encrypt( &StdText, &hash2, schedule , DES_ENCRYPT );
+	des_ecb_encrypt( &StdText, &PasswordHash2, schedule , DES_ENCRYPT );
 	
-	sprintf( lmhash, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
-		hash1[0],hash1[1],hash1[2],hash1[3],hash1[4],hash1[5],hash1[6],hash1[7],
-		hash2[0],hash2[1],hash2[2],hash2[3],hash2[4],hash2[5],hash2[6],hash2[7] );
+	sprintf( PasswordHash, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
+		PasswordHash1[0],PasswordHash1[1],PasswordHash1[2],PasswordHash1[3],
+		PasswordHash1[4],PasswordHash1[5],PasswordHash1[6],PasswordHash1[7],
+		PasswordHash2[0],PasswordHash2[1],PasswordHash2[2],PasswordHash2[3],
+		PasswordHash2[4],PasswordHash2[5],PasswordHash2[6],PasswordHash2[7] );
 	
-	hash.bv_val = lmhash;
-	hash.bv_len = 32;
+	hash->bv_val = PasswordHash;
+	hash->bv_len = 32;
 	
-	return pw_string( scheme, &hash );
+	return pw_string( scheme, hash );
 }
 #endif /* SLAPD_LMHASH */
 
 #ifdef SLAPD_CRYPT
-static struct berval *hash_crypt(
+static int hash_crypt(
 	const struct berval *scheme,
 	const struct berval *passwd,
+	struct berval *hash,
 	const char **text )
 {
-	struct berval hash;
 	unsigned char salt[32];	/* salt suitable for most anything */
 	unsigned int i;
 
 	for( i=0; i<passwd->bv_len; i++) {
 		if(passwd->bv_val[i] == '\0') {
-			return NULL;	/* NUL character in password */
+			return LUTIL_PASSWD_ERR;	/* NUL character in password */
 		}
 	}
 
 	if( passwd->bv_val[i] != '\0' ) {
-		return NULL;	/* passwd must behave like a string */
+		return LUTIL_PASSWD_ERR;	/* passwd must behave like a string */
 	}
 
 	if( lutil_entropy( salt, sizeof( salt ) ) < 0 ) {
-		return NULL; 
+		return LUTIL_PASSWD_ERR; 
 	}
 
 	for( i=0; i< ( sizeof(salt) - 1 ); i++ ) {
@@ -1302,17 +1121,17 @@ static struct berval *hash_crypt(
 		snprintf( (char *) salt, sizeof(entropy), salt_format, entropy );
 	}
 
-	hash.bv_val = crypt( passwd->bv_val, (char *) salt );
+	hash->bv_val = crypt( passwd->bv_val, (char *) salt );
 
-	if( hash.bv_val == NULL ) return NULL;
+	if( hash->bv_val == NULL ) return -1;
 
-	hash.bv_len = strlen( hash.bv_val );
+	hash->bv_len = strlen( hash->bv_val );
 
-	if( hash.bv_len == 0 ) {
-		return NULL;
+	if( hash->bv_len == 0 ) {
+		return LUTIL_PASSWD_ERR;
 	}
 
-	return pw_string( scheme, &hash );
+	return pw_string( scheme, hash );
 }
 #endif
 
@@ -1328,12 +1147,14 @@ int lutil_salt_format(const char *format)
 }
 
 #ifdef SLAPD_CLEARTEXT
-static struct berval *hash_clear(
+static int hash_clear(
 	const struct berval *scheme,
 	const struct berval  *passwd,
+	struct berval *hash,
 	const char **text )
 {
-	return ber_bvdup( (struct berval *) passwd );
+	ber_dupbv( hash, (struct berval *)passwd );
+	return LUTIL_PASSWD_OK;
 }
 #endif
 
