@@ -1,26 +1,21 @@
-/******************************************************************************
+/* $OpenLDAP: pkg/ldap/libraries/librewrite/session.c,v 1.11 2004/01/01 18:15:29 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright (C) 2000 Pierangelo Masarati, <ando@sys-net.it>
+ * Copyright 2000-2004 The OpenLDAP Foundation.
  * All rights reserved.
  *
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
  *
- * 1. The author is not responsible for the consequences of use of this
- * software, no matter how awful, even if they arise from flaws in it.
- *
- * 2. The origin of this software must not be misrepresented, either by
- * explicit claim or by omission.  Since few users ever read sources,
- * credits should appear in the documentation.
- *
- * 3. Altered versions must be plainly marked as such, and must not be
- * misrepresented as being the original software.  Since few users
- * ever read sources, credits should appear in the documentation.
- * 
- * 4. This notice may not be removed or altered.
- *
- ******************************************************************************/
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENT:
+ * This work was initially developed by Pierangelo Masarati for
+ * inclusion in OpenLDAP Software.
+ */
 
 #include <portable.h>
 
@@ -35,16 +30,16 @@ rewrite_cookie_cmp(
                 const void *c2
 )
 {
-	struct rewrite_session *s1, *s2;
+	const struct rewrite_session *s1, *s2;
 
-	s1 = ( struct rewrite_session * )c1;
-	s2 = ( struct rewrite_session * )c2;
+	s1 = ( const struct rewrite_session * )c1;
+	s2 = ( const struct rewrite_session * )c2;
 
 	assert( s1 != NULL );
 	assert( s2 != NULL );
 	assert( s1->ls_cookie != NULL );
 	assert( s2->ls_cookie != NULL );
-	
+
         return ( ( s1->ls_cookie < s2->ls_cookie ) ? -1 :
 			( ( s1->ls_cookie > s2->ls_cookie ) ? 1 : 0 ) );
 }
@@ -68,6 +63,8 @@ rewrite_cookie_dup(
 	assert( s1->ls_cookie != NULL );
 	assert( s2->ls_cookie != NULL );
 	
+	assert( s1->ls_cookie != s2->ls_cookie );
+	
         return ( ( s1->ls_cookie == s2->ls_cookie ) ? -1 : 0 );
 }
 
@@ -80,24 +77,44 @@ rewrite_session_init(
 		const void *cookie
 )
 {
-	struct rewrite_session *session;
-	int rc;
+	struct rewrite_session 	*session, tmp;
+	int			rc;
 
 	assert( info != NULL );
 	assert( cookie != NULL );
-	
+
+#ifdef USE_REWRITE_LDAP_PVT_THREADS
+	ldap_pvt_thread_rdwr_wlock( &info->li_cookies_mutex );
+#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+
+	tmp.ls_cookie = ( void * )cookie;
+	session = ( struct rewrite_session * )avl_find( info->li_cookies, 
+			( caddr_t )&tmp, rewrite_cookie_cmp );
+	if ( session ) {
+		session->ls_count++;
+#ifdef USE_REWRITE_LDAP_PVT_THREADS
+		ldap_pvt_thread_rdwr_wunlock( &info->li_cookies_mutex );
+#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+		return session;
+	}
+		
 	session = calloc( sizeof( struct rewrite_session ), 1 );
 	if ( session == NULL ) {
 		return NULL;
 	}
 	session->ls_cookie = ( void * )cookie;
+	session->ls_count = 1;
 	
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
+	if ( ldap_pvt_thread_mutex_init( &session->ls_mutex ) ) {
+		free( session );
+		return NULL;
+	}
 	if ( ldap_pvt_thread_rdwr_init( &session->ls_vars_mutex ) ) {
+		ldap_pvt_thread_mutex_destroy( &session->ls_mutex );
 		free( session );
 		return NULL;
 	}
-	ldap_pvt_thread_rdwr_wlock( &info->li_cookies_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
 
 	rc = avl_insert( &info->li_cookies, ( caddr_t )session,
@@ -109,6 +126,11 @@ rewrite_session_init(
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
 	
 	if ( rc != 0 ) {
+#ifdef USE_REWRITE_LDAP_PVT_THREADS
+		ldap_pvt_thread_rdwr_destroy( &session->ls_vars_mutex );
+		ldap_pvt_thread_mutex_destroy( &session->ls_mutex );
+#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+
 		free( session );
 		return NULL;
 	}
@@ -137,22 +159,38 @@ rewrite_session_find(
 	session = ( struct rewrite_session * )avl_find( info->li_cookies,
 			( caddr_t )&tmp, rewrite_cookie_cmp );
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
+	if ( session ) {
+		ldap_pvt_thread_mutex_lock( &session->ls_mutex );
+	}
 	ldap_pvt_thread_rdwr_runlock( &info->li_cookies_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
 
 	return session;
-		
+}
+
+/*
+ * Returns a session
+ */
+void
+rewrite_session_return(
+		struct rewrite_info *info,
+		struct rewrite_session *session
+)
+{
+	assert( session );
+	ldap_pvt_thread_mutex_unlock( &session->ls_mutex );
 }
 
 /*
  * Defines and inits a var with session scope
  */
 int
-rewrite_session_var_set(
+rewrite_session_var_set_f(
 		struct rewrite_info *info,
 		const void *cookie,
 		const char *name,
-		const char *value
+		const char *value,
+		int flags
 )
 {
 	struct rewrite_session *session;
@@ -175,15 +213,16 @@ rewrite_session_var_set(
 	var = rewrite_var_find( session->ls_vars, name );
 	if ( var != NULL ) {
 		assert( var->lv_value.bv_val != NULL );
-		free( var->lv_value.bv_val );
-		var->lv_value.bv_val = strdup( value );
-		var->lv_value.bv_len = strlen( value );
+
+		(void)rewrite_var_replace( var, value, flags );
+
 	} else {
-		var = rewrite_var_insert( &session->ls_vars, name, value );
+		var = rewrite_var_insert_f( &session->ls_vars, name, value, flags );
 		if ( var == NULL ) {
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
 			ldap_pvt_thread_rdwr_wunlock( &session->ls_vars_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
+			rewrite_session_return( info, session );
 			return REWRITE_ERR;
 		}
 	}	
@@ -192,6 +231,8 @@ rewrite_session_var_set(
 	ldap_pvt_thread_rdwr_wunlock( &session->ls_vars_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
 
+	rewrite_session_return( info, session );
+
 	return REWRITE_SUCCESS;
 }
 
@@ -236,7 +277,9 @@ rewrite_session_var_get(
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
 	        ldap_pvt_thread_rdwr_runlock( &session->ls_vars_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
-		
+
+		rewrite_session_return( info, session );
+
 		return REWRITE_ERR;
 	} else {
 		value->bv_val = strdup( var->lv_value.bv_val );
@@ -246,6 +289,8 @@ rewrite_session_var_get(
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
         ldap_pvt_thread_rdwr_runlock( &session->ls_vars_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
+
+	rewrite_session_return( info, session );
 	
 	return REWRITE_SUCCESS;
 }
@@ -268,16 +313,28 @@ rewrite_session_delete(
 	
 	session = rewrite_session_find( info, cookie );
 
-	if ( session != NULL ) {
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-		ldap_pvt_thread_rdwr_wlock( &session->ls_vars_mutex );
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
-		rewrite_var_delete( session->ls_vars );
-#ifdef USE_REWRITE_LDAP_PVT_THREADS
-		ldap_pvt_thread_rdwr_wunlock( &session->ls_vars_mutex );
-#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+	if ( session == NULL ) {
+		return REWRITE_SUCCESS;
 	}
 
+	if ( --session->ls_count > 0 ) {
+		rewrite_session_return( info, session );
+		return REWRITE_SUCCESS;
+	}
+
+#ifdef USE_REWRITE_LDAP_PVT_THREADS
+	ldap_pvt_thread_rdwr_wlock( &session->ls_vars_mutex );
+#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+
+	rewrite_var_delete( session->ls_vars );
+
+#ifdef USE_REWRITE_LDAP_PVT_THREADS
+	ldap_pvt_thread_rdwr_wunlock( &session->ls_vars_mutex );
+	ldap_pvt_thread_rdwr_destroy( &session->ls_vars_mutex );
+	ldap_pvt_thread_mutex_unlock( &session->ls_mutex );
+	ldap_pvt_thread_mutex_destroy( &session->ls_mutex );
+#endif /* USE_REWRITE_LDAP_PVT_THREADS */
+
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
 	ldap_pvt_thread_rdwr_wlock( &info->li_cookies_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
@@ -289,6 +346,8 @@ rewrite_session_delete(
 	 * There is nothing to delete in the return value
 	 */
 	avl_delete( &info->li_cookies, ( caddr_t )&tmp, rewrite_cookie_cmp );
+	free( session );
+
 #ifdef USE_REWRITE_LDAP_PVT_THREADS
 	ldap_pvt_thread_rdwr_wunlock( &info->li_cookies_mutex );
 #endif /* USE_REWRITE_LDAP_PVT_THREADS */
@@ -318,6 +377,12 @@ rewrite_session_destroy(
 	
 	count = avl_free( info->li_cookies, NULL );
 	info->li_cookies = NULL;
+
+#if 0
+	fprintf( stderr, "count = %d; num_cookies = %d\n", 
+			count, info->li_num_cookies );
+#endif
+	
 	assert( count == info->li_num_cookies );
 	info->li_num_cookies = 0;
 
