@@ -1,9 +1,28 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/attr.c,v 1.88 2003/05/16 01:58:28 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
 /* attr.c - routines for dealing with attributes */
+/* $OpenLDAP: pkg/ldap/servers/slapd/attr.c,v 1.100.2.1 2004/12/30 21:29:56 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
+ */
 
 #include "portable.h"
 
@@ -19,17 +38,35 @@
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 
 void
 attr_free( Attribute *a )
 {
+	if ( a->a_nvals && a->a_nvals != a->a_vals ) {
+		ber_bvarray_free( a->a_nvals );
+	}
 	ber_bvarray_free( a->a_vals );
-	if (a->a_nvals != a->a_vals) ber_bvarray_free( a->a_nvals );
 	free( a );
 }
 
+#ifdef LDAP_COMP_MATCH
+void
+comp_tree_free( Attribute *a )
+{
+	Attribute *next;
+
+	for( ; a != NULL ; a = next ) {
+		next = a->a_next;
+		if ( component_destructor && a->a_comp_data ) {
+			if ( a->a_comp_data->cd_mem_op )
+				component_destructor( a->a_comp_data->cd_mem_op );
+			free ( a->a_comp_data );
+		}
+	}
+}
+#endif
+
 void
 attrs_free( Attribute *a )
 {
@@ -41,35 +78,41 @@ attrs_free( Attribute *a )
 	}
 }
 
-Attribute *attr_dup( Attribute *a )
+Attribute *
+attr_dup( Attribute *a )
 {
 	Attribute *tmp;
 
-	if( a == NULL) return NULL;
+	if ( a == NULL) return NULL;
 
 	tmp = ch_malloc( sizeof(Attribute) );
 
-	if( a->a_vals != NULL ) {
+	if ( a->a_vals != NULL ) {
 		int i;
 
-		for( i=0; a->a_vals[i].bv_val != NULL; i++ ) {
+		for ( i = 0; !BER_BVISNULL( &a->a_vals[i] ); i++ ) {
 			/* EMPTY */ ;
 		}
 
-		tmp->a_vals = ch_malloc((i+1) * sizeof(struct berval));
-		for( i=0; a->a_vals[i].bv_val != NULL; i++ ) {
+		tmp->a_vals = ch_malloc( (i + 1) * sizeof(struct berval) );
+		for ( i = 0; !BER_BVISNULL( &a->a_vals[i] ); i++ ) {
 			ber_dupbv( &tmp->a_vals[i], &a->a_vals[i] );
-			if( tmp->a_vals[i].bv_val == NULL ) break;
+			if ( BER_BVISNULL( &tmp->a_vals[i] ) ) break;
+			/* FIXME: error? */
 		}
-		tmp->a_vals[i].bv_val = NULL;
+		BER_BVZERO( &tmp->a_vals[i] );
 
-		if( a->a_nvals != a->a_vals ) {
-			tmp->a_nvals = ch_malloc((i+1) * sizeof(struct berval));
-			for( i=0; a->a_nvals[i].bv_val != NULL; i++ ) {
+		/* a_nvals must be non null; it may be equal to a_vals */
+		assert( a->a_nvals );
+
+		if ( a->a_nvals != a->a_vals ) {
+			tmp->a_nvals = ch_malloc( (i + 1) * sizeof(struct berval) );
+			for ( i = 0; !BER_BVISNULL( &a->a_nvals[i] ); i++ ) {
 				ber_dupbv( &tmp->a_nvals[i], &a->a_nvals[i] );
-				if( tmp->a_nvals[i].bv_val == NULL ) break;
+				if ( BER_BVISNULL( &tmp->a_nvals[i] ) ) break;
+				/* FIXME: error? */
 			}
-			tmp->a_nvals[i].bv_val = NULL;
+			BER_BVZERO( &tmp->a_nvals[i] );
 
 		} else {
 			tmp->a_nvals = tmp->a_vals;
@@ -83,11 +126,15 @@ Attribute *attr_dup( Attribute *a )
 	tmp->a_desc = a->a_desc;
 	tmp->a_next = NULL;
 	tmp->a_flags = 0;
+#ifdef LDAP_COMP_MATCH
+	tmp->a_comp_data = NULL;
+#endif
 
 	return tmp;
 }
 
-Attribute *attrs_dup( Attribute *a )
+Attribute *
+attrs_dup( Attribute *a )
 {
 	Attribute *tmp, **next;
 
@@ -123,8 +170,8 @@ attr_merge(
 	Entry		*e,
 	AttributeDescription *desc,
 	BerVarray	vals,
-	BerVarray	nvals
-) {
+	BerVarray	nvals )
+{
 	int rc;
 
 	Attribute	**a;
@@ -142,12 +189,21 @@ attr_merge(
 		(*a)->a_nvals = NULL;
 		(*a)->a_next = NULL;
 		(*a)->a_flags = 0;
+#ifdef LDAP_COMP_MATCH
+		(*a)->a_comp_data = NULL;
+#endif
 	}
 
 	rc = value_add( &(*a)->a_vals, vals );
 
-	if( !rc && nvals ) rc = value_add( &(*a)->a_nvals, nvals );
-	else (*a)->a_nvals = (*a)->a_vals;
+	if ( rc == LDAP_SUCCESS ) {
+		if ( nvals ) {
+			rc = value_add( &(*a)->a_nvals, nvals );
+			/* FIXME: what if rc != LDAP_SUCCESS ? */
+		} else {
+			(*a)->a_nvals = (*a)->a_vals;
+		}
+	}
 
 	return rc;
 }
@@ -157,37 +213,39 @@ attr_merge_normalize(
 	Entry		*e,
 	AttributeDescription *desc,
 	BerVarray	vals,
-	void	 *memctx
-) {
+	void	 *memctx )
+{
 	BerVarray	nvals = NULL;
 	int		rc;
 
-	if ( desc->ad_type->sat_equality && desc->ad_type->sat_equality->smr_normalize ) {
+	if ( desc->ad_type->sat_equality &&
+		desc->ad_type->sat_equality->smr_normalize )
+	{
 		int	i;
 		
-		for ( i = 0; vals[i].bv_val; i++ );
+		for ( i = 0; !BER_BVISNULL( &vals[i] ); i++ );
 
-		nvals = ch_calloc( sizeof(struct berval), i + 1 );
-		for ( i = 0; vals[i].bv_val; i++ ) {
+		nvals = slap_sl_calloc( sizeof(struct berval), i + 1, memctx );
+		for ( i = 0; !BER_BVISNULL( &vals[i] ); i++ ) {
 			rc = (*desc->ad_type->sat_equality->smr_normalize)(
-					0,
+					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
 					desc->ad_type->sat_syntax,
 					desc->ad_type->sat_equality,
 					&vals[i], &nvals[i], memctx );
 
 			if ( rc != LDAP_SUCCESS ) {
-				nvals[i+1].bv_val = NULL;
+				BER_BVZERO( &nvals[i + 1] );
 				goto error_return;
 			}
 		}
-		nvals[i].bv_val = NULL;
+		BER_BVZERO( &nvals[i] );
 	}
 
 	rc = attr_merge( e, desc, vals, nvals );
 
 error_return:;
 	if ( nvals != NULL ) {
-		ber_bvarray_free( nvals );
+		ber_bvarray_free_x( nvals, memctx );
 	}
 	return rc;
 }
@@ -197,8 +255,8 @@ attr_merge_one(
 	Entry		*e,
 	AttributeDescription *desc,
 	struct berval	*val,
-	struct berval	*nval
-) {
+	struct berval	*nval )
+{
 	int rc;
 	Attribute	**a;
 
@@ -215,12 +273,21 @@ attr_merge_one(
 		(*a)->a_nvals = NULL;
 		(*a)->a_next = NULL;
 		(*a)->a_flags = 0;
+#ifdef LDAP_COMP_MATCH
+		(*a)->a_comp_data = NULL;
+#endif
 	}
 
 	rc = value_add_one( &(*a)->a_vals, val );
 
-	if( !rc && nval ) rc = value_add_one( &(*a)->a_nvals, nval );
-	else (*a)->a_nvals = (*a)->a_vals;
+	if ( rc == LDAP_SUCCESS ) {
+		if ( nval ) {
+			rc = value_add_one( &(*a)->a_nvals, nval );
+			/* FIXME: what if rc != LDAP_SUCCESS ? */
+		} else {
+			(*a)->a_nvals = (*a)->a_vals;
+		}
+	}
 	return rc;
 }
 
@@ -229,15 +296,17 @@ attr_merge_normalize_one(
 	Entry		*e,
 	AttributeDescription *desc,
 	struct berval	*val,
-	void		*memctx
-) {
+	void		*memctx )
+{
 	struct berval	nval;
-	struct berval	*nvalp;
+	struct berval	*nvalp = NULL;
 	int		rc;
 
-	if ( desc->ad_type->sat_equality && desc->ad_type->sat_equality->smr_normalize ) {
+	if ( desc->ad_type->sat_equality &&
+		desc->ad_type->sat_equality->smr_normalize )
+	{
 		rc = (*desc->ad_type->sat_equality->smr_normalize)(
-				0,
+				SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
 				desc->ad_type->sat_syntax,
 				desc->ad_type->sat_equality,
 				val, &nval, memctx );
@@ -246,13 +315,11 @@ attr_merge_normalize_one(
 			return rc;
 		}
 		nvalp = &nval;
-	} else {
-		nvalp = NULL;
 	}
 
 	rc = attr_merge_one( e, desc, val, nvalp );
 	if ( nvalp != NULL ) {
-		ch_free( nval.bv_val );
+		slap_sl_free( nval.bv_val, memctx );
 	}
 	return rc;
 }
@@ -265,8 +332,7 @@ attr_merge_normalize_one(
 Attribute *
 attrs_find(
     Attribute	*a,
-	AttributeDescription *desc
-)
+	AttributeDescription *desc )
 {
 	for ( ; a != NULL; a = a->a_next ) {
 		if ( is_ad_subtype( a->a_desc, desc ) ) {
@@ -284,8 +350,7 @@ attrs_find(
 Attribute *
 attr_find(
     Attribute	*a,
-	AttributeDescription *desc
-)
+	AttributeDescription *desc )
 {
 	for ( ; a != NULL; a = a->a_next ) {
 		if ( ad_cmp( a->a_desc, desc ) == 0 ) {
@@ -306,8 +371,7 @@ attr_find(
 int
 attr_delete(
     Attribute	**attrs,
-	AttributeDescription *desc
-)
+	AttributeDescription *desc )
 {
 	Attribute	**a;
 
