@@ -1,8 +1,17 @@
 /* back-bdb.h - bdb back-end header file */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/back-bdb.h,v 1.92 2003/05/01 21:39:29 jongchoi Exp $ */
-/*
- * Copyright 2000-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/back-bdb.h,v 1.117.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #ifndef _BACK_BDB_H_
@@ -14,9 +23,11 @@
 
 LDAP_BEGIN_DECL
 
-#ifdef LDAP_SYNCREPL
+#undef BDB_PSEARCH
+
+#define DB_VERSION_FULL ((DB_VERSION_MAJOR << 24) | (DB_VERSION_MINOR << 16) | DB_VERSION_PATCH)
+
 #define BDB_SUBENTRIES 1
-#endif
 
 #define DN_BASE_PREFIX		SLAP_INDEX_EQUALITY_PREFIX
 #define DN_ONE_PREFIX	 	'%'
@@ -32,8 +43,6 @@ LDAP_BEGIN_DECL
 
 #define BDB_MAX_ADD_LOOP	30
 
-#define	BDB_ALIASES	1
-
 #ifdef BDB_SUBDIRS
 #define BDB_TMP_SUBDIR	"tmp"
 #define BDB_LG_SUBDIR	"log"
@@ -66,10 +75,6 @@ LDAP_BEGIN_DECL
 /* The minimum we can function with */
 #define MINIMUM_SEARCH_STACK_DEPTH	8
 
-/* for the IDL cache */
-#define SLAP_IDL_CACHE	1
-
-#ifdef SLAP_IDL_CACHE
 typedef struct bdb_idl_cache_entry_s {
 	struct berval kstr;
 	ldap_pvt_thread_rdwr_t idl_entry_rwlock;
@@ -78,17 +83,24 @@ typedef struct bdb_idl_cache_entry_s {
 	struct bdb_idl_cache_entry_s* idl_lru_prev;
 	struct bdb_idl_cache_entry_s* idl_lru_next;
 } bdb_idl_cache_entry_t;
-#endif
 
 /* BDB backend specific entry info */
 typedef struct bdb_entry_info {
 	struct bdb_entry_info *bei_parent;
 	ID bei_id;
 
-	int bei_state;
+	/* we use the bei_id as a lockobj, but we need to make the size != 4
+	 * to avoid conflicting with BDB's internal locks. So add a byte here
+	 * that is always zero.
+	 */
+	char bei_lockpad;
+						
+	short bei_state;
 #define	CACHE_ENTRY_DELETED	1
 #define	CACHE_ENTRY_NO_KIDS	2
 #define	CACHE_ENTRY_NOT_LINKED	4
+#define CACHE_ENTRY_NO_GRANDKIDS	8
+#define	CACHE_ENTRY_LOADING	0x10
 
 	/*
 	 * remaining fields require backend cache lock to access
@@ -96,10 +108,16 @@ typedef struct bdb_entry_info {
 	struct berval bei_nrdn;
 #ifdef BDB_HIER
 	struct berval bei_rdn;
-	int	bei_modrdns;
+	int	bei_modrdns;	/* track renames */
+	int	bei_ckids;	/* number of kids cached */
+	int	bei_dkids;	/* number of kids on-disk, plus 1 */
 #endif
 	Entry	*bei_e;
 	Avlnode	*bei_kids;
+#ifdef SLAP_ZONE_ALLOC
+	struct bdb_info *bei_bdb;
+	int bei_zseq;	
+#endif
 	ldap_pvt_thread_mutex_t	bei_kids_mutex;
 	
 	struct bdb_entry_info	*bei_lrunext;	/* for cache lru list */
@@ -112,12 +130,18 @@ typedef struct bdb_entry_info {
 typedef struct bdb_cache {
 	int             c_maxsize;
 	int             c_cursize;
+	int		c_eiused;	/* EntryInfo's in use */
 	EntryInfo	c_dntree;
+	EntryInfo	*c_eifree;	/* free list */
 	Avlnode         *c_idtree;
 	EntryInfo	*c_lruhead;	/* lru - add accessed entries here */
 	EntryInfo	*c_lrutail;	/* lru - rem lru entries from here */
 	ldap_pvt_thread_rdwr_t c_rwlock;
 	ldap_pvt_thread_mutex_t lru_mutex;
+	u_int32_t	c_locker;	/* used by lru cleaner */
+#ifdef SLAP_ZONE_ALLOC
+	void *c_zctx;
+#endif
 } Cache;
  
 #define CACHE_READ_LOCK                0
@@ -141,6 +165,7 @@ struct bdb_info {
 
 	int			bi_ndatabases;
 	struct bdb_db_info **bi_databases;
+	ldap_pvt_thread_mutex_t	bi_database_mutex;
 	int		bi_db_opflags;	/* db-specific flags */
 
 	slap_mask_t	bi_defaultmask;
@@ -148,6 +173,7 @@ struct bdb_info {
 	Avlnode		*bi_attrs;
 	void		*bi_search_stack;
 	int		bi_search_stack_depth;
+	int		bi_linear_index;
 
 	int			bi_txn_cp;
 	u_int32_t	bi_txn_cp_min;
@@ -158,10 +184,11 @@ struct bdb_info {
 
 	ID			bi_lastid;
 	ldap_pvt_thread_mutex_t	bi_lastid_mutex;
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+#ifdef BDB_PSEARCH
 	LDAP_LIST_HEAD(pl, slap_op) bi_psearch_list;
+	ldap_pvt_thread_rdwr_t bi_pslist_rwlock;
+	LDAP_LIST_HEAD(se, slap_session_entry) bi_session_list;
 #endif
-#ifdef SLAP_IDL_CACHE
 	int		bi_idl_cache_max_size;
 	int		bi_idl_cache_size;
 	Avlnode		*bi_idl_tree;
@@ -169,22 +196,27 @@ struct bdb_info {
 	bdb_idl_cache_entry_t	*bi_idl_lru_tail;
 	ldap_pvt_thread_rdwr_t bi_idl_tree_rwlock;
 	ldap_pvt_thread_mutex_t bi_idl_tree_lrulock;
-#endif
 };
 
 #define bi_id2entry	bi_databases[BDB_ID2ENTRY]
 #define bi_dn2id	bi_databases[BDB_DN2ID]
 
+struct bdb_lock_info {
+	struct bdb_lock_info *bli_next;
+	ID		bli_id;
+	DB_LOCK	bli_lock;
+};
+
 struct bdb_op_info {
 	BackendDB*	boi_bdb;
 	DB_TXN*		boi_txn;
-	DB_LOCK		boi_lock;	/* used when no txn */
 	u_int32_t	boi_err;
 	u_int32_t	boi_locker;
 	int		boi_acl_cache;
+	struct bdb_lock_info *boi_locks;	/* used when no txn */
 };
 
-#define	DB_OPEN(db, txn, file, name, type, flags, mode) \
+#define	DB_OPEN(db, file, name, type, flags, mode) \
 	(db)->open(db, file, name, type, flags, mode)
 
 #if DB_VERSION_MAJOR < 4
@@ -213,16 +245,38 @@ struct bdb_op_info {
 #define XLOCK_ID_FREE(env, locker)	(env)->lock_id_free(env, locker)
 
 /* BDB 4.1.17 adds txn arg to db->open */
-#if DB_VERSION_MINOR > 1 || DB_VERSION_PATCH >= 17
+#if DB_VERSION_FULL >= 0x04010011
 #undef DB_OPEN
-#define	DB_OPEN(db, txn, file, name, type, flags, mode) \
-	(db)->open(db, txn, file, name, type, flags, mode)
+#define	DB_OPEN(db, file, name, type, flags, mode) \
+	(db)->open(db, NULL, file, name, type, (flags)|DB_AUTO_COMMIT, mode)
 #endif
 
 #endif
 
 #define BDB_REUSE_LOCKERS
 
+#define BDB_CSN_COMMIT	0
+#define BDB_CSN_ABORT	1
+#define BDB_CSN_RETRY	2
+
+/* Copy an ID "src" to pointer "dst" in big-endian byte order */
+#define BDB_ID2DISK( src, dst )	\
+	do { int i0; ID tmp; char *ptr;	\
+		tmp = (src); ptr = (char *)(dst);	\
+		for ( i0=sizeof(ID)-1; i0>=0; i0-- ) {	\
+			ptr[i0] = tmp & 0xff; tmp >>= 8;	\
+		} \
+	} while(0);
+
+/* Copy a pointer "src" to a pointer "dst" from big-endian to native order */
+#define BDB_DISK2ID( src, dst ) \
+	do { int i0; ID tmp = 0; unsigned char *ptr;	\
+		ptr = (unsigned char *)(src);	\
+		for ( i0=0; i0<sizeof(ID); i0++ ) {	\
+			tmp <<= 8; tmp |= *ptr++;	\
+		} *(dst) = tmp; \
+	} while (0);
+
 LDAP_END_DECL
 
 #include "proto-bdb.h"
