@@ -1,8 +1,17 @@
 /* cache.c - routines to maintain an in-core cache of entries */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/cache.c,v 1.38 2003/05/24 22:53:08 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/cache.c,v 1.88.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -17,24 +26,43 @@
 
 #include "back-bdb.h"
 
-static int	bdb_cache_delete_internal(Cache *cache, EntryInfo *e);
+#ifdef BDB_HIER
+#define bdb_cache_lru_add	hdb_cache_lru_add
+#endif
+
+static int	bdb_cache_delete_internal(Cache *cache, EntryInfo *e, int decr);
 #ifdef LDAP_DEBUG
 static void	bdb_lru_print(Cache *cache);
 #endif
 
-static EntryInfo *
-bdb_cache_entryinfo_new( )
-{
-	EntryInfo *ei;
+static int bdb_txn_get( Operation *op, DB_ENV *env, DB_TXN **txn, int reset );
 
-	ei = ch_calloc(1, sizeof(struct bdb_entry_info));
-	ldap_pvt_thread_mutex_init( &ei->bei_kids_mutex );
+static EntryInfo *
+bdb_cache_entryinfo_new( Cache *cache )
+{
+	EntryInfo *ei = NULL;
+
+	if ( cache->c_eifree ) {
+		ldap_pvt_thread_rdwr_wlock( &cache->c_rwlock );
+		if ( cache->c_eifree ) {
+			ei = cache->c_eifree;
+			cache->c_eifree = ei->bei_lrunext;
+		}
+		ldap_pvt_thread_rdwr_wunlock( &cache->c_rwlock );
+	}
+	if ( ei ) {
+		ei->bei_lrunext = NULL;
+		ei->bei_state = 0;
+	} else {
+		ei = ch_calloc(1, sizeof(struct bdb_entry_info));
+		ldap_pvt_thread_mutex_init( &ei->bei_kids_mutex );
+	}
 
 	return ei;
 }
 
 /* Atomically release and reacquire a lock */
-static int
+int
 bdb_cache_entry_db_relock(
 	DB_ENV *env,
 	u_int32_t locker,
@@ -52,8 +80,8 @@ bdb_cache_entry_db_relock(
 
 	if ( !lock ) return 0;
 
-	lockobj.data = ei;
-	lockobj.size = sizeof(ei->bei_parent) + sizeof(ei->bei_id);
+	lockobj.data = &ei->bei_id;
+	lockobj.size = sizeof(ei->bei_id) + 1;
 
 	list[0].op = DB_LOCK_PUT;
 	list[0].lock = *lock;
@@ -64,25 +92,20 @@ bdb_cache_entry_db_relock(
 	rc = env->lock_vec(env, locker, tryOnly ? DB_LOCK_NOWAIT : 0,
 		list, 2, NULL );
 
-	if (rc) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( CACHE, DETAIL1, 
-			"bdb_cache_entry_db_relock: entry %ld, rw %d, rc %d\n",
-			ei->bei_id, rw, rc );
-#else
+	if (rc && !tryOnly) {
 		Debug( LDAP_DEBUG_TRACE,
 			"bdb_cache_entry_db_relock: entry %ld, rw %d, rc %d\n",
 			ei->bei_id, rw, rc );
-#endif
 	} else {
 		*lock = list[1].lock;
 	}
 	return rc;
 #endif
 }
+
 static int
-bdb_cache_entry_db_lock
-( DB_ENV *env, u_int32_t locker, EntryInfo *ei, int rw, int tryOnly, DB_LOCK *lock )
+bdb_cache_entry_db_lock( DB_ENV *env, u_int32_t locker, EntryInfo *ei,
+	int rw, int tryOnly, DB_LOCK *lock )
 {
 #ifdef NO_THREADS
 	return 0;
@@ -98,35 +121,30 @@ bdb_cache_entry_db_lock
 	else
 		db_rw = DB_LOCK_READ;
 
-	lockobj.data = ei;
-	lockobj.size = sizeof(ei->bei_parent) + sizeof(ei->bei_id);
+	lockobj.data = &ei->bei_id;
+	lockobj.size = sizeof(ei->bei_id) + 1;
 
 	rc = LOCK_GET(env, locker, tryOnly ? DB_LOCK_NOWAIT : 0,
 					&lockobj, db_rw, lock);
-	if (rc) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( CACHE, DETAIL1, 
-			"bdb_cache_entry_db_lock: entry %ld, rw %d, rc %d\n",
-			ei->bei_id, rw, rc );
-#else
+	if (rc && !tryOnly) {
 		Debug( LDAP_DEBUG_TRACE,
 			"bdb_cache_entry_db_lock: entry %ld, rw %d, rc %d\n",
 			ei->bei_id, rw, rc );
-#endif
 	}
 	return rc;
 #endif /* NO_THREADS */
 }
 
 int
-bdb_cache_entry_db_unlock
-( DB_ENV *env, DB_LOCK *lock )
+bdb_cache_entry_db_unlock ( DB_ENV *env, DB_LOCK *lock )
 {
 #ifdef NO_THREADS
 	return 0;
 #else
 	int rc;
 
+	if ( !lock ) return 0;
+
 	rc = LOCK_PUT ( env, lock );
 	return rc;
 #endif
@@ -155,6 +173,7 @@ bdb_cache_entryinfo_destroy( EntryInfo *e )
 	} else { \
 		(cache)->c_lrutail = (ei)->bei_lruprev; \
 	} \
+	(ei)->bei_lrunext = (ei)->bei_lruprev = NULL; \
 } while(0)
 
 #define LRU_ADD( cache, ei ) do { \
@@ -175,8 +194,10 @@ bdb_rdn_cmp( const void *v_e1, const void *v_e2 )
 {
 	const EntryInfo *e1 = v_e1, *e2 = v_e2;
 	int rc = e1->bei_nrdn.bv_len - e2->bei_nrdn.bv_len;
-	if (rc == 0) rc = strncmp( e1->bei_nrdn.bv_val, e2->bei_nrdn.bv_val,
-		e1->bei_nrdn.bv_len );
+	if (rc == 0) {
+		rc = strncmp( e1->bei_nrdn.bv_val, e2->bei_nrdn.bv_val,
+			e1->bei_nrdn.bv_len );
+	}
 	return rc;
 }
 
@@ -193,103 +214,32 @@ static int
 bdb_entryinfo_add_internal(
 	struct bdb_info *bdb,
 	EntryInfo *ei,
-	EntryInfo **res,
-	u_int32_t locker
-)
+	EntryInfo **res )
 {
-	Cache *cache = &bdb->bi_cache;
-	DB_ENV *env = bdb->bi_dbenv;
 	EntryInfo *ei2 = NULL;
-	int incr = 1;
-	int addkid = 1;
-	int rc;
-	DB_LOCK lock;
 
 	*res = NULL;
 
+	ei2 = bdb_cache_entryinfo_new( &bdb->bi_cache );
+
 	ldap_pvt_thread_rdwr_wlock( &bdb->bi_cache.c_rwlock );
 	bdb_cache_entryinfo_lock( ei->bei_parent );
 
-	/* if parent was previously considered a leaf node,
-	 * it was on the LRU list. Now it's going to have
-	 * kids, take it off the LRU list.
-	 */
-	ldap_pvt_thread_mutex_lock( &cache->lru_mutex );
-	if ( ei->bei_parent->bei_id && !ei->bei_parent->bei_kids ) {
-		LRU_DELETE( cache, ei->bei_parent );
-		incr = 0;
-	}
-
-	cache->c_cursize += incr;
-
-	/* See if we're above the cache size limit */
-	if ( cache->c_cursize > cache->c_maxsize ) {
-		EntryInfo *elru, *elprev;
-		int i = 0;
-
-		/* Look for an unused entry to remove */
-		for (elru = cache->c_lrutail; elru; elru = elprev, i++ ) {
-			elprev = elru->bei_lruprev;
-
-			/* Too many probes, not enough idle, give up */
-			if (i > 10) break;
-
-			/* If we can successfully writelock it, then
-			 * the object is idle.
-			 */
-			if ( bdb_cache_entry_db_lock( env, locker, elru, 1, 1,
-				&lock ) == 0 ) {
-				/* Need to lock parent to delete child */
-				if ( ldap_pvt_thread_mutex_trylock(
-					&elru->bei_parent->bei_kids_mutex )) {
-					bdb_cache_entry_db_unlock( env, &lock );
-					continue;
-				}
-				bdb_cache_delete_internal( cache, elru );
-				bdb_cache_entryinfo_unlock( elru->bei_parent );
-				elru->bei_e->e_private = NULL;
-				bdb_entry_return( elru->bei_e );
-				bdb_cache_entry_db_unlock( env, &lock );
-				if (ei2) {
-					bdb_cache_entryinfo_destroy( elru );
-				} else {
-					/* re-use this one */
-					ch_free(elru->bei_nrdn.bv_val);
-					elru->bei_nrdn.bv_val = NULL;
-					elru->bei_e = NULL;
-					elru->bei_kids = NULL;
-					elru->bei_lrunext = NULL;
-					elru->bei_lruprev = NULL;
-					elru->bei_state = 0;
-#ifdef BDB_HIER
-					ch_free(elru->bei_rdn.bv_val);
-					elru->bei_rdn.bv_val = NULL;
-					elru->bei_modrdns = 0;
-#endif
-					ei2 = elru;
-				}
-				if (cache->c_cursize < cache->c_maxsize)
-					break;
-			}
-		}
-	}
-	if (!ei2) {
-		ei2 = bdb_cache_entryinfo_new();
-	}
 	ei2->bei_id = ei->bei_id;
 	ei2->bei_parent = ei->bei_parent;
 #ifdef BDB_HIER
 	ei2->bei_rdn = ei->bei_rdn;
 #endif
+#ifdef SLAP_ZONE_ALLOC
+	ei2->bei_bdb = bdb;
+#endif
 
 	/* Add to cache ID tree */
-	if (avl_insert( &cache->c_idtree, ei2, bdb_id_cmp, avl_dup_error )) {
+	if (avl_insert( &bdb->bi_cache.c_idtree, ei2, bdb_id_cmp, avl_dup_error )) {
 		EntryInfo *eix;
-		eix = avl_find( cache->c_idtree, ei2, bdb_id_cmp );
+		eix = avl_find( bdb->bi_cache.c_idtree, ei2, bdb_id_cmp );
 		bdb_cache_entryinfo_destroy( ei2 );
 		ei2 = eix;
-		addkid = 0;
-		cache->c_cursize -= incr;
 #ifdef BDB_HIER
 		/* It got freed above because its value was
 		 * assigned to ei2.
@@ -297,17 +247,15 @@ bdb_entryinfo_add_internal(
 		ei->bei_rdn.bv_val = NULL;
 #endif
 	} else {
-		LRU_ADD( cache, ei2 );
+		bdb->bi_cache.c_eiused++;
 		ber_dupbv( &ei2->bei_nrdn, &ei->bei_nrdn );
-	}
-
-	if ( addkid ) {
 		avl_insert( &ei->bei_parent->bei_kids, ei2, bdb_rdn_cmp,
 			avl_dup_error );
+#ifdef BDB_HIER
+		ei->bei_parent->bei_ckids++;
+#endif
 	}
 
-	ldap_pvt_thread_mutex_unlock( &cache->lru_mutex );
-
 	*res = ei2;
 	return 0;
 }
@@ -323,9 +271,7 @@ bdb_cache_find_ndn(
 	Operation	*op,
 	DB_TXN		*txn,
 	struct berval	*ndn,
-	EntryInfo	**res,
-	u_int32_t	locker
-)
+	EntryInfo	**res )
 {
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	EntryInfo	ei, *eip, *ei2;
@@ -343,6 +289,16 @@ bdb_cache_find_ndn(
 		ptr = ndn->bv_val + ndn->bv_len - op->o_bd->be_nsuffix[0].bv_len;
 		ei.bei_nrdn.bv_val = ptr;
 		ei.bei_nrdn.bv_len = op->o_bd->be_nsuffix[0].bv_len;
+		/* Skip to next rdn if suffix is empty */
+		if ( ei.bei_nrdn.bv_len == 0 ) {
+			for (ptr = ei.bei_nrdn.bv_val - 2; ptr > ndn->bv_val
+				&& !DN_SEPARATOR(*ptr); ptr--) /* empty */;
+			if ( ptr >= ndn->bv_val ) {
+				if (DN_SEPARATOR(*ptr)) ptr++;
+				ei.bei_nrdn.bv_len = ei.bei_nrdn.bv_val - ptr;
+				ei.bei_nrdn.bv_val = ptr;
+			}
+		}
 		eip = &bdb->bi_cache.c_dntree;
 	}
 	
@@ -352,7 +308,8 @@ bdb_cache_find_ndn(
 		if ( !ei2 ) {
 			int len = ei.bei_nrdn.bv_len;
 				
-			ei.bei_nrdn.bv_len = ndn->bv_len - (ei.bei_nrdn.bv_val - ndn->bv_val);
+			ei.bei_nrdn.bv_len = ndn->bv_len -
+				(ei.bei_nrdn.bv_val - ndn->bv_val);
 			bdb_cache_entryinfo_unlock( eip );
 
 			rc = bdb_dn2id( op, txn, &ei.bei_nrdn, &ei );
@@ -364,8 +321,7 @@ bdb_cache_find_ndn(
 
 			/* DN exists but needs to be added to cache */
 			ei.bei_nrdn.bv_len = len;
-			rc = bdb_entryinfo_add_internal( bdb, &ei, &ei2,
-				locker );
+			rc = bdb_entryinfo_add_internal( bdb, &ei, &ei2 );
 			/* add_internal left eip and c_rwlock locked */
 			ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
 			if ( rc ) {
@@ -389,7 +345,7 @@ bdb_cache_find_ndn(
 
 		/* Advance to next lower RDN */
 		for (ptr = ei.bei_nrdn.bv_val - 2; ptr > ndn->bv_val
-			&& !DN_SEPARATOR(*ptr); ptr--);
+			&& !DN_SEPARATOR(*ptr); ptr--) /* empty */;
 		if ( ptr >= ndn->bv_val ) {
 			if (DN_SEPARATOR(*ptr)) ptr++;
 			ei.bei_nrdn.bv_len = ei.bei_nrdn.bv_val - ptr - 1;
@@ -413,16 +369,18 @@ hdb_cache_find_parent(
 	Operation *op,
 	DB_TXN *txn,
 	ID id,
-	EntryInfo **res
-)
+	EntryInfo **res )
 {
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	EntryInfo ei, eip, *ei2 = NULL, *ein = NULL, *eir = NULL;
+	char ndn[SLAP_LDAPDN_MAXLEN];
 	ID parent;
 	int rc;
+	int addlru = 1;
 
 	ei.bei_id = id;
 	ei.bei_kids = NULL;
+	ei.bei_ckids = 0;
 
 	for (;;) {
 		rc = hdb_dn2id_parent( op, txn, &ei, &eip.bei_id );
@@ -432,30 +390,44 @@ hdb_cache_find_parent(
 		ei2 = ein;
 
 		/* Create a new node for the current ID */
-		ein = bdb_cache_entryinfo_new();
+		ein = bdb_cache_entryinfo_new( &bdb->bi_cache );
 		ein->bei_id = ei.bei_id;
 		ein->bei_kids = ei.bei_kids;
 		ein->bei_nrdn = ei.bei_nrdn;
 		ein->bei_rdn = ei.bei_rdn;
+		ein->bei_ckids = ei.bei_ckids;
+#ifdef SLAP_ZONE_ALLOC
+		ein->bei_bdb = bdb;
+#endif
+		ei.bei_ckids = 0;
 		
 		/* This node is not fully connected yet */
 		ein->bei_state = CACHE_ENTRY_NOT_LINKED;
 
 		/* Insert this node into the ID tree */
-		ldap_pvt_thread_rdwr_rlock( &bdb->bi_cache.c_rwlock );
+		ldap_pvt_thread_rdwr_wlock( &bdb->bi_cache.c_rwlock );
 		if ( avl_insert( &bdb->bi_cache.c_idtree, (caddr_t)ein,
 			bdb_id_cmp, avl_dup_error ) ) {
 
-			/* Hm, can this really happen? */
+			/* Someone else created this node just before us.
+			 * Free our new copy and use the existing one.
+			 */
 			bdb_cache_entryinfo_destroy( ein );
 			ein = (EntryInfo *)avl_find( bdb->bi_cache.c_idtree,
 				(caddr_t) &ei, bdb_id_cmp );
+			
+			/* Link in any kids we've already processed */
 			if ( ei2 ) {
 				bdb_cache_entryinfo_lock( ein );
 				avl_insert( &ein->bei_kids, (caddr_t)ei2,
 					bdb_rdn_cmp, avl_dup_error );
+				ein->bei_ckids++;
 				bdb_cache_entryinfo_unlock( ein );
 			}
+
+			if ( !eir ) {
+				addlru = 0;
+			}
 		}
 
 		/* If this is the first time, save this node
@@ -466,38 +438,162 @@ hdb_cache_find_parent(
 		/* If there was a previous node, link it to this one */
 		if ( ei2 ) ei2->bei_parent = ein;
 
+		/* Look for this node's parent */
 		if ( eip.bei_id ) {
 			ei2 = (EntryInfo *) avl_find( bdb->bi_cache.c_idtree,
 					(caddr_t) &eip, bdb_id_cmp );
 		} else {
 			ei2 = &bdb->bi_cache.c_dntree;
 		}
+		bdb->bi_cache.c_eiused++;
+		ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
 
+		/* Got the parent, link in and we're done. */
 		if ( ei2 ) {
-			ein->bei_parent = ei2;
 			bdb_cache_entryinfo_lock( ei2 );
+			ein->bei_parent = ei2;
 			avl_insert( &ei2->bei_kids, (caddr_t)ein, bdb_rdn_cmp,
 				avl_dup_error);
+			ei2->bei_ckids++;
 			bdb_cache_entryinfo_unlock( ei2 );
-			*res = eir;
 			bdb_cache_entryinfo_lock( eir );
-		}
-		ldap_pvt_thread_rdwr_runlock( &bdb->bi_cache.c_rwlock );
-		if ( ei2 ) {
-			/* Found a link. Reset all the state info */
+
+			/* Reset all the state info */
 			for (ein = eir; ein != ei2; ein=ein->bei_parent)
 				ein->bei_state &= ~CACHE_ENTRY_NOT_LINKED;
+			*res = eir;
 			break;
 		}
 		ei.bei_kids = NULL;
 		ei.bei_id = eip.bei_id;
+		ei.bei_ckids = 1;
 		avl_insert( &ei.bei_kids, (caddr_t)ein, bdb_rdn_cmp,
 			avl_dup_error );
 	}
 	return rc;
 }
+
+/* Used by hdb_dn2idl when loading the EntryInfo for all the children
+ * of a given node
+ */
+int hdb_cache_load(
+	struct bdb_info *bdb,
+	EntryInfo *ei,
+	EntryInfo **res )
+{
+	EntryInfo *ei2;
+	int rc;
+
+	/* See if we already have this one */
+	bdb_cache_entryinfo_lock( ei->bei_parent );
+	ei2 = (EntryInfo *)avl_find( ei->bei_parent->bei_kids, ei, bdb_rdn_cmp );
+	bdb_cache_entryinfo_unlock( ei->bei_parent );
+
+	if ( !ei2 ) {
+		/* Not found, add it */
+		struct berval bv;
+
+		/* bei_rdn was not malloc'd before, do it now */
+		ber_dupbv( &bv, &ei->bei_rdn );
+		ei->bei_rdn = bv;
+
+		rc = bdb_entryinfo_add_internal( bdb, ei, res );
+		bdb_cache_entryinfo_unlock( ei->bei_parent );
+		ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
+	} else {
+		/* Found, return it */
+		*res = ei2;
+		return 0;
+	}
+	return rc;
+}
 #endif
 
+/* caller must have lru_mutex locked. mutex
+ * will be unlocked on return.
+ */
+static void
+bdb_cache_lru_add(
+	struct bdb_info *bdb,
+	u_int32_t	locker,
+	EntryInfo *ei )
+{
+	DB_LOCK		lock, *lockp;
+
+	if ( locker ) {
+		lockp = &lock;
+	} else {
+		lockp = NULL;
+	}
+
+	/* See if we're above the cache size limit */
+	if ( bdb->bi_cache.c_cursize > bdb->bi_cache.c_maxsize ) {
+		EntryInfo *elru, *elprev;
+		int i = 0;
+
+		/* Look for an unused entry to remove */
+		for (elru = bdb->bi_cache.c_lrutail; elru; elru = elprev, i++ ) {
+			elprev = elru->bei_lruprev;
+
+			/* Too many probes, not enough idle, give up */
+			if (i > 10)
+				break;
+
+			/* If we can successfully writelock it, then
+			 * the object is idle.
+			 */
+			if ( bdb_cache_entry_db_lock( bdb->bi_dbenv,
+					bdb->bi_cache.c_locker, elru, 1, 1, lockp ) == 0 ) {
+				int stop = 0;
+
+				/* If there's no entry, or this node is in
+				 * the process of linking into the cache,
+				 * or this node is being deleted, skip it.
+				 */
+				if ( !elru->bei_e || (elru->bei_state &
+					( CACHE_ENTRY_NOT_LINKED | CACHE_ENTRY_DELETED ))) {
+					bdb_cache_entry_db_unlock( bdb->bi_dbenv, lockp );
+					continue;
+				}
+				LRU_DELETE( &bdb->bi_cache, elru );
+				elru->bei_e->e_private = NULL;
+#ifdef SLAP_ZONE_ALLOC
+				bdb_entry_return( bdb, elru->bei_e, elru->bei_zseq );
+#else
+				bdb_entry_return( elru->bei_e );
+#endif
+				elru->bei_e = NULL;
+				ldap_pvt_thread_rdwr_wlock( &bdb->bi_cache.c_rwlock );
+				--bdb->bi_cache.c_cursize;
+				if (bdb->bi_cache.c_cursize <= bdb->bi_cache.c_maxsize)
+					stop = 1;
+				ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
+				bdb_cache_entry_db_unlock( bdb->bi_dbenv, lockp );
+				if (stop) break;
+			}
+		}
+	}
+	LRU_ADD( &bdb->bi_cache, ei );
+	ldap_pvt_thread_mutex_unlock( &bdb->bi_cache.lru_mutex );
+}
+
+EntryInfo *
+bdb_cache_find_info(
+	struct bdb_info *bdb,
+	ID id )
+{
+	EntryInfo	ei = { 0 },
+			*ei2;
+
+	ei.bei_id = id;
+
+	ldap_pvt_thread_rdwr_rlock( &bdb->bi_cache.c_rwlock );
+	ei2 = (EntryInfo *) avl_find( bdb->bi_cache.c_idtree,
+					(caddr_t) &ei, bdb_id_cmp );
+	ldap_pvt_thread_rdwr_runlock( &bdb->bi_cache.c_rwlock );
+	return ei2;
+}
+
 /*
  * cache_find_id - find an entry in the cache, given id.
  * The entry is locked for Read upon return. Call with islocked TRUE if
@@ -512,23 +608,47 @@ bdb_cache_find_id(
 	EntryInfo	**eip,
 	int		islocked,
 	u_int32_t	locker,
-	DB_LOCK		*lock
-)
+	DB_LOCK		*lock )
 {
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	Entry	*ep = NULL;
-	int	rc = 0;
-	EntryInfo ei;
+	int	rc = 0, load = 0;
+	EntryInfo ei = { 0 };
 
 	ei.bei_id = id;
 
+#ifdef SLAP_ZONE_ALLOC
+	slap_zh_rlock(bdb->bi_cache.c_zctx);
+#endif
 	/* If we weren't given any info, see if we have it already cached */
 	if ( !*eip ) {
-		ldap_pvt_thread_rdwr_rlock( &bdb->bi_cache.c_rwlock );
+again:	ldap_pvt_thread_rdwr_rlock( &bdb->bi_cache.c_rwlock );
 		*eip = (EntryInfo *) avl_find( bdb->bi_cache.c_idtree,
-					(caddr_t) &ei, bdb_id_cmp );
+			(caddr_t) &ei, bdb_id_cmp );
 		if ( *eip ) {
-			bdb_cache_entryinfo_lock( *eip );
+			/* If the lock attempt fails, the info is in use */
+			if ( ldap_pvt_thread_mutex_trylock(
+					&(*eip)->bei_kids_mutex )) {
+				ldap_pvt_thread_rdwr_runlock( &bdb->bi_cache.c_rwlock );
+				/* If this node is being deleted, treat
+				 * as if the delete has already finished
+				 */
+				if ( (*eip)->bei_state & CACHE_ENTRY_DELETED ) {
+					return DB_NOTFOUND;
+				}
+				/* otherwise, wait for the info to free up */
+				ldap_pvt_thread_yield();
+				goto again;
+			}
+			/* If this info isn't hooked up to its parent yet,
+			 * unlock and wait for it to be fully initialized
+			 */
+			if ( (*eip)->bei_state & CACHE_ENTRY_NOT_LINKED ) {
+				bdb_cache_entryinfo_unlock( *eip );
+				ldap_pvt_thread_rdwr_runlock( &bdb->bi_cache.c_rwlock );
+				ldap_pvt_thread_yield();
+				goto again;
+			}
 			islocked = 1;
 		}
 		ldap_pvt_thread_rdwr_runlock( &bdb->bi_cache.c_rwlock );
@@ -540,18 +660,20 @@ bdb_cache_find_id(
 		rc = bdb_id2entry( op->o_bd, tid, id, &ep );
 		if ( rc == 0 ) {
 			rc = bdb_cache_find_ndn( op, tid,
-				&ep->e_nname, eip, locker );
-			if ( *eip )
-				islocked = 1;
+				&ep->e_nname, eip );
+			if ( *eip ) islocked = 1;
 			if ( rc ) {
+#ifdef SLAP_ZONE_ALLOC
+				bdb_entry_return( bdb, ep, (*eip)->bei_zseq );
+#else
 				bdb_entry_return( ep );
+#endif
 				ep = NULL;
 			}
 		}
 #else
 		rc = hdb_cache_find_parent(op, tid, id, eip );
-		if ( rc == 0 && *eip )
-			islocked = 1;
+		if ( rc == 0 && *eip ) islocked = 1;
 #endif
 	}
 
@@ -559,51 +681,164 @@ bdb_cache_find_id(
 	if ( *eip && rc == 0 ) {
 		if ( (*eip)->bei_state & CACHE_ENTRY_DELETED ) {
 			rc = DB_NOTFOUND;
-		} else if (!(*eip)->bei_e ) {
-			if (!ep) {
-				rc = bdb_id2entry( op->o_bd, tid, id, &ep );
-			}
-			if ( rc == 0 ) {
-				bdb_cache_entry_db_lock( bdb->bi_dbenv, locker,
-					*eip, 1, 0, lock );
-				ep->e_private = *eip;
-#ifdef BDB_HIER
-				bdb_fix_dn( ep, 0 );
-#endif
-				(*eip)->bei_e = ep;
-				bdb_cache_entry_db_relock( bdb->bi_dbenv, locker,
-					*eip, 0, 0, lock );
-			}
 		} else {
-#ifdef BDB_HIER
-			rc = bdb_fix_dn( (*eip)->bei_e, 1 );
-			if ( rc ) {
-				bdb_cache_entry_db_lock( bdb->bi_dbenv,
-					locker, *eip, 1, 0, lock );
-				rc = bdb_fix_dn( (*eip)->bei_e, 2 );
-				bdb_cache_entry_db_relock( bdb->bi_dbenv,
-					locker, *eip, 0, 0, lock );
-			} else {
-				bdb_cache_entry_db_lock( bdb->bi_dbenv,
-					locker, *eip, 0, 0, lock );
+			/* Make sure only one thread tries to load the entry */
+load1:
+#ifdef SLAP_ZONE_ALLOC
+			if ((*eip)->bei_e && !slap_zn_validate(
+					bdb->bi_cache.c_zctx, (*eip)->bei_e, (*eip)->bei_zseq)) {
+				(*eip)->bei_e = NULL;
+				(*eip)->bei_zseq = 0;
 			}
-#else
-			bdb_cache_entry_db_lock( bdb->bi_dbenv, locker,
-					*eip, 0, 0, lock );
 #endif
+			if ( !(*eip)->bei_e && !((*eip)->bei_state & CACHE_ENTRY_LOADING)) {
+				load = 1;
+				(*eip)->bei_state |= CACHE_ENTRY_LOADING;
+			}
+			if ( islocked ) {
+				bdb_cache_entryinfo_unlock( *eip );
+				islocked = 0;
+			}
+			rc = bdb_cache_entry_db_lock( bdb->bi_dbenv, locker, *eip, 0, 0, lock );
+			if ( (*eip)->bei_state & CACHE_ENTRY_DELETED ) {
+				rc = DB_NOTFOUND;
+				bdb_cache_entry_db_unlock( bdb->bi_dbenv, lock );
+			} else if ( rc == 0 ) {
+				if ( load ) {
+					DB_TXN *ltid;
+					u_int32_t locker2 = locker;
+
+					/* We don't wrap entire read operations in txn's, but
+					 * we need our cache entry lock and any DB page locks
+					 * to be associated, in order for deadlock detection
+					 * to work properly. So if we need to read from the DB,
+					 * we use a long-lived per-thread txn for this step.
+					 */
+					if ( !ep && !tid ) {
+						rc = bdb_txn_get( op, bdb->bi_dbenv, &ltid, 0 );
+						if ( ltid )
+							locker2 = TXN_ID( ltid );
+					} else {
+						ltid = tid;
+					}
+					/* Give up original read lock, obtain write lock with
+					 * (possibly) new locker ID.
+					 */
+				    if ( rc == 0 ) {
+						rc = bdb_cache_entry_db_relock( bdb->bi_dbenv, locker2,
+							*eip, 1, 0, lock );
+					}
+					if ( rc == 0 && !ep) {
+						rc = bdb_id2entry( op->o_bd, ltid, id, &ep );
+					}
+					if ( rc == 0 ) {
+						ep->e_private = *eip;
+#ifdef BDB_HIER
+						bdb_fix_dn( ep, 0 );
+#endif
+						(*eip)->bei_e = ep;
+#ifdef SLAP_ZONE_ALLOC
+						(*eip)->bei_zseq = *((ber_len_t *)ep - 2);
+#endif
+						ep = NULL;
+					}
+					(*eip)->bei_state ^= CACHE_ENTRY_LOADING;
+					if ( rc == 0 ) {
+						/* If we succeeded, downgrade back to a readlock. */
+						rc = bdb_cache_entry_db_relock( bdb->bi_dbenv, locker,
+							*eip, 0, 0, lock );
+					} else {
+						/* Otherwise, release the lock. */
+						bdb_cache_entry_db_unlock( bdb->bi_dbenv, lock );
+					}
+					if ( locker2 != locker ) {
+						/* If we're using the per-thread txn, release all
+						 * of its page locks now.
+						 */
+						DB_LOCKREQ list;
+						list.op = DB_LOCK_PUT_ALL;
+						list.obj = NULL;
+						bdb->bi_dbenv->lock_vec( bdb->bi_dbenv, locker2,
+							0, &list, 1, NULL );
+						/* If this txn was deadlocked, we must abort it
+						 * and invalidate this per-thread txn.
+						 */
+						if ( rc == DB_LOCK_DEADLOCK ) {
+							bdb_txn_get( op, bdb->bi_dbenv, &ltid, 1 );
+						}
+					}
+				} else if ( !(*eip)->bei_e ) {
+					/* Some other thread is trying to load the entry,
+					 * give it a chance to finish.
+					 */
+					bdb_cache_entry_db_unlock( bdb->bi_dbenv, lock );
+					ldap_pvt_thread_yield();
+					bdb_cache_entryinfo_lock( *eip );
+					islocked = 1;
+					goto load1;
+#ifdef BDB_HIER
+				} else {
+					/* Check for subtree renames
+					 */
+					rc = bdb_fix_dn( (*eip)->bei_e, 1 );
+					if ( rc ) {
+						bdb_cache_entry_db_relock( bdb->bi_dbenv,
+							locker, *eip, 1, 0, lock );
+						/* check again in case other modifier did it already */
+						if ( bdb_fix_dn( (*eip)->bei_e, 1 ) )
+							rc = bdb_fix_dn( (*eip)->bei_e, 2 );
+						bdb_cache_entry_db_relock( bdb->bi_dbenv,
+							locker, *eip, 0, 0, lock );
+					}
+#endif
+				}
+
+			}
 		}
 	}
-	if ( rc == 0 && (*eip)->bei_kids == NULL ) {
-		/* set lru mutex */
-		ldap_pvt_thread_mutex_lock( &bdb->bi_cache.lru_mutex );
-		LRU_DELETE( &bdb->bi_cache, *eip );
-		LRU_ADD( &bdb->bi_cache, *eip );
-		ldap_pvt_thread_mutex_unlock( &bdb->bi_cache.lru_mutex );
-	}
-
 	if ( islocked ) {
 		bdb_cache_entryinfo_unlock( *eip );
 	}
+	if ( ep ) {
+#ifdef SLAP_ZONE_ALLOC
+		bdb_entry_return( bdb, ep, (*eip)->bei_zseq );
+#else
+		bdb_entry_return( ep );
+#endif
+	}
+	if ( rc == 0 ) {
+
+		if ( load ) {
+			ldap_pvt_thread_rdwr_wlock( &bdb->bi_cache.c_rwlock );
+			bdb->bi_cache.c_cursize++;
+			ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
+		}
+
+		ldap_pvt_thread_mutex_lock( &bdb->bi_cache.lru_mutex );
+
+		/* If the LRU list has only one entry and this is it, it
+		 * doesn't need to be added again.
+		 */
+		if ( bdb->bi_cache.c_lruhead == bdb->bi_cache.c_lrutail &&
+			bdb->bi_cache.c_lruhead == *eip ) {
+			ldap_pvt_thread_mutex_unlock( &bdb->bi_cache.lru_mutex );
+		} else {
+
+			/* if entry is on LRU list, remove from old spot */
+			if ( (*eip)->bei_lrunext || (*eip)->bei_lruprev ) {
+				LRU_DELETE( &bdb->bi_cache, *eip );
+			}
+			/* lru_mutex is unlocked for us */
+			bdb_cache_lru_add( bdb, locker, *eip );
+		}
+	}
+
+#ifdef SLAP_ZONE_ALLOC
+	if (rc == 0 && (*eip)->bei_e) {
+		slap_zn_rlock(bdb->bi_cache.c_zctx, (*eip)->bei_e);
+	}
+	slap_zh_runlock(bdb->bi_cache.c_zctx);
+#endif
 	return rc;
 }
 
@@ -611,8 +846,7 @@ int
 bdb_cache_children(
 	Operation *op,
 	DB_TXN *txn,
-	Entry *e
-)
+	Entry *e )
 {
 	int rc;
 
@@ -624,7 +858,7 @@ bdb_cache_children(
 	}
 	rc = bdb_dn2id_children( op, txn, e );
 	if ( rc == DB_NOTFOUND ) {
-		BEI(e)->bei_state |= CACHE_ENTRY_NO_KIDS;
+		BEI(e)->bei_state |= CACHE_ENTRY_NO_KIDS | CACHE_ENTRY_NO_GRANDKIDS;
 	}
 	return rc;
 }
@@ -636,30 +870,63 @@ bdb_cache_add(
 	EntryInfo *eip,
 	Entry *e,
 	struct berval *nrdn,
-	u_int32_t locker
-)
+	u_int32_t locker )
 {
 	EntryInfo *new, ei;
 	struct berval rdn = e->e_name;
+	DB_LOCK lock;
 	int rc;
 
 	ei.bei_id = e->e_id;
 	ei.bei_parent = eip;
 	ei.bei_nrdn = *nrdn;
+	ei.bei_lockpad = 0;
+
+	/* Lock this entry so that bdb_add can run to completion.
+	 * It can only fail if BDB has run out of lock resources.
+	 */
+	rc = bdb_cache_entry_db_lock( bdb->bi_dbenv, locker, &ei, 1, 0, &lock );
+	if ( rc ) {
+		bdb_cache_entryinfo_unlock( eip );
+		return rc;
+	}
+
 #ifdef BDB_HIER
 	if ( nrdn->bv_len != e->e_nname.bv_len ) {
 		char *ptr = strchr( rdn.bv_val, ',' );
 		rdn.bv_len = ptr - rdn.bv_val;
 	}
 	ber_dupbv( &ei.bei_rdn, &rdn );
+	if ( eip->bei_dkids ) eip->bei_dkids++;
 #endif
-	rc = bdb_entryinfo_add_internal( bdb, &ei, &new, locker );
+
+	rc = bdb_entryinfo_add_internal( bdb, &ei, &new );
+	/* bdb_csn_commit can cause this when adding the database root entry */
+	if ( new->bei_e ) {
+		new->bei_e->e_private = NULL;
+#ifdef SLAP_ZONE_ALLOC
+		bdb_entry_return( bdb, new->bei_e, new->bei_zseq );
+#else
+		bdb_entry_return( new->bei_e );
+#endif
+	}
 	new->bei_e = e;
 	e->e_private = new;
-	new->bei_state = CACHE_ENTRY_NO_KIDS;
+	new->bei_state = CACHE_ENTRY_NO_KIDS | CACHE_ENTRY_NO_GRANDKIDS;
 	eip->bei_state &= ~CACHE_ENTRY_NO_KIDS;
+	if (eip->bei_parent) {
+		eip->bei_parent->bei_state &= ~CACHE_ENTRY_NO_GRANDKIDS;
+	}
 	bdb_cache_entryinfo_unlock( eip );
+
+	/* set lru mutex */
+	ldap_pvt_thread_mutex_lock( &bdb->bi_cache.lru_mutex );
+	++bdb->bi_cache.c_cursize;
 	ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
+
+	/* lru_mutex is unlocked for us */
+	bdb_cache_lru_add( bdb, locker, new );
+
 	return rc;
 }
 
@@ -669,23 +936,23 @@ bdb_cache_modify(
 	Attribute *newAttrs,
 	DB_ENV *env,
 	u_int32_t locker,
-	DB_LOCK *lock
-)
+	DB_LOCK *lock )
 {
 	EntryInfo *ei = BEI(e);
-	
+	int rc;
 	/* Get write lock on data */
-	bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
+	rc = bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
 
 	/* If we've done repeated mods on a cached entry, then e_attrs
 	 * is no longer contiguous with the entry, and must be freed.
 	 */
-	if ( (void *)e->e_attrs != (void *)(e+1) ) {
-		attrs_free( e->e_attrs );
+	if ( ! rc ) {
+		if ( (void *)e->e_attrs != (void *)(e+1) ) {
+			attrs_free( e->e_attrs ); 
+		}
+		e->e_attrs = newAttrs;
 	}
-	e->e_attrs = newAttrs;
-
-	return 0;
+	return rc;
 }
 
 /*
@@ -699,15 +966,15 @@ bdb_cache_modrdn(
 	EntryInfo *ein,
 	DB_ENV *env,
 	u_int32_t locker,
-	DB_LOCK *lock
-)
+	DB_LOCK *lock )
 {
 	EntryInfo *ei = BEI(e), *pei;
 	struct berval rdn;
-	int rc = 0;
+	int rc;
 
 	/* Get write lock on data */
-	bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
+	rc =  bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
+	if ( rc ) return rc;
 
 	/* If we've done repeated mods on a cached entry, then e_attrs
 	 * is no longer contiguous with the entry, and must be freed.
@@ -716,8 +983,9 @@ bdb_cache_modrdn(
 		attrs_free( e->e_attrs );
 	}
 	e->e_attrs = new->e_attrs;
-	if( e->e_nname.bv_val < e->e_bv.bv_val || e->e_nname.bv_val >
-		e->e_bv.bv_val + e->e_bv.bv_len ) {
+	if( e->e_nname.bv_val < e->e_bv.bv_val ||
+		e->e_nname.bv_val > e->e_bv.bv_val + e->e_bv.bv_len )
+	{
 		ch_free(e->e_name.bv_val);
 		ch_free(e->e_nname.bv_val);
 	}
@@ -749,11 +1017,11 @@ bdb_cache_modrdn(
 		bdb_cache_entryinfo_lock( ein );
 	}
 #ifdef BDB_HIER
-	{ int max = ei->bei_modrdns;
-	/* Record the generation number of this change */
+	{
+		int max = ei->bei_modrdns;
+		/* Record the generation number of this change */
 		for ( pei = ein; pei->bei_parent; pei = pei->bei_parent ) {
-			if ( pei->bei_modrdns > max )
-				max = pei->bei_modrdns;
+			if ( pei->bei_modrdns > max ) max = pei->bei_modrdns;
 		}
 		ei->bei_modrdns = max + 1;
 	}
@@ -775,8 +1043,7 @@ bdb_cache_delete(
     Entry		*e,
     DB_ENV	*env,
     u_int32_t	locker,
-    DB_LOCK	*lock
-)
+    DB_LOCK	*lock )
 {
 	EntryInfo *ei = BEI(e);
 	int	rc;
@@ -786,68 +1053,94 @@ bdb_cache_delete(
 	/* Set this early, warn off any queriers */
 	ei->bei_state |= CACHE_ENTRY_DELETED;
 
+	/* Lock the entry's info */
+	bdb_cache_entryinfo_lock( ei );
+
 	/* Get write lock on the data */
-	bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
+	rc = bdb_cache_entry_db_relock( env, locker, ei, 1, 0, lock );
+	if ( rc ) {
+		/* couldn't lock, undo and give up */
+		ei->bei_state ^= CACHE_ENTRY_DELETED;
+		bdb_cache_entryinfo_unlock( ei );
+		return rc;
+	}
 
-	/* set cache write lock */
-	ldap_pvt_thread_rdwr_wlock( &cache->c_rwlock );
-
-	/* Lock the parent's kids tree */
-	bdb_cache_entryinfo_lock( ei->bei_parent );
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( CACHE, ENTRY, 
-		"bdb_cache_delete: delete %ld.\n", e->e_id, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "====> bdb_cache_delete( %ld )\n",
 		e->e_id, 0, 0 );
-#endif
 
 	/* set lru mutex */
 	ldap_pvt_thread_mutex_lock( &cache->lru_mutex );
-	rc = bdb_cache_delete_internal( cache, e->e_private );
+	rc = bdb_cache_delete_internal( cache, e->e_private, 1 );
 	/* free lru mutex */
 	ldap_pvt_thread_mutex_unlock( &cache->lru_mutex );
 
-	/* free cache write lock */
-	ldap_pvt_thread_rdwr_wunlock( &cache->c_rwlock );
-	bdb_cache_entryinfo_unlock( ei->bei_parent );
+	/* Leave entry info locked */
+
 	return( rc );
 }
 
 void
 bdb_cache_delete_cleanup(
-	Entry *e
-)
+	Cache *cache,
+	EntryInfo *ei )
 {
-	bdb_cache_entryinfo_destroy( e->e_private );
-	e->e_private = NULL;
-	bdb_entry_return( e );
+	if ( ei->bei_e ) {
+		ei->bei_e->e_private = NULL;
+#ifdef SLAP_ZONE_ALLOC
+		bdb_entry_return( ei->bei_bdb, ei->bei_e, ei->bei_zseq );
+#else
+		bdb_entry_return( ei->bei_e );
+#endif
+		ei->bei_e = NULL;
+	}
+
+	free( ei->bei_nrdn.bv_val );
+	ei->bei_nrdn.bv_val = NULL;
+#ifdef BDB_HIER
+	free( ei->bei_rdn.bv_val );
+	ei->bei_rdn.bv_val = NULL;
+	ei->bei_modrdns = 0;
+	ei->bei_ckids = 0;
+	ei->bei_dkids = 0;
+#endif
+	ei->bei_parent = NULL;
+	ei->bei_kids = NULL;
+	ei->bei_lruprev = NULL;
+
+	ldap_pvt_thread_rdwr_wlock( &cache->c_rwlock );
+	ei->bei_lrunext = cache->c_eifree;
+	cache->c_eifree = ei;
+	ldap_pvt_thread_rdwr_wunlock( &cache->c_rwlock );
+	bdb_cache_entryinfo_unlock( ei );
 }
-	
+
 static int
 bdb_cache_delete_internal(
     Cache	*cache,
-    EntryInfo		*e
-)
+    EntryInfo		*e,
+    int		decr )
 {
 	int rc = 0;	/* return code */
 
+	/* set cache write lock */
+	ldap_pvt_thread_rdwr_wlock( &cache->c_rwlock );
+
+	/* Lock the parent's kids tree */
+	bdb_cache_entryinfo_lock( e->bei_parent );
+
+#ifdef BDB_HIER
+	e->bei_parent->bei_ckids--;
+	if ( decr && e->bei_parent->bei_dkids ) e->bei_parent->bei_dkids--;
+#endif
 	/* dn tree */
-	if ( avl_delete( &e->bei_parent->bei_kids, (caddr_t) e, bdb_rdn_cmp ) == NULL )
+	if ( avl_delete( &e->bei_parent->bei_kids, (caddr_t) e, bdb_rdn_cmp )
+		== NULL )
 	{
 		rc = -1;
 	}
 
-	/* If parent has no more kids, put in on LRU list */
-	if ( e->bei_parent->bei_kids == NULL ) {
-		LRU_ADD( cache, e->bei_parent );
-		cache->c_cursize++;
-	}
-
 	/* id tree */
-	if ( avl_delete( &cache->c_idtree, (caddr_t) e, bdb_id_cmp ) == NULL )
-	{
+	if ( avl_delete( &cache->c_idtree, (caddr_t) e, bdb_id_cmp ) == NULL ) {
 		rc = -1;
 	}
 
@@ -855,14 +1148,15 @@ bdb_cache_delete_internal(
 		return rc;
 	}
 
+	cache->c_eiused--;
+
 	/* lru */
 	LRU_DELETE( cache, e );
-	cache->c_cursize--;
+	if ( e->bei_e ) cache->c_cursize--;
 
-	/*
-	 * flag entry to be freed later by a call to cache_return_entry()
-	 */
-	e->bei_state |= CACHE_ENTRY_DELETED;
+	/* free cache write lock */
+	ldap_pvt_thread_rdwr_wunlock( &cache->c_rwlock );
+	bdb_cache_entryinfo_unlock( e->bei_parent );
 
 	return( 0 );
 }
@@ -876,7 +1170,11 @@ bdb_entryinfo_release( void *data )
 	}
 	if ( ei->bei_e ) {
 		ei->bei_e->e_private = NULL;
+#ifdef SLAP_ZONE_ALLOC
+		bdb_entry_return( ei->bei_bdb, ei->bei_e, ei->bei_zseq );
+#else
 		bdb_entry_return( ei->bei_e );
+#endif
 	}
 	bdb_cache_entryinfo_destroy( ei );
 }
@@ -889,14 +1187,14 @@ bdb_cache_release_all( Cache *cache )
 	/* set lru mutex */
 	ldap_pvt_thread_mutex_lock( &cache->lru_mutex );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( CACHE, ENTRY, "bdb_cache_release_all: enter\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "====> bdb_cache_release_all\n", 0, 0, 0 );
-#endif
 
 	avl_free( cache->c_dntree.bei_kids, NULL );
 	avl_free( cache->c_idtree, bdb_entryinfo_release );
+	for (;cache->c_eifree;cache->c_eifree = cache->c_lruhead) {
+		cache->c_lruhead = cache->c_eifree->bei_lrunext;
+		bdb_cache_entryinfo_destroy(cache->c_eifree);
+	}
 	cache->c_lruhead = NULL;
 	cache->c_lrutail = NULL;
 
@@ -925,6 +1223,62 @@ bdb_lru_print( Cache *cache )
 }
 #endif
 
+static void
+bdb_txn_free( void *key, void *data )
+{
+	DB_TXN *txn = data;
+	TXN_ABORT( txn );
+}
+
+/* Obtain a long-lived transaction for the current thread.
+ * If reset == 1, remove the current transaction. */
+static int
+bdb_txn_get( Operation *op, DB_ENV *env, DB_TXN **txn, int reset )
+{
+	int i, rc, lockid;
+	void *ctx, *data = NULL;
+
+	/* If no op was provided, try to find the ctx anyway... */
+	if ( op ) {
+		ctx = op->o_threadctx;
+	} else {
+		ctx = ldap_pvt_thread_pool_context();
+	}
+
+	/* Shouldn't happen unless we're single-threaded */
+	if ( !ctx ) {
+		*txn = NULL;
+		return 0;
+	}
+
+	if ( reset ) {
+		TXN_ABORT( *txn );
+		return ldap_pvt_thread_pool_setkey( ctx, ((char *)env)+1, NULL, NULL );
+	}
+
+	if ( ldap_pvt_thread_pool_getkey( ctx, ((char *)env)+1, &data, NULL ) ||
+		data == NULL ) {
+		for ( i=0, rc=1; rc != 0 && i<4; i++ ) {
+			rc = TXN_BEGIN( env, NULL, txn, 0 );
+			if (rc) ldap_pvt_thread_yield();
+		}
+		if ( rc != 0) {
+			return rc;
+		}
+		if ( ( rc = ldap_pvt_thread_pool_setkey( ctx, ((char *)env)+1,
+			*txn, bdb_txn_free ) ) ) {
+			TXN_ABORT( *txn );
+			Debug( LDAP_DEBUG_ANY, "bdb_txn_get: err %s(%d)\n",
+				db_strerror(rc), rc, 0 );
+
+			return rc;
+		}
+	} else {
+		*txn = data;
+	}
+	return 0;
+}
+
 #ifdef BDB_REUSE_LOCKERS
 static void
 bdb_locker_id_free( void *key, void *data )
@@ -933,23 +1287,15 @@ bdb_locker_id_free( void *key, void *data )
 	int lockid = (int) data;
 	int rc;
 
-
 	rc = XLOCK_ID_FREE( env, lockid );
 	if ( rc == EINVAL ) {
 		DB_LOCKREQ lr;
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_BDB, ERR,
-			"bdb_locker_id_free: %d err %s(%d)\n",
-			lockid, db_strerror(rc), rc );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"bdb_locker_id_free: %d err %s(%d)\n",
 			lockid, db_strerror(rc), rc );
-#endif
-		memset( &lr, 0, sizeof(lr) );
-
 		/* release all locks held by this locker. */
 		lr.op = DB_LOCK_PUT_ALL;
+		lr.obj = NULL;
 		env->lock_vec( env, lockid, 0, &lr, 1, NULL );
 		XLOCK_ID_FREE( env, lockid );
 	}
@@ -989,13 +1335,8 @@ bdb_locker_id( Operation *op, DB_ENV *env, int *locker )
 		if ( ( rc = ldap_pvt_thread_pool_setkey( ctx, env,
 			data, bdb_locker_id_free ) ) ) {
 			XLOCK_ID_FREE( env, lockid );
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, "bdb_locker_id: err %s(%d)\n",
-				db_strerror(rc), rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "bdb_locker_id: err %s(%d)\n",
 				db_strerror(rc), rc, 0 );
-#endif
 
 			return rc;
 		}
@@ -1006,3 +1347,29 @@ bdb_locker_id( Operation *op, DB_ENV *env, int *locker )
 	return 0;
 }
 #endif
+
+void
+bdb_cache_delete_entry(
+	struct bdb_info *bdb,
+	EntryInfo *ei,
+	u_int32_t locker,
+	DB_LOCK *lock )
+{
+	ldap_pvt_thread_rdwr_wlock( &bdb->bi_cache.c_rwlock );
+	if ( bdb_cache_entry_db_lock( bdb->bi_dbenv, bdb->bi_cache.c_locker, ei, 1, 1, lock ) == 0 )
+	{
+		if ( ei->bei_e && !(ei->bei_state & CACHE_ENTRY_NOT_LINKED )) {
+			LRU_DELETE( &bdb->bi_cache, ei );
+			ei->bei_e->e_private = NULL;
+#ifdef SLAP_ZONE_ALLOC
+			bdb_entry_return( bdb, ei->bei_e, ei->bei_zseq );
+#else
+			bdb_entry_return( ei->bei_e );
+#endif
+			ei->bei_e = NULL;
+			--bdb->bi_cache.c_cursize;
+		}
+		bdb_cache_entry_db_unlock( bdb->bi_dbenv, lock );
+	}
+	ldap_pvt_thread_rdwr_wunlock( &bdb->bi_cache.c_rwlock );
+}
