@@ -1,8 +1,17 @@
 /* dn2entry.c - routines to deal with the dn2id / id2entry glue */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2entry.c,v 1.19 2003/05/24 22:53:08 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2entry.c,v 1.25 2004/09/04 02:54:31 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -12,7 +21,6 @@
 
 #include "back-bdb.h"
 
-
 /*
  * dn2entry - look up dn in the cache/indexes and return the corresponding
  * entry. If the requested DN is not found and matched is TRUE, return info
@@ -30,28 +38,25 @@ bdb_dn2entry(
 	DB_LOCK *lock )
 {
 	EntryInfo *ei = NULL;
-	int rc;
+	int rc, rc2;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( CACHE, ARGS, "bdb_dn2entry(\"%s\")\n", dn->bv_val, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_TRACE, "bdb_dn2entry(\"%s\")\n",
 		dn->bv_val, 0, 0 );
-#endif
 
 	*e = NULL;
 
-	rc = bdb_cache_find_ndn( op, tid, dn, &ei, locker );
+	rc = bdb_cache_find_ndn( op, tid, dn, &ei );
 	if ( rc ) {
 		if ( matched && rc == DB_NOTFOUND ) {
 			/* Set the return value, whether we have its entry
 			 * or not.
 			 */
 			*e = ei;
-			if ( ei && ei->bei_id )
-				bdb_cache_find_id( op, tid, ei->bei_id,
+			if ( ei && ei->bei_id ) {
+				rc2 = bdb_cache_find_id( op, tid, ei->bei_id,
 					&ei, 1, locker, lock );
-			else if ( ei )
+				if ( rc2 ) rc = rc2;
+			} else if ( ei )
 				bdb_cache_entryinfo_unlock( ei );
 		} else if ( ei ) {
 			bdb_cache_entryinfo_unlock( ei );
@@ -64,8 +69,9 @@ bdb_dn2entry(
 		} else if ( matched && rc == DB_NOTFOUND ) {
 			/* always return EntryInfo */
 			ei = ei->bei_parent;
-			bdb_cache_find_id( op, tid, ei->bei_id, &ei, 1,
+			rc2 = bdb_cache_find_id( op, tid, ei->bei_id, &ei, 1,
 				locker, lock );
+			if ( rc2 ) rc = rc2;
 			*e = ei;
 		}
 	}
