@@ -1,8 +1,17 @@
 /* dn2id.c - routines to deal with the dn2id index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.84 2003/05/24 22:53:08 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/dn2id.c,v 1.106 2004/12/08 19:55:35 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -26,16 +35,12 @@ bdb_dn2id_add(
 	DB *db = bdb->bi_dn2id->bdi_db;
 	int		rc;
 	DBT		key, data;
+	ID		nid;
 	char		*buf;
 	struct berval	ptr, pdn;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, "bdb_dn2id_add( \"%s\", 0x%08lx )\n",
-		e->e_ndn, (long) e->e_id, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id_add( \"%s\", 0x%08lx )\n",
 		e->e_ndn, (long) e->e_id, 0 );
-#endif
 	assert( e->e_id != NOID );
 
 	DBTzero( &key );
@@ -51,43 +56,35 @@ bdb_dn2id_add(
 	ptr.bv_val[ptr.bv_len] = '\0';
 
 	DBTzero( &data );
-	data.data = (char *) &e->e_id;
-	data.size = sizeof( e->e_id );
+	data.data = &nid;
+	data.size = sizeof( nid );
+	BDB_ID2DISK( e->e_id, &nid );
 
 	/* store it -- don't override */
 	rc = db->put( db, txn, &key, &data, DB_NOOVERWRITE );
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, ERR, "bdb_dn2id_add: put failed: %s %d\n",
-			db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "=> bdb_dn2id_add: put failed: %s %d\n",
 			db_strerror(rc), rc, 0 );
-#endif
 		goto done;
 	}
 
 #ifndef BDB_MULTIPLE_SUFFIXES
-	if( !be_issuffix( op->o_bd, &ptr )) {
+	if( !be_issuffix( op->o_bd, &ptr ))
 #endif
+	{
 		buf[0] = DN_SUBTREE_PREFIX;
 		rc = db->put( db, txn, &key, &data, DB_NOOVERWRITE );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_add: subtree (%s) put failed: %d\n",
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			"=> bdb_dn2id_add: subtree (%s) put failed: %d\n",
 			ptr.bv_val, rc, 0 );
-#endif
 			goto done;
 		}
 		
 #ifdef BDB_MULTIPLE_SUFFIXES
-	if( !be_issuffix( op->o_bd, &ptr )) {
+	if( !be_issuffix( op->o_bd, &ptr ))
 #endif
+	{
 		dnParent( &ptr, &pdn );
 	
 		key.size = pdn.bv_len + 2;
@@ -99,38 +96,27 @@ bdb_dn2id_add(
 		rc = bdb_idl_insert_key( op->o_bd, db, txn, &key, e->e_id );
 
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_add: parent (%s) insert failed: %d\n",
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"=> bdb_dn2id_add: parent (%s) insert failed: %d\n",
 					ptr.bv_val, rc, 0 );
-#endif
 			goto done;
 		}
-#ifndef BDB_MULTIPLE_SUFFIXES
 	}
 
-	while( !be_issuffix( op->o_bd, &ptr )) {
+#ifndef BDB_MULTIPLE_SUFFIXES
+	while( !be_issuffix( op->o_bd, &ptr ))
 #else
-	for (;;) {
+	for (;;)
 #endif
+	{
 		ptr.bv_val[-1] = DN_SUBTREE_PREFIX;
 
 		rc = bdb_idl_insert_key( op->o_bd, db, txn, &key, e->e_id );
 
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_add: subtree (%s) insert failed: %d\n",
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"=> bdb_dn2id_add: subtree (%s) insert failed: %d\n",
 					ptr.bv_val, rc, 0 );
-#endif
 			break;
 		}
 #ifdef BDB_MULTIPLE_SUFFIXES
@@ -143,17 +129,11 @@ bdb_dn2id_add(
 		key.data = pdn.bv_val - 1;
 		ptr = pdn;
 	}
-#ifdef BDB_MULTIPLE_SUFFIXES
 	}
-#endif
 
 done:
 	op->o_tmpfree( buf, op->o_tmpmemctx );
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, RESULTS, "<= bdb_dn2id_add: %d\n", rc, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id_add: %d\n", rc, 0, 0 );
-#endif
 	return rc;
 }
 
@@ -171,13 +151,8 @@ bdb_dn2id_delete(
 	char		*buf;
 	struct berval	pdn, ptr;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, 
-		"=> bdb_dn2id_delete ( \"%s\", 0x%08lx )\n", e->e_ndn, e->e_id, 0);
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id_delete( \"%s\", 0x%08lx )\n",
 		e->e_ndn, e->e_id, 0 );
-#endif
 
 	DBTzero( &key );
 	key.size = e->e_nname.bv_len + 2;
@@ -193,38 +168,28 @@ bdb_dn2id_delete(
 	/* delete it */
 	rc = db->del( db, txn, &key, 0 );
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, ERR, 
-			"=> bdb_dn2id_delete: delete failed: %s %d\n", 
-			db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "=> bdb_dn2id_delete: delete failed: %s %d\n",
 			db_strerror(rc), rc, 0 );
-#endif
 		goto done;
 	}
 
 #ifndef BDB_MULTIPLE_SUFFIXES
-	if( !be_issuffix( op->o_bd, &ptr )) {
+	if( !be_issuffix( op->o_bd, &ptr ))
 #endif
+	{
 		buf[0] = DN_SUBTREE_PREFIX;
 		rc = db->del( db, txn, &key, 0 );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_delete: subtree (%s) delete failed: %d\n", 
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			"=> bdb_dn2id_delete: subtree (%s) delete failed: %d\n",
 			ptr.bv_val, rc, 0 );
-#endif
 			goto done;
 		}
 
 #ifdef BDB_MULTIPLE_SUFFIXES
-	if( !be_issuffix( op->o_bd, &ptr )) {
+	if( !be_issuffix( op->o_bd, &ptr ))
 #endif
+	{
 		dnParent( &ptr, &pdn );
 
 		key.size = pdn.bv_len + 2;
@@ -236,37 +201,26 @@ bdb_dn2id_delete(
 		rc = bdb_idl_delete_key( op->o_bd, db, txn, &key, e->e_id );
 
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_delete: parent (%s) delete failed: %d\n", 
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"=> bdb_dn2id_delete: parent (%s) delete failed: %d\n",
 				ptr.bv_val, rc, 0 );
-#endif
 			goto done;
 		}
-#ifndef BDB_MULTIPLE_SUFFIXES
 	}
 
-	while( !be_issuffix( op->o_bd, &ptr )) {
+#ifndef BDB_MULTIPLE_SUFFIXES
+	while( !be_issuffix( op->o_bd, &ptr ))
 #else
-	for (;;) {
+	for (;;)
 #endif
+	{
 		ptr.bv_val[-1] = DN_SUBTREE_PREFIX;
 
 		rc = bdb_idl_delete_key( op->o_bd, db, txn, &key, e->e_id );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( INDEX, ERR, 
-				"=> bdb_dn2id_delete: subtree (%s) delete failed: %d\n", 
-				ptr.bv_val, rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"=> bdb_dn2id_delete: subtree (%s) delete failed: %d\n",
 				ptr.bv_val, rc, 0 );
-#endif
 			goto done;
 		}
 #ifdef BDB_MULTIPLE_SUFFIXES
@@ -279,17 +233,11 @@ bdb_dn2id_delete(
 		key.data = pdn.bv_val - 1;
 		ptr = pdn;
 	}
-#ifdef BDB_MULTIPLE_SUFFIXES
 	}
-#endif
 
 done:
 	op->o_tmpfree( buf, op->o_tmpmemctx );
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, RESULTS, "<= bdb_dn2id_delete %d\n", rc, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id_delete %d\n", rc, 0, 0 );
-#endif
 	return rc;
 }
 
@@ -300,16 +248,13 @@ bdb_dn2id(
 	struct berval	*dn,
 	EntryInfo *ei )
 {
-	int		rc;
-	DBT		key, data;
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	DB *db = bdb->bi_dn2id->bdi_db;
+	int		rc;
+	DBT		key, data;
+	ID		nid;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, "=> bdb_dn2id( \"%s\" )\n", dn->bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id( \"%s\" )\n", dn->bv_val, 0, 0 );
-#endif
+	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id(\"%s\")\n", dn->bv_val, 0, 0 );
 	DBTzero( &key );
 	key.size = dn->bv_len + 2;
 	key.data = op->o_tmpalloc( key.size, op->o_tmpmemctx );
@@ -318,7 +263,7 @@ bdb_dn2id(
 
 	/* store the ID */
 	DBTzero( &data );
-	data.data = &ei->bei_id;
+	data.data = &nid;
 	data.ulen = sizeof(ID);
 	data.flags = DB_DBT_USERMEM;
 
@@ -326,21 +271,12 @@ bdb_dn2id(
 	rc = db->get( db, txn, &key, &data, bdb->bi_db_opflags );
 
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, ERR, "<= bdb_dn2id: get failed %s (%d)\n", 
-			db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id: get failed: %s (%d)\n",
 			db_strerror( rc ), rc, 0 );
-#endif
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, RESULTS, 
-			"<= bdb_dn2id: got id=0x%08lx\n", ei->bei_id, 0, 0 );
-#else
+		BDB_DISK2ID( &nid, &ei->bei_id );
 		Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id: got id=0x%08lx\n",
 			ei->bei_id, 0, 0 );
-#endif
 	}
 
 	op->o_tmpfree( key.data, op->o_tmpmemctx );
@@ -359,20 +295,14 @@ bdb_dn2id_children(
 	ID		id;
 	int		rc;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, 
-		"=> bdb_dn2id_children( %s )\n", e->e_nname.bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id_children( %s )\n",
+	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2id_children(\"%s\")\n",
 		e->e_nname.bv_val, 0, 0 );
-#endif
 	DBTzero( &key );
 	key.size = e->e_nname.bv_len + 2;
 	key.data = op->o_tmpalloc( key.size, op->o_tmpmemctx );
 	((char *)key.data)[0] = DN_ONE_PREFIX;
 	AC_MEMCPY( &((char *)key.data)[1], e->e_nname.bv_val, key.size - 1 );
 
-#ifdef SLAP_IDL_CACHE
 	if ( bdb->bi_idl_cache_size ) {
 		rc = bdb_idl_cache_get( bdb, db, &key, NULL );
 		if ( rc != LDAP_NO_SUCH_OBJECT ) {
@@ -380,7 +310,6 @@ bdb_dn2id_children(
 			return rc;
 		}
 	}
-#endif
 	/* we actually could do a empty get... */
 	DBTzero( &data );
 	data.data = &id;
@@ -392,17 +321,10 @@ bdb_dn2id_children(
 	rc = db->get( db, txn, &key, &data, bdb->bi_db_opflags );
 	op->o_tmpfree( key.data, op->o_tmpmemctx );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, DETAIL1, 
-		"<= bdb_dn2id_children( %s ): %s (%d)\n", 
-		e->e_nname.bv_val, rc == 0 ? "" : ( rc == DB_NOTFOUND ? "no " :
-		db_strerror(rc)), rc );
-#else
-	Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id_children( %s ): %s (%d)\n",
+	Debug( LDAP_DEBUG_TRACE, "<= bdb_dn2id_children(\"%s\"): %s (%d)\n",
 		e->e_nname.bv_val,
 		rc == 0 ? "" : ( rc == DB_NOTFOUND ? "no " :
 			db_strerror(rc) ), rc );
-#endif
 
 	return rc;
 }
@@ -418,19 +340,14 @@ bdb_dn2idl(
 	DBT		key;
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	DB *db = bdb->bi_dn2id->bdi_db;
-	int prefix = op->ors_scope == LDAP_SCOPE_SUBTREE ? DN_SUBTREE_PREFIX :
-			DN_ONE_PREFIX;
+	int prefix = ( op->ors_scope == LDAP_SCOPE_ONELEVEL )
+		? DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, 
-		"=> bdb_dn2ididl( \"%s\" )\n", e->e_nname.bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2idl( \"%s\" )\n", e->e_nname.bv_val, 0, 0 );
-#endif
+	Debug( LDAP_DEBUG_TRACE, "=> bdb_dn2idl(\"%s\")\n",
+		e->e_nname.bv_val, 0, 0 );
 
 #ifndef	BDB_MULTIPLE_SUFFIXES
-	if (prefix == DN_SUBTREE_PREFIX && BEI(e)->bei_parent->bei_id == 0 )
-	{
+	if ( prefix == DN_SUBTREE_PREFIX && BEI(e)->bei_parent->bei_id == 0 ) {
 		BDB_IDL_ALL(bdb, ids);
 		return 0;
 	}
@@ -444,37 +361,25 @@ bdb_dn2idl(
 	((char *)key.data)[0] = prefix;
 	AC_MEMCPY( &((char *)key.data)[1], e->e_nname.bv_val, key.size - 1 );
 
-	rc = bdb_idl_fetch_key( op->o_bd, db, NULL, &key, ids );
+	rc = bdb_idl_fetch_key( op->o_bd, db, NULL, &key, ids, NULL, 0 );
 
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, ERR, 
-			"<= bdb_dn2ididl: get failed: %s (%d)\n", db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"<= bdb_dn2idl: get failed: %s (%d)\n",
 			db_strerror( rc ), rc, 0 );
-#endif
 
 	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( INDEX, RESULTS, 
-			"<= bdb_dn2ididl: id=%ld first=%ld last=%ld\n", 
-			(long) ids[0], (long) BDB_IDL_FIRST( ids ), 
-			(long) BDB_IDL_LAST( ids ) );
-#else
 		Debug( LDAP_DEBUG_TRACE,
 			"<= bdb_dn2idl: id=%ld first=%ld last=%ld\n",
 			(long) ids[0],
 			(long) BDB_IDL_FIRST( ids ), (long) BDB_IDL_LAST( ids ) );
-#endif
 	}
 
 	op->o_tmpfree( key.data, op->o_tmpmemctx );
 	return rc;
 }
-#else	/* BDB_HIER */
 
+#else	/* BDB_HIER */
 /* Experimental management routines for a hierarchically structured database.
  *
  * Unsupported! Use at your own risk!
@@ -506,11 +411,10 @@ int
 hdb_dup_compare(
 	DB *db, 
 	const DBT *usrkey,
-	const DBT *curkey
-)
+	const DBT *curkey )
 {
-	char *u = (char *)&(((diskNode *)(usrkey->data))->nrdnlen);
-	char *c = (char *)&(((diskNode *)(curkey->data))->nrdnlen);
+	signed char *u = (signed char *)&(((diskNode *)(usrkey->data))->nrdnlen);
+	signed char *c = (signed char *)&(((diskNode *)(curkey->data))->nrdnlen);
 	int rc, i;
 
 	/* data is not aligned, cannot compare directly */
@@ -530,8 +434,7 @@ hdb_dup_compare(
  */
 int hdb_fix_dn(
 	Entry *e,
-	int checkit
-)
+	int checkit )
 {
 	EntryInfo *ei;
 	int rlen = 0, nrlen = 0;
@@ -556,6 +459,7 @@ int hdb_fix_dn(
 		}
 		/* checkit == 2. do the fix. */
 		free( e->e_name.bv_val );
+		free( e->e_nname.bv_val );
 	}
 
 	e->e_name.bv_len = rlen - 1;
@@ -619,11 +523,21 @@ hdb_dn2id_add(
 	key.size = sizeof(ID);
 	key.flags = DB_DBT_USERMEM;
 
-#ifdef SLAP_IDL_CACHE
+	/* Need to make dummy root node once. Subsequent attempts
+	 * will fail harmlessly.
+	 */
+	if ( eip->bei_id == 0 ) {
+		diskNode dummy = {0};
+		data.data = &dummy;
+		data.size = sizeof(diskNode);
+		data.flags = DB_DBT_USERMEM;
+
+		db->put( db, txn, &key, &data, DB_NODUPDATA );
+	}
+
 	if ( bdb->bi_idl_cache_size ) {
 		bdb_idl_cache_del( bdb, db, &key );
 	}
-#endif
 	data.data = d;
 	data.size = sizeof(diskNode) + rlen + nrlen;
 	data.flags = DB_DBT_USERMEM;
@@ -669,11 +583,9 @@ hdb_dn2id_delete(
 	data.dlen = data.size;
 	data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
 
-#ifdef SLAP_IDL_CACHE
 	if ( bdb->bi_idl_cache_size ) {
 		bdb_idl_cache_del( bdb, db, &key );
 	}
-#endif
 	rc = db->cursor( db, txn, &cursor, bdb->bi_db_opflags );
 	if ( rc ) return rc;
 
@@ -704,6 +616,7 @@ hdb_dn2id_delete(
 	return rc;
 }
 
+
 int
 hdb_dn2id(
 	Operation	*op,
@@ -744,15 +657,22 @@ hdb_dn2id(
 	data.data = d;
 
 	rc = cursor->c_get( cursor, &key, &data, DB_GET_BOTH );
-	cursor->c_close( cursor );
-
 	if ( rc == 0 ) {
-		AC_MEMCPY( &ei->bei_id, &d->entryID, sizeof(ID) );
+		ei->bei_id = d->entryID;
 		ei->bei_rdn.bv_len = data.size - sizeof(diskNode) - nrlen;
 		ptr = d->nrdn + nrlen + 1;
-		ei->bei_rdn.bv_val = ch_malloc( ei->bei_rdn.bv_len + 1 );
-		strcpy( ei->bei_rdn.bv_val, ptr );
+		ber_str2bv( ptr, ei->bei_rdn.bv_len, 1, &ei->bei_rdn );
+		if ( !ei->bei_parent->bei_dkids ) {
+			db_recno_t dkids;
+			/* How many children does the parent have? */
+			/* FIXME: do we need to lock the parent
+			 * entryinfo? Seems safe...
+			 */
+			cursor->c_count( cursor, &dkids, 0 );
+			ei->bei_parent->bei_dkids = dkids;
+		}
 	}
+	cursor->c_close( cursor );
 	op->o_tmpfree( d, op->o_tmpmemctx );
 
 	return rc;
@@ -791,19 +711,24 @@ hdb_dn2id_parent(
 	data.data = d;
 
 	rc = cursor->c_get( cursor, &key, &data, DB_SET );
-	cursor->c_close( cursor );
 	if ( rc == 0 ) {
 		if (d->nrdnlen >= 0) {
-			return LDAP_OTHER;
+			rc = LDAP_OTHER;
+		} else {
+			db_recno_t dkids;
+			*idp = d->entryID;
+			ei->bei_nrdn.bv_len = 0 - d->nrdnlen;
+			ber_str2bv( d->nrdn, ei->bei_nrdn.bv_len, 1, &ei->bei_nrdn );
+			ei->bei_rdn.bv_len = data.size - sizeof(diskNode) -
+				ei->bei_nrdn.bv_len;
+			ptr = d->nrdn + ei->bei_nrdn.bv_len + 1;
+			ber_str2bv( ptr, ei->bei_rdn.bv_len, 1, &ei->bei_rdn );
+			/* How many children does this node have? */
+			cursor->c_count( cursor, &dkids, 0 );
+			ei->bei_dkids = dkids;
 		}
-		AC_MEMCPY( idp, &d->entryID, sizeof(ID) );
-		ei->bei_nrdn.bv_len = 0 - d->nrdnlen;
-		ber_str2bv( d->nrdn, ei->bei_nrdn.bv_len, 1, &ei->bei_nrdn );
-		ei->bei_rdn.bv_len = data.size - sizeof(diskNode) -
-			ei->bei_nrdn.bv_len;
-		ptr = d->nrdn + ei->bei_nrdn.bv_len + 1;
-		ber_str2bv( ptr, ei->bei_rdn.bv_len, 1, &ei->bei_rdn );
 	}
+	cursor->c_close( cursor );
 	op->o_tmpfree( d, op->o_tmpmemctx );
 	return rc;
 }
@@ -827,14 +752,12 @@ hdb_dn2id_children(
 	key.data = &e->e_id;
 	key.flags = DB_DBT_USERMEM;
 
-#ifdef SLAP_IDL_CACHE
 	if ( bdb->bi_idl_cache_size ) {
 		rc = bdb_idl_cache_get( bdb, db, &key, NULL );
 		if ( rc != LDAP_NO_SUCH_OBJECT ) {
 			return rc;
 		}
 	}
-#endif
 	DBTzero(&data);
 	data.data = &d;
 	data.ulen = sizeof(d);
@@ -846,7 +769,12 @@ hdb_dn2id_children(
 
 	rc = cursor->c_get( cursor, &key, &data, DB_SET );
 	if ( rc == 0 ) {
-		rc = cursor->c_get( cursor, &key, &data, DB_NEXT_DUP );
+		db_recno_t dkids;
+		rc = cursor->c_count( cursor, &dkids, 0 );
+		if ( rc == 0 ) {
+			BEI(e)->bei_dkids = dkids;
+			if ( dkids < 2 ) rc = DB_NOTFOUND;
+		}
 	}
 	cursor->c_close( cursor );
 	return rc;
@@ -867,6 +795,7 @@ struct dn2id_cookie {
 	DB *db;
 	int prefix;
 	int rc;
+	EntryInfo *ei;
 	ID id;
 	ID dbuf;
 	ID *ids;
@@ -879,12 +808,23 @@ struct dn2id_cookie {
 	Operation *op;
 };
 
+static int
+apply_func(
+	void *data,
+	void *arg )
+{
+	EntryInfo *ei = data;
+	ID *idl = arg;
+
+	bdb_idl_insert( idl, ei->bei_id );
+	return 0;
+}
+
 static int
 hdb_dn2idl_internal(
 	struct dn2id_cookie *cx
 )
 {
-#ifdef SLAP_IDL_CACHE
 	if ( cx->bdb->bi_idl_cache_size ) {
 		cx->rc = bdb_idl_cache_get(cx->bdb, cx->db, &cx->key, cx->tmp);
 		if ( cx->rc == DB_NOTFOUND ) {
@@ -894,72 +834,135 @@ hdb_dn2idl_internal(
 			goto gotit;
 		}
 	}
-#endif
-
-	cx->rc = cx->db->cursor( cx->db, NULL, &cx->dbc,
-		cx->bdb->bi_db_opflags );
-	if ( cx->rc ) return cx->rc;
 	BDB_IDL_ZERO( cx->tmp );
 
-	cx->data.data = &cx->dbuf;
-	cx->data.ulen = sizeof(ID);
-	cx->data.dlen = sizeof(ID);
-	cx->data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
-
-	/* The first item holds the parent ID. Ignore it. */
-	cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data, DB_SET );
-	if ( cx->rc == DB_NOTFOUND ) goto saveit;
-	if ( cx->rc ) return cx->rc;
-
-	cx->data.data = cx->buf;
-	cx->data.ulen = BDB_IDL_UM_SIZE * sizeof(ID);
-	cx->data.flags = DB_DBT_USERMEM;
-
-	/* Fetch the rest of the IDs in a loop... */
-	while ( (cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data,
-		DB_MULTIPLE | DB_NEXT_DUP )) == 0 ) {
-		u_int8_t *j;
-		size_t len;
-		DB_MULTIPLE_INIT( cx->ptr, &cx->data );
-		while (cx->ptr) {
-			DB_MULTIPLE_NEXT( cx->ptr, &cx->data, j, len );
-			if (j) {
-				AC_MEMCPY( &cx->dbuf, j, sizeof(ID) );
-				bdb_idl_insert( cx->tmp, cx->dbuf );
-			}
+	if ( !cx->ei ) {
+		cx->ei = bdb_cache_find_info( cx->bdb, cx->id );
+		if ( !cx->ei ) {
+			cx->rc = DB_NOTFOUND;
+			goto saveit;
 		}
 	}
-	cx->dbc->c_close( cx->dbc );
 
-	/* If we got some records, treat as success */
-	if (!BDB_IDL_IS_ZERO(cx->tmp)) {
+	bdb_cache_entryinfo_lock( cx->ei );
+
+	/* If number of kids in the cache differs from on-disk, load
+	 * up all the kids from the database
+	 */
+	if ( cx->ei->bei_ckids+1 != cx->ei->bei_dkids ) {
+		EntryInfo ei;
+		db_recno_t dkids = cx->ei->bei_dkids;
+		ei.bei_parent = cx->ei;
+
+		bdb_cache_entryinfo_unlock( cx->ei );
+
+		cx->rc = cx->db->cursor( cx->db, NULL, &cx->dbc,
+			cx->bdb->bi_db_opflags );
+		if ( cx->rc ) return cx->rc;
+
+		cx->data.data = &cx->dbuf;
+		cx->data.ulen = sizeof(ID);
+		cx->data.dlen = sizeof(ID);
+		cx->data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
+
+		/* The first item holds the parent ID. Ignore it. */
+		cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data, DB_SET );
+		if ( cx->rc ) {
+			cx->dbc->c_close( cx->dbc );
+			if ( cx->rc == DB_NOTFOUND ) goto saveit;
+			return cx->rc;
+		}
+
+		/* If the on-disk count is zero we've never checked it.
+		 * Count it now.
+		 */
+		if ( !dkids ) {
+			cx->dbc->c_count( cx->dbc, &dkids, 0 );
+			cx->ei->bei_dkids = dkids;
+		}
+
+		cx->data.data = cx->buf;
+		cx->data.ulen = BDB_IDL_UM_SIZE * sizeof(ID);
+		cx->data.flags = DB_DBT_USERMEM;
+
+		/* Fetch the rest of the IDs in a loop... */
+		while ( (cx->rc = cx->dbc->c_get( cx->dbc, &cx->key, &cx->data,
+			DB_MULTIPLE | DB_NEXT_DUP )) == 0 ) {
+			u_int8_t *j;
+			size_t len;
+			DB_MULTIPLE_INIT( cx->ptr, &cx->data );
+			while (cx->ptr) {
+				DB_MULTIPLE_NEXT( cx->ptr, &cx->data, j, len );
+				if (j) {
+					EntryInfo *ei2;
+					diskNode *d = (diskNode *)j;
+					short nrlen;
+
+					AC_MEMCPY( &ei.bei_id, &d->entryID, sizeof(ID) );
+					AC_MEMCPY( &nrlen, &d->nrdnlen, sizeof(d->nrdnlen) );
+					ei.bei_nrdn.bv_len = nrlen;
+					/* nrdn/rdn are set in-place.
+					 * hdb_cache_load will copy them as needed
+					 */
+					ei.bei_nrdn.bv_val = d->nrdn;
+					ei.bei_rdn.bv_len = len - sizeof(diskNode)
+						- ei.bei_nrdn.bv_len;
+					ei.bei_rdn.bv_val = d->nrdn + ei.bei_nrdn.bv_len + 1;
+					bdb_idl_insert( cx->tmp, ei.bei_id );
+					hdb_cache_load( cx->bdb, &ei, &ei2 );
+				}
+			}
+		}
+		cx->rc = cx->dbc->c_close( cx->dbc );
+	} else {
+		/* The in-memory cache is in sync with the on-disk data.
+		 * do we have any kids?
+		 */
 		cx->rc = 0;
+		if ( cx->ei->bei_ckids > 0 ) {
+			/* Walk the kids tree; order is irrelevant since bdb_idl_insert
+			 * will insert in sorted order.
+			 */
+			avl_apply( cx->ei->bei_kids, apply_func,
+				cx->tmp, -1, AVL_POSTORDER );
+		}
+		bdb_cache_entryinfo_unlock( cx->ei );
 	}
 
 saveit:
-#ifdef SLAP_IDL_CACHE
 	if ( cx->bdb->bi_idl_cache_max_size ) {
 		bdb_idl_cache_put( cx->bdb, cx->db, &cx->key, cx->tmp, cx->rc );
 	}
-#endif
 	;
 gotit:
-	if ( cx->rc == 0 ) {
+	if ( !BDB_IDL_IS_ZERO( cx->tmp )) {
 		if ( cx->prefix == DN_SUBTREE_PREFIX ) {
-			ID *save, idcurs;
+			if (cx->ei->bei_state & CACHE_ENTRY_NO_GRANDKIDS) {
+				bdb_idl_union( cx->ids, cx->tmp );
+			} else {
+				ID *save, idcurs;
+				EntryInfo *ei = cx->ei;
+				int nokids = 1;
+				save = cx->op->o_tmpalloc( BDB_IDL_SIZEOF( cx->tmp ),
+					cx->op->o_tmpmemctx );
+				BDB_IDL_CPY( save, cx->tmp );
+				bdb_idl_union( cx->ids, cx->tmp );
 
-			save = cx->op->o_tmpalloc( BDB_IDL_SIZEOF( cx->tmp ),
-				cx->op->o_tmpmemctx );
-			BDB_IDL_CPY( save, cx->tmp );
-			bdb_idl_union( cx->ids, cx->tmp );
-	
-			idcurs = 0;
-			for ( cx->id = bdb_idl_first( save, &idcurs );
-				cx->id != NOID;
-				cx->id = bdb_idl_next( save, &idcurs )) {
-				hdb_dn2idl_internal( cx );
+				idcurs = 0;
+				for ( cx->id = bdb_idl_first( save, &idcurs );
+					cx->id != NOID;
+					cx->id = bdb_idl_next( save, &idcurs )) {
+					cx->ei = NULL;
+					hdb_dn2idl_internal( cx );
+					if ( !BDB_IDL_IS_ZERO( cx->tmp ))
+						nokids = 0;
+				}
+				cx->op->o_tmpfree( save, cx->op->o_tmpmemctx );
+				if ( nokids ) ei->bei_state |= CACHE_ENTRY_NO_GRANDKIDS;
 			}
-			cx->op->o_tmpfree( save, cx->op->o_tmpmemctx );
+			/* Make sure caller knows it had kids! */
+			cx->tmp[0]=1;
+
 			cx->rc = 0;
 		} else {
 			BDB_IDL_CPY( cx->ids, cx->tmp );
@@ -978,26 +981,24 @@ hdb_dn2idl(
 	struct bdb_info *bdb = (struct bdb_info *)op->o_bd->be_private;
 	struct dn2id_cookie cx;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( INDEX, ARGS, 
-		"=> hdb_dn2ididl( \"%s\" )\n", e->e_nname.bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> hdb_dn2idl( \"%s\" )\n", e->e_nname.bv_val, 0, 0 );
-#endif
+	Debug( LDAP_DEBUG_TRACE, "=> hdb_dn2idl(\"%s\")\n",
+		e->e_nname.bv_val, 0, 0 );
 
 #ifndef BDB_MULTIPLE_SUFFIXES
-	if ( op->ors_scope == LDAP_SCOPE_SUBTREE && 
-		BEI(e)->bei_parent->bei_id == 0 ) {
+	if ( op->ors_scope != LDAP_SCOPE_ONELEVEL && 
+		BEI(e)->bei_parent->bei_id == 0 )
+	{
 		BDB_IDL_ALL( bdb, ids );
 		return 0;
 	}
 #endif
 
 	cx.id = e->e_id;
+	cx.ei = e->e_id ? BEI(e) : &bdb->bi_cache.c_dntree;
 	cx.bdb = bdb;
 	cx.db = cx.bdb->bi_dn2id->bdi_db;
-	cx.prefix = op->ors_scope == LDAP_SCOPE_SUBTREE ? DN_SUBTREE_PREFIX :
-			DN_ONE_PREFIX;
+	cx.prefix = op->ors_scope == LDAP_SCOPE_ONELEVEL
+		? DN_ONE_PREFIX : DN_SUBTREE_PREFIX;
 	cx.ids = ids;
 	cx.buf = stack;
 	cx.op = op;
