@@ -1,8 +1,17 @@
 /* init.c - initialize bdb backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.132 2003/05/11 05:03:48 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/init.c,v 1.177.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -13,8 +22,8 @@
 #include <ac/stdlib.h>
 
 #include "back-bdb.h"
-#include "external.h"
 #include <lutil.h>
+#include <ldap_rq.h>
 
 static const struct bdbi_database {
 	char *file;
@@ -27,7 +36,7 @@ static const struct bdbi_database {
 	{ NULL, NULL, 0, 0 }
 };
 
-struct berval bdb_uuid = { 0, NULL };
+struct berval bdb_uuid = BER_BVNULL;
 
 typedef void * db_malloc(size_t);
 typedef void * db_realloc(void *, size_t);
@@ -58,23 +67,9 @@ bdb_db_init( BackendDB *be )
 {
 	struct bdb_info	*bdb;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_BDB, ENTRY, "bdb_db_init", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY,
-		"bdb_db_init: Initializing BDB database\n",
-		0, 0, 0 );
-#endif
-
-	/* indicate system schema supported */
-	be->be_flags |=
-#ifdef BDB_SUBENTRIES
-		SLAP_BFLAG_SUBENTRIES |
-#endif
-#ifdef BDB_ALIASES
-		SLAP_BFLAG_ALIASES |
-#endif
-		SLAP_BFLAG_REFERRALS;
+		LDAP_XSTRING(bdb_db_init) ": Initializing "
+		BDB_UCTYPE " database\n", 0, 0, 0 );
 
 	/* allocate backend-database-specific stuff */
 	bdb = (struct bdb_info *) ch_calloc( 1, sizeof(struct bdb_info) );
@@ -90,10 +85,12 @@ bdb_db_init( BackendDB *be )
 	bdb->bi_search_stack_depth = DEFAULT_SEARCH_STACK_DEPTH;
 	bdb->bi_search_stack = NULL;
 
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+#ifdef BDB_PSEARCH
 	LDAP_LIST_INIT (&bdb->bi_psearch_list);
+	ldap_pvt_thread_rdwr_init ( &bdb->bi_pslist_rwlock );
 #endif
 
+	ldap_pvt_thread_mutex_init( &bdb->bi_database_mutex );
 	ldap_pvt_thread_mutex_init( &bdb->bi_lastid_mutex );
 	ldap_pvt_thread_mutex_init( &bdb->bi_cache.lru_mutex );
 	ldap_pvt_thread_mutex_init( &bdb->bi_cache.c_dntree.bei_kids_mutex );
@@ -108,8 +105,7 @@ int
 bdb_bt_compare(
 	DB *db, 
 	const DBT *usrkey,
-	const DBT *curkey
-)
+	const DBT *curkey )
 {
 	unsigned char *u, *c;
 	int i, x;
@@ -130,6 +126,17 @@ bdb_bt_compare(
 	return 0;
 }
 
+static void *
+bdb_checkpoint( void *ctx, void *arg )
+{
+	struct re_s *rtask = arg;
+	struct bdb_info *bdb = rtask->arg;
+	
+	TXN_CHECKPOINT( bdb->bi_dbenv, bdb->bi_txn_cp_kbyte,
+		bdb->bi_txn_cp_min, 0 );
+	return NULL;
+}
+
 static int
 bdb_db_open( BackendDB *be )
 {
@@ -140,24 +147,14 @@ bdb_db_open( BackendDB *be )
 	char path[MAXPATHLEN];
 #endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_BDB, ARGS, 
-		"bdb_db_open: %s\n", be->be_suffix[0].bv_val, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ARGS,
 		"bdb_db_open: %s\n",
 		be->be_suffix[0].bv_val, 0, 0 );
-#endif
 
 #ifndef BDB_MULTIPLE_SUFFIXES
 	if ( be->be_suffix[1].bv_val ) {
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_BDB, ERR, 
-		"bdb_db_open: only one suffix allowed\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY,
 		"bdb_db_open: only one suffix allowed\n", 0, 0, 0 );
-#endif
 		return -1;
 	}
 #endif
@@ -165,15 +162,9 @@ bdb_db_open( BackendDB *be )
 
 	rc = db_env_create( &bdb->bi_dbenv, 0 );
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_BDB, ERR, 
-			"bdb_db_open: db_env_create failed: %s (%d)\n", 
-			db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"bdb_db_open: db_env_create failed: %s (%d)\n",
 			db_strerror(rc), rc, 0 );
-#endif
 		return rc;
 	}
 
@@ -198,13 +189,25 @@ bdb_db_open( BackendDB *be )
 	bdb->bi_dbenv->set_errcall( bdb->bi_dbenv, bdb_errcall );
 	bdb->bi_dbenv->set_lk_detect( bdb->bi_dbenv, bdb->bi_lock_detect );
 
-#ifdef SLAP_IDL_CACHE
+	/* One long-lived TXN per thread, two TXNs per write op */
+	bdb->bi_dbenv->set_tx_max( bdb->bi_dbenv, connection_pool_max * 3 );
+
+#ifdef SLAP_ZONE_ALLOC
+	if ( bdb->bi_cache.c_maxsize ) {
+		bdb->bi_cache.c_zctx = slap_zn_mem_create(
+								SLAP_ZONE_INITSIZE,
+								SLAP_ZONE_MAXSIZE,
+								SLAP_ZONE_DELTA,
+								SLAP_ZONE_SIZE);
+	}
+#endif
+
 	if ( bdb->bi_idl_cache_max_size ) {
+		bdb->bi_idl_tree = NULL;
 		ldap_pvt_thread_rdwr_init( &bdb->bi_idl_tree_rwlock );
 		ldap_pvt_thread_mutex_init( &bdb->bi_idl_tree_lrulock );
 		bdb->bi_idl_cache_size = 0;
 	}
-#endif
 
 #ifdef BDB_SUBDIRS
 	{
@@ -231,15 +234,9 @@ bdb_db_open( BackendDB *be )
 #endif
 		rc = bdb->bi_dbenv->set_tmp_dir( bdb->bi_dbenv, dir );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: set_tmp_dir(%s) failed: %s (%d)\n", 
-				dir, db_strerror(rc), rc );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_open: set_tmp_dir(%s) failed: %s (%d)\n",
 				dir, db_strerror(rc), rc );
-#endif
 			return rc;
 		}
 
@@ -249,15 +246,9 @@ bdb_db_open( BackendDB *be )
 #endif
 		rc = bdb->bi_dbenv->set_lg_dir( bdb->bi_dbenv, dir );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: set_lg_dir(%s) failed: %s (%d)\n", 
-				dir, db_strerror(rc), rc );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_open: set_lg_dir(%s) failed: %s (%d)\n",
 				dir, db_strerror(rc), rc );
-#endif
 			return rc;
 		}
 
@@ -267,28 +258,28 @@ bdb_db_open( BackendDB *be )
 #endif
 		rc = bdb->bi_dbenv->set_data_dir( bdb->bi_dbenv, dir );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: set_data_dir(%s) failed: %s (%d)\n",
-				dir, db_strerror(rc), rc );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_open: set_data_dir(%s) failed: %s (%d)\n",
 				dir, db_strerror(rc), rc );
-#endif
 			return rc;
 		}
 	}
 #endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_BDB, DETAIL1, 
-		"bdb_db_open: dbenv_open %s\n", bdb->bi_dbenv_home, 0, 0 );
-#else
+	if( bdb->bi_dbenv_xflags != 0 ) {
+		rc = bdb->bi_dbenv->set_flags( bdb->bi_dbenv,
+			bdb->bi_dbenv_xflags, 1);
+		if( rc != 0 ) {
+			Debug( LDAP_DEBUG_ANY,
+				"bdb_db_open: dbenv_set_flags failed: %s (%d)\n",
+				db_strerror(rc), rc, 0 );
+			return rc;
+		}
+	}
+
 	Debug( LDAP_DEBUG_TRACE,
 		"bdb_db_open: dbenv_open(%s)\n",
 		bdb->bi_dbenv_home, 0, 0);
-#endif
 
 #ifdef HAVE_EBCDIC
 	strcpy( path, bdb->bi_dbenv_home );
@@ -304,35 +295,12 @@ bdb_db_open( BackendDB *be )
 		bdb->bi_dbenv_mode );
 #endif
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_BDB, ERR, 
-			"bdb_db_open: dbenv_open failed: %s (%d)\n", 
-			db_strerror(rc), rc, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"bdb_db_open: dbenv_open failed: %s (%d)\n",
 			db_strerror(rc), rc, 0 );
-#endif
 		return rc;
 	}
 
-	if( bdb->bi_dbenv_xflags != 0 ) {
-		rc = bdb->bi_dbenv->set_flags( bdb->bi_dbenv,
-			bdb->bi_dbenv_xflags, 1);
-		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: dbenv_set_flags failed: %s (%d)\n", 
-				db_strerror(rc), rc, 0 );
-#else
-			Debug( LDAP_DEBUG_ANY,
-				"bdb_db_open: dbenv_set_flags failed: %s (%d)\n",
-				db_strerror(rc), rc, 0 );
-#endif
-			return rc;
-		}
-	}
-
 	flags = DB_THREAD | bdb->bi_db_opflags;
 
 	bdb->bi_databases = (struct bdb_db_info **) ch_malloc(
@@ -346,21 +314,17 @@ bdb_db_open( BackendDB *be )
 
 		rc = db_create( &db->bdi_db, bdb->bi_dbenv, 0 );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: db_create(%s) failed: %s (%d)\n", 
-				bdb->bi_dbenv_home, db_strerror(rc), rc );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_open: db_create(%s) failed: %s (%d)\n",
 				bdb->bi_dbenv_home, db_strerror(rc), rc );
-#endif
 			return rc;
 		}
 
 		if( i == BDB_ID2ENTRY ) {
+#if 0
 			rc = db->bdi_db->set_bt_compare( db->bdi_db,
 				bdb_bt_compare );
+#endif
 			rc = db->bdi_db->set_pagesize( db->bdi_db,
 				BDB_ID2ENTRY_PAGESIZE );
 			if ( slapMode & SLAP_TOOL_READMAIN ) {
@@ -372,8 +336,10 @@ bdb_db_open( BackendDB *be )
 			rc = db->bdi_db->set_flags( db->bdi_db, 
 				DB_DUP | DB_DUPSORT );
 #ifndef BDB_HIER
+#if 0
 			rc = db->bdi_db->set_dup_compare( db->bdi_db,
 				bdb_bt_compare );
+#endif
 			if ( slapMode & SLAP_TOOL_READONLY ) {
 				flags |= DB_RDONLY;
 			} else {
@@ -397,31 +363,25 @@ bdb_db_open( BackendDB *be )
 #ifdef HAVE_EBCDIC
 		strcpy( path, bdbi_databases[i].file );
 		__atoe( path );
-		rc = DB_OPEN( db->bdi_db, NULL,
+		rc = DB_OPEN( db->bdi_db,
 			path,
 		/*	bdbi_databases[i].name, */ NULL,
 			bdbi_databases[i].type,
-			bdbi_databases[i].flags | flags | DB_AUTO_COMMIT,
+			bdbi_databases[i].flags | flags,
 			bdb->bi_dbenv_mode );
 #else
-		rc = DB_OPEN( db->bdi_db, NULL,
+		rc = DB_OPEN( db->bdi_db,
 			bdbi_databases[i].file,
 		/*	bdbi_databases[i].name, */ NULL,
 			bdbi_databases[i].type,
-			bdbi_databases[i].flags | flags | DB_AUTO_COMMIT,
+			bdbi_databases[i].flags | flags,
 			bdb->bi_dbenv_mode );
 #endif
 
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: db_create(%s) failed: %s (%d)\n", 
-				bdb->bi_dbenv_home, db_strerror(rc), rc );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_open: db_open(%s) failed: %s (%d)\n",
 				bdb->bi_dbenv_home, db_strerror(rc), rc );
-#endif
 			return rc;
 		}
 
@@ -436,18 +396,25 @@ bdb_db_open( BackendDB *be )
 	/* get nextid */
 	rc = bdb_last_id( be, NULL );
 	if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_open: last_id(%s) failed: %s (%d)\n", 
-				bdb->bi_dbenv_home, db_strerror(rc), rc );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"bdb_db_open: last_id(%s) failed: %s (%d)\n",
 			bdb->bi_dbenv_home, db_strerror(rc), rc );
-#endif
 		return rc;
 	}
 
+	XLOCK_ID(bdb->bi_dbenv, &bdb->bi_cache.c_locker);
+
+	/* If we're in server mode and time-based checkpointing is enabled,
+	 * submit a task to perform periodic checkpoints.
+	 */
+	if ( slapMode & SLAP_SERVER_MODE && bdb->bi_txn_cp &&
+		bdb->bi_txn_cp_min )  {
+		ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
+		ldap_pvt_runqueue_insert( &slapd_rq, bdb->bi_txn_cp_min*60,
+			bdb_checkpoint, bdb );
+		ldap_pvt_thread_mutex_unlock( &slapd_rq.rq_mutex );
+	}
+
 	/* <insert> open (and create) index databases */
 	return 0;
 }
@@ -458,9 +425,7 @@ bdb_db_close( BackendDB *be )
 	int rc;
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	struct bdb_db_info *db;
-#ifdef SLAP_IDL_CACHE
 	bdb_idl_cache_entry_t *entry, *next_entry;
-#endif
 
 	while( bdb->bi_ndatabases-- ) {
 		db = bdb->bi_databases[bdb->bi_ndatabases];
@@ -475,7 +440,6 @@ bdb_db_close( BackendDB *be )
 
 	bdb_cache_release_all (&bdb->bi_cache);
 
-#ifdef SLAP_IDL_CACHE
 	if ( bdb->bi_idl_cache_max_size ) {
 		ldap_pvt_thread_rdwr_wlock ( &bdb->bi_idl_tree_rwlock );
 		avl_free( bdb->bi_idl_tree, NULL );
@@ -490,7 +454,8 @@ bdb_db_close( BackendDB *be )
 		}
 		ldap_pvt_thread_rdwr_wunlock ( &bdb->bi_idl_tree_rwlock );
 	}
-#endif
+
+	XLOCK_ID_FREE(bdb->bi_dbenv, bdb->bi_cache.c_locker);
 
 	return 0;
 }
@@ -500,35 +465,26 @@ bdb_db_destroy( BackendDB *be )
 {
 	int rc;
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
+	Operation *ps = NULL;
+	Operation *psn = NULL;
+	void *saved_tmpmemctx = NULL;
 
 	/* close db environment */
 	if( bdb->bi_dbenv ) {
 		/* force a checkpoint */
 		rc = TXN_CHECKPOINT( bdb->bi_dbenv, 0, 0, DB_FORCE );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_destroy: txn_checkpoint failed: %s (%d)\n",
-				db_strerror(rc), rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_destroy: txn_checkpoint failed: %s (%d)\n",
 				db_strerror(rc), rc, 0 );
-#endif
 		}
 
 		rc = bdb->bi_dbenv->close( bdb->bi_dbenv, 0 );
 		bdb->bi_dbenv = NULL;
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_destroy: close failed: %s (%d)\n", 
-				db_strerror(rc), rc, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"bdb_db_destroy: close failed: %s (%d)\n",
 				db_strerror(rc), rc, 0 );
-#endif
 			return rc;
 		}
 	}
@@ -539,11 +495,72 @@ bdb_db_destroy( BackendDB *be )
 	ldap_pvt_thread_mutex_destroy( &bdb->bi_cache.lru_mutex );
 	ldap_pvt_thread_mutex_destroy( &bdb->bi_cache.c_dntree.bei_kids_mutex );
 	ldap_pvt_thread_mutex_destroy( &bdb->bi_lastid_mutex );
-#ifdef SLAP_IDL_CACHE
+	ldap_pvt_thread_mutex_destroy( &bdb->bi_database_mutex );
 	if ( bdb->bi_idl_cache_max_size ) {
 		ldap_pvt_thread_rdwr_destroy( &bdb->bi_idl_tree_rwlock );
 		ldap_pvt_thread_mutex_destroy( &bdb->bi_idl_tree_lrulock );
 	}
+
+#ifdef BDB_PSEARCH
+	ldap_pvt_thread_rdwr_destroy ( &bdb->bi_pslist_rwlock );
+	ps = LDAP_LIST_FIRST( &bdb->bi_psearch_list );
+
+	if ( ps ) {
+		psn = LDAP_LIST_NEXT( ps, o_ps_link );
+
+		saved_tmpmemctx = ps->o_tmpmemctx;
+
+		if (!BER_BVISNULL(&ps->o_req_dn)) {
+			slap_sl_free( ps->o_req_dn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->o_req_ndn)) {
+			slap_sl_free( ps->o_req_ndn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->ors_filterstr)) {
+			slap_sl_free(ps->ors_filterstr.bv_val, ps->o_tmpmemctx);
+		}
+		if (ps->ors_filter != NULL) {
+			filter_free_x(ps, ps->ors_filter);
+		}
+		if ( ps->ors_attrs != NULL) {
+			ps->o_tmpfree(ps->ors_attrs, ps->o_tmpmemctx);
+		}
+
+		slap_op_free( ps );
+
+		if ( saved_tmpmemctx ) {
+			slap_sl_mem_destroy( NULL, saved_tmpmemctx );
+		}
+	}
+
+	while ( psn ) {
+		ps = psn;
+		psn = LDAP_LIST_NEXT( ps, o_ps_link );
+
+		saved_tmpmemctx = ps->o_tmpmemctx;
+
+		if (!BER_BVISNULL(&ps->o_req_dn)) {
+			slap_sl_free( ps->o_req_dn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->o_req_ndn)) {
+			slap_sl_free( ps->o_req_ndn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->ors_filterstr)) {
+			slap_sl_free(ps->ors_filterstr.bv_val, ps->o_tmpmemctx);
+		}
+		if (ps->ors_filter != NULL) {
+			filter_free_x(ps, ps->ors_filter);
+		}
+		if ( ps->ors_attrs != NULL) {
+			ps->o_tmpfree(ps->ors_attrs, ps->o_tmpmemctx);
+		}
+
+		slap_op_free( ps );
+
+		if ( saved_tmpmemctx ) {
+			slap_sl_mem_destroy( NULL, saved_tmpmemctx );
+		}
+	}
 #endif
 
 	ch_free( bdb );
@@ -552,52 +569,42 @@ bdb_db_destroy( BackendDB *be )
 	return 0;
 }
 
-#ifdef SLAPD_BDB_DYNAMIC
-int back_bdb_LTX_init_module( int argc, char *argv[] ) {
-	BackendInfo bi;
-
-	memset( &bi, '\0', sizeof(bi) );
-	bi.bi_type = "bdb";
-	bi.bi_init = bdb_initialize;
-
-	backend_add( &bi );
-	return 0;
-}
-#endif /* SLAPD_BDB_DYNAMIC */
-
 int
-bdb_initialize(
-	BackendInfo	*bi
-)
+bdb_back_initialize(
+	BackendInfo	*bi )
 {
 	static char *controls[] = {
+		LDAP_CONTROL_ASSERT,
 		LDAP_CONTROL_MANAGEDSAIT,
 		LDAP_CONTROL_NOOP,
-#ifdef LDAP_CONTROL_PAGEDRESULTS
 		LDAP_CONTROL_PAGEDRESULTS,
-#endif
- 		LDAP_CONTROL_VALUESRETURNFILTER,
 #ifdef LDAP_CONTROL_SUBENTRIES
 		LDAP_CONTROL_SUBENTRIES,
 #endif
-#ifdef LDAP_CLIENT_UPDATE
-		LDAP_CONTROL_CLIENT_UPDATE,
+ 		LDAP_CONTROL_VALUESRETURNFILTER,
+#ifdef LDAP_CONTROL_X_PERMISSIVE_MODIFY
+		LDAP_CONTROL_X_PERMISSIVE_MODIFY,
 #endif
 		NULL
 	};
 
+	/* initialize the underlying database system */
+	Debug( LDAP_DEBUG_TRACE,
+		LDAP_XSTRING(bdb_back_initialize) ": initialize " 
+		BDB_UCTYPE " backend\n", 0, 0, 0 );
+
+	bi->bi_flags |=
+		SLAP_BFLAG_INCREMENT |
+#ifdef BDB_SUBENTRIES
+		SLAP_BFLAG_SUBENTRIES |
+#endif
+		SLAP_BFLAG_ALIASES |
+		SLAP_BFLAG_REFERRALS;
+
 	bi->bi_controls = controls;
 
-	/* initialize the underlying database system */
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_BDB, ENTRY, "bdb_db_initialize\n", 0, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "bdb_initialize: initialize BDB backend\n",
-		0, 0, 0 );
-#endif
-
 	{	/* version check */
-		int major, minor, patch;
+		int major, minor, patch, ver;
 		char *version = db_version( &major, &minor, &patch );
 #ifdef HAVE_EBCDIC
 		char v2[1024];
@@ -611,29 +618,18 @@ bdb_initialize(
 		version = v2;
 #endif
 
-		if( major != DB_VERSION_MAJOR ||
-			minor != DB_VERSION_MINOR ||
-			patch < DB_VERSION_PATCH )
+		ver = (major << 24) | (minor << 16) | patch;
+		if( ver < DB_VERSION_FULL )
 		{
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_BDB, ERR, 
-				"bdb_db_initialize: version mismatch: "
-				"\texpected: %s \tgot: %s\n", DB_VERSION_STRING, version, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
-				"bdb_initialize: version mismatch\n"
-				"\texpected: " DB_VERSION_STRING "\n"
-				"\tgot: %s \n", version, 0, 0 );
-#endif
+				LDAP_XSTRING(bdb_back_initialize) ": "
+				"BDB library version mismatch:"
+				" expected " DB_VERSION_STRING ","
+				" got %s\n", version, 0, 0 );
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_BDB, DETAIL1, 
-			"bdb_db_initialize: %s\n", version, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "bdb_initialize: %s\n",
-			version, 0, 0 );
-#endif
+		Debug( LDAP_DEBUG_ANY, LDAP_XSTRING(bdb_back_initialize)
+			": %s\n", version, 0, 0 );
 	}
 
 	db_env_set_func_free( ber_memfree );
@@ -674,12 +670,9 @@ bdb_initialize(
 
 	bi->bi_op_unbind = 0;
 
-#ifdef LDAP_CLIENT_UPDATE
+#if 0	/* DELETE ME */
 	bi->bi_op_abandon = bdb_abandon;
 	bi->bi_op_cancel = bdb_cancel;
-#else
-	bi->bi_op_abandon = 0;
-	bi->bi_op_cancel = 0;
 #endif
 
 	bi->bi_extended = bdb_extended;
@@ -701,9 +694,25 @@ bdb_initialize(
 	bi->bi_tool_entry_put = bdb_tool_entry_put;
 	bi->bi_tool_entry_reindex = bdb_tool_entry_reindex;
 	bi->bi_tool_sync = 0;
+	bi->bi_tool_dn2id_get = bdb_tool_dn2id_get;
+	bi->bi_tool_id2entry_get = bdb_tool_id2entry_get;
+	bi->bi_tool_entry_modify = bdb_tool_entry_modify;
 
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = 0;
 
 	return 0;
 }
+
+#if	(SLAPD_BDB == SLAPD_MOD_DYNAMIC && !defined(BDB_HIER)) || \
+	(SLAPD_HDB == SLAPD_MOD_DYNAMIC && defined(BDB_HIER))
+
+/* conditionally define the init_module() function */
+#ifdef BDB_HIER
+SLAP_BACKEND_INIT_MODULE( hdb )
+#else /* !BDB_HIER */
+SLAP_BACKEND_INIT_MODULE( bdb )
+#endif /* !BDB_HIER */
+
+#endif /* SLAPD_[BH]DB == SLAPD_MOD_DYNAMIC */
+
