@@ -1,8 +1,17 @@
 /* modify.c - bdb backend modify routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modify.c,v 1.76 2003/05/24 22:53:08 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modify.c,v 1.124 2004/12/05 01:00:19 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -12,10 +21,11 @@
 #include <ac/time.h>
 
 #include "back-bdb.h"
-#include "external.h"
 
-#define INDEXED	0x2000
-#define NULLIFIED	0x4000
+static struct berval scbva[] = {
+	BER_BVC("glue"),
+	BER_BVNULL
+};
 
 int bdb_modify_internal(
 	Operation *op,
@@ -31,14 +41,10 @@ int bdb_modify_internal(
 	Modifications	*ml;
 	Attribute	*save_attrs;
 	Attribute 	*ap;
+	int			glue_attr_delete = 0;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "bdb_modify_internal: 0x%08lx: %s\n", 
-		e->e_id, e->e_dn, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "bdb_modify_internal: 0x%08lx: %s\n",
 		e->e_id, e->e_dn, 0);
-#endif
 
 	if ( !acl_check_modlist( op, e, modlist )) {
 		return LDAP_INSUFFICIENT_ACCESS;
@@ -48,77 +54,92 @@ int bdb_modify_internal(
 	save_attrs = e->e_attrs;
 	e->e_attrs = attrs_dup( e->e_attrs );
 
+	for ( ml = modlist; ml != NULL; ml = ml->sml_next ) {
+		int match;
+		mod = &ml->sml_mod;
+		switch( mod->sm_op ) {
+		case LDAP_MOD_ADD:
+		case LDAP_MOD_REPLACE:
+			if ( mod->sm_desc == slap_schema.si_ad_structuralObjectClass ) {
+				value_match( &match, slap_schema.si_ad_structuralObjectClass,
+				slap_schema.si_ad_structuralObjectClass->ad_type->sat_equality,
+				SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+				&mod->sm_values[0], &scbva[0], text );
+				if ( !match )
+					glue_attr_delete = 1;
+			}
+		}
+		if ( glue_attr_delete )
+			break;
+	}
+
+	if ( glue_attr_delete ) {
+		Attribute	**app = &e->e_attrs;
+		while ( *app != NULL ) {
+			if ( !is_at_operational( (*app)->a_desc->ad_type )) {
+				Attribute *save = *app;
+				*app = (*app)->a_next;
+				attr_free( save );
+				continue;
+			}
+			app = &(*app)->a_next;
+		}
+	}
+
 	for ( ml = modlist; ml != NULL; ml = ml->sml_next ) {
 		mod = &ml->sml_mod;
 
 		switch ( mod->sm_op ) {
 		case LDAP_MOD_ADD:
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, "bdb_modify_internal: add\n", 0,0,0);
-#else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: add\n", 0, 0, 0);
-#endif
 			err = modify_add_values( e, mod, get_permissiveModify(op),
 				text, textbuf, textlen );
 			if( err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: %d %s\n", err, *text, 0 );
-#else
 				Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: %d %s\n",
 					err, *text, 0);
-#endif
 			}
 			break;
 
 		case LDAP_MOD_DELETE:
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"bdb_modify_internal: delete\n", 0, 0, 0 );
-#else
+			if ( glue_attr_delete ) {
+				err = LDAP_SUCCESS;
+				break;
+			}
+
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: delete\n", 0, 0, 0);
-#endif
 			err = modify_delete_values( e, mod, get_permissiveModify(op),
 				text, textbuf, textlen );
 			assert( err != LDAP_TYPE_OR_VALUE_EXISTS );
 			if( err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: %d %s\n", err, *text, 0 );
-#else
 				Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: %d %s\n",
 					err, *text, 0);
-#endif
 			}
 			break;
 
 		case LDAP_MOD_REPLACE:
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"bdb_modify_internal: replace\n", 0, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: replace\n", 0, 0, 0);
-#endif
 			err = modify_replace_values( e, mod, get_permissiveModify(op),
 				text, textbuf, textlen );
 			if( err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: %d %s\n", err, *text, 0 );
-#else
 				Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: %d %s\n",
 					err, *text, 0);
-#endif
+			}
+			break;
+
+		case LDAP_MOD_INCREMENT:
+			Debug(LDAP_DEBUG_ARGS,
+				"bdb_modify_internal: increment\n", 0, 0, 0);
+			err = modify_increment_values( e, mod, get_permissiveModify(op),
+				text, textbuf, textlen );
+			if( err != LDAP_SUCCESS ) {
+				Debug(LDAP_DEBUG_ARGS,
+					"bdb_modify_internal: %d %s\n",
+					err, *text, 0);
 			}
 			break;
 
 		case SLAP_MOD_SOFTADD:
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, DETAIL1, 
-				"bdb_modify_internal: softadd\n",0,0,0 );
-#else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: softadd\n", 0, 0, 0);
-#endif
  			/* Avoid problems in index_add_mods()
  			 * We need to add index if necessary.
  			 */
@@ -134,33 +155,18 @@ int bdb_modify_internal(
  			}
 
 			if( err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: %d %s\n", err, *text, 0 );
-#else
 				Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: %d %s\n",
 					err, *text, 0);
-#endif
 			}
  			break;
 
 		default:
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: invalid op %d\n", mod->sm_op, 0, 0 );
-#else
 			Debug(LDAP_DEBUG_ANY, "bdb_modify_internal: invalid op %d\n",
 				mod->sm_op, 0, 0);
-#endif
 			*text = "Invalid modify operation";
 			err = LDAP_OTHER;
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: %d %s\n", err, *text, 0 );
-#else
 			Debug(LDAP_DEBUG_ARGS, "bdb_modify_internal: %d %s\n",
 				err, *text, 0);
-#endif
 		}
 
 		if ( err != LDAP_SUCCESS ) {
@@ -174,24 +180,37 @@ int bdb_modify_internal(
 		if ( mod->sm_desc == slap_schema.si_ad_objectClass ) {
 			e->e_ocflags = 0;
 		}
+
+		if ( glue_attr_delete ) {
+			e->e_ocflags = 0;
+		}
+
+		/* check if modified attribute was indexed
+		 * but not in case of NOOP... */
+		err = bdb_index_is_indexed( op->o_bd, mod->sm_desc );
+		if ( err == LDAP_SUCCESS && !op->o_noop ) {
+			ap = attr_find( save_attrs, mod->sm_desc );
+			if ( ap ) ap->a_flags |= SLAP_ATTR_IXDEL;
+
+			ap = attr_find( e->e_attrs, mod->sm_desc );
+			if ( ap ) ap->a_flags |= SLAP_ATTR_IXADD;
+		}
 	}
 
 	/* check that the entry still obeys the schema */
 	rc = entry_schema_check( op->o_bd, e, save_attrs, text, textbuf, textlen );
 	if ( rc != LDAP_SUCCESS || op->o_noop ) {
 		attrs_free( e->e_attrs );
+		/* clear the indexing flags */
+		for ( ap = save_attrs; ap != NULL; ap = ap->a_next ) {
+			ap->a_flags = 0;
+		}
 		e->e_attrs = save_attrs;
 
 		if ( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, "bdb_modify_internal: "
-				"entry failed schema check %s\n", 
-				*text, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"entry failed schema check: %s\n",
 				*text, 0, 0 );
-#endif
 		}
 
 		/* if NOOP then silently revert to saved attrs */
@@ -199,84 +218,40 @@ int bdb_modify_internal(
 	}
 
 	/* update the indices of the modified attributes */
-	if ( !op->o_noop ) {
-		Modifications *m2;
 
-		/* First look for any deletes that would nullify any adds
-		 * in this request. I.e., deleting an entire attribute after
-		 * assigning some values to it.
-		 */
-		for ( ml = modlist; ml != NULL; ml = ml->sml_next ) {
-			if (bdb_index_is_indexed( op->o_bd, ml->sml_desc ))
-				continue;
-			switch ( ml->sml_op ) {
-			case LDAP_MOD_DELETE:
-				/* If just deleting specific values, ignore */
-				if ( ml->sml_bvalues ) break;
-			case LDAP_MOD_REPLACE:
-				for ( m2 = modlist; m2 != ml; m2 = m2->sml_next ) {
-					if ( m2->sml_desc == ml->sml_desc &&
-						m2->sml_op != LDAP_MOD_DELETE )
-						m2->sml_op |= NULLIFIED;
-				}
-				break;
-			}
-			ml->sml_op |= INDEXED;
-		}
-		/* Now index the modifications */
-		for ( ml = modlist; ml != NULL; ml = ml->sml_next ) {
-			if ( ! (ml->sml_op & INDEXED) ) continue;
-			ml->sml_op ^= INDEXED;
-			switch ( ml->sml_op ) {
-			case LDAP_MOD_DELETE:
-				if ( ml->sml_bvalues ) {
-					rc = bdb_index_values( op, tid, ml->sml_desc,
-						ml->sml_nvalues ? ml->sml_nvalues : ml->sml_bvalues,
-						e->e_id, SLAP_INDEX_DELETE_OP );
-					break;
-				}
-				/* FALLTHRU */
-			case LDAP_MOD_REPLACE:
-			/* A nullified replace still does its delete action */
-			case LDAP_MOD_REPLACE | NULLIFIED:
-				ap = attr_find( save_attrs, ml->sml_desc );
-				if ( ap != NULL ) {
-					rc = bdb_index_values( op, tid, ap->a_desc,
-						ap->a_nvals,
-						e->e_id, SLAP_INDEX_DELETE_OP );
-				} else {
-					rc = LDAP_SUCCESS;
-				}
-				if ( rc || ml->sml_op == LDAP_MOD_DELETE ||
-					(ml->sml_op & NULLIFIED))
-					break;
-				/* FALLTHRU */
-			case LDAP_MOD_ADD:
-			case SLAP_MOD_SOFTADD:
-				rc = bdb_index_values( op, tid, ml->sml_desc,
-					ml->sml_nvalues ? ml->sml_nvalues : ml->sml_bvalues,
-					e->e_id, SLAP_INDEX_ADD_OP );
-				break;
-			}
-			ml->sml_op &= ~NULLIFIED;
+	/* start with deleting the old index entries */
+	for ( ap = save_attrs; ap != NULL; ap = ap->a_next ) {
+		if ( ap->a_flags & SLAP_ATTR_IXDEL ) {
+			rc = bdb_index_values( op, tid, ap->a_desc,
+				ap->a_nvals,
+				e->e_id, SLAP_INDEX_DELETE_OP );
 			if ( rc != LDAP_SUCCESS ) {
 				attrs_free( e->e_attrs );
 				e->e_attrs = save_attrs;
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"bdb_modify_internal: attribute index update failure\n",
-					0, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
-				       "Attribute index update failure",
+				       "Attribute index delete failure",
 				       0, 0, 0 );
-#endif
-				/* reset our flags */
-				for (; ml; ml=ml->sml_next ) {
-					ml->sml_op &= ~(INDEXED | NULLIFIED);
-				}
-				break;
+				return rc;
 			}
+			ap->a_flags &= ~SLAP_ATTR_IXDEL;
+		}
+	}
+
+	/* add the new index entries */
+	for ( ap = e->e_attrs; ap != NULL; ap = ap->a_next ) {
+		if (ap->a_flags & SLAP_ATTR_IXADD) {
+			rc = bdb_index_values( op, tid, ap->a_desc,
+				ap->a_nvals,
+				e->e_id, SLAP_INDEX_ADD_OP );
+			if ( rc != LDAP_SUCCESS ) {
+				attrs_free( e->e_attrs );
+				e->e_attrs = save_attrs;
+				Debug( LDAP_DEBUG_ANY,
+				       "Attribute index add failure",
+				       0, 0, 0 );
+				return rc;
+			}
+			ap->a_flags &= ~SLAP_ATTR_IXADD;
 		}
 	}
 
@@ -294,39 +269,45 @@ bdb_modify( Operation *op, SlapReply *rs )
 	char textbuf[SLAP_TEXT_BUFLEN];
 	size_t textlen = sizeof textbuf;
 	DB_TXN	*ltid = NULL, *lt2;
-	struct bdb_op_info opinfo;
-	Entry		dummy;
+	struct bdb_op_info opinfo = {0};
+	Entry		dummy = {0};
 
 	u_int32_t	locker = 0;
 	DB_LOCK		lock;
 
-	int		noop = 0;
+	int		num_retries = 0;
+
+	LDAPControl **preread_ctrl = NULL;
+	LDAPControl **postread_ctrl = NULL;
+	LDAPControl *ctrls[SLAP_MAX_RESPONSE_CONTROLS];
+	int num_ctrls = 0;
 
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
 	Operation* ps_list;
 	struct psid_entry *pm_list, *pm_prev;
-#endif
+	int rc;
+	EntryInfo	*suffix_ei;
+	Entry		*ctxcsn_e;
+	int			ctxcsn_added = 0;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ENTRY, "bdb_modify: %s\n", op->o_req_dn.bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_ARGS, "bdb_modify: %s\n", op->o_req_dn.bv_val, 0, 0 );
-#endif
+	Debug( LDAP_DEBUG_ARGS, LDAP_XSTRING(bdb_modify) ": %s\n",
+		op->o_req_dn.bv_val, 0, 0 );
+
+	ctrls[num_ctrls] = NULL;
 
 	if( 0 ) {
 retry:	/* transaction retry */
+		if ( dummy.e_attrs ) {
+			attrs_free( dummy.e_attrs );
+			dummy.e_attrs = NULL;
+		}
 		if( e != NULL ) {
 			bdb_unlocked_cache_return_entry_w(&bdb->bi_cache, e);
 			e = NULL;
 		}
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, "bdb_modify: retrying...\n", 0, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_TRACE,
-			"bdb_modify: retrying...\n", 0, 0, 0);
-#endif
+			LDAP_XSTRING(bdb_modify) ": retrying...\n", 0, 0, 0);
 
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+#ifdef BDB_PSEARCH
 		pm_list = LDAP_LIST_FIRST(&op->o_pm_list);
 		while ( pm_list != NULL ) {
 			LDAP_LIST_REMOVE ( pm_list, ps_link );
@@ -346,6 +327,7 @@ retry:	/* transaction retry */
 			goto return_results;
 		}
 		ldap_pvt_thread_yield();
+		bdb_trans_backoff( ++num_retries );
 	}
 
 	/* begin transaction */
@@ -353,14 +335,9 @@ retry:	/* transaction retry */
 		bdb->bi_db_opflags );
 	rs->sr_text = NULL;
 	if( rs->sr_err != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"bdb_modify: txn_begin failed: %s (%d)\n", db_strerror(rs->sr_err), rs->sr_err, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: txn_begin failed: %s (%d)\n",
-			db_strerror(rs->sr_err), rs->sr_err, 0 );
-#endif
+			LDAP_XSTRING(bdb_modify) ": txn_begin failed: "
+			"%s (%d)\n", db_strerror(rs->sr_err), rs->sr_err, 0 );
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "internal error";
 		goto return_results;
@@ -380,14 +357,9 @@ retry:	/* transaction retry */
 		locker, &lock );
 
 	if ( rs->sr_err != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"bdb_modify: dn2entry failed: (%d)\n", rs->sr_err, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: dn2entry failed (%d)\n",
+			LDAP_XSTRING(bdb_modify) ": dn2entry failed (%d)\n",
 			rs->sr_err, 0, 0 );
-#endif
 		switch( rs->sr_err ) {
 		case DB_LOCK_DEADLOCK:
 		case DB_LOCK_NOTGRANTED:
@@ -406,7 +378,11 @@ retry:	/* transaction retry */
 
 	e = ei->bei_e;
 	/* acquire and lock entry */
-	if ( rs->sr_err == DB_NOTFOUND ) {
+	/* FIXME: dn2entry() should return non-glue entry */
+	if (( rs->sr_err == DB_NOTFOUND ) ||
+		( !manageDSAit && e && is_entry_glue( e )))
+	{
+		BerVarray deref = NULL;
 		if ( e != NULL ) {
 			rs->sr_matched = ch_strdup( e->e_dn );
 			rs->sr_ref = is_entry_referral( e )
@@ -416,14 +392,29 @@ retry:	/* transaction retry */
 			e = NULL;
 
 		} else {
-			rs->sr_ref = referral_rewrite( default_referral,
-				NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
+			if ( !LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo )) {
+				syncinfo_t *si;
+				LDAP_STAILQ_FOREACH( si, &op->o_bd->be_syncinfo, si_next ) {
+					struct berval tmpbv;
+					ber_dupbv( &tmpbv, &si->si_provideruri_bv[0] );
+					ber_bvarray_add( &deref, &tmpbv );
+                }
+			} else {
+				deref = default_referral;
+			}
+			rs->sr_ref = referral_rewrite( deref, NULL, &op->o_req_dn,
+					LDAP_SCOPE_DEFAULT );
 		}
 
 		rs->sr_err = LDAP_REFERRAL;
 		send_ldap_result( op, rs );
 
-		ber_bvarray_free( rs->sr_ref );
+		if ( rs->sr_ref != default_referral ) {
+			ber_bvarray_free( rs->sr_ref );
+		}
+		if ( deref != default_referral ) {
+			ber_bvarray_free( deref );
+		}
 		free( (char *)rs->sr_matched );
 		rs->sr_ref = NULL;
 		rs->sr_matched = NULL;
@@ -435,13 +426,9 @@ retry:	/* transaction retry */
 		/* entry is a referral, don't allow modify */
 		rs->sr_ref = get_entry_referrals( op, e );
 
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, "bdb_modify: entry is referral\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: entry is referral\n",
+			LDAP_XSTRING(bdb_modify) ": entry is referral\n",
 			0, 0, 0 );
-#endif
 
 		rs->sr_err = LDAP_REFERRAL;
 		rs->sr_matched = e->e_name.bv_val;
@@ -453,27 +440,56 @@ retry:	/* transaction retry */
 		goto done;
 	}
 
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
-	if ( rs->sr_err == LDAP_SUCCESS && !op->o_noop ) {
+	if ( get_assert( op ) &&
+		( test_filter( op, e, get_assertion( op )) != LDAP_COMPARE_TRUE ))
+	{
+		rs->sr_err = LDAP_ASSERTION_FAILED;
+		goto return_results;
+	}
+
+#ifdef BDB_PSEARCH
+	if ( rs->sr_err == LDAP_SUCCESS && !op->o_noop && !op->o_no_psearch ) {
+		ldap_pvt_thread_rdwr_wlock( &bdb->bi_pslist_rwlock );
 		LDAP_LIST_FOREACH ( ps_list, &bdb->bi_psearch_list, o_ps_link ) {
-			bdb_psearch(op, rs, ps_list, e, LDAP_PSEARCH_BY_PREMODIFY );
+			rc = bdb_psearch(op, rs, ps_list, e, LDAP_PSEARCH_BY_PREMODIFY );
+			if ( rc == LDAP_BUSY && op->o_ps_send_wait ) {
+				ldap_pvt_thread_rdwr_wunlock( &bdb->bi_pslist_rwlock );
+				goto retry;
+			} else if ( rc ) {
+				Debug( LDAP_DEBUG_TRACE,
+					LDAP_XSTRING(bdb_modify)
+					": persistent search failed (%d,%d)\n",
+					rc, rs->sr_err, 0 );
+			}
 		}
+		ldap_pvt_thread_rdwr_wunlock( &bdb->bi_pslist_rwlock );
 	}
 #endif
-	
+
+	if( op->o_preread ) {
+		if( preread_ctrl == NULL ) {
+			preread_ctrl = &ctrls[num_ctrls++];
+			ctrls[num_ctrls] = NULL;
+		}
+		if ( slap_read_controls( op, rs, e,
+			&slap_pre_read_bv, preread_ctrl ) )
+		{
+			Debug( LDAP_DEBUG_TRACE,
+				"<=- " LDAP_XSTRING(bdb_modify)
+				": pre-read failed!\n", 0, 0, 0 );
+			goto return_results;
+		}
+	}
+
 	/* nested transaction */
 	rs->sr_err = TXN_BEGIN( bdb->bi_dbenv, ltid, &lt2, 
 		bdb->bi_db_opflags );
 	rs->sr_text = NULL;
 	if( rs->sr_err != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ERR, 
-			"bdb_modify: txn_begin(2) failed: %s (%d)\n", db_strerror(rs->sr_err), rs->sr_err, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: txn_begin(2) failed: %s (%d)\n",
+			LDAP_XSTRING(bdb_modify) ": txn_begin(2) failed: "
+			"%s (%d)\n",
 			db_strerror(rs->sr_err), rs->sr_err, 0 );
-#endif
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "internal error";
 		goto return_results;
@@ -484,17 +500,14 @@ retry:	/* transaction retry */
 		&dummy, &rs->sr_text, textbuf, textlen );
 
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ERR, 
-			"bdb_modify: modify failed (%d)\n", rs->sr_err, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: modify failed (%d)\n",
+			LDAP_XSTRING(bdb_modify) ": modify failed (%d)\n",
 			rs->sr_err, 0, 0 );
-#endif
 		if ( (rs->sr_err == LDAP_INSUFFICIENT_ACCESS) && opinfo.boi_err ) {
 			rs->sr_err = opinfo.boi_err;
 		}
+		/* Only free attrs if they were dup'd.  */
+		if ( dummy.e_attrs == e->e_attrs ) dummy.e_attrs = NULL;
 		switch( rs->sr_err ) {
 		case DB_LOCK_DEADLOCK:
 		case DB_LOCK_NOTGRANTED:
@@ -506,14 +519,9 @@ retry:	/* transaction retry */
 	/* change the entry itself */
 	rs->sr_err = bdb_id2entry_update( op->o_bd, lt2, &dummy );
 	if ( rs->sr_err != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ERR, 
-			"bdb_modify: id2entry update failed (%d)\n", rs->sr_err, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: id2entry update failed (%d)\n",
-			rs->sr_err, 0, 0 );
-#endif
+			LDAP_XSTRING(bdb_modify) ": id2entry update failed "
+			"(%d)\n", rs->sr_err, 0, 0 );
 		switch( rs->sr_err ) {
 		case DB_LOCK_DEADLOCK:
 		case DB_LOCK_NOTGRANTED:
@@ -522,75 +530,128 @@ retry:	/* transaction retry */
 		rs->sr_text = "entry update failed";
 		goto return_results;
 	}
+
 	if ( TXN_COMMIT( lt2, 0 ) != 0 ) {
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "txn_commit(2) failed";
 		goto return_results;
 	}
 
+#ifdef BDB_PSEARCH
+	if ( LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo )) {
+		rc = bdb_csn_commit( op, rs, ltid, ei, &suffix_ei,
+			&ctxcsn_e, &ctxcsn_added, locker );
+		switch ( rc ) {
+		case BDB_CSN_ABORT :
+			goto return_results;
+		case BDB_CSN_RETRY :
+			goto retry;
+		}
+	}
+#endif
+
+	if( op->o_postread ) {
+		if( postread_ctrl == NULL ) {
+			postread_ctrl = &ctrls[num_ctrls++];
+			ctrls[num_ctrls] = NULL;
+		}
+		if( slap_read_controls( op, rs, &dummy,
+			&slap_post_read_bv, postread_ctrl ) )
+		{
+			Debug( LDAP_DEBUG_TRACE,
+				"<=- " LDAP_XSTRING(bdb_modify)
+				": post-read failed!\n", 0, 0, 0 );
+			goto return_results;
+		}
+	}
+
 	if( op->o_noop ) {
 		if ( ( rs->sr_err = TXN_ABORT( ltid ) ) != 0 ) {
 			rs->sr_text = "txn_abort (no-op) failed";
 		} else {
-			noop = 1;
-			rs->sr_err = LDAP_SUCCESS;
+			rs->sr_err = LDAP_NO_OPERATION;
+			goto return_results;
 		}
 	} else {
-		bdb_cache_modify( e, dummy.e_attrs, bdb->bi_dbenv, locker, &lock );
+		rc = bdb_cache_modify( e, dummy.e_attrs, bdb->bi_dbenv, locker, &lock );
+		switch( rc ) {
+		case DB_LOCK_DEADLOCK:
+		case DB_LOCK_NOTGRANTED:
+			goto retry;
+		}
+		dummy.e_attrs = NULL;
+
+#ifdef BDB_PSEARCH
+		if ( LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo )) {
+			if ( ctxcsn_added ) {
+				bdb_cache_add( bdb, suffix_ei, ctxcsn_e,
+					(struct berval *)&slap_ldapsync_cn_bv, locker );
+			}
+		}
+
+		if ( rs->sr_err == LDAP_SUCCESS ) {
+			/* Loop through in-scope entries for each psearch spec */
+			ldap_pvt_thread_rdwr_wlock( &bdb->bi_pslist_rwlock );
+			LDAP_LIST_FOREACH ( ps_list, &bdb->bi_psearch_list, o_ps_link ) {
+				rc = bdb_psearch( op, rs, ps_list, e, LDAP_PSEARCH_BY_MODIFY );
+				if ( rc ) {
+					Debug( LDAP_DEBUG_TRACE,
+						LDAP_XSTRING(bdb_modify)
+						": persistent search failed "
+						"(%d,%d)\n",
+						rc, rs->sr_err, 0 );
+				}
+			}
+			pm_list = LDAP_LIST_FIRST(&op->o_pm_list);
+			while ( pm_list != NULL ) {
+				rc = bdb_psearch(op, rs, pm_list->ps_op,
+							e, LDAP_PSEARCH_BY_SCOPEOUT);
+				if ( rc ) {
+					Debug( LDAP_DEBUG_TRACE,
+						LDAP_XSTRING(bdb_modify)
+						": persistent search failed "
+						"(%d,%d)\n",
+						rc, rs->sr_err, 0 );
+				}
+				LDAP_LIST_REMOVE ( pm_list, ps_link );
+				pm_prev = pm_list;
+				pm_list = LDAP_LIST_NEXT ( pm_list, ps_link );
+				ch_free( pm_prev );
+			}
+			ldap_pvt_thread_rdwr_wunlock( &bdb->bi_pslist_rwlock );
+		}
+#endif
+
 		rs->sr_err = TXN_COMMIT( ltid, 0 );
 	}
 	ltid = NULL;
 	op->o_private = NULL;
 
 	if( rs->sr_err != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, ERR, 
-			"bdb_modify: txn_%s failed %s (%d)\n", 
-			op->o_noop ? "abort (no_op)" : "commit", db_strerror(rs->sr_err), rs->sr_err );
-#else
 		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: txn_%s failed: %s (%d)\n",
+			LDAP_XSTRING(bdb_modify) ": txn_%s failed: %s (%d)\n",
 			op->o_noop ? "abort (no-op)" : "commit",
 			db_strerror(rs->sr_err), rs->sr_err );
-#endif
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "commit failed";
 
-	} else {
-#ifdef NEW_LOGGING
-		LDAP_LOG ( OPERATION, DETAIL1, 
-			"bdb_modify: updated%s id=%08lx dn=\"%s\"\n", 
-			op->o_noop ? " (no_op)" : "", e->e_id, e->e_dn );
-#else
-		Debug( LDAP_DEBUG_TRACE,
-			"bdb_modify: updated%s id=%08lx dn=\"%s\"\n",
-			op->o_noop ? " (no-op)" : "",
-			e->e_id, e->e_dn );
-#endif
-		rs->sr_err = LDAP_SUCCESS;
-		rs->sr_text = NULL;
+		goto return_results;
 	}
 
+	Debug( LDAP_DEBUG_TRACE,
+		LDAP_XSTRING(bdb_modify) ": updated%s id=%08lx dn=\"%s\"\n",
+		op->o_noop ? " (no-op)" : "",
+		dummy.e_id, op->o_req_dn.bv_val );
+
+	rs->sr_err = LDAP_SUCCESS;
+	rs->sr_text = NULL;
+	if( num_ctrls ) rs->sr_ctrls = ctrls;
+
 return_results:
-	send_ldap_result( op, rs );
-
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
-	if ( rs->sr_err == LDAP_SUCCESS && !op->o_noop ) {
-		/* Loop through in-scope entries for each psearch spec */
-		LDAP_LIST_FOREACH ( ps_list, &bdb->bi_psearch_list, o_ps_link ) {
-			bdb_psearch( op, rs, ps_list, e, LDAP_PSEARCH_BY_MODIFY );
-		}
-		pm_list = LDAP_LIST_FIRST(&op->o_pm_list);
-		while ( pm_list != NULL ) {
-			bdb_psearch(op, rs, pm_list->ps_op,
-						e, LDAP_PSEARCH_BY_SCOPEOUT);
-			LDAP_LIST_REMOVE ( pm_list, ps_link );
-			pm_prev = pm_list;
-			pm_list = LDAP_LIST_NEXT ( pm_list, ps_link );
-			ch_free( pm_prev );
-		}
+	if( dummy.e_attrs ) {
+		attrs_free( dummy.e_attrs );
 	}
-#endif
+	send_ldap_result( op, rs );
 
 	if( rs->sr_err == LDAP_SUCCESS && bdb->bi_txn_cp ) {
 		ldap_pvt_thread_yield();
@@ -600,7 +661,7 @@ return_results:
 
 done:
 	if( ltid != NULL ) {
-#if defined(LDAP_CLIENT_UPDATE) || defined(LDAP_SYNC)
+#ifdef BDB_PSEARCH
 		pm_list = LDAP_LIST_FIRST(&op->o_pm_list);
 		while ( pm_list != NULL ) {
 			LDAP_LIST_REMOVE ( pm_list, ps_link );
@@ -616,5 +677,14 @@ done:
 	if( e != NULL ) {
 		bdb_unlocked_cache_return_entry_w (&bdb->bi_cache, e);
 	}
-	return ( ( rs->sr_err == LDAP_SUCCESS ) ? noop : rs->sr_err );
+
+	if( preread_ctrl != NULL ) {
+		slap_sl_free( (*preread_ctrl)->ldctl_value.bv_val, op->o_tmpmemctx );
+		slap_sl_free( *preread_ctrl, op->o_tmpmemctx );
+	}
+	if( postread_ctrl != NULL ) {
+		slap_sl_free( (*postread_ctrl)->ldctl_value.bv_val, op->o_tmpmemctx );
+		slap_sl_free( *postread_ctrl, op->o_tmpmemctx );
+	}
+	return rs->sr_err;
 }
