@@ -1,8 +1,17 @@
 /* init.c - initialize bdb backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/nextid.c,v 1.16 2002/01/04 20:17:49 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/nextid.c,v 1.23 2004/12/06 06:28:47 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2000-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -26,15 +35,16 @@ int bdb_next_id( BackendDB *be, DB_TXN *tid, ID *out )
 int bdb_last_id( BackendDB *be, DB_TXN *tid )
 {
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
-	int rc;
+	int i, rc;
 	ID id = 0;
+	unsigned char idbuf[sizeof(ID)];
 	DBT key, data;
 	DBC *cursor;
 
 	DBTzero( &key );
 	key.flags = DB_DBT_USERMEM;
-	key.data = (char *) &id;
-	key.ulen = sizeof( id );
+	key.data = (char *) idbuf;
+	key.ulen = sizeof( idbuf );
 
 	DBTzero( &data );
 	data.flags = DB_DBT_USERMEM | DB_DBT_PARTIAL;
@@ -50,10 +60,10 @@ int bdb_last_id( BackendDB *be, DB_TXN *tid )
 
 	switch(rc) {
 	case DB_NOTFOUND:
-		id = 0;
 		rc = 0;
-		/* FALLTHROUGH */
+		break;
 	case 0:
+		BDB_DISK2ID( idbuf, &id );
 		break;
 
 	default:
