@@ -1,8 +1,17 @@
 /* add.c - ldap ldbm back-end add routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/add.c,v 1.73 2003/04/10 23:29:30 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/add.c,v 1.92 2004/10/06 03:51:41 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -29,69 +38,50 @@ ldbm_back_add(
 	AttributeDescription *entry = slap_schema.si_ad_entry;
 	char textbuf[SLAP_TEXT_BUFLEN];
 	size_t textlen = sizeof textbuf;
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, ENTRY, "ldbm_back_add: %s\n", op->o_req_dn.bv_val, 0, 0 );
-#else
-	Debug(LDAP_DEBUG_ARGS, "==> ldbm_back_add: %s\n", op->o_req_dn.bv_val, 0, 0);
+#ifdef LDBM_SUBENTRIES
+	int subentry;
 #endif
+
+	Debug(LDAP_DEBUG_ARGS, "==> ldbm_back_add: %s\n",
+		op->o_req_dn.bv_val, 0, 0);
 	
-#ifndef LDAP_CACHING
-	rs->sr_err = entry_schema_check( op->o_bd, op->oq_add.rs_e, NULL, &rs->sr_text, textbuf, textlen );
-#else /* LDAP_CACHING */
-        if ( !op->o_caching_on ) {
-		rs->sr_err = entry_schema_check( op->o_bd, op->oq_add.rs_e, NULL, &rs->sr_text, textbuf, textlen );
-	} else {
-		rs->sr_err = LDAP_SUCCESS;
-	}
-#endif /* LDAP_CACHING */
+	rs->sr_err = entry_schema_check( op->o_bd, op->oq_add.rs_e, NULL,
+		&rs->sr_text, textbuf, textlen );
 
 	if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR, 
-			"ldbm_back_add: entry (%s) failed schema check.\n", op->o_req_dn.bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "entry failed schema check: %s\n",
 			rs->sr_text, 0, 0 );
-#endif
 
 		send_ldap_result( op, rs );
-		return( -1 );
+		return rs->sr_err;
 	}
 
-#ifdef LDAP_CACHING
-	if ( !op->o_caching_on ) {
-#endif /* LDAP_CACHING */
+#ifdef LDBM_SUBENTRIES
+	subentry = is_entry_subentry( op->oq_add.rs_e );
+#endif
+
 	if ( !access_allowed( op, op->oq_add.rs_e,
 				entry, NULL, ACL_WRITE, NULL ) )
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR, 
-			"ldbm_back_add: No write access to entry (%s).\n", 
-			op->o_req_dn.bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "no write access to entry\n", 0,
 		    0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_INSUFFICIENT_ACCESS,
 		    "no write access to entry" );
 
-		return -1;
+		return LDAP_INSUFFICIENT_ACCESS;
 	}
-#ifdef LDAP_CACHING
-	}
-#endif /* LDAP_CACHING */
 
 	/* grab giant lock for writing */
 	ldap_pvt_thread_rdwr_wlock(&li->li_giant_rwlock);
 
-	if ( ( rs->sr_err = dn2id( op->o_bd, &op->o_req_ndn, &id ) ) || id != NOID ) {
+	rs->sr_err = dn2id( op->o_bd, &op->o_req_ndn, &id );
+	if ( rs->sr_err || id != NOID )	{
 		/* if (rs->sr_err) something bad happened to ldbm cache */
 		ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 		rs->sr_err = rs->sr_err ? LDAP_OTHER : LDAP_ALREADY_EXISTS;
 		send_ldap_result( op, rs );
-		return( -1 );
+		return rs->sr_err;
 	}
 
 	/*
@@ -106,12 +96,7 @@ ldbm_back_add(
 		dnParent( &op->o_req_ndn, &pdn );
 	}
 
-#ifndef LDAP_CACHING
-	if( pdn.bv_len )
-#else /* LDAP_CACHING */
-	if( pdn.bv_len && !op->o_caching_on )
-#endif /* LDAP_CACHING */
-	{
+	if( pdn.bv_len ) {
 		Entry *matched = NULL;
 
 		/* get parent with writer lock */
@@ -130,47 +115,45 @@ ldbm_back_add(
 
 			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, ERR, 
-				"ldbm_back_add: Parent of (%s) does not exist.\n", 
-				op->o_req_dn.bv_val, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "parent does not exist\n",
 				0, 0, 0 );
-#endif
 
-			rs->sr_text = rs->sr_ref ? "parent is referral" : "parent does not exist";
+			rs->sr_text = rs->sr_ref
+				? "parent is referral" : "parent does not exist";
 			rs->sr_err = LDAP_REFERRAL;
 			send_ldap_result( op, rs );
 
 			ber_bvarray_free( rs->sr_ref );
 			free( (char *)rs->sr_matched );
-
-			return -1;
+			rs->sr_ref = NULL;
+			rs->sr_matched = NULL;
+			return rs->sr_err;
 		}
 
-		if ( ! access_allowed( op, p,
-			children, NULL, ACL_WRITE, NULL ) )
-		{
+		if ( ! access_allowed( op, p, children, NULL, ACL_WRITE, NULL ) ) {
 			/* free parent and writer lock */
 			cache_return_entry_w( &li->li_cache, p ); 
 			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, ERR, 
-				"ldbm_back_add: No write access to parent (%s).\n", 
-				op->o_req_dn.bv_val, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "no write access to parent\n", 0,
 			    0, 0 );
-#endif
 
 			send_ldap_error( op, rs, LDAP_INSUFFICIENT_ACCESS,
 			    "no write access to parent" );
 
-			return -1;
+			return LDAP_INSUFFICIENT_ACCESS;
 		}
 
+#ifdef LDBM_SUBENTRIES
+		if ( is_entry_subentry( p )) {
+			Debug( LDAP_DEBUG_TRACE, "bdb_add: parent is subentry\n",
+				0, 0, 0 );
+			rs->sr_err = LDAP_OBJECT_CLASS_VIOLATION;
+			rs->sr_text = "parent is a subentry";
+			goto return_results;
+		}
+#endif
+
 		if ( is_entry_alias( p ) ) {
 			/* parent is an alias, don't allow add */
 
@@ -178,19 +161,13 @@ ldbm_back_add(
 			cache_return_entry_w( &li->li_cache, p );
 			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-			LDAP_LOG(BACK_LDBM, ERR, 
-				"ldbm_back_add:  Parent is an alias.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "parent is alias\n", 0,
 			    0, 0 );
-#endif
-
 
 			send_ldap_error( op, rs, LDAP_ALIAS_PROBLEM,
 			    "parent is an alias" );
 
-			return -1;
+			return LDAP_ALIAS_PROBLEM;
 		}
 
 		if ( is_entry_referral( p ) ) {
@@ -204,84 +181,38 @@ ldbm_back_add(
 			cache_return_entry_w( &li->li_cache, p );
 			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, ERR,
-				   "ldbm_back_add: Parent is referral.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE, "parent is referral\n", 0,
 			    0, 0 );
-#endif
 			rs->sr_err = LDAP_REFERRAL;
 			send_ldap_result( op, rs );
 
 			ber_bvarray_free( rs->sr_ref );
 			free( (char *)rs->sr_matched );
-			return -1;
+			rs->sr_ref = NULL;
+			rs->sr_matched = NULL;
+			return rs->sr_err;
 		}
 
+#ifdef LDBM_SUBENTRIES
+		if ( subentry ) {
+			/* FIXME: */
+			/* parent must be an administrative point of the required kind */
+		}
+#endif
+
 	} else {
-#ifndef LDAP_CACHING
-		if( pdn.bv_val != NULL )
-#else /* LDAP_CACHING */
-	        if( pdn.bv_val != NULL && !op->o_caching_on )
-#endif /* LDAP_CACHING */
+		assert( pdn.bv_val == NULL || *pdn.bv_val == '\0' );
+
+		if (( !be_isroot(op) && !be_shadow_update(op) )
+			&& !is_entry_glue( op->oq_add.rs_e ))
 		{
-			assert( *pdn.bv_val == '\0' );
-		}
+			ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-		/* no parent, must be adding entry to root */
-#ifndef LDAP_CACHING
-		if ( !be_isroot( op->o_bd, &op->o_ndn ) )
-#else /* LDAP_CACHING */
-		if ( !be_isroot( op->o_bd, &op->o_ndn ) && !op->o_caching_on )
-#endif /* LDAP_CACHING */
-		{
-			if ( be_issuffix( op->o_bd, (struct berval *)&slap_empty_bv ) || be_isupdate( op->o_bd, &op->o_ndn ) ) {
-				p = (Entry *)&slap_entry_root;
-				
-				rs->sr_err = access_allowed( op, p,
-					children, NULL, ACL_WRITE, NULL );
-				p = NULL;
-				
-				if ( ! rs->sr_err ) {
-					ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
+			Debug( LDAP_DEBUG_TRACE, "%s add denied\n",
+				pdn.bv_val == NULL ? "suffix" : "entry at root", 0, 0 );
 
-#ifdef NEW_LOGGING
-					LDAP_LOG( BACK_LDBM, ERR,
-						"ldbm_back_add: No write "
-						"access to parent (\"\").\n", 0, 0, 0 );
-#else
-					Debug( LDAP_DEBUG_TRACE, 
-						"no write access to parent\n", 
-						0, 0, 0 );
-#endif
-
-					send_ldap_error( op, rs,
-						LDAP_INSUFFICIENT_ACCESS,
-						"no write access to parent" );
-
-					return -1;
-				}
-
-			} else {
-				ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
-
-#ifdef NEW_LOGGING
-				LDAP_LOG( BACK_LDBM, ERR,
-					   "ldbm_back_add: %s add denied.\n",
-					   pdn.bv_val == NULL ? "suffix" 
-					   : "entry at root", 0, 0 );
-#else
-				Debug( LDAP_DEBUG_TRACE, "%s add denied\n",
-						pdn.bv_val == NULL ? "suffix" 
-						: "entry at root", 0, 0 );
-#endif
-
-				send_ldap_error( op, rs,
-						LDAP_INSUFFICIENT_ACCESS, NULL );
-
-				return -1;
-			}
+			send_ldap_error( op, rs, LDAP_NO_SUCH_OBJECT, NULL );
+			return LDAP_NO_SUCH_OBJECT;
 		}
 	}
 
@@ -293,24 +224,20 @@ ldbm_back_add(
 
 		ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR,
-			"ldbm_back_add: next_id failed.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "ldbm_add: next_id failed\n",
 			0, 0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_OTHER,
 			"next_id add failed" );
 
-		return( -1 );
+		return LDAP_OTHER;
 	}
 
 	/*
 	 * Try to add the entry to the cache, assign it a new dnid.
 	 */
-	rs->sr_err = cache_add_entry_rw(&li->li_cache, op->oq_add.rs_e, CACHE_WRITE_LOCK);
+	rs->sr_err = cache_add_entry_rw( &li->li_cache, op->oq_add.rs_e,
+		CACHE_WRITE_LOCK );
 
 	if ( rs->sr_err != 0 ) {
 		if( p != NULL) {
@@ -320,32 +247,22 @@ ldbm_back_add(
 
 		ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
 
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR,
-			"ldbm_back_add: cache_add_entry_lock failed.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "cache_add_entry_lock failed\n", 0, 0,
 		    0 );
-#endif
 
 		rs->sr_text = rs->sr_err > 0 ? NULL : "cache add failed";
 		rs->sr_err = rs->sr_err > 0 ? LDAP_ALREADY_EXISTS : LDAP_OTHER;
 		send_ldap_result( op, rs );
 
-		return( -1 );
+		return rs->sr_err;
 	}
 
 	rs->sr_err = -1;
 
 	/* attribute indexes */
 	if ( index_entry_add( op, op->oq_add.rs_e ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR,
-			"ldbm_back_add: index_entry_add failed.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "index_entry_add failed\n", 0,
 		    0, 0 );
-#endif
 		
 		send_ldap_error( op, rs, LDAP_OTHER,
 			"index generation failed" );
@@ -354,14 +271,11 @@ ldbm_back_add(
 	}
 
 	/* dn2id index */
-	if ( dn2id_add( op->o_bd, &op->oq_add.rs_e->e_nname, op->oq_add.rs_e->e_id ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR,
-			"ldbm_back_add: dn2id_add failed.\n", 0, 0, 0 );
-#else
+	if ( dn2id_add( op->o_bd, &op->oq_add.rs_e->e_nname,
+		op->oq_add.rs_e->e_id ) != 0 )
+	{
 		Debug( LDAP_DEBUG_TRACE, "dn2id_add failed\n", 0,
 		    0, 0 );
-#endif
 		/* FIXME: delete attr indices? */
 
 		send_ldap_error( op, rs, LDAP_OTHER,
@@ -372,16 +286,12 @@ ldbm_back_add(
 
 	/* id2entry index */
 	if ( id2entry_add( op->o_bd, op->oq_add.rs_e ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( BACK_LDBM, ERR,
-			   "ldbm_back_add: id2entry_add failed.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_TRACE, "id2entry_add failed\n", 0,
 		    0, 0 );
-#endif
 
 		/* FIXME: delete attr indices? */
-		(void) dn2id_delete( op->o_bd, &op->oq_add.rs_e->e_nname, op->oq_add.rs_e->e_id );
+		(void) dn2id_delete( op->o_bd, &op->oq_add.rs_e->e_nname,
+			op->oq_add.rs_e->e_id );
 		
 		send_ldap_error( op, rs, LDAP_OTHER,
 			"entry store failed" );
@@ -390,6 +300,7 @@ ldbm_back_add(
 	}
 
 	rs->sr_err = LDAP_SUCCESS;
+	rs->sr_text = NULL;
 	send_ldap_result( op, rs );
 
 	/* marks the entry as committed, so it is added to the cache;
