@@ -1,8 +1,17 @@
 /* config.c - ldbm backend configuration file routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/config.c,v 1.33 2003/01/03 19:20:57 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/config.c,v 1.37 2004/09/04 02:54:32 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -106,32 +115,18 @@ ldbm_back_db_config(
 #ifndef NO_THREADS
 		int i;
 		if ( argc < 2 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, ERR, "ldbm_back_db_config: %s: "
-				"line %d: missing frequency value in \"dbsync <frequency> "
-				"[<wait-times> [wait-interval]]\" line\n", fname, lineno, 0 );
-#else	
 			Debug( LDAP_DEBUG_ANY,
     "%s: line %d: missing frquency value in \"dbsync <frequency> [<wait-times> [wait-interval]]\" line\n",
 			    fname, lineno, 0 );
-#endif
 			return 1;
 		}
 
 		i = atoi( argv[1] );
 
 		if( i < 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( CONFIG, ERR, 
-				"ldbm_back_db_config: %s: "
-				"line %d: frequency value (%d) invalid \"dbsync "
-				"<frequency> [<wait-times> [wait-interval]]\" line\n", 
-				fname, lineno, i );
-#else	
 			Debug( LDAP_DEBUG_ANY,
     "%s: line %d: frquency value (%d) invalid \"dbsync <frequency> [<wait-times> [wait-interval]]\" line\n",
 			    fname, lineno, i );
-#endif
 			return 1;
 		}
 
@@ -140,16 +135,9 @@ ldbm_back_db_config(
 		if ( argc > 2 ) {
 			i = atoi( argv[2] );
 			if ( i < 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( CONFIG,ERR, "ldbm_back_db_config: %s: "
-					"line %d: frequency value (%d) invalid \"dbsync "
-					"<frequency> [<wait-times> [wait-interval]]\" line\n", 
-					fname, lineno, i );
-#else	
 				Debug( LDAP_DEBUG_ANY,
 	    "%s: line %d: frquency value (%d) invalid \"dbsync <frequency> [<wait-times> [wait-interval]]\" line\n",
 				    fname, lineno, i );
-#endif
 				return 1;
 			}
 			li ->li_dbsyncwaitn = i;
@@ -158,16 +146,9 @@ ldbm_back_db_config(
 		if ( argc > 3 ) {
 			i = atoi( argv[3] );
 			if ( i <= 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( CONFIG,ERR, "ldbm_back_db_config: %s: "
-					"line %d: frequency value (%d) invalid \"dbsync "
-					"<frequency> [<wait-times> [wait-interval]]\" line\n", 
-					fname, lineno, i );
-#else	
 				Debug( LDAP_DEBUG_ANY,
 	    "%s: line %d: frquency value (%d) invalid \"dbsync <frequency> [<wait-times> [wait-interval]]\" line\n",
 				    fname, lineno, i );
-#endif
 				return 1;
 			}
 			li ->li_dbsyncwaitinterval = i;
@@ -177,22 +158,15 @@ ldbm_back_db_config(
 		li->li_dbwritesync = 0;
 
 #else
-#ifdef NEW_LOGGING
-		LDAP_LOG ( CONFIG, ERR, "ldbm_back_db_config: \"dbsync\""
-			" policies not supported in non-threaded environments\n", 0, 0, 0 );
-#else	
 		Debug( LDAP_DEBUG_ANY,
     "\"dbsync\" policies not supported in non-threaded environments\n", 0, 0, 0);
-#endif
 		return 1;
 #endif
 
 
 	/* anything else */
 	} else {
-		fprintf( stderr,
-"%s: line %d: unknown directive \"%s\" in ldbm database definition (ignored)\n",
-		    fname, lineno, argv[0] );
+		return SLAP_CONF_UNKNOWN;
 	}
 
 	return 0;
