@@ -1,8 +1,17 @@
 /* ldbmcache.c - maintain a cache of open ldbm files */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/dbcache.c,v 1.47 2002/01/04 20:17:52 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/dbcache.c,v 1.57 2004/10/06 03:51:41 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -53,13 +62,8 @@ ldbm_cache_open(
 		flags |= LDBM_NOSYNC;
 	}
 	
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_ENTRY,
-		   "ldbm_cache_open: \"%s\", %d, %o\n", buf, flags, li->li_mode ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> ldbm_cache_open( \"%s\", %d, %o )\n", buf,
 	    flags, li->li_mode );
-#endif
 
 
 	empty = MAXDBCACHE;
@@ -100,13 +104,8 @@ ldbm_cache_open(
 					break;
 				}
 				li->li_dbcache[i].dbc_refcnt++;
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-					   "ldbm_cache_open: cache %d\n", i ));
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				    "<= ldbm_cache_open (cache %d)\n", i, 0, 0 );
-#endif
 
 				ldap_pvt_thread_mutex_unlock( &li->li_dbcache_mutex );
 				return( &li->li_dbcache[i] );
@@ -131,14 +130,9 @@ ldbm_cache_open(
 				free( li->li_dbcache[i].dbc_name );
 				li->li_dbcache[i].dbc_name = NULL;
 			} else {
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-					   "ldbm_cache_open: no unused db to close - waiting\n" ));
-#else
 				Debug( LDAP_DEBUG_ANY,
 				    "ldbm_cache_open no unused db to close - waiting\n",
 				    0, 0, 0 );
-#endif
 
 				ldap_pvt_thread_cond_wait( &li->li_dbcache_cv,
 					    &li->li_dbcache_mutex );
@@ -154,17 +148,10 @@ ldbm_cache_open(
 	    li->li_dbcachesize )) == NULL )
 	{
 		int err = errno;
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "cache", LDAP_LEVEL_ERR,
-			   "ldbm_cache_open: \"%s\" failed, errono=%d, reason=%s\n",
-			   buf, err, err > -1 && err < sys_nerr ? sys_errlist[err] :
-			   "unknown" ));
-#else
 		Debug( LDAP_DEBUG_TRACE,
 		    "<= ldbm_cache_open NULL \"%s\" errno=%d reason=\"%s\")\n",
 		    buf, err, err > -1 && err < sys_nerr ?
 		    sys_errlist[err] : "unknown" );
-#endif
 
 		ldap_pvt_thread_mutex_unlock( &li->li_dbcache_mutex );
 		return( NULL );
@@ -189,24 +176,12 @@ ldbm_cache_open(
 
 	assert( li->li_dbcache[i].dbc_maxindirect < 256 );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_ARGS,
-		   "ldbm_cache_open: blksize:%ld  maxids:%d  maxindirect:%d\n",
-		   li->li_dbcache[i].dbc_blksize, li->li_dbcache[i].dbc_maxids,
-		   li->li_dbcache[i].dbc_maxindirect ));
-#else
 	Debug( LDAP_DEBUG_ARGS,
 	    "ldbm_cache_open (blksize %ld) (maxids %d) (maxindirect %d)\n",
 	    li->li_dbcache[i].dbc_blksize, li->li_dbcache[i].dbc_maxids,
 	    li->li_dbcache[i].dbc_maxindirect );
-#endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-		   "ldbm_cache_open: opened %d\n", i ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= ldbm_cache_open (opened %d)\n", i, 0, 0 );
-#endif
 
 	ldap_pvt_thread_mutex_init( &li->li_dbcache[i].dbc_write_mutex );
 
@@ -258,39 +233,21 @@ ldbm_cache_flush_all( Backend *be )
 	ldap_pvt_thread_mutex_lock( &li->li_dbcache_mutex );
 	for ( i = 0; i < MAXDBCACHE; i++ ) {
 		if ( li->li_dbcache[i].dbc_name != NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-				   "ldbm_cache_flush_all: flushing db (%s)\n",
-				   li->li_dbcache[i].dbc_name ));
-#else
 			Debug( LDAP_DEBUG_TRACE, "ldbm flushing db (%s)\n",
 			    li->li_dbcache[i].dbc_name, 0, 0 );
-#endif
 
 			ldbm_sync( li->li_dbcache[i].dbc_db );
 			li->li_dbcache[i].dbc_dirty = 0;
 			if ( li->li_dbcache[i].dbc_refcnt != 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "cache", LDAP_LEVEL_INFO,
-					   "ldbm_cache_flush_all: couldn't close db (%s), refcnt=%d\n",
-					   li->li_dbcache[i].dbc_name, li->li_dbcache[i].dbc_refcnt ));
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				       "refcnt = %d, couldn't close db (%s)\n",
 				       li->li_dbcache[i].dbc_refcnt,
 				       li->li_dbcache[i].dbc_name, 0 );
-#endif
 
 			} else {
-#ifdef NEW_LOGGING
-				LDAP_LOG(( "cache", LDAP_LEVEL_DETAIL1,
-					   "ldbm_cache_flush_all: ldbm closing db (%s)\n",
-					   li->li_dbcache[i].dbc_name ));
-#else
 				Debug( LDAP_DEBUG_TRACE,
 				       "ldbm closing db (%s)\n",
 				       li->li_dbcache[i].dbc_name, 0, 0 );
-#endif
 
 				ldap_pvt_thread_cond_signal( &li->li_dbcache_cv );
 				ldbm_close( li->li_dbcache[i].dbc_db );
@@ -320,6 +277,7 @@ ldbm_cache_sync( Backend *be )
 	ldap_pvt_thread_mutex_unlock( &li->li_dbcache_mutex );
 }
 
+#if 0 /* macro in proto-back-ldbm.h */
 Datum
 ldbm_cache_fetch(
     DBCache	*db,
@@ -328,6 +286,7 @@ ldbm_cache_fetch(
 {
 	return ldbm_fetch( db->dbc_db, key );
 }
+#endif /* 0 */
 
 int
 ldbm_cache_store(
@@ -339,24 +298,6 @@ ldbm_cache_store(
 {
 	int	rc;
 
-#ifdef LDBM_DEBUG
-	Statslog( LDAP_DEBUG_STATS,
-		"=> ldbm_cache_store(): key.dptr=%s, key.dsize=%d\n",
-		key.dptr, key.dsize, 0, 0, 0 );
-
-	Statslog( LDAP_DEBUG_STATS,
-		"=> ldbm_cache_store(): key.dptr=0x%08x, data.dptr=0x%0 8x\n",
-		key.dptr, data.dptr, 0, 0, 0 );
-
-	Statslog( LDAP_DEBUG_STATS,
-		"=> ldbm_cache_store(): data.dptr=%s, data.dsize=%d\n",
-		data.dptr, data.dsize, 0, 0, 0 );
-
-	Statslog( LDAP_DEBUG_STATS,
-		"=> ldbm_cache_store(): flags=0x%08x\n",
-		flags, 0, 0, 0, 0 );
-#endif /* LDBM_DEBUG */
-
 	db->dbc_dirty = 1;
 	rc = ldbm_store( db->dbc_db, key, data, flags );
 
