@@ -1,8 +1,17 @@
 /* id2children.c - routines to deal with the id2children index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/id2children.c,v 1.25 2002/01/04 20:17:52 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/id2children.c,v 1.31 2004/09/04 02:54:32 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -15,7 +24,6 @@
 #include "slap.h"
 #include "back-ldbm.h"
 
-
 int
 has_children(
     Backend	*be,
@@ -29,25 +37,14 @@ has_children(
 
 	ldbm_datum_init( key );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "has_children: enter %ld\n", p->e_id ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> has_children( %ld )\n", p->e_id , 0, 0 );
-#endif
 
 
 	if ( (db = ldbm_cache_open( be, "dn2id", LDBM_SUFFIX,
 	    LDBM_WRCREAT )) == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "has_children: could not open \"dn2id%s\"\n",
-			   LDBM_SUFFIX ));
-#else
 		Debug( LDAP_DEBUG_ANY,
 		    "<= has_children -1 could not open \"dn2id%s\"\n",
 		    LDBM_SUFFIX, 0, 0 );
-#endif
 
 		return( 0 );
 	}
@@ -67,14 +64,8 @@ has_children(
 		rc = 1;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "has_children: id (%ld) %s children.\n",
-		   p->e_id, rc ? "has" : "doesn't have" ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= has_children( %ld ): %s\n",
 		p->e_id, rc ? "yes" : "no", 0 );
-#endif
 
 	return( rc );
 }
