@@ -1,8 +1,17 @@
 /* id2entry.c - routines to deal with the id2entry index */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/id2entry.c,v 1.29 2002/01/04 20:17:52 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/id2entry.c,v 1.38 2004/10/06 03:51:41 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -32,25 +41,14 @@ id2entry_add( Backend *be, Entry *e )
 	ldbm_datum_init( key );
 	ldbm_datum_init( data );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_add: (%s)%ld\n", e->e_dn, e->e_id ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> id2entry_add( %ld, \"%s\" )\n", e->e_id,
 	    e->e_dn, 0 );
-#endif
 
 
 	if ( (db = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, LDBM_WRCREAT ))
 	    == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_add: could not open/create id2entry%s\n",
-			   LDBM_SUFFIX ));
-#else
 		Debug( LDAP_DEBUG_ANY, "Could not open/create id2entry%s\n",
 		    LDBM_SUFFIX, 0, 0 );
-#endif
 
 		return( -1 );
 	}
@@ -75,12 +73,7 @@ id2entry_add( Backend *be, Entry *e )
 
 	ldbm_cache_close( be, db );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_add: return %d\n", rc ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= id2entry_add %d\n", rc, 0, 0 );
-#endif
 
 
 	return( rc );
@@ -97,13 +90,8 @@ id2entry_delete( Backend *be, Entry *e )
 	ID		id;
 #endif
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_delete: (%s)%ld\n", e->e_dn, e->e_id ));
-#else
 	Debug(LDAP_DEBUG_TRACE, "=> id2entry_delete( %ld, \"%s\" )\n", e->e_id,
 	    e->e_dn, 0 );
-#endif
 
 
 #ifdef notdef
@@ -117,27 +105,15 @@ id2entry_delete( Backend *be, Entry *e )
 
 	if ( (db = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, LDBM_WRCREAT ))
 		== NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_delete: could not open/create id2entry%s\n",
-			   LDBM_SUFFIX ));
-#else
 		Debug( LDAP_DEBUG_ANY, "Could not open/create id2entry%s\n",
 		    LDBM_SUFFIX, 0, 0 );
-#endif
 
 		return( -1 );
 	}
 
 	if ( cache_delete_entry( &li->li_cache, e ) != 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_delete: Could not delete (%s)%ld from cache\n",
-			   e->e_dn, e->e_id ));
-#else
 		Debug(LDAP_DEBUG_ANY, "could not delete %ld (%s) from cache\n",
 		    e->e_id, e->e_dn, 0 );
-#endif
 
 	}
 
@@ -153,12 +129,7 @@ id2entry_delete( Backend *be, Entry *e )
 
 	ldbm_cache_close( be, db );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_delete: return %d\n", rc ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= id2entry_delete %d\n", rc, 0, 0 );
-#endif
 
 	return( rc );
 }
@@ -178,38 +149,21 @@ id2entry_rw( Backend *be, ID id, int rw )
 	ldbm_datum_init( key );
 	ldbm_datum_init( data );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_rw: %s (%ld)\n",
-		   rw ? "write" : "read", id ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> id2entry_%s( %ld )\n",
 		rw ? "w" : "r", id, 0 );
-#endif
 
 
 	if ( (e = cache_find_entry_id( &li->li_cache, id, rw )) != NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-			   "id2entry_rw: %s (%ld) 0x%lx (cache).\n",
-			   rw ? "write" : "read", id, (unsigned long)e ));
-#else
 		Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) 0x%lx (cache)\n",
 			rw ? "w" : "r", id, (unsigned long) e );
-#endif
 
 		return( e );
 	}
 
 	if ( (db = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, LDBM_WRCREAT ))
 		== NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_rw: could not open id2entry%s\n", LDBM_SUFFIX ));
-#else
 		Debug( LDAP_DEBUG_ANY, "Could not open id2entry%s\n",
 		    LDBM_SUFFIX, 0, 0 );
-#endif
 
 		return( NULL );
 	}
@@ -225,13 +179,8 @@ id2entry_rw( Backend *be, ID id, int rw )
 	data = ldbm_cache_fetch( db, key );
 
 	if ( data.dptr == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_rw: (%ld) not found\n", id ));
-#else
 		Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) not found\n",
 			rw ? "w" : "r", id, 0 );
-#endif
 
 		ldbm_cache_close( be, db );
 		return( NULL );
@@ -242,21 +191,17 @@ id2entry_rw( Backend *be, ID id, int rw )
 	ldbm_cache_close( be, db );
 
 	if ( e == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_rw: %s of %ld failed\n",
-			   rw ? "write" : "read", id ));
-#else
 		Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) (failed)\n",
 			rw ? "w" : "r", id, 0 );
-#endif
 
 		return( NULL );
 	}
 
 	e->e_id = id;
 
-	if( cache_add_entry_rw( &li->li_cache, e, rw ) != 0 ) {
+	if ( slapMode == SLAP_SERVER_MODE
+			&& cache_add_entry_rw( &li->li_cache, e, rw ) != 0 )
+	{
 		entry_free( e );
 
 		/* XXX this is a kludge.
@@ -264,42 +209,26 @@ id2entry_rw( Backend *be, ID id, int rw )
 		 * There are many underlying race condtions in the cache/disk code.
 		 */
 		if ( (e = cache_find_entry_id( &li->li_cache, id, rw )) != NULL ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				   "id2entry_rw: %s of %ld 0x%lx (cache)\n",
-				   rw ? "write" : "read", id, (unsigned long)e ));
-#else
 			Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) 0x%lx (cache)\n",
 				rw ? "w" : "r", id, (unsigned long) e );
-#endif
 
 			return( e );
 		}
 
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "id2entry_rw: %s of %ld (cache add failed)\n",
-			   rw ? "write" : "read", id ));
-#else
 		Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) (cache add failed)\n",
 			rw ? "w" : "r", id, 0 );
-#endif
 
 		return NULL;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ENTRY,
-		   "id2entry_rw: %s of %ld 0x%lx (disk)\n",
-		   rw ? "write" : "read", id, (unsigned long)e ));
-#else
 	Debug( LDAP_DEBUG_TRACE, "<= id2entry_%s( %ld ) 0x%lx (disk)\n",
 		rw ? "w" : "r", id, (unsigned long) e );
-#endif
 
-	/* marks the entry as committed, so it will get added to the cache
-	 * when the lock is released */
-	cache_entry_commit( e );
+	if ( slapMode == SLAP_SERVER_MODE ) {
+		/* marks the entry as committed, so it will get added to the cache
+		 * when the lock is released */
+		cache_entry_commit( e );
+	}
 
 	return( e );
 }
