@@ -1,67 +1,23 @@
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/compare.c,v 1.30.2.1 2004/12/30 21:29:58 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001, Pierangelo Masarati, All rights reserved. <ando@sys-net.it>
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 2001-2003 Pierangelo Masarati.
+ * Portions Copyright 1999-2003 Howard Chu.
+ * All rights reserved.
  *
- * This work has been developed to fulfill the requirements
- * of SysNet s.n.c. <http:www.sys-net.it> and it has been donated
- * to the OpenLDAP Foundation in the hope that it may be useful
- * to the Open Source community, but WITHOUT ANY WARRANTY.
- *
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
- *
- * 1. The author and SysNet s.n.c. are not responsible for the consequences
- *    of use of this software, no matter how awful, even if they arise from 
- *    flaws in it.
- *
- * 2. The origin of this software must not be misrepresented, either by
- *    explicit claim or by omission.  Since few users ever read sources,
- *    credits should appear in the documentation.
- *
- * 3. Altered versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.  Since few users
- *    ever read sources, credits should appear in the documentation.
- *    SysNet s.n.c. cannot be responsible for the consequences of the
- *    alterations.
- * 
- * 4. This notice may not be removed or altered.
- *
- *
- * This software is based on the backend back-ldap, implemented
- * by Howard Chu <hyc@highlandsun.com>, and modified by Mark Valence
- * <kurash@sassafras.com>, Pierangelo Masarati <ando@sys-net.it> and other
- * contributors. The contribution of the original software to the present
- * implementation is acknowledged in this copyright statement.
- *
- * A special acknowledgement goes to Howard for the overall architecture
- * (and for borrowing large pieces of code), and to Mark, who implemented
- * from scratch the attribute/objectclass mapping.
- *
- * The original copyright statement follows.
- *
- * Copyright 1999, Howard Chu, All rights reserved. <hyc@highlandsun.com>
- *
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
- *
- * 1. The author is not responsible for the consequences of use of this
- *    software, no matter how awful, even if they arise from flaws in it.
- *
- * 2. The origin of this software must not be misrepresented, either by
- *    explicit claim or by omission.  Since few users ever read sources,
- *    credits should appear in the documentation.
- *
- * 3. Altered versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.  Since few users
- *    ever read sources, credits should appear in the
- *    documentation.
- *
- * 4. This notice may not be removed or altered.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
  *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by the Howard Chu for inclusion
+ * in OpenLDAP Software and subsequently enhanced by Pierangelo
+ * Masarati.
  */
 
 #include "portable.h"
@@ -82,7 +38,7 @@ meta_back_compare( Operation *op, SlapReply *rs )
 	struct metaconn *lc;
 	struct metasingleconn *lsc;
 	char *match = NULL, *err = NULL;
-	struct berval mmatch = { 0, NULL };
+	struct berval mmatch = BER_BVNULL;
 	int candidates = 0, last = 0, i, count = 0, rc;
        	int cres = LDAP_SUCCESS, rres = LDAP_SUCCESS;
 	int *msgid;
@@ -111,10 +67,10 @@ meta_back_compare( Operation *op, SlapReply *rs )
 	 */
 	dc.conn = op->o_conn;
 	dc.rs = rs;
-	dc.ctx = "compareDn";
+	dc.ctx = "compareDN";
 
 	for ( i = 0, lsc = lc->conns; !META_LAST(lsc); ++i, ++lsc ) {
-		struct berval mdn = { 0, NULL };
+		struct berval mdn = BER_BVNULL;
 		struct berval mapped_attr = op->oq_compare.rs_ava->aa_desc->ad_cname;
 		struct berval mapped_value = op->oq_compare.rs_ava->aa_value;
 
@@ -158,6 +114,21 @@ meta_back_compare( Operation *op, SlapReply *rs )
 			if ( mapped_attr.bv_val == NULL || mapped_attr.bv_val[0] == '\0' ) {
 				continue;
 			}
+
+			if ( op->oq_compare.rs_ava->aa_desc->ad_type->sat_syntax == slap_schema.si_syn_distinguishedName )
+			{
+				dc.ctx = "compareAttrDN";
+
+				switch ( ldap_back_dn_massage( &dc, &op->oq_compare.rs_ava->aa_value, &mapped_value ) )
+				{
+				case LDAP_UNWILLING_TO_PERFORM:
+					rc = 1;
+					goto finish;
+
+				default:
+					break;
+				}
+			}
 		}
 		
 		/*
@@ -167,13 +138,16 @@ meta_back_compare( Operation *op, SlapReply *rs )
 		 */
 		msgid[ i ] = ldap_compare( lc->conns[ i ].ld, mdn.bv_val,
 				mapped_attr.bv_val, mapped_value.bv_val );
+
 		if ( mdn.bv_val != op->o_req_dn.bv_val ) {
 			free( mdn.bv_val );
 			mdn.bv_val = NULL;
 		}
+
 		if ( mapped_attr.bv_val != op->oq_compare.rs_ava->aa_desc->ad_cname.bv_val ) {
 			free( mapped_attr.bv_val );
 		}
+
 		if ( mapped_value.bv_val != op->oq_compare.rs_ava->aa_value.bv_val ) {
 			free( mapped_value.bv_val );
 		}
@@ -238,7 +212,7 @@ meta_back_compare( Operation *op, SlapReply *rs )
 					break;
 
 				default:
-					rres = ldap_back_map_result( rs );
+					rres = slap_map_api2result( rs );
 
 					if ( err != NULL ) {
 						free( err );
@@ -298,11 +272,13 @@ finish:;
 		matched.bv_val = match;
 		matched.bv_len = strlen( match );
 
-		dc.ctx = "matchedDn";
+		dc.ctx = "matchedDN";
 		ldap_back_dn_massage( &dc, &matched, &mmatch );
 	}
 
-	rs->sr_err = rres;
+	if ( rres != LDAP_SUCCESS ) {
+		rs->sr_err = rres;
+	}
 	rs->sr_matched = mmatch.bv_val;
 	send_ldap_result( op, rs );
 	rs->sr_matched = NULL;
