@@ -1,67 +1,23 @@
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/search.c,v 1.84.2.1 2004/12/30 21:29:58 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001, Pierangelo Masarati, All rights reserved. <ando@sys-net.it>
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 2001-2003 Pierangelo Masarati.
+ * Portions Copyright 1999-2003 Howard Chu.
+ * All rights reserved.
  *
- * This work has been developed to fulfill the requirements
- * of SysNet s.n.c. <http:www.sys-net.it> and it has been donated
- * to the OpenLDAP Foundation in the hope that it may be useful
- * to the Open Source community, but WITHOUT ANY WARRANTY.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
  *
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
- *
- * 1. The author and SysNet s.n.c. are not responsible for the consequences
- *    of use of this software, no matter how awful, even if they arise from 
- *    flaws in it.
- *
- * 2. The origin of this software must not be misrepresented, either by
- *    explicit claim or by omission.  Since few users ever read sources,
- *    credits should appear in the documentation.
- *
- * 3. Altered versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.  Since few users
- *    ever read sources, credits should appear in the documentation.
- *    SysNet s.n.c. cannot be responsible for the consequences of the
- *    alterations.
- *
- * 4. This notice may not be removed or altered.
- *
- *
- * This software is based on the backend back-ldap, implemented
- * by Howard Chu <hyc@highlandsun.com>, and modified by Mark Valence
- * <kurash@sassafras.com>, Pierangelo Masarati <ando@sys-net.it> and other
- * contributors. The contribution of the original software to the present
- * implementation is acknowledged in this copyright statement.
- *
- * A special acknowledgement goes to Howard for the overall architecture
- * (and for borrowing large pieces of code), and to Mark, who implemented
- * from scratch the attribute/objectclass mapping.
- *
- * The original copyright statement follows.
- *
- * Copyright 1999, Howard Chu, All rights reserved. <hyc@highlandsun.com>
- *
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
- *
- * 1. The author is not responsible for the consequences of use of this
- *    software, no matter how awful, even if they arise from flaws in it.
- *
- * 2. The origin of this software must not be misrepresented, either by
- *    explicit claim or by omission.  Since few users ever read sources,
- *    credits should appear in the documentation.
- *
- * 3. Altered versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.  Since few users
- *    ever read sources, credits should appear in the
- *    documentation.
- *
- * 4. This notice may not be removed or altered.
- *                
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by the Howard Chu for inclusion
+ * in OpenLDAP Software and subsequently enhanced by Pierangelo
+ * Masarati.
  */
 
 #include "portable.h"
@@ -75,7 +31,6 @@
 #include "slap.h"
 #include "../back-ldap/back-ldap.h"
 #include "back-meta.h"
-#include "ldap_pvt.h"
 #undef ldap_debug	/* silence a warning in ldap-int.h */
 #include "ldap_log.h"
 #include "../../../libraries/libldap/ldap-int.h"
@@ -89,12 +44,6 @@ meta_send_entry(
 		LDAPMessage 	*e
 );
 
-static int
-is_one_level_rdn(
-		const char	*rdn,
-		int		from
-);
-
 int
 meta_back_search( Operation *op, SlapReply *rs )
 {
@@ -102,26 +51,19 @@ meta_back_search( Operation *op, SlapReply *rs )
 	struct metaconn *lc;
 	struct metasingleconn *lsc;
 	struct timeval	tv = { 0, 0 };
-	LDAPMessage	*res, *e;
-	int	rc = 0, *msgid, sres = LDAP_NO_SUCH_OBJECT;
+	LDAPMessage	*res = NULL, *e;
+	int	rc = 0, *msgid, sres = LDAP_SUCCESS;
 	char *err = NULL;
-	struct berval match = { 0, NULL }, mmatch = { 0, NULL };
+	struct berval match = BER_BVNULL, mmatch = BER_BVNULL;
 	BerVarray v2refs = NULL;
 		
 	int i, last = 0, candidates = 0, initial_candidates = 0,
 			candidate_match = 0;
-	struct slap_limits_set *limit = NULL;
-	int isroot = 0;
 	dncookie dc;
 
-#ifdef LDAP_CACHING
-	cache_manager*  cm = li->cm;
+	int	is_scope = 0,
+		is_filter = 0;
 
- 	if (cm->caching) {
- 		return meta_back_cache_search(op, rs);
-	}
-#endif /* LDAP_CACHING */
-	
 	/*
 	 * controls are set in ldap_back_dobind()
 	 * 
@@ -151,54 +93,6 @@ meta_back_search( Operation *op, SlapReply *rs )
 		return -1;
 	}
 	
-	/* if not root, get appropriate limits */
-	if ( be_isroot( op->o_bd, &op->o_ndn ) ) {
-		isroot = 1;
-	} else {
-		( void ) get_limits( op->o_bd, &op->o_ndn, &limit );
-	}
-
-	/* if no time limit requested, rely on remote server limits */
-	/* if requested limit higher than hard limit, abort */
-	if ( !isroot && op->oq_search.rs_tlimit > limit->lms_t_hard ) {
-		/* no hard limit means use soft instead */
-		if ( limit->lms_t_hard == 0
-				&& limit->lms_t_soft > -1
-				&& op->oq_search.rs_tlimit > limit->lms_t_soft ) {
-			op->oq_search.rs_tlimit = limit->lms_t_soft;
-			
-		/* positive hard limit means abort */
-		} else if ( limit->lms_t_hard > 0 ) {
-			rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
-			send_ldap_result( op, rs );
-			rc = 0;
-			goto finish;
-		}
-		
-		/* negative hard limit means no limit */
-	}
-	
-	/* if no size limit requested, rely on remote server limits */
-	/* if requested limit higher than hard limit, abort */
-	if ( !isroot && op->oq_search.rs_slimit > limit->lms_s_hard ) {
-		/* no hard limit means use soft instead */
-		if ( limit->lms_s_hard == 0
-				&& limit->lms_s_soft > -1
-				&& op->oq_search.rs_slimit > limit->lms_s_soft ) {
-			op->oq_search.rs_slimit = limit->lms_s_soft;
-			
-		/* positive hard limit means abort */
-		} else if ( limit->lms_s_hard > 0 ) {
-			rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
-			send_ldap_result( op, rs );
-			rc = 0;
-			goto finish;
-		}
-		
-		/* negative hard limit means no limit */
-	}
-
-
 	dc.conn = op->o_conn;
 	dc.rs = rs;
 
@@ -207,10 +101,10 @@ meta_back_search( Operation *op, SlapReply *rs )
 	 */
 	for ( i = 0, lsc = lc->conns; !META_LAST(lsc); ++i, ++lsc ) {
 		struct berval	realbase = op->o_req_dn;
-		int		realscope = op->oq_search.rs_scope;
+		int		realscope = op->ors_scope;
 		ber_len_t	suffixlen = 0;
-		struct berval	mbase = { 0, NULL }; 
-		struct berval	mfilter = { 0, NULL };
+		struct berval	mbase = BER_BVNULL; 
+		struct berval	mfilter = BER_BVNULL;
 		char		**mapped_attrs = NULL;
 
 		if ( lsc->candidate != META_CANDIDATE ) {
@@ -219,17 +113,17 @@ meta_back_search( Operation *op, SlapReply *rs )
 		}
 
 		/* should we check return values? */
-		if ( op->oq_search.rs_deref != -1 ) {
+		if ( op->ors_deref != -1 ) {
 			ldap_set_option( lsc->ld, LDAP_OPT_DEREF,
-					( void * )&op->oq_search.rs_deref);
+					( void * )&op->ors_deref);
 		}
-		if ( op->oq_search.rs_tlimit != -1 ) {
+		if ( op->ors_tlimit != SLAP_NO_LIMIT ) {
 			ldap_set_option( lsc->ld, LDAP_OPT_TIMELIMIT,
-					( void * )&op->oq_search.rs_tlimit);
+					( void * )&op->ors_tlimit);
 		}
-		if ( op->oq_search.rs_slimit != -1 ) {
+		if ( op->ors_slimit != SLAP_NO_LIMIT ) {
 			ldap_set_option( lsc->ld, LDAP_OPT_SIZELIMIT,
-					( void * )&op->oq_search.rs_slimit);
+					( void * )&op->ors_slimit);
 		}
 
 		dc.rwmap = &li->targets[ i ]->rwmap;
@@ -239,7 +133,7 @@ meta_back_search( Operation *op, SlapReply *rs )
 		 */
 		suffixlen = li->targets[ i ]->suffix.bv_len;
 		if ( suffixlen > op->o_req_ndn.bv_len ) {
-			switch ( op->oq_search.rs_scope ) {
+			switch ( op->ors_scope ) {
 			case LDAP_SCOPE_SUBTREE:
 				/*
 				 * make the target suffix the new base
@@ -250,6 +144,8 @@ meta_back_search( Operation *op, SlapReply *rs )
 				if ( dnIsSuffix( &li->targets[ i ]->suffix,
 						&op->o_req_ndn ) ) {
 					realbase = li->targets[ i ]->suffix;
+					is_scope++;
+
 				} else {
 					/*
 					 * this target is no longer candidate
@@ -260,9 +156,12 @@ meta_back_search( Operation *op, SlapReply *rs )
 				break;
 
 			case LDAP_SCOPE_ONELEVEL:
-				if ( is_one_level_rdn( li->targets[ i ]->suffix.bv_val,
-						suffixlen - op->o_req_ndn.bv_len - 1 ) 
-			&& dnIsSuffix( &li->targets[ i ]->suffix, &op->o_req_ndn ) ) {
+			{
+				struct berval	rdn = li->targets[ i ]->suffix;
+				rdn.bv_len -= op->o_req_ndn.bv_len + STRLENOF( "," );
+				if ( dnIsOneLevelRDN( &rdn )
+						&& dnIsSuffix( &li->targets[ i ]->suffix, &op->o_req_ndn ) )
+				{
 					/*
 					 * if there is exactly one level,
 					 * make the target suffix the new
@@ -270,8 +169,10 @@ meta_back_search( Operation *op, SlapReply *rs )
 					 */
 					realbase = li->targets[ i ]->suffix;
 					realscope = LDAP_SCOPE_BASE;
+					is_scope++;
 					break;
 				} /* else continue with the next case */
+			}
 
 			case LDAP_SCOPE_BASE:
 				/*
@@ -281,6 +182,8 @@ meta_back_search( Operation *op, SlapReply *rs )
 				goto new_candidate;
 			}
 
+		} else {
+			is_scope++;
 		}
 
 		/*
@@ -318,9 +221,17 @@ meta_back_search( Operation *op, SlapReply *rs )
 		 * Maps filter
 		 */
 		rc = ldap_back_filter_map_rewrite( &dc,
-				op->oq_search.rs_filter,
+				op->ors_filter,
 				&mfilter, BACKLDAP_MAP );
-		if ( rc != 0 ) {
+		switch ( rc ) {
+		case LDAP_SUCCESS:
+			is_filter++;
+			break;
+
+		case LDAP_COMPARE_FALSE:
+			rc = 0;
+
+		default:
 			/*
 			 * this target is no longer candidate
 			 */
@@ -332,7 +243,7 @@ meta_back_search( Operation *op, SlapReply *rs )
 		 * Maps required attributes
 		 */
 		rc = ldap_back_map_attrs( &li->targets[ i ]->rwmap.rwm_at,
-				op->oq_search.rs_attrs, BACKLDAP_MAP,
+				op->ors_attrs, BACKLDAP_MAP,
 				&mapped_attrs );
 		if ( rc != LDAP_SUCCESS ) {
 			/*
@@ -347,12 +258,12 @@ meta_back_search( Operation *op, SlapReply *rs )
 		 */
 		msgid[ i ] = ldap_search( lsc->ld, mbase.bv_val, realscope,
 				mfilter.bv_val, mapped_attrs,
-				op->oq_search.rs_attrsonly ); 
+				op->ors_attrsonly ); 
 		if ( mapped_attrs ) {
 			free( mapped_attrs );
 			mapped_attrs = NULL;
 		}
-		if ( mfilter.bv_val != op->oq_search.rs_filterstr.bv_val ) {
+		if ( mfilter.bv_val != op->ors_filterstr.bv_val ) {
 			free( mfilter.bv_val );
 			mfilter.bv_val = NULL;
 		}
@@ -401,8 +312,8 @@ new_candidate:;
 				break;
 			}
 
-			if ( op->oq_search.rs_slimit > 0
-					&& rs->sr_nentries == op->oq_search.rs_slimit ) {
+			if ( op->ors_slimit > 0
+					&& rs->sr_nentries == op->ors_slimit ) {
 				rs->sr_err = LDAP_SIZELIMIT_EXCEEDED;
 				rs->sr_v2ref = v2refs;
 				send_ldap_result( op, rs );
@@ -420,6 +331,11 @@ new_candidate:;
 					0, &tv, &res );
 
 			if ( rc == 0 ) {
+				/* timeout exceeded */
+
+				/* FIXME: res should not need to be freed */
+				assert( res == NULL );
+
 				continue;
 
 			} else if ( rc == -1 ) {
@@ -431,12 +347,19 @@ new_candidate:;
 				send_ldap_result( op, rs );
 				
 				/* anything else needs be done? */
+
+				/* FIXME: res should not need to be freed */
+				assert( res == NULL );
+
 				goto finish;
 
 			} else if ( rc == LDAP_RES_SEARCH_ENTRY ) {
 				e = ldap_first_entry( lsc->ld, res );
 				meta_send_entry( op, rs, lc, i, e );
 
+				ldap_msgfree( res );
+				res = NULL;
+
 				/*
 				 * If scope is BASE, we need to jump out
 				 * as soon as one entry is found; if
@@ -444,26 +367,22 @@ new_candidate:;
 				 * this should correspond to the sole
 				 * entry that has the base DN
 				 */
-				if ( op->oq_search.rs_scope == LDAP_SCOPE_BASE
+				if ( op->ors_scope == LDAP_SCOPE_BASE
 						&& rs->sr_nentries > 0 ) {
 					candidates = 0;
 					sres = LDAP_SUCCESS;
 					break;
 				}
-				ldap_msgfree( res );
+
 				gotit = 1;
 
 			} else if ( rc == LDAP_RES_SEARCH_REFERENCE ) {
 				char		**references = NULL;
 				int		cnt;
 
-				/*
-				 * FIXME: should we collect references
-				 * and send them alltogether at the end?
-				 */
-
 				rc = ldap_parse_reference( lsc->ld, res,
 						&references, &rs->sr_ctrls, 1 );
+				res = NULL;
 
 				if ( rc != LDAP_SUCCESS ) {
 					continue;
@@ -473,24 +392,35 @@ new_candidate:;
 					continue;
 				}
 
+#ifdef ENABLE_REWRITE
+				dc.ctx = "referralDN";
+#else /* ! ENABLE_REWRITE */
+				dc.tofrom = 0;
+				dc.normalized = 0;
+#endif /* ! ENABLE_REWRITE */
 				for ( cnt = 0; references[ cnt ]; cnt++ )
-					/* NO OP */ ;
-				
-				rs->sr_ref = ch_calloc( cnt + 1, sizeof( struct berval ) );
+					;
+
+				rs->sr_ref = ch_calloc( sizeof( struct berval ), cnt + 1 );
 
 				for ( cnt = 0; references[ cnt ]; cnt++ ) {
-					rs->sr_ref[ cnt ].bv_val = references[ cnt ];
-					rs->sr_ref[ cnt ].bv_len = strlen( references[ cnt ] );
+					ber_str2bv( references[ cnt ], 0, 1, &rs->sr_ref[ cnt ] );
 				}
+				BER_BVZERO( &rs->sr_ref[ cnt ] );
 
-				/* ignore return value by now */
-				( void )send_search_reference( op, rs );
+				( void )ldap_back_referral_result_rewrite( &dc, rs->sr_ref );
+
+				if ( rs->sr_ref != NULL && !BER_BVISNULL( &rs->sr_ref[ 0 ] ) ) {
+					/* ignore return value by now */
+					( void )send_search_reference( op, rs );
+
+					ber_bvarray_free( rs->sr_ref );
+					rs->sr_ref = NULL;
+				}
 
 				/* cleanup */
 				if ( references ) {
 					ldap_value_free( references );
-					ch_free( rs->sr_ref );
-					rs->sr_ref = NULL;
 				}
 
 				if ( rs->sr_ctrls ) {
@@ -501,7 +431,9 @@ new_candidate:;
 			} else {
 				rs->sr_err = ldap_result2error( lsc->ld,
 						res, 1 );
-				sres = ldap_back_map_result( rs );
+				res = NULL;
+
+				sres = slap_map_api2result( rs );
 				if ( err != NULL ) {
 					free( err );
 				}
@@ -513,17 +445,10 @@ new_candidate:;
 				ldap_get_option( lsc->ld,
 						LDAP_OPT_MATCHED_DN, &match.bv_val );
 
-#ifdef NEW_LOGGING
-				LDAP_LOG( BACK_META, ERR,
-					"meta_back_search [%d] "
-					"match=\"%s\" err=\"%s\"\n",
-					i, match.bv_val, err );
-#else /* !NEW_LOGGING */
 				Debug( LDAP_DEBUG_ANY,
 					"=>meta_back_search [%d] "
 					"match=\"%s\" err=\"%s\"\n",
      					i, match.bv_val, err );	
-#endif /* !NEW_LOGGING */
 				candidate_match++;
 				last = i;
 				rc = 0;
@@ -566,7 +491,7 @@ new_candidate:;
 	 */
 	if ( candidate_match == initial_candidates
 			&& match.bv_val != NULL && *match.bv_val ) {
-		dc.ctx = "matchedDn";
+		dc.ctx = "matchedDN";
 		dc.rwmap = &li->targets[ last ]->rwmap;
 
 		if ( ldap_back_dn_massage( &dc, &match, &mmatch ) ) {
@@ -581,6 +506,10 @@ new_candidate:;
 	 * FIXME: we should handle error codes and return the more 
 	 * important/reasonable
 	 */
+	if ( is_scope == 0 ) {
+		sres = LDAP_NO_SUCH_OBJECT;
+	}
+
 	if ( sres == LDAP_SUCCESS && v2refs ) {
 		sres = LDAP_REFERRAL;
 	}
@@ -622,10 +551,9 @@ meta_send_entry(
 {
 	struct metainfo 	*li = ( struct metainfo * )op->o_bd->be_private;
 	struct berval		a, mapped;
-	Entry 			ent;
+	Entry 			ent = {0};
 	BerElement 		ber = *e->lm_ber;
 	Attribute 		*attr, **attrp;
-	struct berval 		dummy = { 0, NULL };
 	struct berval 		*bv, bdn;
 	const char 		*text;
 	dncookie		dc;
@@ -655,7 +583,7 @@ meta_send_entry(
 	 * FIXME: should we log anything, or delegate to dnNormalize?
 	 */
 	if ( dnNormalize( 0, NULL, NULL, &ent.e_name, &ent.e_nname,
-		&op->o_tmpmemctx ) != LDAP_SUCCESS )
+		op->o_tmpmemctx ) != LDAP_SUCCESS )
 	{
 		return LDAP_INVALID_DN_SYNTAX;
 	}
@@ -668,13 +596,12 @@ meta_send_entry(
 				&ent.e_nname, target );
 	}
 
-	ent.e_id = 0;
-	ent.e_attrs = 0;
-	ent.e_private = 0;
 	attrp = &ent.e_attrs;
 
 	dc.ctx = "searchAttrDN";
 	while ( ber_scanf( &ber, "{m", &a ) != LBER_ERROR ) {
+		int		last = 0;
+
 		ldap_back_map( &li->targets[ target ]->rwmap.rwm_at, 
 				&a, &mapped, BACKLDAP_REMAP );
 		if ( mapped.bv_val == NULL || mapped.bv_val[0] == '\0' ) {
@@ -691,14 +618,9 @@ meta_send_entry(
 				!= LDAP_SUCCESS) {
 			if ( slap_bv2undef_ad( &mapped, &attr->a_desc, &text ) 
 					!= LDAP_SUCCESS) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( BACK_META, DETAIL1,
-					"slap_bv2undef_ad(%s): %s\n", mapped.bv_val, text, 0 );
-#else /* !NEW_LOGGING */
 				Debug( LDAP_DEBUG_ANY,
 						"slap_bv2undef_ad(%s): "
 						"%s\n%s", mapped.bv_val, text, "" );
-#endif /* !NEW_LOGGING */
 				ch_free( attr );
 				continue;
 			}
@@ -722,11 +644,10 @@ meta_send_entry(
 
 		if ( ber_scanf( &ber, "[W]", &attr->a_vals ) == LBER_ERROR 
 				|| attr->a_vals == NULL ) {
-			attr->a_vals = &dummy;
+			attr->a_vals = &slap_dummy_bv;
 
 		} else if ( attr->a_desc == slap_schema.si_ad_objectClass
 				|| attr->a_desc == slap_schema.si_ad_structuralObjectClass ) {
-			int		last;
 
 			for ( last = 0; attr->a_vals[ last ].bv_val; ++last );
 
@@ -762,20 +683,43 @@ meta_send_entry(
 		} else if ( attr->a_desc->ad_type->sat_syntax ==
 				slap_schema.si_syn_distinguishedName ) {
 			ldap_dnattr_result_rewrite( &dc, attr->a_vals );
+
+		} else if ( attr->a_desc == slap_schema.si_ad_ref ) {
+			ldap_back_referral_result_rewrite( &dc, attr->a_vals );
+		}
+
+		if ( last && attr->a_desc->ad_type->sat_equality &&
+			attr->a_desc->ad_type->sat_equality->smr_normalize ) {
+			int i;
+
+			attr->a_nvals = ch_malloc((last + 1)*sizeof(struct berval));
+			for ( i = 0; i<last; i++ ) {
+				attr->a_desc->ad_type->sat_equality->smr_normalize(
+					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+					attr->a_desc->ad_type->sat_syntax,
+					attr->a_desc->ad_type->sat_equality,
+					&attr->a_vals[i], &attr->a_nvals[i],
+					NULL );
+			}
+			attr->a_nvals[i].bv_val = NULL;
+			attr->a_nvals[i].bv_len = 0;
+		} else {
+			attr->a_nvals = attr->a_vals;
 		}
 
 		*attrp = attr;
 		attrp = &attr->a_next;
 	}
 	rs->sr_entry = &ent;
-	rs->sr_attrs = op->oq_search.rs_attrs;
+	rs->sr_attrs = op->ors_attrs;
+	rs->sr_flags = 0;
 	send_search_entry( op, rs );
 	rs->sr_entry = NULL;
 	rs->sr_attrs = NULL;
 	while ( ent.e_attrs ) {
 		attr = ent.e_attrs;
 		ent.e_attrs = attr->a_next;
-		if ( attr->a_vals != &dummy ) {
+		if ( attr->a_vals != &slap_dummy_bv ) {
 			ber_bvarray_free( attr->a_vals );
 		}
 		free( attr );
@@ -791,18 +735,4 @@ meta_send_entry(
 	return LDAP_SUCCESS;
 }
 
-static int
-is_one_level_rdn(
-		const char 	*rdn,
-		int 		from
-)
-{
-	for ( ; from--; ) {
-		if ( DN_SEPARATOR( rdn[ from ] ) ) {
-			return 0;
-		}
-	}
-
-	return 1;
-}
 
