@@ -1,34 +1,22 @@
 /* log.c - deal with log subsystem */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/log.c,v 1.45 2004/11/08 17:52:27 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2001-2004 The OpenLDAP Foundation.
+ * Portions Copyright 2001-2003 Pierangelo Masarati.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright 2001, Pierangelo Masarati, All rights reserved. <ando@sys-net.it>
- * 
- * This work has beed deveolped for the OpenLDAP Foundation 
- * in the hope that it may be useful to the Open Source community, 
- * but WITHOUT ANY WARRANTY.
- * 
- * Permission is granted to anyone to use this software for any purpose
- * on any computer system, and to alter it and redistribute it, subject
- * to the following restrictions:
- * 
- * 1. The author and SysNet s.n.c. are not responsible for the consequences
- *    of use of this software, no matter how awful, even if they arise from
- *    flaws in it.
- * 
- * 2. The origin of this software must not be misrepresented, either by
- *    explicit claim or by omission.  Since few users ever read sources,
- *    credits should appear in the documentation.
- * 
- * 3. Altered versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.  Since few users
- *    ever read sources, credits should appear in the documentation.
- *    SysNet s.n.c. cannot be responsible for the consequences of the
- *    alterations.
- * 
- * 4. This notice may not be removed or altered.
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Pierangelo Masarati for inclusion
+ * in OpenLDAP Software.
  */
 
 #include "portable.h"
@@ -53,21 +41,21 @@ static struct {
 	struct berval s;
 	struct berval n;
 } int_2_level[] = {
-	{ LDAP_DEBUG_TRACE,	BER_BVC("Trace"),	{ 0, NULL } },
-	{ LDAP_DEBUG_PACKETS,	BER_BVC("Packets"),	{ 0, NULL } },
-	{ LDAP_DEBUG_ARGS,	BER_BVC("Args"),	{ 0, NULL } },
-	{ LDAP_DEBUG_CONNS,	BER_BVC("Conns"),	{ 0, NULL } },
-	{ LDAP_DEBUG_BER,	BER_BVC("BER"),	{ 0, NULL } },
-	{ LDAP_DEBUG_FILTER,	BER_BVC("Filter"),	{ 0, NULL } },
-	{ LDAP_DEBUG_CONFIG,	BER_BVC("Config"),	{ 0, NULL } },	/* useless */
-	{ LDAP_DEBUG_ACL,	BER_BVC("ACL"),	{ 0, NULL } },
-	{ LDAP_DEBUG_STATS,	BER_BVC("Stats"),	{ 0, NULL } },
-	{ LDAP_DEBUG_STATS2,	BER_BVC("Stats2"),	{ 0, NULL } },
-	{ LDAP_DEBUG_SHELL,	BER_BVC("Shell"),	{ 0, NULL } },
-	{ LDAP_DEBUG_PARSE,	BER_BVC("Parse"),	{ 0, NULL } },
-	{ LDAP_DEBUG_CACHE,	BER_BVC("Cache"),	{ 0, NULL } },
-	{ LDAP_DEBUG_INDEX,	BER_BVC("Index"),	{ 0, NULL } },
-	{ 0,			{ 0, NULL },	{ 0, NULL } }
+	{ LDAP_DEBUG_TRACE,	BER_BVC("Trace"),	BER_BVNULL },
+	{ LDAP_DEBUG_PACKETS,	BER_BVC("Packets"),	BER_BVNULL },
+	{ LDAP_DEBUG_ARGS,	BER_BVC("Args"),	BER_BVNULL },
+	{ LDAP_DEBUG_CONNS,	BER_BVC("Conns"),	BER_BVNULL },
+	{ LDAP_DEBUG_BER,	BER_BVC("BER"),	BER_BVNULL },
+	{ LDAP_DEBUG_FILTER,	BER_BVC("Filter"),	BER_BVNULL },
+	{ LDAP_DEBUG_CONFIG,	BER_BVC("Config"),	BER_BVNULL },	/* useless */
+	{ LDAP_DEBUG_ACL,	BER_BVC("ACL"),	BER_BVNULL },
+	{ LDAP_DEBUG_STATS,	BER_BVC("Stats"),	BER_BVNULL },
+	{ LDAP_DEBUG_STATS2,	BER_BVC("Stats2"),	BER_BVNULL },
+	{ LDAP_DEBUG_SHELL,	BER_BVC("Shell"),	BER_BVNULL },
+	{ LDAP_DEBUG_PARSE,	BER_BVC("Parse"),	BER_BVNULL },
+	{ LDAP_DEBUG_CACHE,	BER_BVC("Cache"),	BER_BVNULL },
+	{ LDAP_DEBUG_INDEX,	BER_BVC("Index"),	BER_BVNULL },
+	{ 0,			BER_BVNULL,	BER_BVNULL }
 };
 
 static int loglevel2int( struct berval *l );
@@ -82,47 +70,39 @@ static int replace_values( Entry *e, Modification *mod, int *newlevel );
  */
 int
 monitor_subsys_log_init(
-	BackendDB	*be
+	BackendDB		*be,
+	monitor_subsys_t	*ms
 )
 {
-	struct monitorinfo	*mi;
-	Entry			*e;
-	int			i;
-	struct berval		desc[] = {
+	monitor_info_t	*mi;
+	Entry		*e;
+	int		i;
+	struct berval	desc[] = {
 		BER_BVC("This entry allows to set the log level runtime."),
 		BER_BVC("Set the attribute 'managedInfo' to the desired log levels."),
-		{ 0, NULL }
+		BER_BVNULL
 	};
 
 	ldap_pvt_thread_mutex_init( &monitor_log_mutex );
 
-	mi = ( struct monitorinfo * )be->be_private;
+	mi = ( monitor_info_t * )be->be_private;
 
-	if ( monitor_cache_get( mi, &monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn, 
+	if ( monitor_cache_get( mi, &ms->mss_ndn, 
 				&e ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, CRIT,
-			"monitor_subsys_log_init: "
-			"unable to get entry '%s'\n",
-			monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn.bv_val, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_log_init: "
-			"unable to get entry '%s'\n%s%s",
-			monitor_subsys[SLAPD_MONITOR_LOG].mss_ndn.bv_val, 
-			"", "" );
-#endif
+			"unable to get entry \"%s\"\n",
+			ms->mss_ndn.bv_val, 0, 0 );
 		return( -1 );
 	}
 
 	/* initialize the debug level(s) */
 	for ( i = 0; int_2_level[ i ].i != 0; i++ ) {
-
 		if ( mi->mi_ad_managedInfo->ad_type->sat_equality->smr_normalize ) {
 			int	rc;
 
 			rc = (*mi->mi_ad_managedInfo->ad_type->sat_equality->smr_normalize)(
-					0,
+					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
 					mi->mi_ad_managedInfo->ad_type->sat_syntax,
 					mi->mi_ad_managedInfo->ad_type->sat_equality,
 					&int_2_level[ i ].s,
@@ -152,7 +132,7 @@ monitor_subsys_log_modify(
 	Entry 			*e
 )
 {
-	struct monitorinfo *mi = (struct monitorinfo *)op->o_bd->be_private;
+	monitor_info_t	*mi = ( monitor_info_t * )op->o_bd->be_private;
 	int		rc = LDAP_OTHER;
 	int		newlevel = ldap_syslog;
 	Attribute	*save_attrs;
@@ -172,8 +152,7 @@ monitor_subsys_log_modify(
 		 */
 		if ( is_at_operational( mod->sm_desc->ad_type ) ) {
 			( void ) attr_delete( &e->e_attrs, mod->sm_desc );
-			rc = attr_merge( e, mod->sm_desc, mod->sm_bvalues,
-					mod->sm_nvalues );
+			rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
 			if ( rc != 0 ) {
 				rc = LDAP_OTHER;
 				break;
@@ -295,10 +274,10 @@ check_constraints( Modification *mod, int *newlevel )
 {
 	int		i;
 
-	for ( i = 0; mod->sm_bvalues && mod->sm_bvalues[i].bv_val != NULL; i++ ) {
+	for ( i = 0; mod->sm_values && !BER_BVISNULL( &mod->sm_values[ i ] ); i++ ) {
 		int		l;
 		
-		l = loglevel2int( &mod->sm_bvalues[i] );
+		l = loglevel2int( &mod->sm_values[ i ] );
 		if ( !l ) {
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
@@ -308,13 +287,13 @@ check_constraints( Modification *mod, int *newlevel )
 		}
 
 		assert( int_2_level[ l ].s.bv_len
-				== mod->sm_bvalues[i].bv_len );
+				== mod->sm_values[ i ].bv_len );
 		
-		AC_MEMCPY( mod->sm_bvalues[i].bv_val,
+		AC_MEMCPY( mod->sm_values[ i ].bv_val,
 				int_2_level[ l ].s.bv_val,
 				int_2_level[ l ].s.bv_len );
 
-		AC_MEMCPY( mod->sm_nvalues[i].bv_val,
+		AC_MEMCPY( mod->sm_nvalues[ i ].bv_val,
 				int_2_level[ l ].n.bv_val,
 				int_2_level[ l ].n.bv_len );
 
@@ -344,7 +323,7 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 			return LDAP_INAPPROPRIATE_MATCHING;
 		}
 
-		for ( i = 0; mod->sm_bvalues[i].bv_val != NULL; i++ ) {
+		for ( i = 0; !BER_BVISNULL( &mod->sm_values[ i ] ); i++ ) {
 			int rc;
 			int j;
 			const char *text = NULL;
@@ -352,17 +331,16 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 
 			rc = asserted_value_validate_normalize(
 				mod->sm_desc, mr, SLAP_MR_EQUALITY,
-				&mod->sm_bvalues[i], &asserted, &text, NULL );
+				&mod->sm_values[ i ], &asserted, &text, NULL );
 
 			if ( rc != LDAP_SUCCESS ) {
 				return rc;
 			}
 
-			for ( j = 0; a->a_vals[j].bv_val != NULL; j++ ) {
+			for ( j = 0; !BER_BVISNULL( &a->a_vals[ j ] ); j++ ) {
 				int match;
 				int rc = value_match( &match, mod->sm_desc, mr,
-					0,
-					&a->a_vals[j], &asserted, &text );
+					0, &a->a_vals[ j ], &asserted, &text );
 
 				if ( rc == LDAP_SUCCESS && match == 0 ) {
 					free( asserted.bv_val );
@@ -375,7 +353,7 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 	}
 
 	/* no - add them */
-	rc = attr_merge( e, mod->sm_desc, mod->sm_bvalues, mod->sm_nvalues );
+	rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
 	if ( rc != LDAP_SUCCESS ) {
 		/* this should return result of attr_mergeit */
 		return rc;
@@ -399,7 +377,7 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 	*newlevel &= ~nl;
 
 	/* delete the entire attribute */
-	if ( mod->sm_bvalues == NULL ) {
+	if ( mod->sm_values == NULL ) {
 		int rc = attr_delete( &e->e_attrs, mod->sm_desc );
 
 		if ( rc ) {
@@ -423,7 +401,7 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 	}
 
 	/* find each value to delete */
-	for ( i = 0; mod->sm_bvalues[i].bv_val != NULL; i++ ) {
+	for ( i = 0; !BER_BVISNULL( &mod->sm_values[ i ] ); i++ ) {
 		int rc;
 		const char *text = NULL;
 
@@ -431,16 +409,16 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 
 		rc = asserted_value_validate_normalize(
 				mod->sm_desc, mr, SLAP_MR_EQUALITY,
-				&mod->sm_bvalues[i], &asserted, &text, NULL );
+				&mod->sm_values[ i ], &asserted, &text, NULL );
 
 		if( rc != LDAP_SUCCESS ) return rc;
 
 		found = 0;
-		for ( j = 0; a->a_vals[j].bv_val != NULL; j++ ) {
+		for ( j = 0; !BER_BVISNULL( &a->a_vals[ j ] ); j++ ) {
 			int match;
 			int rc = value_match( &match, mod->sm_desc, mr,
 				0,
-				&a->a_vals[j], &asserted, &text );
+				&a->a_vals[ j ], &asserted, &text );
 
 			if( rc == LDAP_SUCCESS && match != 0 ) {
 				continue;
@@ -450,11 +428,11 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 			found = 1;
 
 			/* delete it */
-			free( a->a_vals[j].bv_val );
-			for ( k = j + 1; a->a_vals[k].bv_val != NULL; k++ ) {
-				a->a_vals[k - 1] = a->a_vals[k];
+			free( a->a_vals[ j ].bv_val );
+			for ( k = j + 1; !BER_BVISNULL( &a->a_vals[ k ] ); k++ ) {
+				a->a_vals[ k - 1 ] = a->a_vals[ k ];
 			}
-			a->a_vals[k - 1].bv_val = NULL;
+			BER_BVZERO( &a->a_vals[ k - 1 ] );
 
 			break;
 		}
@@ -468,7 +446,7 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 	}
 
 	/* if no values remain, delete the entire attribute */
-	if ( a->a_vals[0].bv_val == NULL ) {
+	if ( BER_BVISNULL( &a->a_vals[ 0 ] ) ) {
 		/* should already be zero */
 		*newlevel = 0;
 		
@@ -497,9 +475,8 @@ replace_values( Entry *e, Modification *mod, int *newlevel )
 		return rc;
 	}
 
-	if ( mod->sm_bvalues != NULL ) {
-		rc = attr_merge( e, mod->sm_desc, mod->sm_bvalues,
-		       mod->sm_nvalues );
+	if ( mod->sm_values != NULL ) {
+		rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
 		if ( rc != LDAP_SUCCESS ) {
 			return rc;
 		}
