@@ -1,30 +1,20 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/config.c,v 1.14 2003/05/25 01:56:59 kurt Exp $ */
-/*
- * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
-/*
- *	 Copyright 1999, John C. Quillan, All rights reserved.
- *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/config.c,v 1.20 2004/04/27 05:23:04 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- *	 Redistribution and use in source and binary forms are permitted only
- *	 as authorized by the OpenLDAP Public License.	A copy of this
- *	 license is available at http://www.OpenLDAP.org/license.html or
- *	 in file LICENSE in the top-level directory of the distribution.
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1999 John C. Quillan.
+ * Portions Copyright 2002 myinternet Limited.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
-#include "portable.h"
-	
-#include <stdio.h>
-
-#include "slap.h"
-#ifdef HAVE_WIN32_ASPERL
-#include "asperl_undefs.h"
-#endif
-
-#include <EXTERN.h>
-#include <perl.h>
-
 #include "perl_back.h"
 
 
@@ -118,6 +108,7 @@ perl_back_db_config(
 	} else if ( strcasecmp( argv[0], "filterSearchResults" ) == 0 ) {
 		perl_back->pb_filter_search_results = 1;
 	} else {
+		return_code = SLAP_CONF_UNKNOWN;
 		/*
 		 * Pass it to Perl module if defined
 		 */
@@ -153,12 +144,7 @@ perl_back_db_config(
 
 		}
 
-		/* if the module rejected it then we should reject it */
-		if ( return_code != 0 ) {
-			fprintf( stderr,
-				 "Unknown perl backend config: %s\n", argv[0]);
-			exit( EXIT_FAILURE );
-		}
+		return return_code;
 	}
 
 	return 0;
