@@ -1,56 +1,28 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.30 2003/05/25 01:56:59 kurt Exp $ */
-/*
- * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
- */
-/*
- *	 Copyright 1999, John C. Quillan, All rights reserved.
- *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.40 2004/11/15 18:45:50 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- *	 Redistribution and use in source and binary forms are permitted only
- *	 as authorized by the OpenLDAP Public License.	A copy of this
- *	 license is available at http://www.OpenLDAP.org/license.html or
- *	 in file LICENSE in the top-level directory of the distribution.
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1999 John C. Quillan.
+ * Portions Copyright 2002 myinternet Limited.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
-#include "portable.h"
- /* init.c - initialize shell backend */
-	
-#include <stdio.h>
-
-#include "slap.h"
-#ifdef HAVE_WIN32_ASPERL
-#include "asperl_undefs.h"
-#endif
-
-#include <EXTERN.h>
-#include <perl.h>
-
 #include "perl_back.h"
 
-
 static void perl_back_xs_init LDAP_P((PERL_BACK_XS_INIT_PARAMS));
 EXT void boot_DynaLoader LDAP_P((PERL_BACK_BOOT_DYNALOADER_PARAMS));
 
 PerlInterpreter *PERL_INTERPRETER = NULL;
 ldap_pvt_thread_mutex_t	perl_interpreter_mutex;
 
-#ifdef SLAPD_PERL_DYNAMIC
-
-int back_perl_LTX_init_module(int argc, char *argv[])
-{
-	BackendInfo bi;
-
-	memset( &bi, '\0', sizeof(bi) );
-	bi.bi_type = "perl";
-	bi.bi_init = perl_back_initialize;
-
-	backend_add(&bi);
-	return 0;
-}
-
-#endif /* SLAPD_PERL_DYNAMIC */
-
 
 /**********************************************************
  *
@@ -183,3 +155,12 @@ perl_back_xs_init(PERL_BACK_XS_INIT_PARAMS)
 	dXSUB_SYS;
 	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
 }
+
+#if SLAPD_PERL == SLAPD_MOD_DYNAMIC
+
+/* conditionally define the init_module() function */
+SLAP_BACKEND_INIT_MODULE( perl )
+
+#endif /* SLAPD_PERL == SLAPD_MOD_DYNAMIC */
+
+
