@@ -1,11 +1,32 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/perl_back.h,v 1.9 2003/05/25 01:56:59 kurt Exp $ */
-/*
- * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/perl_back.h,v 1.13 2004/11/10 23:41:54 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1999 John C. Quillan.
+ * Portions Copyright 2002 myinternet Limited.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 #ifndef PERL_BACK_H
 #define PERL_BACK_H 1
 
+#include <EXTERN.h>
+#include <perl.h>
+#undef _	/* #defined by both Perl and ac/localize.h */
+#include "asperl_undefs.h"
+
+#include "portable.h"
+
+#include "slap.h"
+
 LDAP_BEGIN_DECL
 
 /*
@@ -27,7 +48,7 @@ extern ldap_pvt_thread_mutex_t  perl_interpreter_mutex;
 # define na	PL_na
 #endif
 
-#ifdef HAVE_WIN32_ASPERL
+#if defined( HAVE_WIN32_ASPERL ) || defined( USE_ITHREADS )
 /* pTHX is needed often now */
 # define PERL_INTERPRETER			my_perl
 # define PERL_BACK_XS_INIT_PARAMS		pTHX
@@ -49,6 +70,6 @@ typedef struct perl_backend_instance {
 
 LDAP_END_DECL
 
-#include "external.h"
+#include "proto-perl.h"
 
-#endif
+#endif /* PERL_BACK_H */
