@@ -1,8 +1,31 @@
 /* modify.c - shell backend modify function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/modify.c,v 1.27 2003/03/30 07:03:55 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/modify.c,v 1.31 2004/04/08 04:49:19 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
  */
 
 #include "portable.h"
@@ -50,8 +73,7 @@ shell_back_modify(
 		return -1;
 	}
 
-	if ( (op->o_private = (void *) forkandexec( si->si_modify, &rfp, &wfp ))
-	    == (void *) -1 ) {
+	if ( forkandexec( si->si_modify, &rfp, &wfp ) == (pid_t)-1 ) {
 		send_ldap_error( op, rs, LDAP_OTHER,
 		    "could not fork/exec" );
 		return( -1 );
@@ -81,10 +103,10 @@ shell_back_modify(
 			break;
 		}
 
-		if( mod->sm_bvalues != NULL ) {
-			for ( i = 0; mod->sm_bvalues[i].bv_val != NULL; i++ ) {
+		if( mod->sm_values != NULL ) {
+			for ( i = 0; mod->sm_values[i].bv_val != NULL; i++ ) {
 				fprintf( wfp, "%s: %s\n", mod->sm_desc->ad_cname.bv_val,
-					mod->sm_bvalues[i].bv_val /* binary! */ );
+					mod->sm_values[i].bv_val /* binary! */ );
 			}
 		}
 
