@@ -1,27 +1,34 @@
-/*
- *	 Copyright 1999, Dmitry Kovalev <mit@openldap.org>, All rights reserved.
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/util.c,v 1.31 2004/11/15 21:10:10 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- *	 Redistribution and use in source and binary forms are permitted only
- *	 as authorized by the OpenLDAP Public License.	A copy of this
- *	 license is available at http://www.OpenLDAP.org/license.html or
- *	 in file LICENSE in the top-level directory of the distribution.
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1999 Dmitry Kovalev.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Dmitry Kovalev for inclusion
+ * by OpenLDAP Software.
  */
 
 #include "portable.h"
 
-#ifdef SLAPD_SQL
-
 #include <stdio.h>
 #include <sys/types.h>
 #include "ac/string.h"
 #include "ac/ctype.h"
 #include "ac/stdarg.h"
+
 #include "slap.h"
-#include "lber_pvt.h"
-#include "ldap_pvt.h"
-#include "back-sql.h"
-#include "schema-map.h"
-#include "util.h"
+#include "proto-sql.h"
+#include "lutil.h"
 
 #define BACKSQL_MAX(a,b) ((a)>(b)?(a):(b))
 #define BACKSQL_MIN(a,b) ((a)<(b)?(a):(b))
@@ -42,14 +49,18 @@ char backsql_def_delentry_query[] = "DELETE FROM ldap_entries WHERE id=?";
 char backsql_def_insentry_query[] = 
 	"INSERT INTO ldap_entries (dn,oc_map_id,parent,keyval) "
 	"VALUES (?,?,?,?)";
+char backsql_def_delobjclasses_query[] = "DELETE FROM ldap_entry_objclasses "
+	"WHERE entry_id=?";
+char backsql_def_delreferrals_query[] = "DELETE FROM ldap_referrals "
+	"WHERE entry_id=?";
 char backsql_def_subtree_cond[] = "ldap_entries.dn LIKE CONCAT('%',?)";
 char backsql_def_upper_subtree_cond[] = "(ldap_entries.dn) LIKE CONCAT('%',?)";
-char backsql_id_query[] = "SELECT id,keyval,oc_map_id FROM ldap_entries WHERE ";
+char backsql_id_query[] = "SELECT id,keyval,oc_map_id,dn FROM ldap_entries WHERE ";
 /* better ?||? or cast(?||? as varchar) */ 
 char backsql_def_concat_func[] = "CONCAT(?,?)";
 
 /* TimesTen */
-char backsql_check_dn_ru_query[] = "SELECT dn_ru from ldap_entries";
+char backsql_check_dn_ru_query[] = "SELECT dn_ru FROM ldap_entries";
 
 struct berbuf *
 backsql_strcat( struct berbuf *dest, ... )
@@ -63,7 +74,7 @@ backsql_strcat( struct berbuf *dest, ... )
 			|| dest->bb_val.bv_len == strlen( dest->bb_val.bv_val ) );
  
 #ifdef BACKSQL_TRACE
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_strcat()\n" );
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_strcat()\n", 0, 0, 0 );
 #endif /* BACKSQL_TRACE */
 
 	va_start( strs, dest );
@@ -100,7 +111,8 @@ backsql_strcat( struct berbuf *dest, ... )
 
 #ifdef BACKSQL_TRACE
 			Debug( LDAP_DEBUG_TRACE, "backsql_strcat(): "
-				"new buflen=%d, dest=%p\n", dest->bb_len, dest, 0 );
+				"new buflen=%d, dest=%p\n",
+				dest->bb_len, dest, 0 );
 #endif /* BACKSQL_TRACE */
 		}
 		AC_MEMCPY( dest->bb_val.bv_val + cdlen, cstr, cslen + 1 );
@@ -109,8 +121,8 @@ backsql_strcat( struct berbuf *dest, ... )
 	va_end( strs );
 
 #ifdef BACKSQL_TRACE
-	Debug( LDAP_DEBUG_TRACE, "<==backsql_strcat() (dest='%s')\n", 
-			dest, 0, 0 );
+	Debug( LDAP_DEBUG_TRACE, "<==backsql_strcat() (dest=\"%s\")\n", 
+			dest->bb_val.bv_val, 0, 0 );
 #endif /* BACKSQL_TRACE */
 
 	dest->bb_val.bv_len = cdlen;
@@ -131,7 +143,7 @@ backsql_strfcat( struct berbuf *dest, const char *fmt, ... )
 			|| dest->bb_val.bv_len == strlen( dest->bb_val.bv_val ) );
  
 #ifdef BACKSQL_TRACE
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_strfcat()\n" );
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_strfcat()\n", 0, 0, 0 );
 #endif /* BACKSQL_TRACE */
 
 	va_start( strs, fmt );
@@ -162,7 +174,7 @@ backsql_strfcat( struct berbuf *dest, const char *fmt, ... )
 			cslen = va_arg( strs, ber_len_t );
 			cstr = va_arg( strs, char * );
 			break;
-			
+
 		/* string */
 		case 's':
 			cstr = va_arg( strs, char * );
@@ -220,8 +232,8 @@ backsql_strfcat( struct berbuf *dest, const char *fmt, ... )
 	va_end( strs );
 
 #ifdef BACKSQL_TRACE
-	Debug( LDAP_DEBUG_TRACE, "<==backsql_strfcat() (dest='%s')\n", 
-			dest, 0, 0 );
+	Debug( LDAP_DEBUG_TRACE, "<==backsql_strfcat() (dest=\"%s\")\n", 
+			dest->bb_val.bv_val, 0, 0 );
 #endif /* BACKSQL_TRACE */
 
 	dest->bb_val.bv_len = cdlen;
@@ -242,7 +254,7 @@ backsql_entry_addattr(
 
 #ifdef BACKSQL_TRACE
 	Debug( LDAP_DEBUG_TRACE, "backsql_entry_addattr(): "
-		"at_name='%s', at_val='%s'\n", 
+		"at_name=\"%s\", at_val=\"%s\"\n", 
 		at_name->bv_val, at_val->bv_val, 0 );
 #endif /* BACKSQL_TRACE */
 
@@ -250,7 +262,7 @@ backsql_entry_addattr(
 	rc = slap_bv2ad( at_name, &ad, &text );
 	if ( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_entry_addattr(): "
-			"failed to find AttributeDescription for '%s'\n",
+			"failed to find AttributeDescription for \"%s\"\n",
 			at_name->bv_val, 0, 0 );
 		return 0;
 	}
@@ -259,7 +271,7 @@ backsql_entry_addattr(
 
 	if ( rc != 0 ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_entry_addattr(): "
-			"failed to merge value '%s' for attribute '%s'\n",
+			"failed to merge value \"%s\" for attribute \"%s\"\n",
 			at_val->bv_val, at_name->bv_val, 0 );
 		return 0;
 	}
@@ -271,7 +283,7 @@ backsql_entry_addattr(
 	return 1;
 }
 
-char *
+static char *
 backsql_get_table_spec( char **p )
 {
 	char		*s, *q;
@@ -303,18 +315,18 @@ backsql_get_table_spec( char **p )
 	s = q;
 
 	BACKSQL_NEXT_WORD;
-	if ( !strcasecmp( s, "as" ) ) {
+	if ( strcasecmp( s, "AS" ) == 0 ) {
 		s = q;
 		BACKSQL_NEXT_WORD;
 	}
 
-#if 0
-	backsql_strcat( &res, " AS ", s, NULL );
-	/* oracle doesn't understand AS :( */
-#endif
-
-	/* table alias */
-	backsql_strfcat( &res, "cs", ' ', s );
+	/* oracle doesn't understand "AS" :( and other RDBMSes don't need it */
+#ifdef BACKSQL_ALIASING_QUOTE
+	backsql_strfcat( &res, "scsc", " " BACKSQL_ALIASING,
+			BACKSQL_ALIASING_QUOTE, s, BACKSQL_ALIASING_QUOTE );
+#else /* ! BACKSQL_ALIASING */
+	backsql_strcat( &res, " " BACKSQL_ALIASING, s, NULL );
+#endif /* ! BACKSQL_ALIASING */
 
 	return res.bb_val.bv_val;
 }
@@ -329,8 +341,9 @@ backsql_merge_from_clause(
 
 #ifdef BACKSQL_TRACE
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_merge_from_clause(): "
-		"dest_from='%s',src_from='%s'\n",
- 		dest_from ? dest_from->bb_val.bv_val : "<NULL>", src_from, 0 );
+		"dest_from=\"%s\",src_from=\"%s\"\n",
+ 		dest_from ? dest_from->bb_val.bv_val : "<NULL>",
+		src_from->bv_val, 0 );
 #endif /* BACKSQL_TRACE */
 
 	srcc = ch_strdup( src_from->bv_val );
@@ -345,7 +358,7 @@ backsql_merge_from_clause(
 
 #ifdef BACKSQL_TRACE
 		Debug( LDAP_DEBUG_TRACE, "backsql_merge_from_clause(): "
-			"p='%s' s='%s'\n", p, s, 0 );
+			"p=\"%s\" s=\"%s\"\n", p, s, 0 );
 #endif /* BACKSQL_TRACE */
 
 		if ( res.bb_val.bv_val == NULL ) {
@@ -475,5 +488,90 @@ backsql_prepare_pattern(
 	return 0;
 }
 
-#endif /* SLAPD_SQL */
+int
+backsql_entryUUID(
+	backsql_info	*bi,
+	backsql_entryID	*id,
+	struct berval	*entryUUID,
+	void		*memctx )
+{
+	char		uuidbuf[ LDAP_LUTIL_UUIDSTR_BUFSIZE ];
+	struct berval	uuid;
+#ifdef BACKSQL_ARBITRARY_KEY
+	int		i;
+	ber_len_t	l, lmax;
+#endif /* BACKSQL_ARBITRARY_KEY */
+
+	/* entryUUID is generated as "%08x-%04x-%04x-0000-eaddrXXX"
+	 * with eid_oc_id as %08x and hi and lo eid_id as %04x-%04x */
+	assert( bi );
+	assert( id );
+	assert( entryUUID );
+
+#ifdef BACKSQL_ARBITRARY_KEY
+	snprintf( uuidbuf, sizeof( uuidbuf ),
+			"%08x-0000-0000-0000-000000000000",
+			( id->eid_oc_id & 0xFFFFFFFF ) );
+	lmax = id->eid_keyval.bv_len < 12 ? id->eid_keyval.bv_len : 12;
+	for ( l = 0, i = 9; l < lmax; l++, i += 2 ) {
+		switch ( i ) {
+		case STRLENOF( "00000000-0000" ):
+		case STRLENOF( "00000000-0000-0000" ):
+		case STRLENOF( "00000000-0000-0000-0000" ):
+			uuidbuf[ i++ ] = '-';
+		/* FALLTHRU */
+
+		default:
+			snprintf( &uuidbuf[ i ], 3, "%2x", id->eid_keyval.bv_val[ l ] );
+			break;
+		}
+	}
+#else /* ! BACKSQL_ARBITRARY_KEY */
+	/* note: works only with 32 bit architectures... */
+	snprintf( uuidbuf, sizeof( uuidbuf ),
+			"%08x-%04x-%04x-0000-000000000000",
+			( id->eid_oc_id & 0xFFFFFFFF ),
+			( ( id->eid_keyval & 0xFFFF0000 ) >> 16 ),
+			( id->eid_keyval & 0xFFFF ) );
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+
+	uuid.bv_val = uuidbuf;
+	uuid.bv_len = strlen( uuidbuf );
+
+	ber_dupbv_x( entryUUID, &uuid, memctx );
+
+	return 0;
+}
+
+int
+backsql_entryUUID_decode(
+	struct berval	*entryUUID,
+	unsigned long	*oc_id,
+#ifdef BACKSQL_ARBITRARY_KEY
+	struct berval	*keyval
+#else /* ! BACKSQL_ARBITRARY_KEY */
+	unsigned long	*keyval
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+	)
+{
+	fprintf( stderr, "==> backsql_entryUUID_decode()\n" );
+
+	*oc_id = ( entryUUID->bv_val[0] << 3 )
+		+ ( entryUUID->bv_val[1] << 2 )
+		+ ( entryUUID->bv_val[2] << 1 )
+		+ entryUUID->bv_val[3];
+
+#ifdef BACKSQL_ARBITRARY_KEY
+#else /* ! BACKSQL_ARBITRARY_KEY */
+	*keyval = ( entryUUID->bv_val[4] << 3 )
+		+ ( entryUUID->bv_val[5] << 2 )
+		+ ( entryUUID->bv_val[6] << 1 )
+		+ entryUUID->bv_val[7];
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+
+	fprintf( stderr, "<== backsql_entryUUID_decode(): oc=%lu id=%lu\n",
+			*oc_id, *keyval );
+
+	return LDAP_SUCCESS;
+}
 
