@@ -1,8 +1,27 @@
 /* entry.c - routines for dealing with entries */
-/* $OpenLDAP: pkg/ldap/servers/slapd/entry.c,v 1.115 2003/05/02 11:29:29 hallvard Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/entry.c,v 1.129.2.1 2004/12/30 21:29:56 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * provided that this notice is preserved and that due credit is given
+ * to the University of Michigan at Ann Arbor. The name of the University
+ * may not be used to endorse or promote products derived from this
+ * software without specific prior written permission. This software
+ * is provided ``as is'' without express or implied warranty.
  */
 
 #include "portable.h"
@@ -46,9 +65,10 @@ str2entry( char *s )
 	char		*type;
 	struct berval	vals[2];
 	struct berval	nvals[2], *nvalsp;
-	AttributeDescription *ad;
+	AttributeDescription *ad, *ad_prev;
 	const char *text;
 	char	*next;
+	int		attr_cnt;
 
 	/*
 	 * LDIF is used as the string format.
@@ -65,24 +85,16 @@ str2entry( char *s )
 	 * or newline.
 	 */
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "str2entry: \"%s\"\n", s ? s : "NULL", 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE, "=> str2entry\n",
+	Debug( LDAP_DEBUG_TRACE, "=> str2entry: \"%s\"\n",
 		s ? s : "NULL", 0, 0 );
-#endif
 
 	/* initialize reader/writer lock */
 	e = (Entry *) ch_calloc( 1, sizeof(Entry) );
 
 	if( e == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "str2entry: entry allocation failed.\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
-		    "<= str2entry NULL (entry allocation failed)\n",
-		    0, 0, 0 );
-#endif
+			"<= str2entry NULL (entry allocation failed)\n",
+			0, 0, 0 );
 		return( NULL );
 	}
 
@@ -93,6 +105,9 @@ str2entry( char *s )
 	vals[1].bv_len = 0;
 	vals[1].bv_val = NULL;
 
+	ad = NULL;
+	ad_prev = NULL;
+	attr_cnt = 0;
 	next = s;
 	while ( (s = ldif_getline( &next )) != NULL ) {
 		if ( *s == '\n' || *s == '\0' ) {
@@ -100,12 +115,8 @@ str2entry( char *s )
 		}
 
 		if ( ldif_parse_line( s, &type, &vals[0].bv_val, &vals[0].bv_len ) != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1, "str2entry:  NULL (parse_line)\n",0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
-			    "<= str2entry NULL (parse_line)\n", 0, 0, 0 );
-#endif
+				"<= str2entry NULL (parse_line)\n", 0, 0, 0 );
 			continue;
 		}
 
@@ -113,15 +124,9 @@ str2entry( char *s )
 			free( type );
 
 			if ( e->e_dn != NULL ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, DETAIL1, "str2entry: "
-					"entry %ld has multiple DNs \"%s\" and \"%s\"\n",
-					(long) e->e_id, e->e_dn, vals[0].bv_val );
-#else
 				Debug( LDAP_DEBUG_ANY, "str2entry: "
 					"entry %ld has multiple DNs \"%s\" and \"%s\"\n",
-				    (long) e->e_id, e->e_dn, vals[0].bv_val );
-#endif
+					(long) e->e_id, e->e_dn, vals[0].bv_val );
 				free( vals[0].bv_val );
 				entry_free( e );
 				return NULL;
@@ -129,15 +134,9 @@ str2entry( char *s )
 
 			rc = dnPrettyNormal( NULL, &vals[0], &e->e_name, &e->e_nname, NULL );
 			if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, DETAIL1, 
-					"str2entry: entry %ld has invalid DN \"%s\"\n",
-					(long) e->e_id, vals[0].bv_val, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY, "str2entry: "
 					"entry %ld has invalid DN \"%s\"\n",
 					(long) e->e_id, vals[0].bv_val, 0 );
-#endif
 				entry_free( e );
 				free( vals[0].bv_val );
 				return NULL;
@@ -146,18 +145,14 @@ str2entry( char *s )
 			continue;
 		}
 
+		ad_prev = ad;
 		ad = NULL;
 		rc = slap_str2ad( type, &ad, &text );
 
 		if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1, 
-				"str2entry:  str2ad(%s): %s\n", type, text, 0 );
-#else
 			Debug( slapMode & SLAP_TOOL_MODE
 				? LDAP_DEBUG_ANY : LDAP_DEBUG_TRACE,
 				"<= str2entry: str2ad(%s): %s\n", type, text, 0 );
-#endif
 			if( slapMode & SLAP_TOOL_MODE ) {
 				entry_free( e );
 				free( vals[0].bv_val );
@@ -167,14 +162,9 @@ str2entry( char *s )
 
 			rc = slap_str2undef_ad( type, &ad, &text );
 			if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, DETAIL1, 
-					"str2entry: str2undef_ad(%s): %s\n", type, text, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"<= str2entry: str2undef_ad(%s): %s\n",
 						type, text, 0 );
-#endif
 				entry_free( e );
 				free( vals[0].bv_val );
 				free( type );
@@ -182,6 +172,10 @@ str2entry( char *s )
 			}
 		}
 
+		if ( ad != ad_prev ) {
+			attr_cnt = 0;
+		}
+
 		if( slapMode & SLAP_TOOL_MODE ) {
 			struct berval pval;
 			slap_syntax_validate_func *validate =
@@ -200,15 +194,11 @@ str2entry( char *s )
 				rc = validate( ad->ad_type->sat_syntax, &vals[0] );
 
 			} else {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, INFO, 
-					"str2entry: no validator for syntax %s\n", 
-					ad->ad_type->sat_syntax->ssyn_oid, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
-					"str2entry: no validator for syntax %s\n",
-					ad->ad_type->sat_syntax->ssyn_oid, 0, 0 );
-#endif
+					"str2entry: attributeType %s #%d: "
+					"no validator for syntax %s\n", 
+					ad->ad_cname.bv_val, attr_cnt,
+					ad->ad_type->sat_syntax->ssyn_oid );
 				entry_free( e );
 				free( vals[0].bv_val );
 				free( type );
@@ -216,15 +206,11 @@ str2entry( char *s )
 			}
 
 			if( rc != 0 ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, ERR, 
-					"str2entry:  invalid value for attribute %s (syntax %s)\n",
-					ad->ad_cname.bv_val, ad->ad_type->sat_syntax->ssyn_oid, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
-					"str2entry: invalid value for attribute %s (syntax %s)\n",
-					ad->ad_cname.bv_val, ad->ad_type->sat_syntax->ssyn_oid, 0 );
-#endif
+					"str2entry: invalid value "
+					"for attributeType %s #%d (syntax %s)\n",
+					ad->ad_cname.bv_val, attr_cnt,
+					ad->ad_type->sat_syntax->ssyn_oid );
 				entry_free( e );
 				free( vals[0].bv_val );
 				free( type );
@@ -250,13 +236,8 @@ str2entry( char *s )
 				&vals[0], &nvals[0], NULL );
 
 			if( rc ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, DETAIL1,
-					"str2entry:  NULL (smr_normalize %d)\n" , rc, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 			   		"<= str2entry NULL (smr_normalize %d)\n", rc, 0, 0 );
-#endif
 
 				entry_free( e );
 				free( vals[0].bv_val );
@@ -272,13 +253,8 @@ str2entry( char *s )
 
 		rc = attr_merge( e, ad, vals, nvalsp );
 		if( rc != 0 ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, DETAIL1,
-				"str2entry:  NULL (attr_merge)\n" , 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
-			    "<= str2entry NULL (attr_merge)\n", 0, 0, 0 );
-#endif
+				"<= str2entry NULL (attr_merge)\n", 0, 0, 0 );
 			entry_free( e );
 			free( vals[0].bv_val );
 			free( type );
@@ -288,28 +264,20 @@ str2entry( char *s )
 		free( type );
 		free( vals[0].bv_val );
 		free( nvals[0].bv_val );
+
+		attr_cnt++;
 	}
 
 	/* check to make sure there was a dn: line */
 	if ( e->e_dn == NULL ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"str2entry:  entry %ld has no dn.\n", (long) e->e_id, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "str2entry: entry %ld has no dn\n",
-		    (long) e->e_id, 0, 0 );
-#endif
+			(long) e->e_id, 0, 0 );
 		entry_free( e );
 		return NULL;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL2,
-		"str2entry(%s) -> 0x%lx\n", e->e_dn, (unsigned long)e, 0 );
-#else
 	Debug(LDAP_DEBUG_TRACE, "<= str2entry(%s) -> 0x%lx\n",
 		e->e_dn, (unsigned long) e, 0 );
-#endif
 	return( e );
 }
 
@@ -321,7 +289,7 @@ str2entry( char *s )
 			ptrdiff_t	offset; \
 			offset = (int) (ecur - ebuf); \
 			ebuf = (unsigned char *) ch_realloc( (char *) ebuf, \
-			    emaxsize + GRABSIZE ); \
+				emaxsize + GRABSIZE ); \
 			emaxsize += GRABSIZE; \
 			ecur = ebuf + offset; \
 		} \
@@ -329,8 +297,8 @@ str2entry( char *s )
 
 char *
 entry2str(
-    Entry	*e,
-    int		*len )
+	Entry	*e,
+	int		*len )
 {
 	Attribute	*a;
 	struct berval	*bv;
@@ -364,7 +332,7 @@ entry2str(
 			MAKE_SPACE( LDIF_SIZE_NEEDED( tmplen, bv->bv_len ));
 			ldif_sput( (char **) &ecur, LDIF_PUT_VALUE,
 				a->a_desc->ad_cname.bv_val,
-			    bv->bv_val, bv->bv_len );
+				bv->bv_val, bv->bv_len );
 		}
 	}
 	MAKE_SPACE( 1 );
@@ -375,7 +343,7 @@ entry2str(
 }
 
 void
-entry_free( Entry *e )
+entry_clean( Entry *e )
 {
 	/* free an entry structure */
 	assert( e != NULL );
@@ -385,23 +353,29 @@ entry_free( Entry *e )
 	e->e_private = NULL;
 
 	/* free DNs */
-	if ( e->e_dn != NULL ) {
-		free( e->e_dn );
-		e->e_dn = NULL;
+	if ( !BER_BVISNULL( &e->e_name ) ) {
+		free( e->e_name.bv_val );
+		BER_BVZERO( &e->e_name );
 	}
-	if ( e->e_ndn != NULL ) {
-		free( e->e_ndn );
-		e->e_ndn = NULL;
+	if ( !BER_BVISNULL( &e->e_nname ) ) {
+		free( e->e_nname.bv_val );
+		BER_BVZERO( &e->e_nname );
 	}
 
-	if ( e->e_bv.bv_val != NULL ) {
+	if ( !BER_BVISNULL( &e->e_bv ) ) {
 		free( e->e_bv.bv_val );
-		e->e_bv.bv_val = NULL;
+		BER_BVZERO( &e->e_bv );
 	}
 
 	/* free attributes */
 	attrs_free( e->e_attrs );
 	e->e_attrs = NULL;
+}
+
+void
+entry_free( Entry *e )
+{
+	entry_clean( e );
 
 	free( e );
 }
@@ -557,13 +531,8 @@ int entry_encode(Entry *e, struct berval *bv)
 	Attribute *a;
 	unsigned char *ptr;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "entry_encode: id: 0x%08lx  \"%s\"\n",
-		(long) e->e_id, e->e_dn, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "=> entry_encode(0x%08lx): %s\n",
 		(long) e->e_id, e->e_dn, 0 );
-#endif
 	dnlen = e->e_name.bv_len;
 	ndnlen = e->e_nname.bv_len;
 
@@ -589,27 +558,27 @@ int entry_encode(Entry *e, struct berval *bv)
 		ptr += a->a_desc->ad_cname.bv_len;
 		*ptr++ = '\0';
 		if (a->a_vals) {
-		    for (i=0; a->a_vals[i].bv_val; i++);
-		    entry_putlen(&ptr, i);
-		    for (i=0; a->a_vals[i].bv_val; i++) {
+			for (i=0; a->a_vals[i].bv_val; i++);
+			entry_putlen(&ptr, i);
+			for (i=0; a->a_vals[i].bv_val; i++) {
 			entry_putlen(&ptr, a->a_vals[i].bv_len);
 			AC_MEMCPY(ptr, a->a_vals[i].bv_val,
 				a->a_vals[i].bv_len);
 			ptr += a->a_vals[i].bv_len;
 			*ptr++ = '\0';
-		    }
-		    if (a->a_nvals != a->a_vals) {
-		    	entry_putlen(&ptr, i);
-			for (i=0; a->a_nvals[i].bv_val; i++) {
-			    entry_putlen(&ptr, a->a_nvals[i].bv_len);
-			    AC_MEMCPY(ptr, a->a_nvals[i].bv_val,
-				a->a_nvals[i].bv_len);
-			    ptr += a->a_nvals[i].bv_len;
-			    *ptr++ = '\0';
 			}
-		    } else {
-		    	entry_putlen(&ptr, 0);
-		    }
+			if (a->a_nvals != a->a_vals) {
+				entry_putlen(&ptr, i);
+			for (i=0; a->a_nvals[i].bv_val; i++) {
+				entry_putlen(&ptr, a->a_nvals[i].bv_len);
+				AC_MEMCPY(ptr, a->a_nvals[i].bv_val,
+				a->a_nvals[i].bv_len);
+				ptr += a->a_nvals[i].bv_len;
+				*ptr++ = '\0';
+			}
+			} else {
+				entry_putlen(&ptr, 0);
+			}
 		}
 	}
 	*ptr = '\0';
@@ -627,7 +596,11 @@ int entry_encode(Entry *e, struct berval *bv)
  * you can not free individual attributes or names from this
  * structure. Attempting to do so will likely corrupt memory.
  */
+#ifdef SLAP_ZONE_ALLOC
+int entry_decode(struct berval *bv, Entry **e, void *ctx)
+#else
 int entry_decode(struct berval *bv, Entry **e)
+#endif
 {
 	int i, j, count;
 	int rc;
@@ -639,7 +612,20 @@ int entry_decode(struct berval *bv, Entry **e)
 	BerVarray bptr;
 
 	i = entry_getlen(&ptr);
+	if (!i) {
+		Debug( LDAP_DEBUG_ANY,
+			"entry_decode: entry length was zero\n", 0, 0, 0);
+		return LDAP_OTHER;
+	}
+#ifdef SLAP_ZONE_ALLOC
+	x = slap_zn_calloc(1, i + bv->bv_len, ctx);
+	AC_MEMCPY((char*)x + i, bv->bv_val, bv->bv_len);
+	bv->bv_val = (char*)x + i;
+	ptr = (unsigned char *)bv->bv_val;
+	i = entry_getlen(&ptr);
+#else
 	x = ch_calloc(1, i);
+#endif
 	i = entry_getlen(&ptr);
 	x->e_name.bv_val = (char *) ptr;
 	x->e_name.bv_len = i;
@@ -648,13 +634,9 @@ int entry_decode(struct berval *bv, Entry **e)
 	x->e_nname.bv_val = (char *) ptr;
 	x->e_nname.bv_len = i;
 	ptr += i+1;
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL2, "entry_decode: \"%s\"\n", x->e_dn, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE,
-	    "entry_decode: \"%s\"\n",
-	    x->e_dn, 0, 0 );
-#endif
+		"entry_decode: \"%s\"\n",
+		x->e_dn, 0, 0 );
 	x->e_bv = *bv;
 
 	/* A valid entry must have at least one attr, so this
@@ -676,24 +658,14 @@ int entry_decode(struct berval *bv, Entry **e)
 		rc = slap_bv2ad( &bv, &ad, &text );
 
 		if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				"entry_decode: str2ad(%s): %s\n", ptr, text, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"<= entry_decode: str2ad(%s): %s\n", ptr, text, 0 );
-#endif
 			rc = slap_bv2undef_ad( &bv, &ad, &text );
 
 			if( rc != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, INFO, 
-					"entry_decode:  str2undef_ad(%s): %s\n", ptr, text, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"<= entry_decode: str2undef_ad(%s): %s\n",
 						ptr, text, 0 );
-#endif
 				return rc;
 			}
 		}
@@ -702,6 +674,9 @@ int entry_decode(struct berval *bv, Entry **e)
 		bptr = (BerVarray)(a+1);
 		a->a_vals = bptr;
 		a->a_flags = 0;
+#ifdef LDAP_COMP_MATCH
+		a->a_comp_data = NULL;
+#endif
 		count = j = entry_getlen(&ptr);
 
 		while (j) {
@@ -736,12 +711,27 @@ int entry_decode(struct berval *bv, Entry **e)
 	}
 
 	if (a) a->a_next = NULL;
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "entry_decode:  %s\n", x->e_dn, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_TRACE, "<= entry_decode(%s)\n",
 		x->e_dn, 0, 0 );
-#endif
 	*e = x;
 	return 0;
 }
+
+Entry *entry_dup( Entry *e )
+{
+	Entry *ret;
+
+	ret = (Entry *)ch_calloc( 1, sizeof(*ret) );
+
+	ret->e_id = e->e_id;
+	ber_dupbv( &ret->e_name, &e->e_name );
+	ber_dupbv( &ret->e_nname, &e->e_nname );
+	ret->e_attrs = attrs_dup( e->e_attrs );
+	ret->e_ocflags = e->e_ocflags;
+	ret->e_bv.bv_val = NULL;
+	ret->e_bv.bv_len = 0;
+	ret->e_private = NULL;
+
+	return ret;
+}
+
