@@ -1,12 +1,16 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/extended.c,v 1.54.2.2 2003/05/31 22:14:11 kurt Exp $ */
-/* 
- * Copyright 1999-2003 The OpenLDAP Foundation.
+/* $OpenLDAP: pkg/ldap/servers/slapd/extended.c,v 1.78 2004/11/23 12:08:46 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1999-2004 The OpenLDAP Foundation.
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 /*
@@ -36,16 +40,11 @@
 #include "lber_pvt.h"
 
 #ifdef LDAP_SLAPI
-#include "slapi.h"
+#include "slapi/slapi.h"
 #endif
 
-#define UNSUPPORTED_EXTENDEDOP "unsupported extended operation"
+#define UNSUPPORTED_EXOP "unsupported extended operation"
 
-#ifdef LDAP_DEVEL
-#define SLAP_EXOP_HIDE 0x0000
-#else
-#define SLAP_EXOP_HIDE 0x8000
-#endif
 
 static struct extop_list {
 	struct extop_list *next;
@@ -56,14 +55,12 @@ static struct extop_list {
 
 static SLAP_EXTOP_MAIN_FN whoami_extop;
 
-/* this list of built-in extops is for extops that are not part
- * of backends or in external modules.	essentially, this is
+/* This list of built-in extops is for extops that are not part
+ * of backends or in external modules.	Essentially, this is
  * just a way to get built-in extops onto the extop list without
  * having a separate init routine for each built-in extop.
  */
-#ifdef LDAP_EXOP_X_CANCEL
 const struct berval slap_EXOP_CANCEL = BER_BVC(LDAP_EXOP_X_CANCEL);
-#endif
 const struct berval slap_EXOP_WHOAMI = BER_BVC(LDAP_EXOP_X_WHO_AM_I);
 const struct berval slap_EXOP_MODIFY_PASSWD = BER_BVC(LDAP_EXOP_MODIFY_PASSWD);
 const struct berval slap_EXOP_START_TLS = BER_BVC(LDAP_EXOP_START_TLS);
@@ -73,15 +70,13 @@ static struct {
 	slap_mask_t flags;
 	SLAP_EXTOP_MAIN_FN *ext_main;
 } builtin_extops[] = {
-#ifdef LDAP_EXOP_X_CANCEL
 	{ &slap_EXOP_CANCEL, SLAP_EXOP_HIDE, cancel_extop },
-#endif
 	{ &slap_EXOP_WHOAMI, 0, whoami_extop },
-	{ &slap_EXOP_MODIFY_PASSWD, 0, passwd_extop },
+	{ &slap_EXOP_MODIFY_PASSWD, SLAP_EXOP_WRITES, passwd_extop },
 #ifdef HAVE_TLS
 	{ &slap_EXOP_START_TLS, 0, starttls_extop },
 #endif
-	{ NULL, NULL }
+	{ NULL, 0, NULL }
 };
 
 
@@ -138,64 +133,22 @@ do_extended(
 	struct berval reqdata = {0, NULL};
 	ber_tag_t tag;
 	ber_len_t len;
-	struct extop_list *ext = NULL;
 
-#if defined(LDAP_SLAPI) 
- 	Slapi_PBlock    *pb = op->o_pb;
- 	SLAPI_FUNC      funcAddr = NULL;
- 	int             extop_rc;
- 	int             msg_sent = FALSE;
-#endif /* defined(LDAP_SLAPI) */
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_extended: conn %d\n", op->o_connid, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_extended\n", 0, 0, 0 );
-#endif
 
 	if( op->o_protocol < LDAP_VERSION3 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_extended: protocol version (%d) too low.\n", op->o_protocol, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_extended: protocol version (%d) too low\n",
 			op->o_protocol, 0 ,0 );
-#endif
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "requires LDAPv3" );
-		rs->sr_err = -1;
+		rs->sr_err = SLAPD_DISCONNECT;
 		goto done;
 	}
 
 	if ( ber_scanf( op->o_ber, "{m" /*}*/, &op->ore_reqoid ) == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_extended: conn %d  ber_scanf failed\n", 
-			op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_extended: ber_scanf failed\n", 0, 0 ,0 );
-#endif
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
-		rs->sr_err = -1;
-		goto done;
-	}
-
-#ifdef LDAP_SLAPI
-	getPluginFunc( &op->ore_reqoid, &funcAddr ); /* NS-SLAPI extended operation */
-	if( !funcAddr && !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
-#else
-	if( !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
-#endif
-	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_extended: conn %d  unsupported operation \"%s\"\n",
-			op->o_connid, op->ore_reqoid.bv_val, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_extended: unsupported operation \"%s\"\n",
-			op->ore_reqoid.bv_val, 0 ,0 );
-#endif
-		send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
-			"unsupported extended operation" );
+		rs->sr_err = SLAPD_DISCONNECT;
 		goto done;
 	}
 
@@ -203,74 +156,77 @@ do_extended(
 	
 	if( ber_peek_tag( op->o_ber, &len ) == LDAP_TAG_EXOP_REQ_VALUE ) {
 		if( ber_scanf( op->o_ber, "m", &reqdata ) == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR, 
-				"do_extended: conn %d  ber_scanf failed\n", 
-				op->o_connid, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "do_extended: ber_scanf failed\n", 0, 0 ,0 );
-#endif
 			send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
-			rs->sr_err = -1;
+			rs->sr_err = SLAPD_DISCONNECT;
 			goto done;
 		}
 	}
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_extended: conn %d  get_ctrls failed\n", op->o_connid, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_extended: get_ctrls failed\n", 0, 0 ,0 );
-#endif
 		return rs->sr_err;
 	} 
 
 	/* check for controls inappropriate for all extended operations */
-	if( get_manageDSAit( op ) == SLAP_CRITICAL_CONTROL ) {
+	if( get_manageDSAit( op ) == SLAP_CONTROL_CRITICAL ) {
 		send_ldap_error( op, rs,
 			LDAP_UNAVAILABLE_CRITICAL_EXTENSION,
 			"manageDSAit control inappropriate" );
 		goto done;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, 
-		"do_extended: conn %d  oid=%s\n.", op->o_connid, op->ore_reqoid.bv_val, 0 );
-#else
-	Debug( LDAP_DEBUG_ARGS, "do_extended: oid=%s\n", op->ore_reqoid.bv_val, 0 ,0 );
-#endif
+	/* FIXME: temporary? */
+	if ( reqdata.bv_val ) {
+		op->ore_reqdata = &reqdata;
+	}
 
-#if defined(LDAP_SLAPI)
-	if (ext != NULL) { /* OpenLDAP extended operation */
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_extended( op, rs );
+done:
+	return rs->sr_err;
+}
+
+int
+fe_extended( Operation *op, SlapReply *rs )
+{
+	struct extop_list	*ext = NULL;
+	struct berval		reqdata = BER_BVNULL;
+
+#if defined(LDAP_SLAPI) 
+ 	Slapi_PBlock    	*pb = op->o_pb;
+ 	SLAPI_FUNC      	funcAddr = NULL;
+ 	int             	extop_rc;
+ 	int             	msg_sent = FALSE;
 #endif /* defined(LDAP_SLAPI) */
 
-		if (reqdata.bv_val) op->ore_reqdata = &reqdata;
-		rs->sr_err = (ext->ext_main)( op, rs );
+	if (op->ore_reqdata) {
+		reqdata = *op->ore_reqdata;
+	}
 
-		if( rs->sr_err != SLAPD_ABANDON ) {
-			if ( rs->sr_err == LDAP_REFERRAL && rs->sr_ref == NULL ) {
-				rs->sr_ref = referral_rewrite( default_referral,
-					NULL, NULL, LDAP_SCOPE_DEFAULT );
-			}
+#ifdef LDAP_SLAPI
+    /* NS-SLAPI extended operation */
+	slapi_int_get_extop_plugin( &op->ore_reqoid, &funcAddr );
 
-			send_ldap_extended( op, rs );
+	if( !funcAddr && !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
+#else
+	if( !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
+#endif
+	{
+		Debug( LDAP_DEBUG_ANY, "do_extended: unsupported operation \"%s\"\n",
+			op->ore_reqoid.bv_val, 0 ,0 );
+		send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
+			"unsupported extended operation" );
+		goto done;
+	}
 
-			ber_bvarray_free( rs->sr_ref );
-		}
+	op->ore_flags = ext->flags;
 
-		if ( rs->sr_rspoid != NULL ) {
-			free( (char *)rs->sr_rspoid );
-		}
+	Debug( LDAP_DEBUG_ARGS, "do_extended: oid=%s\n",
+		op->ore_reqoid.bv_val, 0 ,0 );
 
-		if ( rs->sr_rspdata != NULL ) {
-			ber_bvfree( rs->sr_rspdata );
-		}
-
-#if defined( LDAP_SLAPI )
-		goto done;  /* end of OpenLDAP extended operation */
-
-	} else { /* start of Netscape extended operation */
+#if defined(LDAP_SLAPI)
+	if ( funcAddr != NULL ) {
 		rs->sr_err = slapi_pblock_set( pb, SLAPI_EXT_OP_REQ_OID,
 				(void *)op->ore_reqoid.bv_val);
 		if ( rs->sr_err != LDAP_SUCCESS ) {
@@ -285,7 +241,7 @@ do_extended(
 			goto done;
 		}
 
-		rs->sr_err = slapi_x_pblock_set_operation( pb, op );
+		rs->sr_err = slapi_int_pblock_set_operation( pb, op );
 		if ( rs->sr_err != LDAP_SUCCESS ) {
 			rs->sr_err = LDAP_OTHER;
 			goto done;
@@ -297,7 +253,7 @@ do_extended(
 
 		} else if ( extop_rc == SLAPI_PLUGIN_EXTENDED_NOT_HANDLED ) {
 			rs->sr_err = LDAP_PROTOCOL_ERROR;
-			rs->sr_text = UNSUPPORTED_EXTENDEDOP;
+			rs->sr_text = UNSUPPORTED_EXOP;
 
 		} else {
 			rs->sr_err = slapi_pblock_get( pb, SLAPI_EXT_OP_RET_OID,
@@ -329,11 +285,40 @@ done2:;
 		if ( rs->sr_rspdata != NULL ) {
 			ber_bvfree( rs->sr_rspdata );
 		}
-
-	} /* end of Netscape extended operation */
+	} else
 #endif /* defined( LDAP_SLAPI ) */
+	{ /* start of OpenLDAP extended operation */
+		rs->sr_err = (ext->ext_main)( op, rs );
 
-done:
+		if( rs->sr_err != SLAPD_ABANDON ) {
+			if ( rs->sr_err == LDAP_REFERRAL && rs->sr_ref == NULL ) {
+				rs->sr_ref = referral_rewrite( default_referral,
+					NULL, NULL, LDAP_SCOPE_DEFAULT );
+				if ( !rs->sr_ref ) rs->sr_ref = default_referral;
+				if ( !rs->sr_ref ) {
+					rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
+					rs->sr_text = "referral missing";
+				}
+			}
+
+			send_ldap_extended( op, rs );
+
+			if ( rs->sr_ref != default_referral ) {
+				ber_bvarray_free( rs->sr_ref );
+				rs->sr_ref = NULL;
+			}
+		}
+
+		if ( rs->sr_rspoid != NULL ) {
+			free( (char *)rs->sr_rspoid );
+		}
+
+		if ( rs->sr_rspdata != NULL ) {
+			ber_bvfree( rs->sr_rspdata );
+		}
+	} /* end of OpenLDAP extended operation */
+
+done:;
 	return rs->sr_err;
 }
 
@@ -428,10 +413,10 @@ whoami_extop (
 
 	bv = (struct berval *) ch_malloc( sizeof(struct berval) );
 	if( op->o_dn.bv_len ) {
-		bv->bv_len = op->o_dn.bv_len + sizeof("dn:")-1;
+		bv->bv_len = op->o_dn.bv_len + STRLENOF( "dn:" );
 		bv->bv_val = ch_malloc( bv->bv_len + 1 );
-		AC_MEMCPY( bv->bv_val, "dn:", sizeof("dn:")-1 );
-		AC_MEMCPY( &bv->bv_val[sizeof("dn:")-1], op->o_dn.bv_val,
+		AC_MEMCPY( bv->bv_val, "dn:", STRLENOF( "dn:" ) );
+		AC_MEMCPY( &bv->bv_val[STRLENOF( "dn:" )], op->o_dn.bv_val,
 			op->o_dn.bv_len );
 		bv->bv_val[bv->bv_len] = '\0';
 
