@@ -1,13 +1,23 @@
 /* index.c - index utilities */
-/* $OpenLDAP: pkg/ldap/servers/slapd/index.c,v 1.8 2002/01/30 23:11:44 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/index.c,v 1.13 2004/01/01 18:15:30 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
 
 #include <stdio.h>
+#include <ac/string.h>
 
 #include "slap.h"
 
@@ -29,8 +39,9 @@ int slap_str2index( const char *str, slap_mask_t *idx )
 		strcasecmp( str, "sub" ) == 0 )
 	{
 		*idx = SLAP_INDEX_SUBSTR_DEFAULT;
-	} else if ( strcasecmp( str, "nolang" ) == 0 ) {
-		*idx = SLAP_INDEX_NOLANG;
+	} else if ( strcasecmp( str, "nolang" ) == 0 ||	/* backwards compat */
+	            strcasecmp( str, "notags" ) == 0 ) {
+		*idx = SLAP_INDEX_NOTAGS;
 	} else if ( strcasecmp( str, "nosubtypes" ) == 0 ) {
 		*idx = SLAP_INDEX_NOSUBTYPES;
 	} else {
