@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.182 2003/05/01 21:39:29 jongchoi Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.227.2.1 2004/12/30 21:29:56 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1995 Regents of the University of Michigan.
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -23,10 +31,9 @@
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 #ifdef LDAP_SLAPI
-#include "slapi.h"
+#include "slapi/slapi.h"
 #endif
 #include "lutil.h"
 
@@ -36,26 +43,17 @@ do_modify(
     Operation	*op,
     SlapReply	*rs )
 {
-	struct berval dn = { 0, NULL };
+	struct berval dn = BER_BVNULL;
 	char		*last;
 	ber_tag_t	tag;
 	ber_len_t	len;
 	Modifications	*modlist = NULL;
 	Modifications	**modtail = &modlist;
-#ifdef LDAP_DEBUG
-	Modifications *tmp;
-#endif
-#ifdef LDAP_SLAPI
-	LDAPMod		**modv = NULL;
-	Slapi_PBlock *pb = op->o_pb;
-#endif
-	int manageDSAit;
+	int		increment = 0;
+	char		textbuf[ SLAP_TEXT_BUFLEN ];
+	size_t		textlen = sizeof( textbuf );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_modify: enter\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_modify\n", 0, 0, 0 );
-#endif
 
 	/*
 	 * Parse the modify request.  It looks like this:
@@ -77,25 +75,15 @@ do_modify(
 	 */
 
 	if ( ber_scanf( op->o_ber, "{m" /*}*/, &dn ) == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modify: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modify: ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
 		return SLAPD_DISCONNECT;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ARGS, "do_modify: dn (%s)\n", dn.bv_val, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_ARGS, "do_modify: dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
-
 
 	/* collect modifications & save for later */
-
 	for ( tag = ber_first_element( op->o_ber, &len, &last );
 	    tag != LBER_DEFAULT;
 	    tag = ber_next_element( op->o_ber, &len, last ) )
@@ -106,10 +94,10 @@ do_modify(
 		tmp.sml_nvalues = NULL;
 
 		if ( ber_scanf( op->o_ber, "{i{m[W]}}", &mop,
-		    &tmp.sml_type, &tmp.sml_values )
-		    == LBER_ERROR )
+		    &tmp.sml_type, &tmp.sml_values ) == LBER_ERROR )
 		{
-			send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding modlist error" );
+			send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR,
+				"decoding modlist error" );
 			rs->sr_err = SLAPD_DISCONNECT;
 			goto cleanup;
 		}
@@ -126,15 +114,9 @@ do_modify(
 		switch( mop ) {
 		case LDAP_MOD_ADD:
 			if ( mod->sml_values == NULL ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, ERR, 
-					"do_modify: modify/add operation (%ld) requires values\n",
-					(long)mop, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
 					"do_modify: modify/add operation (%ld) requires values\n",
 					(long) mop, 0, 0 );
-#endif
 
 				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 					"modify/add operation requires values" );
@@ -147,16 +129,37 @@ do_modify(
 		case LDAP_MOD_REPLACE:
 			break;
 
+		case LDAP_MOD_INCREMENT:
+			if( op->o_protocol >= LDAP_VERSION3 ) {
+				increment++;
+				if ( mod->sml_values == NULL ) {
+					Debug( LDAP_DEBUG_ANY, "do_modify: "
+						"modify/increment operation (%ld) requires value\n",
+						(long) mop, 0, 0 );
+
+					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
+						"modify/increment operation requires value" );
+					goto cleanup;
+				}
+
+				if( mod->sml_values[1].bv_val ) {
+					Debug( LDAP_DEBUG_ANY, "do_modify: modify/increment "
+						"operation (%ld) requires single value\n",
+						(long) mop, 0, 0 );
+
+					send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
+						"modify/increment operation requires single value" );
+					goto cleanup;
+				}
+
+				break;
+			}
+			/* fall thru */
+
 		default: {
-#ifdef NEW_LOGGING
-				LDAP_LOG( OPERATION, ERR, 
-					"do_modify: invalid modify operation (%ld)\n",
-					(long)mop, 0, 0 );
-#else
 				Debug( LDAP_DEBUG_ANY,
-					"do_modify: invalid modify operation (%ld)\n",
+					"do_modify: unrecognized modify operation (%ld)\n",
 					(long) mop, 0, 0 );
-#endif
 
 				send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
 					"unrecognized modify operation" );
@@ -169,11 +172,7 @@ do_modify(
 	*modtail = NULL;
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modify: get_ctrls failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modify: get_ctrls failed\n", 0, 0, 0 );
-#endif
 
 		goto cleanup;
 	}
@@ -181,36 +180,61 @@ do_modify(
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn,
 		op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modify: conn %d  invalid dn (%s)\n",
-			op->o_connid, dn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modify: invalid dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid DN" );
 		goto cleanup;
 	}
 
-	if( op->o_req_ndn.bv_len == 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_modify: attempt to modify root DSE.\n",0, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modify: root dse!\n", 0, 0, 0 );
+	rs->sr_err = slap_mods_check( modlist, &rs->sr_text,
+			textbuf, textlen, NULL );
+
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		send_ldap_result( op, rs );
+		goto cleanup;
+	}
+
+	/* FIXME: needs review */
+	op->orm_modlist = modlist;
+	op->orm_increment = increment;
+
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_modify( op, rs );
+
+cleanup:
+	slap_graduate_commit_csn( op );
+
+	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
+	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+	if ( modlist != NULL ) slap_mods_free( modlist );
+
+	return rs->sr_err;
+}
+
+int
+fe_op_modify( Operation *op, SlapReply *rs )
+{
+#ifdef LDAP_DEBUG
+	Modifications	*tmp;
 #endif
+	int		manageDSAit;
+	Modifications	*modlist = op->orm_modlist;
+	Modifications	**modtail = &modlist;
+#ifdef LDAP_SLAPI
+	LDAPMod		**modv = NULL;
+#endif
+	int		increment = op->orm_increment;
+	int		rc = 0;
+	
+	if( op->o_req_ndn.bv_len == 0 ) {
+		Debug( LDAP_DEBUG_ANY, "do_modify: root dse!\n", 0, 0, 0 );
 
 		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 			"modify upon the root DSE not supported" );
 		goto cleanup;
 
-	} else if ( bvmatch( &op->o_req_ndn, &global_schemandn ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modify: attempt to modify subschema subentry.\n" , 0, 0, 0  );
-#else
+	} else if ( bvmatch( &op->o_req_ndn, &frontendDB->be_schemandn ) ) {
 		Debug( LDAP_DEBUG_ANY, "do_modify: subschema subentry!\n", 0, 0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 			"modification of subschema subentry not supported" );
@@ -218,34 +242,14 @@ do_modify(
 	}
 
 #ifdef LDAP_DEBUG
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, DETAIL1, "do_modify: modifications:\n", 0, 0, 0  );
-#else
 	Debug( LDAP_DEBUG_ARGS, "modifications:\n", 0, 0, 0 );
-#endif
 
 	for ( tmp = modlist; tmp != NULL; tmp = tmp->sml_next ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, DETAIL1, "\t%s:  %s\n", 
-			tmp->sml_op == LDAP_MOD_ADD ?
-			"add" : (tmp->sml_op == LDAP_MOD_DELETE ?
-			"delete" : "replace"), tmp->sml_type.bv_val, 0 );
-
-		if ( tmp->sml_values == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tno values", 0, 0, 0 );
-		} else if ( tmp->sml_values[0].bv_val == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tzero values", 0, 0, 0 );
-		} else if ( tmp->sml_values[1].bv_val == NULL ) {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tone value", 0, 0, 0 );
-		} else {
-			LDAP_LOG( OPERATION, DETAIL1, "\t\tmultiple values", 0, 0, 0 );
-		}
-
-#else
 		Debug( LDAP_DEBUG_ARGS, "\t%s: %s\n",
-			tmp->sml_op == LDAP_MOD_ADD
-				? "add" : (tmp->sml_op == LDAP_MOD_DELETE
-					? "delete" : "replace"), tmp->sml_type.bv_val, 0 );
+			tmp->sml_op == LDAP_MOD_ADD ? "add" :
+				(tmp->sml_op == LDAP_MOD_INCREMENT ? "increment" :
+				(tmp->sml_op == LDAP_MOD_DELETE ? "delete" :
+					"replace")), tmp->sml_type.bv_val, 0 );
 
 		if ( tmp->sml_values == NULL ) {
 			Debug( LDAP_DEBUG_ARGS, "%s\n",
@@ -260,22 +264,28 @@ do_modify(
 			Debug( LDAP_DEBUG_ARGS, "%s\n",
 			   "\t\tmultiple values", NULL, NULL );
 		}
-#endif
 	}
 
 	if ( StatslogTest( LDAP_DEBUG_STATS ) ) {
 		char abuf[BUFSIZ/2], *ptr = abuf;
 		int len = 0;
 
-		Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD dn=\"%s\"\n",
-			op->o_connid, op->o_opid, dn.bv_val, 0, 0 );
+		Statslog( LDAP_DEBUG_STATS, "%s MOD dn=\"%s\"\n",
+			op->o_log_prefix, op->o_req_dn.bv_val, 0, 0, 0 );
 
 		for ( tmp = modlist; tmp != NULL; tmp = tmp->sml_next ) {
 			if (len + 1 + tmp->sml_type.bv_len > sizeof(abuf)) {
-				Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD attr=%s\n",
-				    op->o_connid, op->o_opid, abuf, 0, 0 );
+				Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+				    op->o_log_prefix, abuf, 0, 0, 0 );
+
 	    			len = 0;
 				ptr = abuf;
+
+				if( 1 + tmp->sml_type.bv_len > sizeof(abuf)) {
+					Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+						op->o_log_prefix, tmp->sml_type.bv_val, 0, 0, 0 );
+					continue;
+				}
 			}
 			if (len) {
 				*ptr++ = ' ';
@@ -285,8 +295,8 @@ do_modify(
 			len += tmp->sml_type.bv_len;
 		}
 		if (len) {
-			Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MOD attr=%s\n",
-	    			op->o_connid, op->o_opid, abuf, 0, 0 );
+			Statslog( LDAP_DEBUG_STATS, "%s MOD attr=%s\n",
+	    			op->o_log_prefix, abuf, 0, 0, 0 );
 		}
 	}
 #endif	/* LDAP_DEBUG */
@@ -298,7 +308,7 @@ do_modify(
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 );
+	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 1 );
 	if ( op->o_bd == NULL ) {
 		rs->sr_ref = referral_rewrite( default_referral,
 			NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
@@ -311,7 +321,7 @@ do_modify(
 			if (rs->sr_ref != default_referral) ber_bvarray_free( rs->sr_ref );
 		} else {
 			send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-					"referral missing" );
+				"no global superior knowledge" );
 		}
 		goto cleanup;
 	}
@@ -327,59 +337,65 @@ do_modify(
 		goto cleanup;
 	}
 
-#if defined( LDAP_SLAPI )
-	slapi_x_pblock_set_operation( pb, op );
-	slapi_pblock_set( pb, SLAPI_MODIFY_TARGET, (void *)dn.bv_val );
-	slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
-	modv = slapi_x_modifications2ldapmods( &modlist );
-	slapi_pblock_set( pb, SLAPI_MODIFY_MODS, (void *)modv );
-
-	rs->sr_err = doPluginFNs( op->o_bd, SLAPI_PLUGIN_PRE_MODIFY_FN, pb );
-	if ( rs->sr_err < 0 ) {
-		/*
-		 * A preoperation plugin failure will abort the
-		 * entire operation.
-		 */
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modify: modify preoperation plugin "
-				"failed\n", 0, 0, 0 );
-#else
-		Debug(LDAP_DEBUG_TRACE, "do_modify: modify preoperation plugin failed.\n",
-				0, 0, 0);
-#endif
-		if ( ( slapi_pblock_get( op->o_pb, SLAPI_RESULT_CODE, (void *)&rs->sr_err ) != 0 )  ||
-		     rs->sr_err == LDAP_SUCCESS ) {
-			rs->sr_err = LDAP_OTHER;
-		}
-		slapi_x_free_ldapmods( modv );
-		modv = NULL;
-		goto cleanup;
+	/* check for modify/increment support */
+	if( increment && !SLAP_INCREMENT( op->o_bd ) ) {
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+			"modify/increment not supported in context" );
 	}
 
-	/*
-	 * It's possible that the preoperation plugin changed the
-	 * modification array, so we need to convert it back to
-	 * a Modification list.
-	 *
-	 * Calling slapi_x_modifications2ldapmods() destroyed modlist so
-	 * we don't need to free it.
-	 */
-	slapi_pblock_get( pb, SLAPI_MODIFY_MODS, (void **)&modv );
-	modlist = slapi_x_ldapmods2modifications( modv );
+#if defined( LDAP_SLAPI )
+#define pb	op->o_pb
+	if ( pb ) {
+		slapi_int_pblock_set_operation( pb, op );
+		slapi_pblock_set( pb, SLAPI_MODIFY_TARGET, (void *)op->o_req_dn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
+		modv = slapi_int_modifications2ldapmods( &modlist );
+		slapi_pblock_set( pb, SLAPI_MODIFY_MODS, (void *)modv );
+
+		rs->sr_err = slapi_int_call_plugins( op->o_bd,
+			SLAPI_PLUGIN_PRE_MODIFY_FN, pb );
+
+		/*
+		 * It's possible that the preoperation plugin changed the
+		 * modification array, so we need to convert it back to
+		 * a Modification list.
+		 *
+		 * Calling slapi_int_modifications2ldapmods() destroyed modlist so
+		 * we don't need to free it.
+		 */
+		slapi_pblock_get( pb, SLAPI_MODIFY_MODS, (void **)&modv );
+		modlist = slapi_int_ldapmods2modifications( modv );
+
+		if ( rs->sr_err < 0 ) {
+			/*
+			 * A preoperation plugin failure will abort the
+			 * entire operation.
+			 */
+			Debug(LDAP_DEBUG_TRACE,
+				"do_modify: modify preoperation plugin failed.\n",
+				0, 0, 0);
+			if ( ( slapi_pblock_get( op->o_pb, SLAPI_RESULT_CODE,
+				(void *)&rs->sr_err ) != 0 ) || rs->sr_err == LDAP_SUCCESS )
+			{
+				rs->sr_err = LDAP_OTHER;
+			}
+			slapi_int_free_ldapmods( modv );
+			modv = NULL;
+			goto cleanup;
+		}
+	}
 
 	/*
 	 * NB: it is valid for the plugin to return no modifications
 	 * (for example, a plugin might store some attributes elsewhere
 	 * and remove them from the modification list; if only those
 	 * attribute types were included in the modification request,
-	 * then slapi_x_ldapmods2modifications() above will return
+	 * then slapi_int_ldapmods2modifications() above will return
 	 * NULL).
+	 *
+	 * However, the post-operation plugin should still be 
+	 * called.
 	 */
-	if ( modlist == NULL ) {
-		rs->sr_err = LDAP_SUCCESS;
-		send_ldap_result( op, rs );
-		goto cleanup;
-	}
 #endif /* defined( LDAP_SLAPI ) */
 
 	/*
@@ -390,32 +406,32 @@ do_modify(
 	 */
 	if ( op->o_bd->be_modify ) {
 		/* do the update here */
-		int repl_user = be_isupdate( op->o_bd, &op->o_ndn );
+		int repl_user = be_isupdate( op );
 
 		/* Multimaster slapd does not have to check for replicator dn
 		 * because it accepts each modify request
 		 */
-#if defined(LDAP_SYNCREPL) && !defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->syncinfo &&
-				( !op->o_bd->be_update_ndn.bv_len || repl_user ))
-#elif defined(LDAP_SYNCREPL) && defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->syncinfo )  /* LDAP_SYNCREPL overrides MM */
-#elif !defined(LDAP_SYNCREPL) && !defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->be_update_ndn.bv_len || repl_user )
+#ifndef SLAPD_MULTIMASTER
+		if ( !SLAP_SHADOW(op->o_bd) || repl_user )
 #endif
 		{
-			int update = op->o_bd->be_update_ndn.bv_len;
-			char textbuf[SLAP_TEXT_BUFLEN];
-			size_t textlen = sizeof textbuf;
+			int		update = !BER_BVISEMPTY( &op->o_bd->be_update_ndn );
+			char		textbuf[ SLAP_TEXT_BUFLEN ];
+			size_t		textlen = sizeof( textbuf );
+			slap_callback	cb = { NULL, slap_replog_cb, NULL, NULL };
 
-			rs->sr_err = slap_mods_check( modlist, update, &rs->sr_text,
-				textbuf, textlen, NULL );
 
-			if( rs->sr_err != LDAP_SUCCESS ) {
-				send_ldap_result( op, rs );
-				goto cleanup;
+			if ( !update ) {
+				rs->sr_err = slap_mods_no_update_check( modlist,
+						&rs->sr_text, textbuf, textlen );
+				if ( rs->sr_err != LDAP_SUCCESS ) {
+					send_ldap_result( op, rs );
+					goto cleanup;
+				}
 			}
 
+
+
 			if ( !repl_user ) {
 				for( modtail = &modlist;
 					*modtail != NULL;
@@ -425,7 +441,7 @@ do_modify(
 				}
 
 				rs->sr_err = slap_mods_opattrs( op, modlist, modtail,
-					&rs->sr_text, textbuf, textlen );
+					&rs->sr_text, textbuf, textlen, 1 );
 				if( rs->sr_err != LDAP_SUCCESS ) {
 					send_ldap_result( op, rs );
 					goto cleanup;
@@ -433,42 +449,40 @@ do_modify(
 			}
 
 			op->orm_modlist = modlist;
-			if ( (op->o_bd->be_modify)( op, rs ) == 0
 #ifdef SLAPD_MULTIMASTER
-				&& !repl_user
-#endif
-			) {
-				/* but we log only the ones not from a replicator user */
-				replog( op );
-			}
-
-#if defined(LDAP_SYNCREPL) || !defined(SLAPD_MULTIMASTER)
-		/* send a referral */
-		} else {
-			BerVarray defref = NULL;
-#ifdef LDAP_SYNCREPL
-			if ( op->o_bd->syncinfo ) {
-				defref = op->o_bd->syncinfo->master_bv;
-			} else
+			if ( !repl_user )
 #endif
 			{
-				defref = op->o_bd->be_update_refs
-						? op->o_bd->be_update_refs : default_referral;
+				/* but multimaster slapd logs only the ones 
+				 * not from a replicator user */
+				cb.sc_next = op->o_callback;
+				op->o_callback = &cb;
 			}
+			op->o_bd->be_modify( op, rs );
+
+#ifndef SLAPD_MULTIMASTER
+		/* send a referral */
+		} else {
+			BerVarray defref = op->o_bd->be_update_refs
+				? op->o_bd->be_update_refs : default_referral;
 			if ( defref != NULL ) {
 				rs->sr_ref = referral_rewrite( defref,
 					NULL, &op->o_req_dn,
 					LDAP_SCOPE_DEFAULT );
-				if (!rs->sr_ref) rs->sr_ref = defref;
+				if ( rs->sr_ref == NULL ) {
+					/* FIXME: must duplicate, because
+					 * overlays may muck with it */
+					rs->sr_ref = defref;
+				}
 				rs->sr_err = LDAP_REFERRAL;
 				send_ldap_result( op, rs );
-				if (rs->sr_ref != defref) {
+				if ( rs->sr_ref != defref ) {
 					ber_bvarray_free( rs->sr_ref );
 				}
+
 			} else {
-				send_ldap_error( op, rs,
-						LDAP_UNWILLING_TO_PERFORM,
-		    				"referral missing" );
+				send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+					"shadow context; no update referral" );
 			}
 #endif
 		}
@@ -478,33 +492,51 @@ do_modify(
 	}
 
 #if defined( LDAP_SLAPI )
-	if ( doPluginFNs( op->o_bd, SLAPI_PLUGIN_POST_MODIFY_FN, pb ) < 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modify: modify postoperation plugins "
-				"failed\n", 0, 0, 0 );
-#else
-		Debug(LDAP_DEBUG_TRACE, "do_modify: modify postoperation plugins "
-				"failed.\n", 0, 0, 0);
-#endif
+	if ( pb != NULL && slapi_int_call_plugins( op->o_bd,
+		SLAPI_PLUGIN_POST_MODIFY_FN, pb ) < 0 )
+	{
+		Debug(LDAP_DEBUG_TRACE,
+			"do_modify: modify postoperation plugins failed.\n", 0, 0, 0);
 	}
 #endif /* defined( LDAP_SLAPI ) */
 
-cleanup:
-	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
-	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-	if ( modlist != NULL ) slap_mods_free( modlist );
+cleanup:;
 #if defined( LDAP_SLAPI )
-	if ( modv != NULL ) slapi_x_free_ldapmods( modv );
+	if ( modv != NULL ) slapi_int_free_ldapmods( modv );
 #endif
+
 	return rs->sr_err;
 }
 
+/*
+ * Do non-update constraint checking.
+ */
+int
+slap_mods_no_update_check(
+	Modifications *ml,
+	const char **text,
+	char *textbuf,
+	size_t textlen )
+{
+	for ( ; ml != NULL; ml = ml->sml_next ) {
+		if ( is_at_no_user_mod( ml->sml_desc->ad_type ) ) {
+			/* user modification disallowed */
+			snprintf( textbuf, textlen,
+				"%s: no user modification allowed",
+				ml->sml_type.bv_val );
+			*text = textbuf;
+			return LDAP_CONSTRAINT_VIOLATION;
+		}
+	}
+
+	return LDAP_SUCCESS;
+}
+
 /*
  * Do basic attribute type checking and syntax validation.
  */
 int slap_mods_check(
 	Modifications *ml,
-	int update,
 	const char **text,
 	char *textbuf,
 	size_t textlen,
@@ -541,7 +573,7 @@ int slap_mods_check(
 		if( !slap_syntax_is_binary( ad->ad_type->sat_syntax )
 			&& slap_ad_is_binary( ad ))
 		{
-			/* attribute requires binary transfer */
+			/* attribute does not require binary transfer */
 			snprintf( textbuf, textlen,
 				"%s: disallows ;binary transfer",
 				ml->sml_type.bv_val );
@@ -558,6 +590,8 @@ int slap_mods_check(
 			return LDAP_UNDEFINED_TYPE;
 		}
 
+#if 0
+		/* moved to slap_mods_no_update_check() */
 		if (!update && is_at_no_user_mod( ad->ad_type )) {
 			/* user modification disallowed */
 			snprintf( textbuf, textlen,
@@ -566,9 +600,12 @@ int slap_mods_check(
 			*text = textbuf;
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
+#endif
 
 		if ( is_at_obsolete( ad->ad_type ) &&
-			( ml->sml_op == LDAP_MOD_ADD || ml->sml_values != NULL ) )
+			(( ml->sml_op != LDAP_MOD_REPLACE &&
+				ml->sml_op != LDAP_MOD_DELETE ) ||
+					ml->sml_values != NULL ))
 		{
 			/*
 			 * attribute is obsolete,
@@ -581,6 +618,22 @@ int slap_mods_check(
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
 
+		if ( ml->sml_op == LDAP_MOD_INCREMENT &&
+#ifdef SLAPD_REAL_SYNTAX
+			!is_at_syntax( ad->ad_type, SLAPD_REAL_SYNTAX ) &&
+#endif
+			!is_at_syntax( ad->ad_type, SLAPD_INTEGER_SYNTAX ) )
+		{
+			/*
+			 * attribute values must be INTEGER or REAL
+			 */
+			snprintf( textbuf, textlen,
+				"%s: attribute syntax inappropriate for increment",
+				ml->sml_type.bv_val );
+			*text = textbuf;
+			return LDAP_CONSTRAINT_VIOLATION;
+		}
+
 		/*
 		 * check values
 		 */
@@ -643,6 +696,14 @@ int slap_mods_check(
 				return LDAP_CONSTRAINT_VIOLATION;
 			}
 
+			/* if the type has a normalizer, generate the
+			 * normalized values. otherwise leave them NULL.
+			 *
+			 * this is different from the rule for attributes
+			 * in an entry - in an attribute list, the normalized
+			 * value is set equal to the non-normalized value
+			 * when there is no normalizer.
+			 */
 			if( nvals && ad->ad_type->sat_equality &&
 				ad->ad_type->sat_equality->smr_normalize )
 			{
@@ -651,20 +712,14 @@ int slap_mods_check(
 
 				for( nvals = 0; ml->sml_values[nvals].bv_val; nvals++ ) {
 					rc = ad->ad_type->sat_equality->smr_normalize(
-						0,
+						SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
 						ad->ad_type->sat_syntax,
 						ad->ad_type->sat_equality,
 						&ml->sml_values[nvals], &ml->sml_nvalues[nvals], ctx );
 					if( rc ) {
-#ifdef NEW_LOGGING
-						LDAP_LOG( OPERATION, DETAIL1,
-							"str2entry:  NULL (ssyn_normalize %d)\n",
-							rc, 0, 0 );
-#else
 						Debug( LDAP_DEBUG_ANY,
 							"<= str2entry NULL (ssyn_normalize %d)\n",
 							rc, 0, 0 );
-#endif
 						snprintf( textbuf, textlen,
 							"%s: value #%ld normalization failed",
 							ml->sml_type.bv_val, (long) nvals );
@@ -675,9 +730,66 @@ int slap_mods_check(
 
 				ml->sml_nvalues[nvals].bv_val = NULL;
 				ml->sml_nvalues[nvals].bv_len = 0;
-
-			} else {
 			}
+
+			if( nvals ) {
+				/* check for duplicates */
+				int		i, j;
+				MatchingRule *mr = ad->ad_type->sat_equality;
+
+				/* check if the values we're adding already exist */
+				if( mr == NULL || !mr->smr_match ) {
+					for ( i = 1; ml->sml_values[i].bv_val != NULL; i++ ) {
+						/* test asserted values against themselves */
+						for( j = 0; j < i; j++ ) {
+							if ( bvmatch( &ml->sml_values[i],
+								&ml->sml_values[j] ) )
+							{
+								/* value exists already */
+								snprintf( textbuf, textlen,
+									"%s: value #%d provided more than once",
+									ml->sml_desc->ad_cname.bv_val, j );
+								*text = textbuf;
+								return LDAP_TYPE_OR_VALUE_EXISTS;
+							}
+						}
+					}
+
+				} else {
+					int rc;
+					int match;
+
+					for ( i = 1; ml->sml_values[i].bv_val != NULL; i++ ) {
+						/* test asserted values against themselves */
+						for( j = 0; j < i; j++ ) {
+							rc = value_match( &match, ml->sml_desc, mr,
+								SLAP_MR_EQUALITY
+									| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX
+									| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
+									| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
+								ml->sml_nvalues
+									? &ml->sml_nvalues[i]
+									: &ml->sml_values[i],
+								ml->sml_nvalues
+									? &ml->sml_nvalues[j]
+									: &ml->sml_values[j],
+								text );
+							if ( rc == LDAP_SUCCESS && match == 0 ) {
+								/* value exists already */
+								snprintf( textbuf, textlen,
+									"%s: value #%d provided more than once",
+									ml->sml_desc->ad_cname.bv_val, j );
+								*text = textbuf;
+								return LDAP_TYPE_OR_VALUE_EXISTS;
+
+							} else if ( rc != LDAP_SUCCESS ) {
+								return rc;
+							}
+						}
+					}
+				}
+			}
+
 		}
 	}
 
@@ -689,7 +801,8 @@ int slap_mods_opattrs(
 	Modifications *mods,
 	Modifications **modtail,
 	const char **text,
-	char *textbuf, size_t textlen )
+	char *textbuf, size_t textlen,
+	int manage_ctxcsn )
 {
 	struct berval name, timestamp, csn;
 	struct berval nname;
@@ -703,24 +816,32 @@ int slap_mods_opattrs(
 	assert( modtail != NULL );
 	assert( *modtail == NULL );
 
-	if( SLAP_LASTMOD(op->o_bd) ) {
+	if ( SLAP_LASTMOD( op->o_bd )) {
 		struct tm *ltm;
+#ifdef HAVE_GMTIME_R
+		struct tm ltm_buf;
+#endif
 		time_t now = slap_get_time();
 
+#ifdef HAVE_GMTIME_R
+		ltm = gmtime_r( &now, &ltm_buf );
+#else
 		ldap_pvt_thread_mutex_lock( &gmtime_mutex );
 		ltm = gmtime( &now );
+#endif /* HAVE_GMTIME_R */
 		lutil_gentime( timebuf, sizeof(timebuf), ltm );
 
-		csn.bv_len = lutil_csnstr( csnbuf, sizeof( csnbuf ), 0, 0 );
+		slap_get_csn( op, csnbuf, sizeof(csnbuf), &csn, manage_ctxcsn );
+
+#ifndef HAVE_GMTIME_R
 		ldap_pvt_thread_mutex_unlock( &gmtime_mutex );
-		csn.bv_val = csnbuf;
+#endif
 
 		timestamp.bv_val = timebuf;
 		timestamp.bv_len = strlen(timebuf);
 
 		if( op->o_dn.bv_len == 0 ) {
-			name.bv_val = SLAPD_ANONYMOUS;
-			name.bv_len = sizeof(SLAPD_ANONYMOUS)-1;
+			BER_BVSTR( &name, SLAPD_ANONYMOUS );
 			nname = name;
 		} else {
 			name = op->o_dn;
@@ -734,9 +855,7 @@ int slap_mods_opattrs(
 		if( global_schemacheck ) {
 			int rc = mods_structural_class( mods, &tmpval,
 				text, textbuf, textlen );
-			if( rc != LDAP_SUCCESS ) {
-				return rc;
-			}
+			if( rc != LDAP_SUCCESS ) return rc;
 
 			mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 			mod->sml_op = mop;
@@ -758,7 +877,7 @@ int slap_mods_opattrs(
 			modtail = &mod->sml_next;
 		}
 
-		if( SLAP_LASTMOD(op->o_bd) ) {
+		if ( SLAP_LASTMOD( op->o_bd )) {
 			char uuidbuf[ LDAP_LUTIL_UUIDSTR_BUFSIZE ];
 
 			tmpval.bv_len = lutil_uuidstr( uuidbuf, sizeof( uuidbuf ) );
@@ -774,7 +893,15 @@ int slap_mods_opattrs(
 			mod->sml_values[1].bv_len = 0;
 			mod->sml_values[1].bv_val = NULL;
 			assert( mod->sml_values[0].bv_val );
-			mod->sml_nvalues = NULL;
+			mod->sml_nvalues =
+				(BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
+			(*mod->sml_desc->ad_type->sat_equality->smr_normalize)(
+					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+					mod->sml_desc->ad_type->sat_syntax,
+					mod->sml_desc->ad_type->sat_equality,
+					mod->sml_values, mod->sml_nvalues, NULL );
+			mod->sml_nvalues[1].bv_len = 0;
+			mod->sml_nvalues[1].bv_val = NULL;
 			*modtail = mod;
 			modtail = &mod->sml_next;
 
@@ -782,7 +909,8 @@ int slap_mods_opattrs(
 			mod->sml_op = mop;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_creatorsName;
-			mod->sml_values = (BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
+			mod->sml_values =
+				(BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
 			ber_dupbv( &mod->sml_values[0], &name );
 			mod->sml_values[1].bv_len = 0;
 			mod->sml_values[1].bv_val = NULL;
@@ -800,7 +928,8 @@ int slap_mods_opattrs(
 			mod->sml_op = mop;
 			mod->sml_type.bv_val = NULL;
 			mod->sml_desc = slap_schema.si_ad_createTimestamp;
-			mod->sml_values = (BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
+			mod->sml_values =
+				(BerVarray) ch_malloc( 2 * sizeof( struct berval ) );
 			ber_dupbv( &mod->sml_values[0], &timestamp );
 			mod->sml_values[1].bv_len = 0;
 			mod->sml_values[1].bv_val = NULL;
@@ -811,7 +940,7 @@ int slap_mods_opattrs(
 		}
 	}
 
-	if( SLAP_LASTMOD(op->o_bd) ) {
+	if ( SLAP_LASTMOD( op->o_bd )) {
 		mod = (Modifications *) ch_malloc( sizeof( Modifications ) );
 		mod->sml_op = mop;
 		mod->sml_type.bv_val = NULL;
