@@ -1,10 +1,26 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modrdn.c,v 1.114 2003/05/01 21:39:29 jongchoi Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/modrdn.c,v 1.146.2.1 2004/12/30 21:29:56 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1995 Regents of the University of Michigan.
+/* Portions Copyright 1999, Juan C. Gomez, All rights reserved.
+ * This software is not subject to any license of Silicon Graphics 
+ * Inc. or Purdue University.
+ *
+ * Redistribution and use in source and binary forms are permitted
+ * without restriction or fee of any kind as long as this notice
+ * is preserved.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -15,19 +31,6 @@
  * is provided ``as is'' without express or implied warranty.
  */
 
-/*
- * LDAP v3 newSuperior support.
- *
- * Copyright 1999, Juan C. Gomez, All rights reserved.
- * This software is not subject to any license of Silicon Graphics 
- * Inc. or Purdue University.
- *
- * Redistribution and use in source and binary forms are permitted
- * without restriction or fee of any kind as long as this notice
- * is preserved.
- *
- */
-
 #include "portable.h"
 
 #include <stdio.h>
@@ -35,10 +38,9 @@
 #include <ac/socket.h>
 #include <ac/string.h>
 
-#include "ldap_pvt.h"
 #include "slap.h"
 #ifdef LDAP_SLAPI
-#include "slapi.h"
+#include "slapi/slapi.h"
 #endif
 
 int
@@ -47,28 +49,18 @@ do_modrdn(
     SlapReply	*rs
 )
 {
-	struct berval dn = { 0, NULL };
-	struct berval newrdn = { 0, NULL };
-	struct berval newSuperior = { 0, NULL };
+	struct berval	dn = BER_BVNULL;
+	struct berval	newrdn = BER_BVNULL;
+	struct berval	newSuperior = BER_BVNULL;
 	ber_int_t	deloldrdn;
 
-	struct berval pnewSuperior = { 0, NULL };
+	struct berval pnewSuperior = BER_BVNULL;
 
-	struct berval nnewSuperior = { 0, NULL };
+	struct berval nnewSuperior = BER_BVNULL;
 
-	Backend	*newSuperior_be = NULL;
 	ber_len_t	length;
-	int manageDSAit;
 
-#ifdef LDAP_SLAPI
-	Slapi_PBlock *pb = op->o_pb;
-#endif
-
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ENTRY, "do_modrdn: begin\n", 0, 0, 0 );
-#else
 	Debug( LDAP_DEBUG_TRACE, "do_modrdn\n", 0, 0, 0 );
-#endif
 
 
 	/*
@@ -85,11 +77,7 @@ do_modrdn(
 	if ( ber_scanf( op->o_ber, "{mmb", &dn, &newrdn, &deloldrdn )
 	    == LBER_ERROR )
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs, LDAP_PROTOCOL_ERROR, "decoding error" );
 		return SLAPD_DISCONNECT;
@@ -102,14 +90,9 @@ do_modrdn(
 			/* Conection record indicates v2 but field 
 			 * newSuperior is present: report error.
 			 */
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-				"do_modrdn: (v2) invalid field newSuperior.\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 			    "modrdn(v2): invalid field newSuperior!\n",
 			    0, 0, 0 );
-#endif
 
 			send_ldap_discon( op, rs,
 				LDAP_PROTOCOL_ERROR, "newSuperior requires LDAPv3" );
@@ -120,13 +103,8 @@ do_modrdn(
 		if ( ber_scanf( op->o_ber, "m", &newSuperior ) 
 		     == LBER_ERROR ) {
 
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, ERR,
-				"do_modrdn: ber_scanf(\"m\") failed\n", 0, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY, "ber_scanf(\"m\") failed\n",
 				0, 0, 0 );
-#endif
 
 			send_ldap_discon( op, rs,
 				LDAP_PROTOCOL_ERROR, "decoding error" );
@@ -137,24 +115,13 @@ do_modrdn(
 		op->orr_nnewSup = &nnewSuperior;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( OPERATION, ARGS, 
-		"do_modrdn: dn (%s) newrdn (%s) newsuperior(%s)\n",
-		dn.bv_val, newrdn.bv_val,
-		newSuperior.bv_len ? newSuperior.bv_val : "" );
-#else
 	Debug( LDAP_DEBUG_ARGS,
 	    "do_modrdn: dn (%s) newrdn (%s) newsuperior (%s)\n",
 		dn.bv_val, newrdn.bv_val,
 		newSuperior.bv_len ? newSuperior.bv_val : "" );
-#endif
 
 	if ( ber_scanf( op->o_ber, /*{*/ "}") == LBER_ERROR ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: ber_scanf failed\n", 0, 0, 0 );
-#endif
 
 		send_ldap_discon( op, rs,
 			LDAP_PROTOCOL_ERROR, "decoding error" );
@@ -163,11 +130,7 @@ do_modrdn(
 	}
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, "do_modrdn: get_ctrls failed\n", 0, 0, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: get_ctrls failed\n", 0, 0, 0 );
-#endif
 
 		/* get_ctrls has sent results.	Now clean up. */
 		goto cleanup;
@@ -175,69 +138,25 @@ do_modrdn(
 
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn, op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_modrdn: conn %d  invalid dn (%s)\n",
-			op->o_connid, dn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modrdn: invalid dn (%s)\n", dn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid DN" );
 		goto cleanup;
 	}
 
-	if( op->o_req_ndn.bv_len == 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modrdn:  attempt to modify root DSE.\n", 0, 0, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modrdn: root dse!\n", 0, 0, 0 );
-#endif
-
-		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-			"cannot rename the root DSE" );
-		goto cleanup;
-
-	} else if ( bvmatch( &op->o_req_ndn, &global_schemandn ) ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR,
-			"do_modrdn: attempt to modify subschema subentry: %s (%ld)\n",
-			global_schemandn.bv_val, (long) global_schemandn.bv_len, 0 );
-#else
-		Debug( LDAP_DEBUG_ANY, "do_modrdn: subschema subentry: %s (%ld)\n",
-			global_schemandn.bv_val, (long) global_schemandn.bv_len, 0 );
-#endif
-
-		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-			"cannot rename subschema subentry" );
-		goto cleanup;
-	}
-
 	/* FIXME: should have/use rdnPretty / rdnNormalize routines */
 
 	rs->sr_err = dnPrettyNormal( NULL, &newrdn, &op->orr_newrdn, &op->orr_nnewrdn, op->o_tmpmemctx );
 	if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, 
-			"do_modrdn: conn %d  invalid newrdn (%s)\n",
-			op->o_connid, newrdn.bv_val, 0 );
-#else
 		Debug( LDAP_DEBUG_ANY,
 			"do_modrdn: invalid newrdn (%s)\n", newrdn.bv_val, 0, 0 );
-#endif
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid new RDN" );
 		goto cleanup;
 	}
 
-	if( rdnValidate( &op->orr_newrdn ) != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, ERR, 
-			"do_modrdn: invalid rdn (%s).\n", op->orr_newrdn.bv_val, 0, 0 );
-#else
+	if( rdn_validate( &op->orr_newrdn ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "do_modrdn: invalid rdn (%s)\n",
 			op->orr_newrdn.bv_val, 0, 0 );
-#endif
 
 		send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid new RDN" );
 		goto cleanup;
@@ -247,22 +166,61 @@ do_modrdn(
 		rs->sr_err = dnPrettyNormal( NULL, &newSuperior, &pnewSuperior,
 			&nnewSuperior, op->o_tmpmemctx );
 		if( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG( OPERATION, INFO, 
-				"do_modrdn: conn %d  invalid newSuperior (%s)\n",
-				op->o_connid, newSuperior.bv_val, 0 );
-#else
 			Debug( LDAP_DEBUG_ANY,
 				"do_modrdn: invalid newSuperior (%s)\n",
 				newSuperior.bv_val, 0, 0 );
-#endif
 			send_ldap_error( op, rs, LDAP_INVALID_DN_SYNTAX, "invalid newSuperior" );
 			goto cleanup;
 		}
 	}
 
-	Statslog( LDAP_DEBUG_STATS, "conn=%lu op=%lu MODRDN dn=\"%s\"\n",
-	    op->o_connid, op->o_opid, op->o_req_dn.bv_val, 0, 0 );
+	/* FIXME: temporary? */
+	op->orr_deleteoldrdn = deloldrdn;
+
+	op->o_bd = frontendDB;
+	rs->sr_err = frontendDB->be_modrdn( op, rs );
+
+cleanup:
+
+	slap_graduate_commit_csn( op );
+
+	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
+	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
+
+	op->o_tmpfree( op->orr_newrdn.bv_val, op->o_tmpmemctx );	
+	op->o_tmpfree( op->orr_nnewrdn.bv_val, op->o_tmpmemctx );	
+
+	if ( pnewSuperior.bv_val ) op->o_tmpfree( pnewSuperior.bv_val, op->o_tmpmemctx );
+	if ( nnewSuperior.bv_val ) op->o_tmpfree( nnewSuperior.bv_val, op->o_tmpmemctx );
+
+	return rs->sr_err;
+}
+
+int
+fe_op_modrdn( Operation *op, SlapReply *rs )
+{
+	Backend		*newSuperior_be = NULL;
+	int		manageDSAit;
+	struct berval	pdn = BER_BVNULL;
+	
+	if( op->o_req_ndn.bv_len == 0 ) {
+		Debug( LDAP_DEBUG_ANY, "do_modrdn: root dse!\n", 0, 0, 0 );
+
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+			"cannot rename the root DSE" );
+		goto cleanup;
+
+	} else if ( bvmatch( &op->o_req_ndn, &frontendDB->be_schemandn ) ) {
+		Debug( LDAP_DEBUG_ANY, "do_modrdn: subschema subentry: %s (%ld)\n",
+			frontendDB->be_schemandn.bv_val, (long)frontendDB->be_schemandn.bv_len, 0 );
+
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+			"cannot rename subschema subentry" );
+		goto cleanup;
+	}
+
+	Statslog( LDAP_DEBUG_STATS, "%s MODRDN dn=\"%s\"\n",
+	    op->o_log_prefix, op->o_req_dn.bv_val, 0, 0, 0 );
 
 	manageDSAit = get_manageDSAit( op );
 
@@ -271,7 +229,8 @@ do_modrdn(
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	if ( (op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 0 )) == NULL ) {
+	op->o_bd = select_backend( &op->o_req_ndn, manageDSAit, 1 );
+	if ( op->o_bd == NULL ) {
 		rs->sr_ref = referral_rewrite( default_referral,
 			NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
 		if (!rs->sr_ref) rs->sr_ref = default_referral;
@@ -283,7 +242,7 @@ do_modrdn(
 			if (rs->sr_ref != default_referral) ber_bvarray_free( rs->sr_ref );
 		} else {
 			send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
-					"referral missing" );
+				"no global superior knowledge" );
 		}
 		goto cleanup;
 	}
@@ -303,7 +262,7 @@ do_modrdn(
 	 * the same backend, otherwise we return an error.
 	 */
 	if( op->orr_newSup ) {
-		newSuperior_be = select_backend( &nnewSuperior, 0, 0 );
+		newSuperior_be = select_backend( op->orr_nnewSup, 0, 0 );
 
 		if ( newSuperior_be != op->o_bd ) {
 			/* newSuperior is in different backend */
@@ -315,32 +274,30 @@ do_modrdn(
 	}
 
 #if defined( LDAP_SLAPI )
-	slapi_x_pblock_set_operation( pb, op );
-	slapi_pblock_set( pb, SLAPI_MODRDN_TARGET, (void *)dn.bv_val );
-	slapi_pblock_set( pb, SLAPI_MODRDN_NEWRDN, (void *)newrdn.bv_val );
-	slapi_pblock_set( pb, SLAPI_MODRDN_NEWSUPERIOR,
-			(void *)newSuperior.bv_val );
-	slapi_pblock_set( pb, SLAPI_MODRDN_DELOLDRDN, (void *)deloldrdn );
-	slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
+#define	pb	op->o_pb
+	if ( pb ) {
+		slapi_int_pblock_set_operation( pb, op );
+		slapi_pblock_set( pb, SLAPI_MODRDN_TARGET, (void *)op->o_req_dn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_NEWRDN, (void *)op->orr_newrdn.bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_NEWSUPERIOR,
+				(void *)op->orr_newSup->bv_val );
+		slapi_pblock_set( pb, SLAPI_MODRDN_DELOLDRDN, (void *)op->orr_deleteoldrdn);
+		slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
 
-	rs->sr_err = doPluginFNs( op->o_bd, SLAPI_PLUGIN_PRE_MODRDN_FN, pb );
-	if ( rs->sr_err < 0 ) {
-		/*
-		 * A preoperation plugin failure will abort the
-		 * entire operation.
-		 */
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modrdn: modrdn preoperation plugin "
-				"failed\n", 0, 0, 0 );
-#else
-		Debug(LDAP_DEBUG_TRACE, "do_modrdn: modrdn preoperation plugin "
-				"failed.\n", 0, 0, 0);
-#endif
-		if ( ( slapi_pblock_get( pb, SLAPI_RESULT_CODE, (void *)&rs->sr_err ) != 0 ) ||
-		     rs->sr_err == LDAP_SUCCESS ) {
-			rs->sr_err = LDAP_OTHER;
+		rs->sr_err = slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_PRE_MODRDN_FN, pb );
+		if ( rs->sr_err < 0 ) {
+			/*
+			 * A preoperation plugin failure will abort the
+			 * entire operation.
+			 */
+			Debug(LDAP_DEBUG_TRACE, "do_modrdn: modrdn preoperation plugin "
+					"failed.\n", 0, 0, 0);
+			if ( ( slapi_pblock_get( pb, SLAPI_RESULT_CODE, (void *)&rs->sr_err ) != 0 ) ||
+				 rs->sr_err == LDAP_SUCCESS ) {
+				rs->sr_err = LDAP_OTHER;
+			}
+			goto cleanup;
 		}
-		goto cleanup;
 	}
 #endif /* defined( LDAP_SLAPI ) */
 
@@ -352,36 +309,65 @@ do_modrdn(
 	 */
 	if ( op->o_bd->be_modrdn ) {
 		/* do the update here */
-		int repl_user = be_isupdate( op->o_bd, &op->o_ndn );
-#if defined(LDAP_SYNCREPL) && !defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->syncinfo &&
-					( !op->o_bd->be_update_ndn.bv_len || repl_user ))
-#elif defined(LDAP_SYNCREPL) && defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->syncinfo )  /* LDAP_SYNCREPL overrides MM */
-#elif !defined(LDAP_SYNCREPL) && !defined(SLAPD_MULTIMASTER)
-		if ( !op->o_bd->be_update_ndn.bv_len || repl_user )
+		int repl_user = be_isupdate( op );
+#ifndef SLAPD_MULTIMASTER
+		if ( !SLAP_SHADOW(op->o_bd) || repl_user )
 #endif
 		{
-			op->orr_deleteoldrdn = deloldrdn;
-			if ( (op->o_bd->be_modrdn)( op, rs ) == 0
+			slap_callback cb = { NULL, slap_replog_cb, NULL, NULL };
 #ifdef SLAPD_MULTIMASTER
-				&& ( !op->o_bd->be_update_ndn.bv_len || !repl_user )
-#endif
-			) {
-				replog( op );
-			}
-#if defined(LDAP_SYNCREPL) || !defined(SLAPD_MULTIMASTER)
-		} else {
-			BerVarray defref = NULL;
-#ifdef LDAP_SYNCREPL
-			if ( op->o_bd->syncinfo ) {
-				defref = op->o_bd->syncinfo->master_bv;
-			} else
+			if ( !op->o_bd->be_update_ndn.bv_len || !repl_user )
 #endif
 			{
-				defref = op->o_bd->be_update_refs
-						? op->o_bd->be_update_refs : default_referral;
+				cb.sc_next = op->o_callback;
+				op->o_callback = &cb;
 			}
+			op->o_bd->be_modrdn( op, rs );
+
+			if ( op->o_bd->be_delete ) {
+				struct berval	org_req_dn = BER_BVNULL;
+				struct berval	org_req_ndn = BER_BVNULL;
+				struct berval	org_dn = BER_BVNULL;
+				struct berval	org_ndn = BER_BVNULL;
+				int		org_managedsait;
+
+				org_req_dn = op->o_req_dn;
+				org_req_ndn = op->o_req_ndn;
+				org_dn = op->o_dn;
+				org_ndn = op->o_ndn;
+				org_managedsait = get_manageDSAit( op );
+				op->o_dn = op->o_bd->be_rootdn;
+				op->o_ndn = op->o_bd->be_rootndn;
+				op->o_managedsait = SLAP_CONTROL_NONCRITICAL;
+
+				while ( rs->sr_err == LDAP_SUCCESS &&
+						op->o_delete_glue_parent ) {
+					op->o_delete_glue_parent = 0;
+					if ( !be_issuffix( op->o_bd, &op->o_req_ndn )) {
+						slap_callback cb = { NULL };
+						cb.sc_response = slap_null_cb;
+						dnParent( &op->o_req_ndn, &pdn );
+						op->o_req_dn = pdn;
+						op->o_req_ndn = pdn;
+						op->o_callback = &cb;
+						op->o_bd->be_delete( op, rs );
+					} else {
+						break;
+					}
+				}
+				op->o_managedsait = org_managedsait;
+	            		op->o_dn = org_dn;
+				op->o_ndn = org_ndn;
+				op->o_req_dn = org_req_dn;
+				op->o_req_ndn = org_req_ndn;
+				op->o_delete_glue_parent = 0;
+			}
+
+#ifndef SLAPD_MULTIMASTER
+		} else {
+			BerVarray defref = op->o_bd->be_update_refs
+				? op->o_bd->be_update_refs : default_referral;
+
 			if ( defref != NULL ) {
 				rs->sr_ref = referral_rewrite( defref,
 					NULL, &op->o_req_dn, LDAP_SCOPE_DEFAULT );
@@ -392,9 +378,8 @@ do_modrdn(
 
 				if (rs->sr_ref != defref) ber_bvarray_free( rs->sr_ref );
 			} else {
-				send_ldap_error( op, rs,
-					LDAP_UNWILLING_TO_PERFORM,
-					"referral missing" );
+				send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+					"shadow context; no update referral" );
 			}
 #endif
 		}
@@ -404,27 +389,13 @@ do_modrdn(
 	}
 
 #if defined( LDAP_SLAPI )
-	if ( doPluginFNs( op->o_bd, SLAPI_PLUGIN_POST_MODRDN_FN, pb ) < 0 ) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( OPERATION, INFO, "do_modrdn: modrdn postoperation plugins "
-				"failed\n", 0, 0, 0 );
-#else
+	if ( pb != NULL && slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_POST_MODRDN_FN, pb ) < 0 ) {
 		Debug(LDAP_DEBUG_TRACE, "do_modrdn: modrdn postoperation plugins "
 				"failed.\n", 0, 0, 0);
-#endif
 	}
 #endif /* defined( LDAP_SLAPI ) */
 
-cleanup:
-	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
-	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-
-	op->o_tmpfree( op->orr_newrdn.bv_val, op->o_tmpmemctx );	
-	op->o_tmpfree( op->orr_nnewrdn.bv_val, op->o_tmpmemctx );	
-
-	if ( pnewSuperior.bv_val ) op->o_tmpfree( pnewSuperior.bv_val, op->o_tmpmemctx );
-	if ( nnewSuperior.bv_val ) op->o_tmpfree( nnewSuperior.bv_val, op->o_tmpmemctx );
-
+cleanup:;
 	return rs->sr_err;
 }
 
@@ -438,11 +409,15 @@ slap_modrdn2mods(
 	Modifications	**pmod )
 {
 	Modifications	*mod = NULL;
+	Modifications	**modtail = &mod;
 	int		a_cnt, d_cnt;
+	int repl_user;
 
 	assert( new_rdn != NULL );
 	assert( !op->orr_deleteoldrdn || old_rdn != NULL );
 
+	repl_user = be_isupdate( op );
+
 	/* Add new attribute values to the entry */
 	for ( a_cnt = 0; new_rdn[a_cnt]; a_cnt++ ) {
 		AttributeDescription	*desc = NULL;
@@ -451,34 +426,21 @@ slap_modrdn2mods(
 		rs->sr_err = slap_bv2ad( &new_rdn[a_cnt]->la_attr, &desc, &rs->sr_text );
 
 		if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-				"slap_modrdn2modlist: %s: %s (new)\n", 
-				rs->sr_text, 
-				new_rdn[ a_cnt ]->la_attr.bv_val, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"slap_modrdn2modlist: %s: %s (new)\n",
 				rs->sr_text, 
 				new_rdn[ a_cnt ]->la_attr.bv_val, 0 );
-#endif
 			goto done;		
 		}
 
 		/* ACL check of newly added attrs */
 		if ( op->o_bd && !access_allowed( op, e, desc,
 			&new_rdn[a_cnt]->la_value, ACL_WRITE, NULL ) ) {
-#ifdef NEW_LOGGING
-			LDAP_LOG ( OPERATION, ERR, 
-				"slap_modrdn2modlist: access to attr \"%s\" "
-				"(new) not allowed\n", 
-				new_rdn[a_cnt]->la_attr.bv_val, 0, 0 );
-#else
 			Debug( LDAP_DEBUG_TRACE,
 				"slap_modrdn2modlist: access to attr \"%s\" "
 				"(new) not allowed\n", 
 				new_rdn[ a_cnt ]->la_attr.bv_val, 0, 0 );
-#endif
+			rs->sr_text = "access to naming attributes (new) not allowed";
 			rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
 			goto done;
 		}
@@ -493,11 +455,11 @@ slap_modrdn2mods(
 		if( desc->ad_type->sat_equality->smr_normalize) {
 			mod_tmp->sml_nvalues = &mod_tmp->sml_values[2];
 			(void) (*desc->ad_type->sat_equality->smr_normalize)(
-				SLAP_MR_EQUALITY,
+				SLAP_MR_EQUALITY|SLAP_MR_VALUE_OF_ASSERTION_SYNTAX,
 				desc->ad_type->sat_syntax,
 				desc->ad_type->sat_equality,
 				&mod_tmp->sml_values[0],
-				&mod_tmp->sml_nvalues[0], op->o_tmpmemctx );
+				&mod_tmp->sml_nvalues[0], NULL );
 			mod_tmp->sml_nvalues[1].bv_val = NULL;
 		} else {
 			mod_tmp->sml_nvalues = NULL;
@@ -515,19 +477,11 @@ slap_modrdn2mods(
 
 			rs->sr_err = slap_bv2ad( &old_rdn[d_cnt]->la_attr, &desc, &rs->sr_text );
 			if ( rs->sr_err != LDAP_SUCCESS ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"slap_modrdn2modlist: %s: %s (old)\n", 
-					rs->sr_text, 
-					old_rdn[d_cnt]->la_attr.bv_val, 
-					0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 					"slap_modrdn2modlist: %s: %s (old)\n",
 					rs->sr_text, 
 					old_rdn[d_cnt]->la_attr.bv_val, 
 					0 );
-#endif
 				goto done;		
 			}
 
@@ -535,19 +489,12 @@ slap_modrdn2mods(
 			if ( op->o_bd && !access_allowed( op, e, desc,
 				&old_rdn[d_cnt]->la_value, ACL_WRITE, 
 				NULL ) ) {
-#ifdef NEW_LOGGING
-				LDAP_LOG ( OPERATION, ERR, 
-					"slap_modrdn2modlist: access "
-					"to attr \"%s\" (old) not allowed\n", 
-					old_rdn[ d_cnt ]->la_attr.bv_val, 
-					0, 0 );
-#else
 				Debug( LDAP_DEBUG_TRACE,
 					"slap_modrdn2modlist: access "
 					"to attr \"%s\" (old) not allowed\n", 
 					old_rdn[ d_cnt ]->la_attr.bv_val,
 					0, 0 );
-#endif
+				rs->sr_text = "access to naming attributes (old) not allowed";
 				rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
 				goto done;
 			}
@@ -562,7 +509,7 @@ slap_modrdn2mods(
 			if( desc->ad_type->sat_equality->smr_normalize) {
 				mod_tmp->sml_nvalues = &mod_tmp->sml_values[2];
 				(void) (*desc->ad_type->sat_equality->smr_normalize)(
-					SLAP_MR_EQUALITY,
+					SLAP_MR_EQUALITY|SLAP_MR_VALUE_OF_ASSERTION_SYNTAX,
 					desc->ad_type->sat_syntax,
 					desc->ad_type->sat_equality,
 					&mod_tmp->sml_values[0],
@@ -578,6 +525,22 @@ slap_modrdn2mods(
 	}
 	
 done:
+
+	if ( rs->sr_err == LDAP_SUCCESS && !repl_user ) {
+		char textbuf[ SLAP_TEXT_BUFLEN ];
+		size_t textlen = sizeof textbuf;
+
+		for( modtail = &mod;
+			*modtail != NULL;
+			modtail = &(*modtail)->sml_next )
+		{
+			/* empty */
+		}
+
+		rs->sr_err = slap_mods_opattrs( op, mod, modtail,
+						&rs->sr_text, textbuf, textlen, 1 );
+	}
+
 	/* LDAP v2 supporting correct attribute handling. */
 	if ( rs->sr_err != LDAP_SUCCESS && mod != NULL ) {
 		Modifications *tmp;
