@@ -1,9 +1,18 @@
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/mods.c,v 1.47 2004/09/27 20:16:21 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1995 Regents of the University of Michigan.
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -16,40 +25,25 @@
 
 #include "portable.h"
 
+#include <ac/string.h>
+
 #include "slap.h"
 
-int
-modify_check_duplicates(
-	AttributeDescription	*ad,
-	MatchingRule		*mr,
-	BerVarray		vals,
-	BerVarray		mods,
-	int			permissive,
-	const char	**text,
-	char *textbuf, size_t textlen )
-{
-	int		i, j, numvals = 0, nummods,
-			rc = LDAP_SUCCESS, matched;
-	/* this function is no longer used */
-	return rc;
-}
-
 int
 modify_add_values(
-	Entry	*e,
+	Entry		*e,
 	Modification	*mod,
-	int	permissive,
+	int		permissive,
 	const char	**text,
-	char *textbuf, size_t textlen
-)
+	char		*textbuf,
+	size_t		textlen )
 {
-	int		i, j;
-	int		matched;
+	int		rc;
+	const char	*op;
 	Attribute	*a;
-	MatchingRule *mr = mod->sm_desc->ad_type->sat_equality;
-	const char *op;
+	Modification	pmod = *mod;
 
-	switch( mod->sm_op ) {
+	switch ( mod->sm_op ) {
 	case LDAP_MOD_ADD:
 		op = "add";
 		break;
@@ -61,17 +55,14 @@ modify_add_values(
 		assert( 0 );
 	}
 
+	/* check if values to add exist in attribute */
 	a = attr_find( e->e_attrs, mod->sm_desc );
+	if ( a != NULL ) {
+		int		rc, i, j, p;
+		MatchingRule	*mr;
 
-	/*
-	 * With permissive set, as long as the attribute being added
-	 * has the same value(s?) as the existing attribute, then the
-	 * modify will succeed.
-	 */
-
-	/* check if the values we're adding already exist */
-	if( mr == NULL || !mr->smr_match ) {
-		if ( a != NULL ) {
+		mr = mod->sm_desc->ad_type->sat_equality;
+		if( mr == NULL || !mr->smr_match ) {
 			/* do not allow add of additional attribute
 				if no equality rule exists */
 			*text = textbuf;
@@ -81,117 +72,85 @@ modify_add_values(
 			return LDAP_INAPPROPRIATE_MATCHING;
 		}
 
-		for ( i = 0; mod->sm_bvalues[i].bv_val != NULL; i++ ) {
-			/* test asserted values against existing values */
-			if( a ) {
-				for( matched = 0, j = 0; a->a_vals[j].bv_val != NULL; j++ ) {
-					if ( bvmatch( &mod->sm_bvalues[i], &a->a_vals[j] ) ) {
-						if ( permissive ) {
-							matched++;
-							continue;
-						}
-						/* value exists already */
-						*text = textbuf;
-						snprintf( textbuf, textlen,
-							"modify/%s: %s: value #%i already exists",
-							op, mod->sm_desc->ad_cname.bv_val, j );
-						return LDAP_TYPE_OR_VALUE_EXISTS;
-					}
-				}
-				if ( permissive && matched == j ) {
-					/* values already exist; do nothing */
-					return LDAP_SUCCESS;
-				}
-			}
-
-			/* test asserted values against themselves */
-			for( j = 0; j < i; j++ ) {
-				if ( bvmatch( &mod->sm_bvalues[i], &mod->sm_bvalues[j] ) ) {
-					/* value exists already */
-					*text = textbuf;
-					snprintf( textbuf, textlen,
-						"modify/%s: %s: value #%i already exists",
-						op, mod->sm_desc->ad_cname.bv_val, j );
-					return LDAP_TYPE_OR_VALUE_EXISTS;
-				}
+		if ( permissive ) {
+			for ( i = 0; !BER_BVISNULL( &mod->sm_values[i] ); i++ ) /* count 'em */;
+			pmod.sm_values = (BerVarray)ch_malloc( (i + 1)*sizeof( struct berval ) );
+			if ( pmod.sm_nvalues != NULL ) {
+				pmod.sm_nvalues = (BerVarray)ch_malloc(
+					(i + 1)*sizeof( struct berval ) );
 			}
 		}
 
-	} else {
 		/* no normalization is done in this routine nor
 		 * in the matching routines called by this routine. 
 		 * values are now normalized once on input to the
 		 * server (whether from LDAP or from the underlying
 		 * database).
-		 * This should outperform the old code.  No numbers
-		 * are available yet.
 		 */
+		for ( p = i = 0; !BER_BVISNULL( &mod->sm_values[i] ); i++ ) {
+			int	match;
 
-		int		rc;
-
-		if ( mod->sm_bvalues[1].bv_val == 0 ) {
-			if ( a != NULL ) {
-				int		i;
-
-				for ( matched = 0, i = 0; a->a_vals[ i ].bv_val; i++ ) {
-					int	match;
-
-					if( mod->sm_nvalues ) {
-						rc = value_match( &match, mod->sm_desc, mr,
-							SLAP_MR_EQUALITY
-								| SLAP_MR_VALUE_OF_ASSERTION_SYNTAX
-								| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
-								| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
-							&a->a_nvals[i],
-							&mod->sm_nvalues[0],
-							text );
-
-					} else {
-						rc = value_match( &match, mod->sm_desc, mr,
-							SLAP_MR_EQUALITY
-								| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
-							&a->a_vals[i],
-							&mod->sm_values[0],
-							text );
-					}
-
-
-					if( rc == LDAP_SUCCESS && match == 0 ) {
-						if ( permissive ) {
-							matched++;
-							continue;
-						}
-						*text = textbuf;
-						snprintf( textbuf, textlen,
-							"modify/%s: %s: value #0 already exists",
-							op, mod->sm_desc->ad_cname.bv_val );
-						return LDAP_TYPE_OR_VALUE_EXISTS;
-					}
+			assert( a->a_vals[0].bv_val );
+			for ( j = 0; !BER_BVISNULL( &a->a_vals[j] ); j++ ) {
+				if ( mod->sm_nvalues ) {
+					rc = value_match( &match, mod->sm_desc, mr,
+						SLAP_MR_EQUALITY | SLAP_MR_VALUE_OF_ASSERTION_SYNTAX
+							| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
+							| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
+						&a->a_nvals[j], &mod->sm_nvalues[i], text );
+				} else {
+					rc = value_match( &match, mod->sm_desc, mr,
+						SLAP_MR_EQUALITY | SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+						&a->a_vals[j], &mod->sm_values[i], text );
 				}
-				if ( permissive && matched == i ) {
-					/* values already exist; do nothing */
-					return LDAP_SUCCESS;
+
+				if ( rc == LDAP_SUCCESS && match == 0 ) {
+					/* value already exists */
+					if ( permissive ) break;
+
+					*text = textbuf;
+					snprintf( textbuf, textlen,
+						"modify/%s: %s: value #%d already exists",
+						op, mod->sm_desc->ad_cname.bv_val, i );
+					return LDAP_TYPE_OR_VALUE_EXISTS;
+
+				} else if ( rc != LDAP_SUCCESS ) {
+					return rc;
 				}
 			}
 
-		} else {
-			rc = modify_check_duplicates( mod->sm_desc, mr,
-					a ? a->a_vals : NULL, mod->sm_bvalues,
-					permissive, text, textbuf, textlen );
+			if ( permissive && match != 0 ) {
+				if ( pmod.sm_nvalues ) {
+					pmod.sm_nvalues[p] = mod->sm_nvalues[i];
+				}
+				pmod.sm_values[p++] = mod->sm_values[i];
+			}
+		}
 
-			if ( permissive && rc == LDAP_TYPE_OR_VALUE_EXISTS ) {
+		if ( permissive ) {
+			if ( p == 0 ) {
+				/* all new values match exist */
+				ch_free( pmod.sm_values );
+				if ( pmod.sm_nvalues ) ch_free( pmod.sm_nvalues );
 				return LDAP_SUCCESS;
 			}
 
-			if ( rc != LDAP_SUCCESS ) {
-				return rc;
+			BER_BVZERO( &pmod.sm_values[p] );
+			if ( pmod.sm_nvalues ) {
+				BER_BVZERO( &pmod.sm_nvalues[p] );
 			}
 		}
 	}
 
 	/* no - add them */
-	if( attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues ) != 0 )
-	{
+	rc = attr_merge( e, mod->sm_desc, pmod.sm_values, pmod.sm_nvalues );
+
+	if ( a != NULL && permissive ) {
+		ch_free( pmod.sm_values );
+		if ( pmod.sm_nvalues ) ch_free( pmod.sm_nvalues );
+	}
+
+	if ( rc != 0 ) {
 		/* this should return result of attr_merge */
 		*text = textbuf;
 		snprintf( textbuf, textlen,
@@ -209,13 +168,13 @@ modify_delete_values(
 	Modification	*mod,
 	int	permissive,
 	const char	**text,
-	char *textbuf, size_t textlen
-)
+	char *textbuf, size_t textlen )
 {
 	int		i, j, k, rc = LDAP_SUCCESS;
 	Attribute	*a;
 	MatchingRule 	*mr = mod->sm_desc->ad_type->sat_equality;
 	char		dummy = '\0';
+	int		match = 0;
 
 	/*
 	 * If permissive is set, then the non-existence of an 
@@ -223,7 +182,7 @@ modify_delete_values(
 	 */
 
 	/* delete the entire attribute */
-	if ( mod->sm_bvalues == NULL ) {
+	if ( mod->sm_values == NULL ) {
 		rc = attr_delete( &e->e_attrs, mod->sm_desc );
 
 		if( permissive ) {
@@ -260,12 +219,13 @@ modify_delete_values(
 		return LDAP_NO_SUCH_ATTRIBUTE;
 	}
 
-
-	for ( i = 0; mod->sm_values[i].bv_val != NULL; i++ ) {
+	for ( i = 0; !BER_BVISNULL( &mod->sm_values[i] ); i++ ) {
 		int	found = 0;
-		for ( j = 0; a->a_vals[j].bv_val != NULL; j++ )
-		{
-			int match;
+		for ( j = 0; !BER_BVISNULL( &a->a_vals[j] ); j++ ) {
+			/* skip already deleted values */
+			if ( a->a_vals[j].bv_val == &dummy ) {
+				continue;
+			}
 
 			if( mod->sm_nvalues ) {
 				assert( a->a_nvals );
@@ -292,7 +252,7 @@ modify_delete_values(
 				snprintf( textbuf, textlen,
 					"%s: matching rule failed",
 					mod->sm_desc->ad_cname.bv_val );
-				goto return_results;
+				break;
 			}
 
 			if ( match != 0 ) {
@@ -312,20 +272,22 @@ modify_delete_values(
 			break;
 		}
 
-
 		if ( found == 0 ) {
 			*text = textbuf;
 			snprintf( textbuf, textlen,
 				"modify/delete: %s: no such value",
 				mod->sm_desc->ad_cname.bv_val );
 			rc = LDAP_NO_SUCH_ATTRIBUTE;
-			goto return_results;
+			if ( i > 0 ) {
+				break;
+			} else {
+				goto return_results;
+			}
 		}
 	}
 
 	/* compact array skipping dummies */
-	for ( k = 0, j = 0; a->a_vals[k].bv_val != NULL; k++ )
-	{
+	for ( k = 0, j = 0; !BER_BVISNULL( &a->a_vals[k] ); k++ ) {
 		/* skip dummies */
 		if( a->a_vals[k].bv_val == &dummy ) {
 			assert( a->a_nvals == NULL || a->a_nvals[k].bv_val == &dummy );
@@ -341,11 +303,13 @@ modify_delete_values(
 		j++;
 	}
 
-	a->a_vals[j].bv_val = NULL;
-	if (a->a_nvals != a->a_vals) a->a_nvals[j].bv_val = NULL;
+	BER_BVZERO( &a->a_vals[j] );
+	if (a->a_nvals != a->a_vals) {
+		BER_BVZERO( &a->a_nvals[j] );
+	}
 
 	/* if no values remain, delete the entire attribute */
-	if ( a->a_vals[0].bv_val == NULL ) {
+	if ( BER_BVISNULL( &a->a_vals[0] ) ) {
 		if ( attr_delete( &e->e_attrs, mod->sm_desc ) ) {
 			*text = textbuf;
 			snprintf( textbuf, textlen,
@@ -366,23 +330,77 @@ modify_replace_values(
 	Modification	*mod,
 	int		permissive,
 	const char	**text,
-	char *textbuf, size_t textlen
-)
+	char *textbuf, size_t textlen )
 {
 	(void) attr_delete( &e->e_attrs, mod->sm_desc );
 
-	if ( mod->sm_bvalues ) {
+	if ( mod->sm_values ) {
 		return modify_add_values( e, mod, permissive, text, textbuf, textlen );
 	}
 
 	return LDAP_SUCCESS;
 }
 
+int
+modify_increment_values(
+	Entry	*e,
+	Modification	*mod,
+	int	permissive,
+	const char	**text,
+	char *textbuf, size_t textlen )
+{
+	Attribute *a;
+
+	a = attr_find( e->e_attrs, mod->sm_desc );
+	if( a == NULL ) {
+		*text = textbuf;
+		snprintf( textbuf, textlen,
+			"modify/increment: %s: no such attribute",
+			mod->sm_desc->ad_cname.bv_val );
+		return LDAP_NO_SUCH_ATTRIBUTE;
+	}
+
+	if ( !strcmp( a->a_desc->ad_type->sat_syntax_oid, SLAPD_INTEGER_SYNTAX )) {
+		int i;
+		char str[sizeof(long)*3 + 2]; /* overly long */
+		long incr = atol( mod->sm_values[0].bv_val );
+
+		/* treat zero and errors as a no-op */
+		if( incr == 0 ) {
+			return LDAP_SUCCESS;
+		}
+
+		for( i = 0; !BER_BVISNULL( &a->a_nvals[i] ); i++ ) {
+			char *tmp;
+			long value = atol( a->a_nvals[i].bv_val );
+			size_t strln = snprintf( str, sizeof(str), "%ld", value+incr );
+
+			tmp = SLAP_REALLOC( a->a_nvals[i].bv_val, strln+1 );
+			if( tmp == NULL ) {
+				*text = "modify/increment: reallocation error";
+				return LDAP_OTHER;;
+			}
+			a->a_nvals[i].bv_val = tmp;
+			a->a_nvals[i].bv_len = strln;
+
+			AC_MEMCPY( a->a_nvals[i].bv_val, str, strln+1 );
+		}
+
+	} else {
+		snprintf( textbuf, textlen,
+			"modify/increment: %s: increment not supported for value syntax %s",
+			mod->sm_desc->ad_cname.bv_val,
+			a->a_desc->ad_type->sat_syntax_oid );
+		return LDAP_CONSTRAINT_VIOLATION;
+	}
+
+	return LDAP_SUCCESS;
+}
+
 void
 slap_mod_free(
 	Modification	*mod,
-	int				freeit
-)
+	int				freeit )
 {
 	if ( mod->sm_values != NULL ) ber_bvarray_free( mod->sm_values );
 	mod->sm_values = NULL;
@@ -395,8 +413,7 @@ slap_mod_free(
 
 void
 slap_mods_free(
-    Modifications	*ml
-)
+    Modifications	*ml )
 {
 	Modifications *next;
 
