@@ -1,8 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/module.c,v 1.21 2003/05/25 01:56:58 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/module.c,v 1.26 2004/09/04 02:54:30 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+
 #include "portable.h"
 #include <stdio.h>
 #include "slap.h"
@@ -40,16 +50,20 @@ module_loaded_t *module_list = NULL;
 
 static int module_unload (module_loaded_t *module);
 
+#ifdef HAVE_EBCDIC
+static char ebuf[BUFSIZ];
+#endif
+
 int module_init (void)
 {
 	if (lt_dlinit()) {
 		const char *error = lt_dlerror();
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_init: lt_ldinit failed: %s\n", error, 0, 0 );
-#else
-		Debug(LDAP_DEBUG_ANY, "lt_dlinit failed: %s\n", error, 0, 0);
+#ifdef HAVE_EBCDIC
+		strcpy( ebuf, error );
+		__etoa( ebuf );
+		error = ebuf;
 #endif
+		Debug(LDAP_DEBUG_ANY, "lt_dlinit failed: %s\n", error, 0, 0);
 
 		return -1;
 	}
@@ -65,12 +79,12 @@ int module_kill (void)
 
 	if (lt_dlexit()) {
 		const char *error = lt_dlerror();
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, "module_kill: lt_dlexit failed: %s\n", 
-			error, 0, 0 );
-#else
-		Debug(LDAP_DEBUG_ANY, "lt_dlexit failed: %s\n", error, 0, 0);
+#ifdef HAVE_EBCDIC
+		strcpy( ebuf, error );
+		__etoa( ebuf );
+		error = ebuf;
 #endif
+		Debug(LDAP_DEBUG_ANY, "lt_dlexit failed: %s\n", error, 0, 0);
 
 		return -1;
 	}
@@ -83,56 +97,55 @@ int module_load(const char* file_name, int argc, char *argv[])
 	const char *error;
 	int rc;
 	MODULE_INIT_FN initialize;
+#ifdef HAVE_EBCDIC
+#define	file	ebuf
+#else
+#define	file	file_name
+#endif
 
 	module = (module_loaded_t *)ch_calloc(1, sizeof(module_loaded_t));
 	if (module == NULL) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_load:  (%s) out of memory.\n", file_name, 0, 0 );
-#else
 		Debug(LDAP_DEBUG_ANY, "module_load failed: (%s) out of memory\n", file_name,
 			0, 0);
-#endif
 
 		return -1;
 	}
 
+#ifdef HAVE_EBCDIC
+	strcpy( file, file_name );
+	__atoe( file );
+#endif
 	/*
 	 * The result of lt_dlerror(), when called, must be cached prior
 	 * to calling Debug. This is because Debug is a macro that expands
 	 * into multiple function calls.
 	 */
-	if ((module->lib = lt_dlopen(file_name)) == NULL) {
+	if ((module->lib = lt_dlopenext(file)) == NULL) {
 		error = lt_dlerror();
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, CRIT, 
-			"module_load: lt_dlopen failed: (%s) %s.\n", 
-			file_name, error, 0 );
-#else
-		Debug(LDAP_DEBUG_ANY, "lt_dlopen failed: (%s) %s\n", file_name,
-			error, 0);
+#ifdef HAVE_EBCDIC
+		strcpy( ebuf, error );
+		__etoa( ebuf );
+		error = ebuf;
 #endif
+		Debug(LDAP_DEBUG_ANY, "lt_dlopenext failed: (%s) %s\n", file_name,
+			error, 0);
 
 		ch_free(module);
 		return -1;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, "module_load: loaded module %s\n", file_name, 0, 0 );
-#else
 	Debug(LDAP_DEBUG_CONFIG, "loaded module %s\n", file_name, 0, 0);
-#endif
 
    
+#ifdef HAVE_EBCDIC
+#pragma convlit(suspend)
+#endif
 	if ((initialize = lt_dlsym(module->lib, "init_module")) == NULL) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s : no init_module() function found\n",
-		    file_name, 0, 0 );
-#else
+#ifdef HAVE_EBCDIC
+#pragma convlit(resume)
+#endif
 		Debug(LDAP_DEBUG_CONFIG, "module %s: no init_module() function found\n",
 			file_name, 0, 0);
-#endif
 
 		lt_dlclose(module->lib);
 		ch_free(module);
@@ -156,13 +169,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 	 */
 	rc = initialize(argc, argv);
 	if (rc == -1) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load:  module %s init_module() failed\n", file_name, 0, 0);
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: init_module() failed\n",
 			file_name, 0, 0);
-#endif
 
 		lt_dlclose(module->lib);
 		ch_free(module);
@@ -172,14 +180,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 	if (rc >= (int)(sizeof(module_regtable) / sizeof(struct module_regtable_t))
 		|| module_regtable[rc].proc == NULL)
 	{
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s: unknown registration type (%d).\n", 
-			file_name, rc, 0);
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: unknown registration type (%d)\n",
 			file_name, rc, 0);
-#endif
 
 		module_unload(module);
 		return -1;
@@ -187,14 +189,8 @@ int module_load(const char* file_name, int argc, char *argv[])
 
 	rc = (module_regtable[rc].proc)(module, file_name);
 	if (rc != 0) {
-#ifdef NEW_LOGGING
-		LDAP_LOG( SLAPD, ERR, 
-			"module_load: module %s:%s could not be registered.\n",
-			file_name, module_regtable[rc].type, 0 );
-#else
 		Debug(LDAP_DEBUG_CONFIG, "module %s: %s module could not be registered\n",
 			file_name, module_regtable[rc].type, 0);
-#endif
 
 		module_unload(module);
 		return rc;
@@ -203,25 +199,29 @@ int module_load(const char* file_name, int argc, char *argv[])
 	module->next = module_list;
 	module_list = module;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG( SLAPD, INFO, 
-		"module_load: module %s:%s registered\n", file_name,
-		module_regtable[rc].type, 0 );
-#else
 	Debug(LDAP_DEBUG_CONFIG, "module %s: %s module registered\n",
 		file_name, module_regtable[rc].type, 0);
-#endif
 
 	return 0;
 }
 
 int module_path(const char *path)
 {
+#ifdef HAVE_EBCDIC
+	strcpy(ebuf, path);
+	__atoe(ebuf);
+	path = ebuf;
+#endif
 	return lt_dlsetsearchpath( path );
 }
 
 void *module_resolve (const void *module, const char *name)
 {
+#ifdef HAVE_EBCDIC
+	strcpy(ebuf, name);
+	__atoe(ebuf);
+	name = ebuf;
+#endif
 	if (module == NULL || name == NULL)
 		return(NULL);
 	return(lt_dlsym(((module_loaded_t *)module)->lib, name));
@@ -246,7 +246,13 @@ static int module_unload (module_loaded_t *module)
 		}
 
 		/* call module's terminate routine, if present */
+#ifdef HAVE_EBCDIC
+#pragma convlit(suspend)
+#endif
 		if ((terminate = lt_dlsym(module->lib, "term_module"))) {
+#ifdef HAVE_EBCDIC
+#pragma convlit(resume)
+#endif
 			terminate();
 		}
 
