@@ -1,13 +1,23 @@
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/plugin.c,v 1.26 2004/01/01 18:15:39 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2002-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1997,2002-2003 IBM Corporation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * (C) Copyright IBM Corp. 1997,2002
- * Redistribution and use in source and binary forms are permitted
- * provided that this notice is preserved and that due credit is 
- * given to IBM Corporation. This software is provided ``as is'' 
- * without express or implied warranty.
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by IBM Corporation for use in
+ * IBM products and subsequently ported to OpenLDAP Software by
+ * Steve Omrani.  Additional significant contributors include:
+ *    Luke Howard
  */
 
 #include "portable.h"
@@ -20,7 +30,7 @@
  */
 #include <ltdl.h>
 
-static int loadPlugin( Slapi_PBlock *, const char *, const char *, int, 
+static int slapi_int_load_plugin( Slapi_PBlock *, const char *, const char *, int, 
 	SLAPI_FUNC *, lt_dlhandle * );
 
 /* pointer to link list of extended objects */
@@ -29,7 +39,7 @@ static ExtendedOp *pGExtendedOps = NULL;
 static Slapi_PBlock *pGPlugins = NULL;
 
 /*********************************************************************
- * Function Name:      newPlugin
+ * Function Name:      plugin_pblock_new
  *
  * Description:        This routine creates a new Slapi_PBlock structure,
  *                     loads in the plugin module and executes the init
@@ -51,8 +61,8 @@ static Slapi_PBlock *pGPlugins = NULL;
  * Messages:           None
  *********************************************************************/
 
-Slapi_PBlock *
-newPlugin(
+static Slapi_PBlock *
+plugin_pblock_new(
 	int type, 
 	const char *path, 
 	const char *initfunc, 
@@ -60,6 +70,7 @@ newPlugin(
 	char *argv[] ) 
 {
 	Slapi_PBlock	*pPlugin = NULL; 
+	Slapi_PluginDesc *pPluginDesc = NULL;
 	lt_dlhandle	hdLoadHandle;
 	int		rc;
 
@@ -70,28 +81,37 @@ newPlugin(
 	}
 
 	rc = slapi_pblock_set( pPlugin, SLAPI_PLUGIN_TYPE, (void *)type );
-	if ( rc != LDAP_SUCCESS ) {
+	if ( rc != 0 ) {
 		goto done;
 	}
 
 	rc = slapi_pblock_set( pPlugin, SLAPI_PLUGIN_ARGC, (void *)argc );
-	if ( rc != LDAP_SUCCESS ) {
+	if ( rc != 0 ) {
 		goto done;
 	}
 
 	rc = slapi_pblock_set( pPlugin, SLAPI_PLUGIN_ARGV, (void *)argv );
-	if ( rc != LDAP_SUCCESS ) { 
+	if ( rc != 0 ) { 
 		goto done;
 	}
 
-	rc = loadPlugin( pPlugin, path, initfunc, TRUE, NULL, &hdLoadHandle );
+	rc = slapi_int_load_plugin( pPlugin, path, initfunc, TRUE, NULL, &hdLoadHandle );
 	if ( rc != 0 ) {
-		rc = LDAP_OTHER;
 		goto done;
 	}
 
+	if ( slapi_pblock_get( pPlugin, SLAPI_PLUGIN_DESCRIPTION, (void **)&pPluginDesc ) == 0 &&
+	     pPluginDesc != NULL ) {
+		slapi_log_error(SLAPI_LOG_TRACE, "plugin_pblock_new",
+				"Registered plugin %s %s [%s] (%s)\n",
+				pPluginDesc->spd_id,
+				pPluginDesc->spd_version,
+				pPluginDesc->spd_vendor,
+				pPluginDesc->spd_description);
+	}
+
 done:
-	if ( rc != LDAP_SUCCESS && pPlugin != NULL ) {
+	if ( rc != 0 && pPlugin != NULL ) {
 		slapi_pblock_destroy( pPlugin );
 		pPlugin = NULL;
 	}
@@ -100,7 +120,7 @@ done:
 } 
 
 /*********************************************************************
- * Function Name:      insertPlugin
+ * Function Name:      slapi_int_register_plugin
  *
  * Description:        insert the slapi_pblock structure to the end of the plugin
  *                     list 
@@ -116,7 +136,7 @@ done:
  * Messages:           None
  *********************************************************************/
 int 
-insertPlugin(
+slapi_int_register_plugin(
 	Backend *be, 
 	Slapi_PBlock *pPB )
 { 
@@ -154,7 +174,7 @@ insertPlugin(
 }
        
 /*********************************************************************
- * Function Name:      getAllPluginFuncs
+ * Function Name:      slapi_int_get_plugins
  *
  * Description:        get the desired type of function pointers defined 
  *                     in all the plugins 
@@ -170,7 +190,7 @@ insertPlugin(
  * Messages:           None
  *********************************************************************/
 int 
-getAllPluginFuncs(
+slapi_int_get_plugins(
 	Backend *be, 		
 	int functype, 
 	SLAPI_FUNC **ppFuncPtrs )
@@ -311,7 +331,7 @@ createExtendedOp()
 
 
 /*********************************************************************
- * Function Name:      removeExtendedOp
+ * Function Name:      slapi_int_unregister_extop
  *
  * Description:        This routine removes the ExtendedOp structures 
  *					   asscoiated with a particular extended operation 
@@ -329,7 +349,7 @@ createExtendedOp()
  * Messages:           None
  *********************************************************************/
 void
-removeExtendedOp(
+slapi_int_unregister_extop(
 	Backend *pBE, 
 	ExtendedOp **opList, 
 	Slapi_PBlock *pPB )
@@ -378,7 +398,7 @@ removeExtendedOp(
 
 
 /*********************************************************************
- * Function Name:      newExtendedOp
+ * Function Name:      slapi_int_register_extop
  *
  * Description:        This routine creates a new ExtendedOp structure, loads
  *                     in the extended op module and put the extended op function address
@@ -397,7 +417,7 @@ removeExtendedOp(
  * Messages:           None
  *********************************************************************/
 int 
-newExtendedOp(
+slapi_int_register_extop(
 	Backend *pBE, 	
 	ExtendedOp **opList, 
 	Slapi_PBlock *pPB )
@@ -429,7 +449,7 @@ newExtendedOp(
 	}
 
 	rc = slapi_pblock_get( pPB,SLAPI_PLUGIN_EXT_OP_OIDLIST, &pTmpOIDs );
-	if ( rc != LDAP_SUCCESS ) {
+	if ( rc != 0 ) {
 		rc = LDAP_OTHER;
 		goto error_return;
 	}
@@ -465,7 +485,7 @@ error_return:
 }
 
 /*********************************************************************
- * Function Name:      getPluginFunc
+ * Function Name:      slapi_int_get_extop_plugin
  *
  * Description:        This routine gets the function address for a given function
  *                     name.
@@ -481,7 +501,7 @@ error_return:
  * Messages:           None
  *********************************************************************/
 int 
-getPluginFunc(
+slapi_int_get_extop_plugin(
 	struct berval *reqoid, 		
 	SLAPI_FUNC *pFuncAddr ) 
 {
@@ -512,12 +532,12 @@ getPluginFunc(
 }
 
 /***************************************************************************
- * This function is similar to getPluginFunc above. except it returns one OID
+ * This function is similar to slapi_int_get_extop_plugin above. except it returns one OID
  * per call. It is called from root_dse_info (root_dse.c).
  * The function is a modified version of get_supported_extop (file extended.c).
  ***************************************************************************/
 struct berval *
-ns_get_supported_extop( int index )
+slapi_int_get_supported_extop( int index )
 {
         ExtendedOp	*ext;
 
@@ -534,7 +554,7 @@ ns_get_supported_extop( int index )
 }
 
 /*********************************************************************
- * Function Name:      loadPlugin
+ * Function Name:      slapi_int_load_plugin
  *
  * Description:        This routine loads the specified DLL, gets and executes the init function
  *                     if requested.
@@ -558,7 +578,7 @@ ns_get_supported_extop( int index )
  *********************************************************************/
 
 static int 
-loadPlugin(
+slapi_int_load_plugin(
 	Slapi_PBlock	*pPlugin,
 	const char	*path,
 	const char	*initfunc, 
@@ -604,7 +624,7 @@ loadPlugin(
  * Special support for computed attribute plugins
  */
 int 
-doPluginFNs(
+slapi_int_call_plugins(
 	Backend		*be, 	
 	int		funcType, 
 	Slapi_PBlock	*pPB )
@@ -617,7 +637,7 @@ doPluginFNs(
 		return 1;
 	}
 
-	rc = getAllPluginFuncs( be, funcType, &tmpPlugin );
+	rc = slapi_int_get_plugins( be, funcType, &tmpPlugin );
 	if ( rc != LDAP_SUCCESS || tmpPlugin == NULL ) {
 		/* Nothing to do, front-end should ignore. */
 		return 1;
@@ -637,8 +657,11 @@ doPluginFNs(
 		 * failure (confirmed with SLAPI specification).
 		 */
 		if ( !SLAPI_PLUGIN_IS_POST_FN( funcType ) && rc != 0 ) {
-			/* make sure errors are negative */
-			if ( rc > 0 ) rc = 0 - rc;
+			/*
+			 * Plugins generally return negative error codes
+			 * to indicate failure, although in the case of
+			 * bind plugins they may return SLAPI_BIND_xxx
+			 */
 			break;
 		}
 	}
@@ -649,7 +672,7 @@ doPluginFNs(
 }
 
 int
-netscape_plugin(
+slapi_int_read_config(
 	Backend		*be, 		
 	const char	*fname, 
 	int		lineno, 
@@ -697,24 +720,24 @@ netscape_plugin(
 		int rc;
 		Slapi_PBlock *pPlugin;
 
-		pPlugin = newPlugin( iType, argv[2], argv[3], 
+		pPlugin = plugin_pblock_new( iType, argv[2], argv[3], 
 					numPluginArgc, ppPluginArgv );
 		if (pPlugin == NULL) {
 			return 1;
 		}
 
 		if (iType == SLAPI_PLUGIN_EXTENDEDOP) {
-			rc = newExtendedOp(be, &pGExtendedOps, pPlugin);
+			rc = slapi_int_register_extop(be, &pGExtendedOps, pPlugin);
 			if ( rc != LDAP_SUCCESS ) {
 				slapi_pblock_destroy( pPlugin );
 				return 1;
 			}
 		}
 
-		rc = insertPlugin( be, pPlugin );
+		rc = slapi_int_register_plugin( be, pPlugin );
 		if ( rc != LDAP_SUCCESS ) {
 			if ( iType == SLAPI_PLUGIN_EXTENDEDOP ) {
-				removeExtendedOp( be, &pGExtendedOps, pPlugin );
+				slapi_int_unregister_extop( be, &pGExtendedOps, pPlugin );
 			}
 			slapi_pblock_destroy( pPlugin );
 			return 1;
@@ -725,7 +748,7 @@ netscape_plugin(
 }
 
 int
-slapi_init(void)
+slapi_int_initialize(void)
 {
 	if ( ldap_pvt_thread_mutex_init( &slapi_hn_mutex ) ) {
 		return -1;
@@ -744,6 +767,10 @@ slapi_init(void)
 		return -1;
 	}
 
+	if ( slapi_int_init_object_extensions() != 0 ) {
+		return -1;
+	}
+
 	return 0;
 }
 
