@@ -1,14 +1,23 @@
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/proto-slapi.h,v 1.26 2004/06/01 20:07:58 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2002-2004 The OpenLDAP Foundation.
+ * Portions Copyright 1997,2002-2003 IBM Corporation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- *  Copyright IBM Corp. 1997,2002
- *  Use of this source code is subject to the terms of The OpenLDAP Public 
- *  License (version 2.7 or later).
- *  No trademarks of the IBM Corporation are to be used to identify, endorse 
- *  or promote  any products derived from this code without the prior 
- *  written consent of IBM 
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by IBM Corporation for use in
+ * IBM products and subsequently ported to OpenLDAP Software by
+ * Steve Omrani.  Additional significant contributors include:
+ *   Luke Howard
  */
 
 #ifndef _PROTO_SLAPI_H
@@ -28,9 +37,9 @@ extern int slapi_entry_attr_find( Slapi_Entry *e, char *type, Slapi_Attr **attr
 extern char *slapi_entry_attr_get_charptr( const Slapi_Entry *e, const char *type );
 extern int slapi_entry_attr_delete( Slapi_Entry *e, char *type );
 extern int slapi_entry_attr_get_int( const Slapi_Entry *e, const char *type );
-extern int slapi_entry_attr_get_long( const Slapi_Entry *e, const char *type );
-extern int slapi_entry_attr_get_uint( const Slapi_Entry *e, const char *type );
-extern int slapi_entry_attr_get_ulong( const Slapi_Entry *e, const char *type );
+extern long slapi_entry_attr_get_long( const Slapi_Entry *e, const char *type );
+extern unsigned int slapi_entry_attr_get_uint( const Slapi_Entry *e, const char *type );
+extern unsigned long slapi_entry_attr_get_ulong( const Slapi_Entry *e, const char *type );
 extern int slapi_entry_attr_hasvalue( Slapi_Entry *e, const char *type, const char *value );
 extern int slapi_entry_attr_merge_sv( Slapi_Entry *e, const char *type, Slapi_Value **vals );
 extern void slapi_entry_attr_set_charptr(Slapi_Entry* e, const char *type, const char *value);
@@ -39,6 +48,8 @@ extern void slapi_entry_attr_set_uint( Slapi_Entry* e, const char *type, unsigne
 extern void slapi_entry_attr_set_long(Slapi_Entry* e, const char *type, long l);
 extern void slapi_entry_attr_set_ulong(Slapi_Entry* e, const char *type, unsigned long l);
 extern int slapi_is_rootdse( const char *dn );
+extern int slapi_entry_has_children(const Slapi_Entry *e);
+size_t slapi_entry_size(Slapi_Entry *e);
 extern int slapi_entry_attr_merge_sv( Slapi_Entry *e, const char *type, Slapi_Value **vals );
 extern int slapi_entry_add_values_sv( Slapi_Entry *e, const char *type, Slapi_Value **vals );
 extern int slapi_entry_add_valueset(Slapi_Entry *e, const char *type, Slapi_ValueSet *vs);
@@ -50,6 +61,12 @@ extern int slapi_entry_add_string(Slapi_Entry *e, const char *type, const char *
 extern int slapi_entry_delete_string(Slapi_Entry *e, const char *type, const char *value);
 extern int slapi_entry_first_attr( const Slapi_Entry *e, Slapi_Attr **attr );
 extern int slapi_entry_next_attr( const Slapi_Entry *e, Slapi_Attr *prevattr, Slapi_Attr **attr );
+extern const char *slapi_entry_get_uniqueid( const Slapi_Entry *e );
+extern void slapi_entry_set_uniqueid( Slapi_Entry *e, char *uniqueid );
+extern int slapi_entry_schema_check( Slapi_PBlock *pb, Slapi_Entry *e );
+extern int slapi_entry_rdn_values_present( const Slapi_Entry *e );
+extern int slapi_entry_add_rdn_values( Slapi_Entry *e );
+
 extern char *slapi_entry_get_dn( Slapi_Entry *e );
 extern int slapi_x_entry_get_id( Slapi_Entry *e );
 extern void slapi_entry_set_dn( Slapi_Entry *e, char *dn );
@@ -132,6 +149,9 @@ extern void slapi_destroy_condvar( Slapi_CondVar *cvar );
 extern int slapi_wait_condvar( Slapi_CondVar *cvar, struct timeval *timeout );
 extern int slapi_notify_condvar( Slapi_CondVar *cvar, int notify_all );
 
+extern LDAP *slapi_ldap_init( char *ldaphost, int ldapport, int secure, int shared );
+extern void slapi_ldap_unbind( LDAP *ld );
+
 extern char *slapi_ch_malloc( unsigned long size );
 extern void slapi_ch_free( void **ptr );
 extern void slapi_ch_free_string( char **s );
@@ -156,6 +176,9 @@ extern char * slapi_esc_dn_normalize( char *dn );
 extern char * slapi_esc_dn_normalize_case( char *dn );
 extern int slapi_dn_isroot( Slapi_PBlock *pb, char *dn );
 extern int slapi_dn_issuffix( char *dn, char *suffix );
+char *slapi_dn_beparent( Slapi_PBlock *pb, const char *dn );
+char *slapi_dn_parent( const char *dn );
+int slapi_dn_isparent( const char *parentdn, const char *childdn );
 extern char *slapi_dn_ignore_case( char *dn );
 extern char *slapi_get_hostname();
 extern void slapi_register_supported_saslmechanism( char *mechanism );
@@ -165,6 +188,9 @@ extern int slapi_send_ldap_extended_response(Connection *conn, Operation *op,
 			int errornum, char *respName, struct berval *response);
 extern int slapi_send_ldap_search_entry( Slapi_PBlock *pb, Slapi_Entry *e, 
 			LDAPControl **ectrls, char **attrs, int attrsonly ); 
+extern int slapi_send_ldap_search_reference( Slapi_PBlock *pb, Slapi_Entry *e,
+	struct berval **references, LDAPControl **ectrls, struct berval **v2refs );
+
 extern void slapi_register_supported_control(char *controloid, 
 					unsigned long controlops);
 extern int slapi_get_supported_controls(char ***ctrloidsp, unsigned long **ctrlopsp);
@@ -207,19 +233,21 @@ extern int slapi_audit_init_header( Connection *conn, Operation *op,
 extern int slapi_audit_send_record( Slapi_PBlock *pb, Connection *conn, 
 						Operation *op, int rc);
 
-extern int slapi_x_pblock_set_operation( Slapi_PBlock *pb, Operation *op );
+extern int slapi_int_pblock_set_operation( Slapi_PBlock *pb, Operation *op );
 
-extern LDAPMod **slapi_x_modifications2ldapmods(Modifications **);
-extern Modifications *slapi_x_ldapmods2modifications(LDAPMod **);
-extern void slapi_x_free_ldapmods(LDAPMod **);
+extern LDAPMod **slapi_int_modifications2ldapmods(Modifications **);
+extern Modifications *slapi_int_ldapmods2modifications(LDAPMod **);
+extern void slapi_int_free_ldapmods(LDAPMod **);
 
 extern int slapi_compute_add_evaluator(slapi_compute_callback_t function);
 extern int slapi_compute_add_search_rewriter(slapi_search_rewrite_callback_t function);
 extern int compute_rewrite_search_filter(Slapi_PBlock *pb);
 extern int compute_evaluator(computed_attr_context *c, char *type, Slapi_Entry *e, slapi_compute_output_t outputfn);
-extern int slapi_x_compute_output_ber(computed_attr_context *c, Slapi_Attr *a, Slapi_Entry *e);
+extern int slapi_int_compute_output_ber(computed_attr_context *c, Slapi_Attr *a, Slapi_Entry *e);
 extern int slapi_x_compute_get_pblock(computed_attr_context *c, Slapi_PBlock **pb);
 
+extern int slapi_int_access_allowed(Operation *op, Entry *entry, AttributeDescription *desc, struct berval *val, slap_access_t access, AccessControlState *state);
+
 extern ldap_pvt_thread_mutex_t	slapi_hn_mutex;
 extern ldap_pvt_thread_mutex_t	slapi_time_mutex;
 extern ldap_pvt_thread_mutex_t	slapi_printmessage_mutex; 
@@ -242,24 +270,22 @@ extern void slapi_pblock_clear( Slapi_PBlock *pb );
 /*
  * OpenLDAP extensions
  */
-extern int slapi_x_pblock_get_first( Backend *be, Slapi_PBlock **pb );
-extern int slapi_x_pblock_get_next( Slapi_PBlock **pb );
+extern int slapi_int_pblock_get_first( Backend *be, Slapi_PBlock **pb );
+extern int slapi_int_pblock_get_next( Slapi_PBlock **pb );
 
 
 /*
  * Was: plugin.h
  */
 
-extern Slapi_PBlock *newPlugin ( int type, const char *path, const char *initfunc,
-		int argc, char *argv[] );
-extern int insertPlugin(Backend *be, Slapi_PBlock *pPB);
-extern int doPluginFNs(Backend *be, int funcType, Slapi_PBlock * pPB);
-extern int getAllPluginFuncs(Backend *be, int functype, SLAPI_FUNC **ppFuncPtrs);
-extern int newExtendedOp(Backend *pBE, ExtendedOp **opList, Slapi_PBlock *pPB);
-extern int getPluginFunc(struct berval  *reqoid, SLAPI_FUNC *pFuncAddr );
-extern int netscape_plugin(Backend *be, const char *fname, int lineno,
+extern int slapi_int_register_plugin(Backend *be, Slapi_PBlock *pPB);
+extern int slapi_int_call_plugins(Backend *be, int funcType, Slapi_PBlock * pPB);
+extern int slapi_int_get_plugins(Backend *be, int functype, SLAPI_FUNC **ppFuncPtrs);
+extern int slapi_int_register_extop(Backend *pBE, ExtendedOp **opList, Slapi_PBlock *pPB);
+extern int slapi_int_get_extop_plugin(struct berval  *reqoid, SLAPI_FUNC *pFuncAddr );
+extern int slapi_int_read_config(Backend *be, const char *fname, int lineno,
 		int argc, char **argv );
-extern int slapi_init(void);
+extern int slapi_int_initialize(void);
 
 
 /*
@@ -268,9 +294,6 @@ extern int slapi_init(void);
 
 extern Slapi_PBlock *slapi_search_internal( char *base, int scope, char *filter, 
 		LDAPControl **controls, char **attrs, int attrsonly );
-extern Slapi_PBlock *slapi_search_internal_bind( char *bindDN, char *base,
-		int scope, char *filter, LDAPControl **controls,
-		char **attrs, int attrsonly ); /* d58508 */
 extern Slapi_PBlock *slapi_modify_internal( char *dn, LDAPMod **mods,
         LDAPControl **controls, int log_change );
 extern Slapi_PBlock *slapi_add_entry_internal( Slapi_Entry * e,
@@ -283,14 +306,10 @@ extern Slapi_PBlock *slapi_delete_internal( char * dn,  LDAPControl **controls,
 		int log_change );
 extern Slapi_PBlock *slapi_modrdn_internal( char * olddn, char * newrdn,
 		int deloldrdn, LDAPControl **controls, int log_change);
-#if 0
-extern Slapi_PBlock *slapi_modrdn_internal( char * olddn, char * newrdn,
-		char *newParent, int deloldrdn, LDAPControl **controls,
-		int log_change);
-#endif
 extern char **slapi_get_supported_extended_ops(void);
-extern int duplicateBVMod( LDAPMod *pMod, LDAPMod **ppNewMod );
-
+extern struct berval *slapi_int_get_supported_extop( int );
+extern Connection *slapi_int_init_connection(char *DN, int OpType);
+extern void slapi_int_connection_destroy( Connection **pConn );
 
 /*
  * Was: slapi_cl.h
@@ -311,6 +330,11 @@ extern int slapi_modifyrdn_changelog(char *olddn, char *newRdn, int delRdn,
 		char *suffix, char *chNum, Operation* op);
 extern Backend * slapi_cl_get_be(char *dn);
 
+int slapi_int_init_object_extensions(void);
+int slapi_int_free_object_extensions(int objecttype, void *object);
+int slapi_int_create_object_extensions(int objecttype, void *object);
+int slapi_int_clear_object_extensions(int objecttype, void *object);
+
 LDAP_END_DECL
 
 #endif /* _PROTO_SLAPI_H */
