@@ -1,7 +1,19 @@
 /* unbind.c - decode an ldap unbind operation and pass it to a backend db */
-
-/*
- * Copyright (c) 1995 Regents of the University of Michigan.
+/* $OpenLDAP: pkg/ldap/servers/slapd/unbind.c,v 1.23 2004/09/26 21:00:00 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Portions Copyright (c) 1995 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -21,12 +33,8 @@
 
 #include "slap.h"
 
-
 int
-do_unbind(
-    Connection	*conn,
-    Operation	*op
-)
+do_unbind( Operation *op, SlapReply *rs )
 {
 	Debug( LDAP_DEBUG_TRACE, "do_unbind\n", 0, 0, 0 );
 
@@ -36,11 +44,11 @@ do_unbind(
 	 *	UnBindRequest ::= NULL
 	 */
 
-	Statslog( LDAP_DEBUG_STATS, "conn=%d op=%d UNBIND\n", op->o_connid,
-	    op->o_opid, 0, 0, 0 );
+	Statslog( LDAP_DEBUG_STATS, "%s UNBIND\n", op->o_log_prefix,
+		0, 0, 0, 0 );
 
 	/* pass the unbind to all backends */
-	backend_unbind( conn, op );
+	backend_unbind( op, rs );
 
 	return 0;
 }
