@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/args.c,v 1.25 2003/03/18 16:52:48 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/args.c,v 1.29 2004/03/19 07:06:42 hyc Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +22,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 /*
  * args.c - process command-line arguments, and set appropriate globals.
@@ -29,6 +41,7 @@
 #include <ac/unistd.h>
 
 #include <ldap.h>
+#include <lutil.h>
 
 #include "slurp.h"
 #include "globals.h"
@@ -62,13 +75,9 @@ doargs(
     int		i;
     int		rflag = 0;
 
-    if ( (g->myname = strrchr( argv[0], LDAP_DIRSEP[0] )) == NULL ) {
-	g->myname = strdup( argv[0] );
-    } else {
-	g->myname = strdup( g->myname + 1 );
-    }
+    g->myname = strdup( lutil_progname( "slurpd", argc, argv ));
 
-    while ( (i = getopt( argc, argv, "d:f:n:or:t:" )) != EOF ) {
+    while ( (i = getopt( argc, argv, "d:f:n:or:t:V" )) != EOF ) {
 	switch ( i ) {
 	case 'd':	/* set debug level and 'do not detach' flag */
 	    g->no_detach = 1;
@@ -108,6 +117,7 @@ doargs(
 #endif /* LDAP_DEBUG */
 	    break;
 	case 'f':	/* slapd config file */
+	    LUTIL_SLASHPATH( optarg );
 	    g->slapd_configfile = strdup( optarg );
 	    break;
 	case 'n':	/* NT service name */
@@ -118,16 +128,21 @@ doargs(
 	    g->one_shot_mode = 1;
 	    break;
 	case 'r':	/* slapd replog file */
+	    LUTIL_SLASHPATH( optarg );
 		snprintf( g->slapd_replogfile, sizeof g->slapd_replogfile,
 			"%s", optarg );
 	    rflag++;
 	    break;
 	case 't': {	/* dir to use for our copies of replogs */
 		size_t sz;
-	    g->slurpd_rdir = (char *)malloc (sz = (strlen(optarg) + sizeof("/replica")));
+	    LUTIL_SLASHPATH( optarg );
+	    g->slurpd_rdir = (char *)malloc (sz = (strlen(optarg) + sizeof(LDAP_DIRSEP "replica")));
 	    snprintf(g->slurpd_rdir, sz,
 			"%s" LDAP_DIRSEP "replica", optarg);
 	    } break;
+	case 'V':
+	    (g->version)++;
+	    break;
 	default:
 	    usage( g->myname );
 	    return( -1 );
