@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/reject.c,v 1.7 2002/01/04 20:17:57 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/reject.c,v 1.18 2004/09/04 02:54:34 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +22,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 
 /*
@@ -26,6 +38,7 @@
 
 #include <stdio.h>
 
+#include <ac/stdlib.h>
 #include <ac/errno.h>
 #include <ac/unistd.h>
 
@@ -35,6 +48,12 @@
 #include "slurp.h"
 #include "globals.h"
 
+#ifdef _WIN32
+#define	PORTSEP	","
+#else
+#define	PORTSEP	":"
+#endif
+
 /*
  * Write a replication record to a reject file.  The reject file has the
  * same name as the replica's private copy of the file but with ".rej"
@@ -56,13 +75,13 @@ write_reject(
     int		rc;
 
     ldap_pvt_thread_mutex_lock( &sglob->rej_mutex );
-    sprintf( rejfile, "%s/%s:%d.rej", sglob->slurpd_rdir,
-	    ri->ri_hostname, ri->ri_port );
+    snprintf( rejfile, sizeof rejfile, "%s" LDAP_DIRSEP "%s" PORTSEP "%d.rej",
+		sglob->slurpd_rdir, ri->ri_hostname, ri->ri_port );
 
     if ( access( rejfile, F_OK ) < 0 ) {
 	/* Doesn't exist - try to create */
 	int rjfd;
-	if (( rjfd = open( rejfile, O_RDWR | O_APPEND | O_CREAT,
+	if (( rjfd = open( rejfile, O_RDWR|O_APPEND|O_CREAT|O_EXCL,
 		S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP )) < 0 ) {
 	    Debug( LDAP_DEBUG_ANY,
 		"Error: write_reject: Cannot create \"%s\": %s\n",
@@ -78,11 +97,11 @@ write_reject(
 		rejfile, 0, 0 );
     } else {
 	fseek( rfp, 0, 2 );
-	if ( errmsg != NULL ) {
-	    fprintf( rfp, "%s: %s\n", ERROR_STR, errmsg );
-	} else {
-	    fprintf( rfp, "%s: %s\n", ERROR_STR, ldap_err2string( lderr ));
+	fprintf( rfp, "%s: %s", ERROR_STR, ldap_err2string( lderr ));
+	if ( errmsg && *errmsg ) {
+	    fprintf( rfp, ": %s", errmsg );
 	}
+	fprintf( rfp, "\n" );
 	if ((rc = re->re_write( ri, re, rfp )) < 0 ) {
 	    Debug( LDAP_DEBUG_ANY,
 		    "Error: cannot write reject file \"%s\"\n",
