@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/replica.c,v 1.20 2003/01/03 19:21:00 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/replica.c,v 1.23 2004/09/04 02:54:34 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +22,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 
 /*
@@ -39,25 +51,13 @@ replicate(
 {
     Ri		*ri = (Ri *) ri_arg;
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( SLURPD, ARGS, "replicate: "
-		"begin replication thread for %s:%d\n",
-	    ((Ri *)ri)->ri_hostname, ((Ri *)ri)->ri_port, 0 );
-#else
     Debug( LDAP_DEBUG_ARGS, "begin replication thread for %s:%d\n",
 	    ((Ri *)ri)->ri_hostname, ((Ri *)ri)->ri_port, 0 );
-#endif
 
     ri->ri_process( ri );
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( SLURPD, ARGS, "replicate: "
-		"begin replication thread for %s:%d\n",
-	    ri->ri_hostname, ri->ri_port, 0 );
-#else
     Debug( LDAP_DEBUG_ARGS, "end replication thread for %s:%d\n",
 	    ri->ri_hostname, ri->ri_port, 0 );
-#endif
     return NULL;
 }
 
@@ -74,14 +74,8 @@ start_replica_thread(
     /* POSIX_THREADS or compatible */
     if ( ldap_pvt_thread_create( &(ri->ri_tid), 0, replicate,
 	    (void *) ri ) != 0 ) {
-#ifdef NEW_LOGGING
-	LDAP_LOG ( SLURPD, ERR, "start_replica_thread: "
-		"replica %s:%d ldap_pvt_thread_create failed\n",
-	    ri->ri_hostname, ri->ri_port, 0 );
-#else
 	Debug( LDAP_DEBUG_ANY, "replica \"%s:%d\" ldap_pvt_thread_create failed\n",
 		ri->ri_hostname, ri->ri_port, 0 );
-#endif
 	return -1;
     }
 
