@@ -1,10 +1,18 @@
-/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.30 2003/03/03 10:47:17 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slurpd/slurp.h,v 1.37.2.1 2004/12/30 21:29:59 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/*
- * Copyright (c) 1996 Regents of the University of Michigan.
+/* Portions Copyright (c) 1996 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -14,6 +22,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 /* slurp.h - Standalone Ldap Update Replication Daemon (slurpd) */
 
@@ -43,8 +55,10 @@
 #include "ldif.h"
 
 #ifdef HAVE_WINSOCK
+	/* should be moved to portable.h.nt */
 #define ftruncate(a,b) _chsize(a,b)
 #define truncate(a,b) _lclose( _lcreat(a, 0))
+#define mkdir(a,b)	mkdir(a)
 #define S_IRGRP 0
 #define S_IWGRP 0
 #endif
@@ -64,9 +78,6 @@
 /* slurpd dump file - contents of rq struct are written here (debugging) */
 #define	SLURPD_DUMPFILE			LDAP_TMPDIR LDAP_DIRSEP "slurpd.dump"
 
-/* default srvtab file.  Can be overridden */
-#define	SRVTAB				"/etc/srvtab"
-
 /* Amount of time to sleep if no more work to do */
 #define	DEFAULT_NO_WORK_INTERVAL	3
 
@@ -84,11 +95,6 @@
 #define TLS_ON			1
 #define TLS_CRITICAL	2
 
-/* We support simple (plaintext password) and SASL authentication */
-#define	AUTH_SIMPLE	1
-#define	AUTH_KERBEROS	2
-#define	AUTH_SASL 3
-
 /* Rejection records are prefaced with this string */
 #define	ERROR_STR	"ERROR"
 
@@ -117,8 +123,10 @@
 #define	T_MODOPREPLACE		9
 #define	T_MODOPDELETESTR	"delete"
 #define	T_MODOPDELETE		10
+#define	T_MODOPINCREMENTSTR	"increment"
+#define	T_MODOPINCREMENT	11
 #define	T_MODSEPSTR		"-"
-#define	T_MODSEP		11
+#define	T_MODSEP		12
 
 #define	T_NEWRDNSTR		"newrdn"
 #define	T_DELOLDRDNSTR	"deleteoldrdn"
@@ -128,6 +136,7 @@
 
 /* Config file keywords */
 #define	HOSTSTR			"host"
+#define	URISTR			"uri"
 #define	ATTRSTR			"attr"
 #define	SUFFIXSTR		"suffix"
 #define	BINDDNSTR		"binddn"
@@ -143,8 +152,9 @@
 #define	SASLMECHSTR		"saslmech"
 #define	REALMSTR		"realm"
 #define	SECPROPSSTR		"secprops"
+#define STARTTLSSTR		"starttls"
 #define TLSSTR			"tls"
-#define TLSCRITICALSTR	"critical"
+#define CRITICALSTR		"critical"
 
 #define	REPLICA_SLEEP_TIME	( 10 )
 
@@ -213,6 +223,7 @@ struct ri {
     /* Private data */
     char	*ri_hostname;		/* canonical hostname of replica */
     int		ri_port;		/* port where slave slapd running */
+    char	*ri_uri;		/* e.g. "ldaps://ldap-1.example.com:636" */
     LDAP	*ri_ldp;		/* LDAP struct for this replica */
     int		ri_tls;			/* TLS: 0=no, 1=yes, 2=critical */
     int		ri_bind_method;		/* AUTH_SIMPLE or AUTH_KERBEROS */
@@ -234,7 +245,6 @@ struct ri {
     int (*ri_process) LDAP_P(( Ri * ));	/* process the next repl entry */
     void (*ri_wake)   LDAP_P(( Ri * ));	/* wake up a sleeping thread */
 };
-    
 
 
 
@@ -243,12 +253,10 @@ struct ri {
  * be considered private to routines in re.c, and to routines in ri.c.
  */
 typedef struct mi {
-    
     /* Private data */
     char	*mi_type;		/* attr or type */
     char	*mi_val;		/* value */
     int		mi_len;			/* length of mi_val */
-
 } Mi;
 
 
@@ -260,7 +268,6 @@ typedef struct mi {
  */
 typedef struct re Re;
 struct re {
-
     /* Private data */
     ldap_pvt_thread_mutex_t
 		re_mutex;		/* mutex for this Re */
