@@ -1,8 +1,22 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-addel.c,v 1.22 2003/02/22 11:55:05 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-addel.c,v 1.27 2004/09/04 16:31:43 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Kurt Spanier for inclusion
+ * in OpenLDAP Software.
+ */
+ 
 #include "portable.h"
 
 #include <stdio.h>
@@ -16,6 +30,7 @@
 #include <ac/unistd.h>
 #include <ac/wait.h>
 
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 
 #define LOOPS	100
@@ -240,6 +255,7 @@ do_addel(
 	LDAP	*ld = NULL;
 	int  	i;
 	pid_t	pid = getpid();
+	int	rc = LDAP_SUCCESS;
 
 	if ( uri ) {
 		ldap_initialize( &ld, uri );
@@ -270,19 +286,22 @@ do_addel(
 	for ( i = 0; i < maxloop; i++ ) {
 
 		/* add the entry */
-		if ( ldap_add_s( ld, entry, attrs ) != LDAP_SUCCESS ) {
-
+		rc = ldap_add_s( ld, entry, attrs );
+		if ( rc != LDAP_SUCCESS ) {
 			ldap_perror( ld, "ldap_add" );
 			break;
 
 		}
 
+#if 0
 		/* wait a second for the add to really complete */
+		/* This masks some race conditions though. */
 		sleep( 1 );
+#endif
 
 		/* now delete the entry again */
-		if ( ldap_delete_s( ld, entry ) != LDAP_SUCCESS ) {
-
+		rc = ldap_delete_s( ld, entry );
+		if ( rc != LDAP_SUCCESS ) {
 			ldap_perror( ld, "ldap_delete" );
 			break;
 
@@ -290,7 +309,7 @@ do_addel(
 
 	}
 
-	fprintf( stderr, " PID=%ld - Add/Delete done.\n", (long) pid );
+	fprintf( stderr, " PID=%ld - Add/Delete done (%d).\n", (long) pid, rc );
 
 	ldap_unbind( ld );
 }
