@@ -1,8 +1,22 @@
-/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.22 2003/02/22 11:01:59 hyc Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.27 2004/09/04 16:31:43 ando Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1999-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Kurt Spanier for inclusion
+ * in OpenLDAP Software.
+ */
+
 #include "portable.h"
 
 #include <stdio.h>
@@ -25,17 +39,19 @@
 #define READCMD			"slapd-read"
 #define ADDCMD			"slapd-addel"
 #define MODRDNCMD		"slapd-modrdn"
-#define MAXARGS      	100
-#define MAXREQS			20
+#define MODIFYCMD		"slapd-modify"
+#define MAXARGS      		100
+#define MAXREQS			5000
 #define LOOPS			"100"
 
 #define TSEARCHFILE		"do_search.0"
 #define TREADFILE		"do_read.0"
 #define TADDFILE		"do_add."
 #define TMODRDNFILE		"do_modrdn.0"
+#define TMODIFYFILE		"do_modify.0"
 
 static char *get_file_name( char *dirname, char *filename );
-static int  get_search_filters( char *filename, char *filters[] );
+static int  get_search_filters( char *filename, char *filters[], char *bases[] );
 static int  get_read_entries( char *filename, char *entries[] );
 static void fork_child( char *prog, char **args );
 static void	wait4kids( int nkidval );
@@ -54,7 +70,7 @@ static char argbuf[BUFSIZ];
 static void
 usage( char *name )
 {
-	fprintf( stderr, "usage: %s [-h <host>] -p <port> -D <manager> -w <passwd> -d <datadir> -b <baseDN> [-j <maxchild>] [-l <loops>] -P <progdir>\n", name );
+	fprintf( stderr, "usage: %s -H <uri> | ([-h <host>] -p <port>) -D <manager> -w <passwd> -d <datadir> [-j <maxchild>] [-l <loops>] -P <progdir>\n", name );
 	exit( EXIT_FAILURE );
 }
 
@@ -68,13 +84,13 @@ main( int argc, char **argv )
 	char		*manager = NULL;
 	char		*passwd = NULL;
 	char		*dirname = NULL;
-	char        *sbase = NULL;
 	char		*progdir = NULL;
 	char		*loops = LOOPS;
 	DIR			*datadir;
 	struct dirent	*file;
 	char		*sfile = NULL;
 	char		*sreqs[MAXREQS];
+	char		*sbase[MAXREQS];
 	int         snum = 0;
 	char		*rfile = NULL;
 	char		*rreqs[MAXREQS];
@@ -96,6 +112,13 @@ main( int argc, char **argv )
 	char		*margs[MAXARGS];
 	int		manum;
 	char		mcmd[MAXPATHLEN];
+	char		*modargs[MAXARGS];
+	int		modanum;
+	char		modcmd[MAXPATHLEN];
+	char		*modfile = NULL;
+	char		*modreqs[MAXREQS];
+	char		*moddn[MAXREQS];
+	int		modnum = 0;
 
 	while ( (i = getopt( argc, argv, "H:h:p:D:w:b:d:j:l:P:" )) != EOF ) {
 		switch( i ) {
@@ -119,10 +142,6 @@ main( int argc, char **argv )
 				passwd = ArgDup( optarg );
 				break;
 
-			case 'b':		/* the base DN */
-				sbase = ArgDup( optarg );
-				break;
-
 			case 'd':		/* data directory */
 				dirname = strdup( optarg );
 			break;
@@ -145,7 +164,7 @@ main( int argc, char **argv )
 		}
 	}
 
-	if (( dirname == NULL ) || ( sbase == NULL ) || ( port == NULL && uri == NULL ) ||
+	if (( dirname == NULL ) || ( port == NULL && uri == NULL ) ||
 			( manager == NULL ) || ( passwd == NULL ) || ( progdir == NULL ))
 		usage( argv[0] );
 
@@ -173,6 +192,9 @@ main( int argc, char **argv )
 		} else if ( !strcasecmp( file->d_name, TMODRDNFILE )) {
 			mfile = get_file_name( dirname, file->d_name );
 			continue;
+		} else if ( !strcasecmp( file->d_name, TMODIFYFILE )) {
+			modfile = get_file_name( dirname, file->d_name );
+			continue;
 		} else if ( !strncasecmp( file->d_name, TADDFILE, strlen( TADDFILE ))
 			&& ( anum < MAXREQS )) {
 			afiles[anum++] = get_file_name( dirname, file->d_name );
@@ -184,7 +206,7 @@ main( int argc, char **argv )
 
 	/* look for search requests */
 	if ( sfile ) {
-		snum = get_search_filters( sfile, sreqs );
+		snum = get_search_filters( sfile, sreqs, sbase );
 	}
 
 	/* look for read requests */
@@ -196,6 +218,10 @@ main( int argc, char **argv )
 	if ( mfile ) {
 		mnum = get_read_entries( mfile, mreqs );
 	}
+	/* look for modify requests */
+	if ( modfile ) {
+		modnum = get_search_filters( modfile, modreqs, moddn );
+	}
 
 	/*
 	 * generate the search clients
@@ -214,10 +240,14 @@ main( int argc, char **argv )
 		sargs[sanum++] = "-p";
 		sargs[sanum++] = port;
 	}
-	sargs[sanum++] = "-b";
-	sargs[sanum++] = sbase;
+	sargs[sanum++] = "-D";
+	sargs[sanum++] = manager;
+	sargs[sanum++] = "-w";
+	sargs[sanum++] = passwd;
 	sargs[sanum++] = "-l";
 	sargs[sanum++] = loops;
+	sargs[sanum++] = "-b";
+	sargs[sanum++] = NULL;		/* will hold the search base */
 	sargs[sanum++] = "-f";
 	sargs[sanum++] = NULL;		/* will hold the search request */
 	sargs[sanum++] = NULL;
@@ -271,6 +301,35 @@ main( int argc, char **argv )
 	margs[manum++] = "-e";
 	margs[manum++] = NULL;		/* will hold the modrdn entry */
 	margs[manum++] = NULL;
+	
+	/*
+	 * generate the modify clients
+	 */
+
+	modanum = 0;
+	snprintf( modcmd, sizeof modcmd, "%s" LDAP_DIRSEP MODIFYCMD,
+		progdir );
+	modargs[modanum++] = modcmd;
+	if ( uri ) {
+		modargs[modanum++] = "-H";
+		modargs[modanum++] = uri;
+	} else {
+		modargs[modanum++] = "-h";
+		modargs[modanum++] = host;
+		modargs[modanum++] = "-p";
+		modargs[modanum++] = port;
+	}
+	modargs[modanum++] = "-D";
+	modargs[modanum++] = manager;
+	modargs[modanum++] = "-w";
+	modargs[modanum++] = passwd;
+	modargs[modanum++] = "-l";
+	modargs[modanum++] = loops;
+	modargs[modanum++] = "-e";
+	modargs[modanum++] = NULL;		/* will hold the modify entry */
+	modargs[modanum++] = "-a";;
+	modargs[modanum++] = NULL;		/* will hold the ava */
+	modargs[modanum++] = NULL;
 
 	/*
 	 * generate the add/delete clients
@@ -304,6 +363,7 @@ main( int argc, char **argv )
 		if ( j < snum ) {
 
 			sargs[sanum - 2] = sreqs[j];
+			sargs[sanum - 4] = sbase[j];
 			fork_child( scmd, sargs );
 
 		}
@@ -320,6 +380,13 @@ main( int argc, char **argv )
 			margs[manum - 2] = mreqs[j];
 			fork_child( mcmd, margs );
 
+		}
+		if ( j < modnum ) {
+
+			modargs[modanum - 4] = moddn[j];
+			modargs[modanum - 2] = modreqs[j];
+			fork_child( modcmd, modargs );
+
 		}
 
 		if ( j < anum ) {
@@ -348,7 +415,7 @@ get_file_name( char *dirname, char *filename )
 
 
 static int
-get_search_filters( char *filename, char *filters[] )
+get_search_filters( char *filename, char *filters[], char *bases[] )
 {
 	FILE    *fp;
 	int     filter = 0;
@@ -361,6 +428,11 @@ get_search_filters( char *filename, char *filters[] )
 
 			if (( nl = strchr( line, '\r' )) || ( nl = strchr( line, '\n' )))
 				*nl = '\0';
+			bases[filter] = ArgDup( line );
+			fgets( line, BUFSIZ, fp );
+			if (( nl = strchr( line, '\r' )) || ( nl = strchr( line, '\n' )))
+				*nl = '\0';
+
 			filters[filter++] = ArgDup( line );
 
 		}
